#!/bin/sh

#
# make-is-tree cvsdir builddir
# update-is-tree cvsdir builddir
#
# make-is-tree generates a build tree for install shield from a CVS tree.  This
# is needed due to InstallShield *requiring* directories and files with spaces
# in them.  Checking in files with spaces will cause command-line tools grief,
# so we replace the files with with `_' in the source tree.  This script
# creates a build directory that contains the source files with the spaces
# back in them.
#
# If changes are made to the tree, then update-is-tree can be used to copy
# the files back.  It will only copy files that are already the CVS.  New
# file must be added by hand.
#
#  o cvsdir - The project directory (CVS working directory).
#  o builddir - The temporary directory for InstallShield.
# 

REQUIRED_DIRS="
Setup_Files/Compressed_Files/0009-English/Intel_32
Setup_Files/Compressed_Files/0009-English/OS_Independent
Setup_Files/Compressed_Files/Language_Independent/Intel_32
Setup_Files/Compressed_Files/Language_Independent/OS_Independent
Setup_Files/Uncompressed_Files/0009-English/Intel_32
Setup_Files/Uncompressed_Files/0009-English/OS_Independent
Setup_Files/Uncompressed_Files/Language_Independent/Intel_32
Setup_Files/Uncompressed_Files/Language_Independent/OS_Independent"


# Convert underbars to blanks in $1.
toBlanks() {
    echo $1 | sed -e 's/_/ /g'
}

# Entry...
set -e

# Determine if this is the make or update script
case "`basename $0`" in 
    make-is-tree) is_update=NO;;
    update-is-tree) is_update=YES;;
    *) echo "Error: script basename must be make-is-tree or update-is-tree"
       exit 1;;
esac


# Parse arguments
if [ $# != 2 ] ; then
    echo "$0 cvsdir builddir" >&2
    exit 1
fi
cvsdir="$1"
builddir="$2"

if [ ! -r $cvsdir/CVS ] ; then
    echo Error: No directory: $cvsdir/CVS >&2
    echo Must specify top of install CVS working directory >&2
    exit 1
fi

# Check that the paths for spaces.
if [ "X${cvsdir}" != "X`toBlanks ${cvsdir}`" ] ; then
    echo Error: CVS directory path must not contain any blanks >&2
    exit 1
fi
if [ "X${builddir}" != "X`toBlanks ${builddir}`" ] ; then
    echo Error: Build directory path must not contain any blanks >&2
    exit 1
fi

# Get list of directories and files.
dirs=`cd $cvsdir && find . -type d | fgrep -v /CVS`
files=`cd $cvsdir && find . -type f | fgrep -v /CVS`

if [ "X${is_update}" = "XNO" ] ; then
    # Make the directories:
    for d in ${dirs} ; do
        mkdir -p "${builddir}/`toBlanks $d`"
    done

    # Copy files:
    for f in ${files} ; do
        cp -f $cvsdir/$f "${builddir}/`toBlanks $f`"
    done

    # Create empty directories that must be there.
    for f in ${REQUIRED_DIRS} ; do
        mkdir -p "${builddir}/`toBlanks $f`"
    done

else
    # Update modified files:
    for f in ${files} ; do
        s="${builddir}/`toBlanks $f`"
        cmp -s ${cvsdir}/$f "$s" || {
            echo Updating ${cvsdir}/$f
            cp -f "$s" ${cvsdir}/$f 
        }
    done
fi
