##############################################################################
#                                                                            #
#                        RPM spec file for enhydra                           #
#                                                                            #
##############################################################################

##############################################################################
# Preamble -- describe the package                                           #
##############################################################################
Summary: Enhydra is an Open Source Java application server run-time and development environment.
Name: Enhydra
Version: @VERSION@
Release: @RELEASE@
Copyright: FreeBSD-style
Group: Applications/Internet
Source: http://www.enhydra.org/cuts/@BASENAME@
URL: http://www.enhydra.org
Packager: Enhydra <enhydra@enhydra.org>
Icon: Enhydra.gif
vendor: Lutris Technologies
distribution: @VERSION@
#
# Prefix can be defined by the user on install.  All occurances of
# /usr/local will be replaced with the directory following the --prefix flag
#
# ex. rpm -ivh --prefix /home/foo enhydra-*-1.i386.rpm
#
Prefix: @PREFIX@

%description
Enhydra is an Open Source Java application server run-time 
and development environment.  Its run-time modules and tools include:
     o Enhydra Application Framework - Super-servlet run-time
         environment of common services (session, presentation, database
         connectivity) for supporting N-tier Enhydra applications. 
     o Enhydra Multiserver - a servlet running environment with
         embedded servlet management, monitoring and debugging. 
     o Enhydra XMLC - XML Compiler (to-be-released), designed to
         support designer/developer co-development. 
     o Enhydra JDDI - a structured approach to using embedded Java for
         dynamic HTML. 
     o Enhydra DODS - a graphical tool for Object-to-relational database
         mapping.

##############################################################################
# This script get executed prior to package build                            #
#   rpm -ba enhydra-*.spec                                                   #
##############################################################################
%prep
#
# Remove any old sources from previous installs
#
if [ -d $RPM_BUILD_DIR/usr/local/@INFLATE_DIR@ ]
then
    rm -rf $RPM_BUILD_DIR/usr/local/@INFLATE_DIR@
fi


#
# Extract source files to build directory
#
mkdir -p $RPM_BUILD_DIR@PREFIX@
cd $RPM_BUILD_DIR@PREFIX@
tar -zxvf $RPM_SOURCE_DIR/@BASENAME@

chgrp -R root $RPM_BUILD_DIR/usr
chown -R root $RPM_BUILD_DIR/usr

##############################################################################
# This script get executed during package build                              #
#   rpm -ba enhydra-*.spec                                                   #
##############################################################################
%build
#
# Build the Source 
#
# The following libraries and tools are required for full build of Enhydra
#

##############################################################################
# This script get executed during package install                            #
#   rpm -ba enhydra-*.spec                                                   #
##############################################################################
%install
#
# remove source, prepare binaries fro installation
#


##############################################################################
# This script get executed after package install                             #
#   rpm -ba enhydra-*.spec                                                   #
##############################################################################
%clean
#
# nothing to do here
#

##############################################################################
# This script section get executed prior to install                          #
#   rpm -ivh enhydra-*-1.i386.rpm                                            #
##############################################################################
%pre
#
# Make sure we know where the jdk is installed.  This makes for a
# cleaner failure than if we wait until after we've extracted files.
#
# Note: jdk117_v1a is the default directory name for the blackdown.org
# 1.1.7 release.

#
#find path, maybe jdk is in it - will first look for environment variable
#JDK_HOME, if not set, then check path, lastly, look in common system
#locations
#

JDKDIR_LIST="java1.2.2 \
	jdk1.2 \
	jdk1.2.2 \
	jdk122 \
	java1.2 \
	jdk1.3 \
	jdk13 \
	java1.3 \
	java13"

LOCATION_LIST="/usr/local \
	/usr/local/java \
	/usr/java
	/ \
	/usr \
	/home \
	/java \
	/opt \
	/opt/java"

#
# Check JDKDIR_LIST for existing directories, assign and break at first find
# If no match found, keep going, alert user they must run the configure script
#
for j in $JDKDIR_LIST
do
   for l in $LOCATION_LIST
   do
 	if [ -d $l/$j ]
	then
	   JDKDIR=$l/$j
	   break
	fi
   done
done

if [ ! -d $RPM_INSTALL_PREFIX/@INFLATE_DIR@ ]; then
    mkdir -p $RPM_INSTALL_PREFIX/@INFLATE_DIR@
fi

#
# echo the results into a file
#
if [ ! -z $JDKDIR ] && [ $JDKDIR != "" ]
then
    echo $JDKDIR > $RPM_INSTALL_PREFIX/@INFLATE_DIR@/jdkdir.tmp
fi
##############################################################################
# This script section get executed after install                             #
#   rpm -ivh enhydra-*-1.i386.rpm                                            #
##############################################################################
%post
#
# Get the jdk install directory that we determined in the 'pre' phase.
#
JDKDIR=""
if [ -f $RPM_INSTALL_PREFIX/@INFLATE_DIR@/jdkdir.tmp ]
then
    read JDKDIR < $RPM_INSTALL_PREFIX/@INFLATE_DIR@/jdkdir.tmp
    rm $RPM_INSTALL_PREFIX/@INFLATE_DIR@/jdkdir.tmp
fi

# This should never happen; a missing JDK dir should be caught in 
# the 'pre' phase.  But you know how it is with things that should never
# happen.

if [ ! -z $JDKDIR ] && [ $JDKDIR != "" ] 
then
   chmod 755 $RPM_INSTALL_PREFIX/@INFLATE_DIR@/configure
   cd $RPM_INSTALL_PREFIX/@INFLATE_DIR@
   ./configure $JDKDIR
   echo "********************************************************************************" > rpm_config.status
   echo "*     Congratulations - Configuration was completed upon Installation          *" >> rpm_config.status
   echo "*     Enhydra was configured with the jdk located in $JDKDIR                    " >> rpm_config.status
   echo "********************************************************************************" >> rpm_config.status
   echo "* If you wish to reconfigure enhydra to use a different jdk, consult           *" >> rpm_config.status
   echo "* Install.html in the doc directory for instructions on running the configure  *" >> rpm_config.status
   echo "* script                                                                       *" >> rpm_config.status
   echo "********************************************************************************" >> rpm_config.status 
else
   cd $RPM_INSTALL_PREFIX/@INFLATE_DIR@
   echo "********************************************************************************" > rpm_config.status
   echo "*     Error - Enhydra could not be configured upon installation!               *" >> rpm_config.status
   echo "*          Your installation of the jdk could not be located                   *" >> rpm_config.status
   echo "********************************************************************************" >> rpm_config.status
   echo "* For instructions on how to configure Enhydra, please consult the             *" >> rpm_config.status
   echo "* Install.html file in the doc directory                                       *" >> rpm_config.status
   echo "********************************************************************************" >> rpm_config.status 
fi


##############################################################################
# This script section get executed before uninstall                          #
#   rpm -e enhydra-*-1                                                       #
##############################################################################
%preun
#
# nothing to do here
#

##############################################################################
# This script section get executed after uninstall                           #
#   rpm -e enhydra-*-1                                                       #
##############################################################################
%postun
#
# Clean all remaining Enhydra directories.  They are left behind because
# of files that 'configure' creates in them.  But...we only want to remove
# them if we're really doing an erase!  This uninstall script also gets run
# during an upgrade, to uninstall the old version.
# If the enhydra.jar file is missing, assume that we're doing an erase.
#
if [ ! -f $RPM_INSTALL_PREFIX/@INFLATE_DIR@/lib/enhydra.jar ] 
then
    rm -rf $RPM_INSTALL_PREFIX/@INFLATE_DIR@
fi

##############################################################################
# List of files installed by package                                         #
##############################################################################
%files
