/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HtmlTableCell.java,v 1.9.12.1 2000/10/19 17:58:53 jasona Exp $
 */





package com.lutris.html;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This base class is extended to compose a particular type of cell
 * within a row within a HTML table. Normally the entire row will be
 * composed of either data or header cells.
 *
 * @see HtmlTable
 * @see HtmlTableRow
 * @see HtmlTableDataCell
 * @see HtmlTableHeaderCell
 * @author Kyle Clark
 * @author Paul Morgan
 * @version 1.0 (File $Revision: 1.9.12.1 $)
 */
public abstract class HtmlTableCell {

    public static final int ALIGN_LEFT = 1;
    public static final int ALIGN_CENTER = 2;
    public static final int ALIGN_RIGHT = 3;
    public static final int VALIGN_TOP = 1; 
    public static final int VALIGN_MIDDLE = 2;
    public static final int VALIGN_BOTTOM = 3;

    protected String caption;
    protected int rowSpan = 0;
    protected int colSpan = 0;
    protected String tdValue = null;
    protected String align = null;
    protected String valign = null;
    protected String bgColor = null;
    protected int percentWidth = 0;

    public HtmlTableCell(String tdValue)
    {
        this.tdValue = tdValue;
    }

    public HtmlTableCell(String tdValue, String defaultValue)
    {
	if (tdValue.equals("")) {
	    this.tdValue = defaultValue;
	} else {
	    this.tdValue = tdValue;
	}
    }

    public void setHorizontalAlignment(int alignment)
    {
        switch (alignment) {
        case ALIGN_CENTER:
            this.align = "CENTER";
            break;
        case ALIGN_RIGHT:
            this.align = "RIGHT";
            break;
        default:
            this.align = "LEFT";
            break;
        }
    }

    public void setVerticalAlignment(int alignment)
    {
        switch (alignment) {
        case VALIGN_TOP:
            this.valign = "TOP";
            break;
        case VALIGN_MIDDLE:
            this.valign = "MIDDLE";
            break;
        default:
            this.valign = "BOTTOM";
            break;
        }
    }

    public void setRowSpan(int rowSpan)
    {
        if (rowSpan > 0)
            this.rowSpan = rowSpan;
    }

    public void setColSpan(int colSpan)
    {
        if (colSpan > 0)
            this.colSpan = colSpan;
    }
    
    public void setValue(String tdValue)
    {
        this.tdValue = tdValue;
    }

    public void setBackgroundColor(String bgColor)
    {
        this.bgColor = bgColor;
    }

    public void setPercentWidth(int percent)
    {
        if (percent > 0 && percent <= 100)
            this.percentWidth = percent;
    }
}
