/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MimeType.java,v 1.11.2.2.2.1 2000/10/19 17:58:53 jasona Exp $
 */

package com.lutris.http;
import java.util.Hashtable;

/**
 * Class for querying and setting mime types.
 * Provides a default set of well known mime types.
 *
 * @version 1.0
 */
public class MimeType {

    Hashtable types;

    /**
     * Default Constructor
     */
    public MimeType() {
        this.types = new Hashtable();

        /* Set default well known stuff */
        addType("application/octet-stream", "bin");
        addType("application/astound", "asd,asn");
        addType("application/fastman", "lcc");
        addType("application/java-archive", "jar");
        addType("application/java-serialized-object", "ser");
        addType("application/java-vm", "class");
        addType("application/mac-binhex40", "hqx");
        addType("application/x-stuffit", "sit");
        addType("application/mbedlet", "mbd");
        addType("application/msword", "doc");
        addType("application/oda", "oda");
        addType("application/pdf", "pdf");
        addType("application/postscript", "ai");
        addType("application/postscript", "eps");
        addType("application/postscript", "ps");
        addType("application/rtf", "rtf");
        addType("application/studiom", "smp");
        addType("application/timbuktu", "tbt");
        addType("application/vnd.ms-excel", "xls");
        addType("application/vnd.ms-excel", "xlw");
        addType("application/vnd.ms-excel", "xla");
        addType("application/vnd.ms-excel", "xlc");
        addType("application/vnd.ms-excel", "xlm");
        addType("application/vnd.ms-excel", "xlt");
        addType("application/vnd.ms-powerpoint", "ppt");
        addType("application/vnd.ms-powerpoint", "pps");
        addType("application/vnd.ms-powerpoint", "ppt");
        addType("application/vnd.ms-project", "mpp");
        addType("application/winhlp", "hlp");
        addType("application/x-javascript", "js");
        addType("application/x-asap", "asp");
        addType("application/x-csh", "csh");
        addType("application/x-dot", "dot");
        addType("application/x-dvi", "dvi");
        addType("application/x-earthtime", "etc");
        addType("application/x-envoy", "evy");
        addType("application/x-excel", "xls");
        addType("application/x-excel", "xlc");
        addType("application/x-excel", "xll");
        addType("application/x-excel", "xlm");
        addType("application/x-excel", "xlw");
        addType("application/x-gtar", "gtar");
        addType("application/x-cpio", "cpio");
        addType("application/x-hdf", "hdf");
        addType("application/x-latex", "latex");
        addType("application/x-javascript-config", "jsc");
        addType("application/x-maker", "fm");
        addType("application/x-mif", "mif");
        addType("application/x-mif", "mi");
        addType("application/x-mocha", "mocha,moc");
        addType("application/x-msaccess", "mdb");
        addType("application/x-mscardfile", "crd");
        addType("application/x-msclip", "clp");
        addType("application/x-msmediaview", "m13");
        addType("application/x-msmediaview", "m14");
        addType("application/x-msmetafile", "wmf");
        addType("application/x-msmoney", "mny");
        addType("application/x-mspublisher", "pub");
        addType("application/x-msschedule", "scd");
        addType("application/x-msterminal", "trm");
        addType("application/x-mswrite", "wri");
        addType("application/x-NET-Install", "ins");
        addType("application/x-netcdf", "nc");
        addType("application/x-netcdf", "cdf");
        addType("application/x-ns-proxy-autoconfig", "proxy");
/* 	addType("application/x-pointplus", "css");  interfered with text/css */
        addType("application/x-salsa", "slc");
        addType("application/x-sh", "sh");
        addType("application/x-shar", "shar");
        addType("application/x-sprite", "spr");
        addType("application/x-sprite", "sprite");
        addType("application/x-tar", "tar");
        addType("application/x-tcl", "tcl");
        addType("application/x-perl", "pl");
        addType("application/x-tex", "tex");
        addType("application/x-texinfo", "texinfo");
        addType("application/x-texinfo", "texi");
        addType("application/x-timbuktu", "tbp");
        addType("application/x-tkined", "tki");
        addType("application/x-tkined", "tkined");
        addType("application/x-troff-man", "man");
        addType("application/x-troff-me", "me");
        addType("application/x-troff-ms", "ms");
        addType("application/x-troff", "t");
        addType("application/x-troff", "tr");
        addType("application/x-troff", "roff");
        addType("application/x-wais-source", "src");
        addType("application/zip", "zip");

        addType("audio/basic", "au");
        addType("audio/basic", "snd");
        addType("audio/echospeech", "es");
        addType("audio/echospeech", "esl");
        addType("audio/midi", "midi");
        addType("audio/midi", "mid");
        addType("audio/x-aiff", "aif");
        addType("audio/x-aiff", "aiff");
        addType("audio/x-aiff", "aifc");

        addType("audio/x-wav", "wav");
        addType("audio/x-pn-realaudio", "ra");
        addType("audio/x-pn-realaudio", "ram");
        addType("audio/x-pac", "pac");
        addType("audio/x-epac", "pae");
        addType("audio/x-liveaudio", "lam");

        addType("image/fif", "fif");
        addType("image/gif", "gif");
        addType("image/ief", "ief");
        addType("image/ifs", "ifs");
        addType("image/jpeg", "jpeg");
        addType("image/jpeg", "jpg");
        addType("image/jpeg", "jpe");
        addType("image/jpeg", "jfif");
        addType("image/jpeg", "pjpeg");
        addType("image/jpeg", "pjp");
        addType("image/png", "png");
        addType("image/tiff", "tiff");
        addType("image/tiff", "tif");
        addType("image/vnd", "svf");
        addType("image/wavelet", "wi");
        addType("image/bmp", "bmp");
        addType("image/x-photo-cd", "pcd");
        addType("image/x-cmu-raster", "ras");
        addType("image/x-portable-anymap", "pnm");
        addType("image/x-portable-bitmap", "pbm");
        addType("image/x-portable-graymap", "pgm");
        addType("image/x-portable-pixmap", "ppm");
        addType("image/x-rgb", "rgb");
        addType("image/x-xbitmap", "xbm");
        addType("image/x-xpixmap", "xpm");
        addType("image/x-xwindowdump", "xwd");
        addType("image/vnd.wap.wbmp", "wbmp");

        addType("text/html", "htm");
        addType("text/html", "html");
        addType("text/plain", "txt");
        addType("text/richtext", "rtx");
        addType("text/css", "css");
        addType("text/richtext", "rtx");
        addType("text/tab-separated-values", "tsv");
        addType("text/x-setext", "etx");
        addType("text/x-speech", "talk");
        addType("text/vnd.wap.wml", "wml");

        addType("video/isivideo", "fvi");
        addType("video/mpeg", "mpeg");
        addType("video/mpeg", "mpg");
        addType("video/mpeg", "mpe");
        addType("video/mpeg", "mpv");
        addType("video/mpeg", "vbs");
        addType("video/mpeg", "mpegv");
        addType("video/x-mpeg2", "mpv2");
        addType("video/x-mpeg2", "mp2v");
        addType("video/msvideo", "avi");
        addType("video/quicktime", "qt");
        addType("video/quicktime", "mov");
        addType("video/quicktime", "moov");
        addType("video/vivo", "viv");
        addType("video/vivo", "vivo");
        addType("video/wavelet", "wv");
        addType("video/x-sgi-movie", "movie");

        addType("x-world/x-svr", "svr");
        addType("x-world/x-vrml", "wrl");
        addType("x-world/x-vrt", "vrt");

        addType("x-conference/x-cooltalk", "ice");

        addType("x-gzip", "gz");
        addType("x-compress", "z");
        addType("x-uuencode", "uu,uue");

        addType("magnus-internal/imagemap", "map");
        addType("magnus-internal/parsed-html", "shtml");
        addType("magnus-internal/cgi", "cgi,bat");
        addType("application/octet-stream", "exe");

        addType("application/x-x509-ca-cert", "cacert");
        addType("application/x-x509-server-cert", "scert");
        addType("application/x-x509-user-cert", "ucert");
        addType("application/x-x509-email-cert", "ecert");
    }

    /**
     * Register a new mime type.
     */
    public void addType(String mimeType, String extension) {
        types.put ((Object)extension, (Object)mimeType);
    }

    /**
     * Returns the mime type associated with a file.
     */
    public String getType(String fileName) {
        String extension;
        int index = fileName.lastIndexOf(".") + 1;
        if (index > 0) {
            extension = fileName.substring(index);
        } else {
            extension = fileName;
        }

        String mimeType = (String)types.get((Object)extension);
        if (mimeType == null) {
            mimeType = "text/plain";
        }
        return mimeType;
    }

}
 
