/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: URLDecoder.java,v 1.7.12.1 2000/10/19 17:58:53 jasona Exp $
 */




package	com.lutris.http;

/**
 * This class contains a utility method for converting a <code>String</code>
 * from the MIME <code>"x-www-form-urlencoded"</code> into its original
 * format.  This class implements the inverse functionality of
 * <code>java.net.URLEncoder</code>.
 * 
 * @see		java.net.URLEncoder
 */
public class URLDecoder {
    /**
     * Decode a urlencoded string by replacing '+' with space ' ',
     * and "%xx" to the Latin1 character specified by the hex digits
     * "xx".  The input string is assumed to have been broken up into
     * either a key or a value pair, so '=', '?', and '&' are not
     * treated as separators.  This method implements the inverse
     * functionality of <code>java.net.URLEncoder.encode</code>.
     * 
     * @param s The string to decode.
     * @return The decoded string.
     * 
     * @see	java.net.URLEncoder#encode
     */ 
    public static
    String decode(String s)
    {
	StringBuffer buf = new StringBuffer();
	char[] chars = s.toCharArray();
	int i = 0, len = s.length();
	while (i < len) {
	    char ch = chars[i];
	    switch (ch) {
		case '+':
		    buf.append(' ');
		    i++;
		    break;
		case '%':
		    if (i < (len - 2)) {
			int hi = Character.digit(chars[i+1],16);
			int lo = Character.digit(chars[i+2],16);
			if ((hi < 0) || (lo < 0)) {
			    buf.append(ch);
			    i++;
			} else {
			    buf.append((char)((hi << 4) + lo));
			    i += 3;
			}
		    } else {
			buf.append(ch);
			i++;
		    }
		    break;
		default:
		    buf.append(ch);
		    i++;
		    break;
	    }
	}
	return new String(buf);
    }
}
