/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: CookieParserTests.java,v 1.8.8.1 2000/10/19 17:58:53 jasona Exp $
 */




package com.lutris.http.tests;
import com.lutris.http.*;
import java.util.*;
import javax.servlet.http.*;

/**
 * Simple tests for cookie parser.
 *
 * @author Kyle Clark
 */
public class CookieParserTests
{
    /**
     * Where to print test results.
     */
    java.io.PrintStream out;

    /**
     * Constructor.
     */
    public CookieParserTests()
    {
        out = System.out;
        out.println("com.lutris.http.HttpCookieParser");
    }

    /**
     * Main entry point which executes all the tests.
     *
     * @param args
     *   Command line arguments.
     */
    public static void main(String [] args)
        throws Exception
    {
        CookieParserTests parserTests = new CookieParserTests();

        parserTests.testRequest("Test 1 (Request Header)",
                         "firstName=Kyle; lastName=Clark");

        parserTests.testResponse("Test 2 (Response Header)",
                         "firstName=Kyle, lastName=Clark");

        parserTests.testResponse("Test 3 (Response Header)",
                         "firstName=Kyle; " +
                         "path=/bogus/tmp; domain=lutris.com, " +
                         "lastName=Clark");

        parserTests.testResponse("Test 4 (Response Header)",
                         "firstName=Kyle; " +
                         "path=/bogus/tmp; domain=lutris.com; " +
                         "bogus=junk, " +
                         "lastName=Clark");

        parserTests.testResponse("Test 5 (Mixed header)",
                         "firstName=Kyle, " +
                         "path=/bogus/tmp; domain=lutris.com, " +
                         "bogus=junk; " +
                         "lastName=Clark");

    }

    public void testRequest(String testName, String header)
    {
        out.println();
        printTestName(testName);
        out.println("= Request Header  = ");
        out.println("    " + header);
        out.println("= Results = ");
        printResults(HttpCookieParser.parseRequestHeader(header));
        out.println();
    }

    public void testResponse(String testName, String header)
    {
        out.println();
        printTestName(testName);
        out.println("= Response Header  = ");
        out.println("    " + header);
        out.println("= Results = ");
        printResults(HttpCookieParser.parseResponseHeader(header));
        out.println();
    }

    private void printTestName(String s)
    {
        out.println();
        out.println("========== " + s + " ==========");
    }
        
    private void printResults(Vector v)
    {
        int i=0;
        Enumeration e = v.elements();
        while (e.hasMoreElements()) {
            i++;
            Cookie c = (Cookie)e.nextElement();
            out.println("    " + i + ") " + c);
        }
    }
}
