/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LoggerTests.java,v 1.9.8.1 2000/10/19 17:58:53 jasona Exp $
 */





package com.lutris.logging.tests;

import java.io.*;
import com.lutris.logging.*;

/**
 * Logger tests.
 */
public class LoggerTests {
    private static void makeException(String msg, int depth)
        throws Exception {
        if (depth == 0) {
            throw new Exception(msg);
        }
        makeException(msg, depth-1);
    }

      
    private static void doSomeLogging(String facility, String[] levels) {
        LogChannel lchan = Logger.getCentralLogger().getChannel(facility);
        int msgNum = 0;
        
        for (int l = 0; l < levels.length; l++) {
            lchan.write(levels[l], "msg #" + msgNum++);
            try {
                makeException("exception " + msgNum, 4);
            } catch (Exception except) {
                lchan.write(levels[l], "exception #" + msgNum++, except);
            }
            int intLevel = lchan.getLevel(levels[l]);

            lchan.write(intLevel, "msg #" + msgNum++);
            try {
                makeException("exception " + msgNum, 5);
            } catch (Exception except) {
                lchan.write(intLevel, "exception #" + msgNum++, except);
            }
            lchan.write(intLevel, "multi-line\nmessage\nnumber " + msgNum++);
        }
    }


    private static void writerTest(String facility, String level) {
        LogChannel lchan = Logger.getCentralLogger().getChannel(facility);
        LogWriter writer = lchan.getLogWriter(level);

        writer.write("start LogWriter test");
        writer.println("@1 - println");
        writer.print("@2 - print1 ");
        writer.print("@3 - print2 ");
        writer.print(10);
        writer.print(" ");
        writer.print("@4 - print3 ");
        writer.println();
        writer.print("@5 - flush ");
        writer.flush();
        writer.println("@6 - after flush");
    }

    public static void main(String[] args)
            throws java.io.IOException {
        StandardLogger logger = new StandardLogger(true);
        String allLevels[] = {"A_LEVEL", "ALERT", "WARNING", "ERROR", "OFF"};
        String logFileLevels[] = {"A_LEVEL", "ALERT", "WARNING", "ERROR"};
        String stderrLevels[] = {"WARNING", "ERROR"};
        
        logger.configure(new File("logs/test.log"),
                         logFileLevels, stderrLevels);
        
        doSomeLogging("fac1", allLevels);
        doSomeLogging("fac2", allLevels);
        writerTest("fac1", "ALERT");
    }
}
