/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MimeException.java,v 1.8.12.1 2000/10/19 17:58:53 jasona Exp $
 */




package	com.lutris.mime;

/**
 * Public exception class to indicate an error in Mime data.
 * This is never thrown by <code>MultiparMimeInputStream</code> since
 * that class needs to look exactly like an <code>InputStream</code>.
 */
public class MimeException extends Exception
{
    /**
     * Indicates a general error in parsing Mime data.
     */
    public static final int	GENERIC = 0;
    
    /**
     * Indicates incorrect Mime header syntax or illegal parameters.
     */
    public static final int	INVALID_HEADER = 1;
    
    /**
     * Indicates an illegal Mime type for a particular operation.  For
     * example, this code is generated if an attempt is made to create
     * a <code>MultipartMimeInput</code> object with a Mime type other than
     * <b>multipart/*</b>.
     */
    public static final int	INVALID_MIME_TYPE = 2;
    
    /**
     * Holds the reason code for this exception instance.
     */
    public int			reason = GENERIC;

    /**
     * Creates a MimeException object with informational string
     * <code>s</code> and reason code <code>reason</code>.
     * 
     * @param s			Informational string.
     * @param reason		Reason code.
     */
    MimeException(String s, int reason)
    {
	super(s);
	this.reason = reason;
    }

    /**
     * Creates a MimeException object with informational string
     * <code>s</code> and the default reason code, <code>GENERIC</code>.
     * 
     * @param s			Informational string.
     */
    MimeException(String s)
    {
	super(s);
	reason = GENERIC;
    }
    
    /**
     * Creates a MimeException object with informational string
     * <code>s</code> and the default reason code, <code>GENERIC</code>.
     * 
     * @param s			Informational string.
     */
    MimeException()
    {
	super();
	reason = GENERIC;
    }
}
