/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ConfigTests.java,v 1.10.8.1 2000/10/19 17:58:53 jasona Exp $
 */





package com.lutris.util.tests;
import com.lutris.util.*;
import java.io.*;

/**
 * Class that tests the Confi object.
 */
public class ConfigTests {
    private static final    String	NL = "\n";
    private static 	    int		ival;
    private static 	    int[]	ivals;
    private static 	    long	lval;
    private static 	    long[]	lvals;
    private static 	    String	sval;
    private static 	    String[]	svals;
    private static 	    boolean	bval;
    private static 	    boolean[]	bvals;
    private static 	    double	dval;
    private static 	    double[]	dvals;

    private static final String Test1 =
        "#" + NL +
        "# Correct data." + NL +
        "#" + NL +
        "int.a = 1" + NL +
        "int.b = [1, 2, 3]" + NL +
        "int.c = \"123\"" + NL +
        "long.a = 100000000000000" + NL +
        "long.b = [0, 200000000000000]" + NL +
        "long.c = \"300000000000000\"" + NL +
        "boolean.a = true" + NL +
        "boolean.b = false" + NL +
        "boolean.c = [true, false]" + NL +
        "boolean.d = [\"true\", \"false\"]" + NL +
        "string.a = \"Hello World\"" + NL +
        "string.b = Hello-World" + NL +
        "string.c = [\"Hello\", \"World\", \"\"]" + NL +
        "string.d = \"\007\"" + NL +
        "string.e = \"There is a \" + " + NL + "\"newline in here.\"" + NL +
        "string.f = \"Another\\" + NL + " newline.\"" + NL +
        "string.g = \"PartA\" + \"PartB\" + PartC" + NL +
        "double.a = 1.000000000000001" + NL +
        "double.b = [0, 200000000000.0002, 3.1415927, -2.2E+12]" + NL +
        "double.c = \"3000000.000000003\"" + NL +
        "" + NL +
        "" + NL +
        "#" + NL +
        "# Comment" + NL +
        "#" + NL +
        "comment.a = \"Hello world\" # Trailing comment." + NL;
    
    private static
    void beginTest(String name, String reason)
    {
	System.out.println("TEST: " + name + "  --  " + reason);
    }

    private static
    void goodTest(String name)
    {
	System.out.println("SUCCEED: " + name);
    }

    private static
    void failTest(String name, String reason)
    throws Exception
    {
	throw new Exception("FAIL: " + name + ": " + reason);
    }
    
    static
    void testGoodStreamRead()
    throws Exception
    {
	beginTest("testGoodStreamRead", "Read good input from stream.");
	ByteArrayInputStream sis = new ByteArrayInputStream(Test1.getBytes());
	try {
	    ConfigFile cf = new ConfigFile(sis);
	    Config conf = cf.getConfig();
	}
	catch (Exception e) {
	    failTest("testGoodStreamRead", e.toString());
	}
	goodTest("testGoodStreamRead");
    }
    
    static
    void testGoodFileRead()
    throws Exception
    {
	String FileName = "./Config.test.xx";
	beginTest("testGoodFileRead", "Read good input from file.");
	try {
	    FileOutputStream fos = new FileOutputStream(FileName);
	    BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(fos));
	    bw.write(Test1, 0, Test1.length());
	    bw.flush(); bw.close();
	    bw = null;
	    fos = null;
	}
	catch (Exception e) {
	    failTest("testGoodFileRead", "Failed to write out test data.");
	}
	try {
	    FileInputStream cfis = new FileInputStream(FileName);
	    ConfigFile cf = new ConfigFile(cfis);
	    Config conf = cf.getConfig();
	}
	catch (Exception e) {
	    failTest("testGoodFileRead", e.toString());
	}
	goodTest("testGoodStreamRead");
    }
    
    static
    void testGoodInput()
    throws Exception
    {
	Config conf = null;
	beginTest("testGoodInput", "Read good input from stream.");
	ByteArrayInputStream sis = new ByteArrayInputStream(Test1.getBytes());
	try {
	    ConfigFile cf = new ConfigFile(sis);
	    conf = cf.getConfig();
	}
	catch (Exception e) {
	    failTest("testGoodInput", e.toString());
	}
	
	int		thisCase = 0;
	boolean		xfail;
	
	try {
	    thisCase = 1;
	    ival = conf.getInt("int.a");
	    if (ival != 1) throw new Exception("Wrong value");
	    
	    thisCase = 2;
	    ivals = conf.getInts("int.a");
	    if (ivals.length != 1) throw new Exception("Wrong count");
	    if (ivals[0] != 1) throw new Exception("Wrong value");
	    
	    thisCase = 3;
	    ivals = conf.getInts("int.b");
	    if (ivals.length != 3) throw new Exception("Wrong count");
	    if (ivals[0] != 1) throw new Exception("Wrong value");
	    if (ivals[1] != 2) throw new Exception("Wrong value");
	    if (ivals[2] != 3) throw new Exception("Wrong value");
	    
	    thisCase = 4;
	    xfail = false;
	    try {
		ival = conf.getInt("int.b");
		xfail = true;
	    } catch (ConfigException e) {}
	    if (xfail) throw new Exception("Expected ConfigException");

	    thisCase = 5;
	    ival = conf.getInt("int.c");
	    if (ival != 123) throw new Exception("Wrong value");
	    
	    thisCase = 6;
	    ivals = conf.getInts("int.c");
	    if (ivals.length != 1) throw new Exception("Wrong count");
	    if (ivals[0] != 123) throw new Exception("Wrong value");
	    
	    //
	    // Test Longs.
	    // 
	    thisCase = 7;
	    lval = conf.getLong("long.a");
	    if (lval != 100000000000000L)
		throw new Exception("Wrong value");
	    
	    thisCase = 8;
	    lvals = conf.getLongs("long.a");
	    if (lvals.length != 1) throw new Exception("Wrong count");
	    if (lvals[0] != 100000000000000L)
		throw new Exception("Wrong value");
	    
	    thisCase = 9;
	    lvals = conf.getLongs("long.b");
	    if (lvals.length != 2) throw new Exception("Wrong count");
	    if (lvals[0] != 0L) throw new Exception("Wrong value");
	    if (lvals[1] != 200000000000000L)
		throw new Exception("Wrong value");
	    
	    thisCase = 10;
	    xfail = false;
	    try {
		lval = conf.getLong("long.b");
		xfail = true;
	    } catch (ConfigException e) {}
	    if (xfail) throw new Exception("Expected ConfigException");

	    thisCase = 11;
	    lval = conf.getLong("long.c");
	    if (lval != 300000000000000L)
		throw new Exception("Wrong value");
	    
	    thisCase = 12;
	    lvals = conf.getLongs("long.c");
	    if (lvals.length != 1) throw new Exception("Wrong count");
	    if (lvals[0] != 300000000000000L)
		throw new Exception("Wrong value");
	    
	    //
	    // Booleans
	    // 
	    thisCase = 13;
	    bval = conf.getBoolean("boolean.a");
	    if (!bval) throw new Exception("Wrong value");

	    thisCase = 14;
	    bvals = conf.getBooleans("boolean.a");
	    if (bvals.length != 1) throw new Exception("Wrong count");
	    if (!(bvals[0])) throw new Exception("Wrong value");

	    thisCase = 15;
	    bval = conf.getBoolean("boolean.b");
	    if (bval) throw new Exception("Wrong value");
	    
	    thisCase = 16;
	    bvals = conf.getBooleans("boolean.b");
	    if (bvals.length != 1) throw new Exception("Wrong count");
	    if (bvals[0]) throw new Exception("Wrong value");
	
	    thisCase = 17;
	    bvals = conf.getBooleans("boolean.c");
	    if (bvals.length != 2) throw new Exception("Wrong count");
	    if (!(bvals[0])) throw new Exception("Wrong value");
	    if (bvals[1]) throw new Exception("Wrong value");
	    
	    thisCase = 18;
	    xfail = false;
	    try {
		bval = conf.getBoolean("boolean.c");
		xfail = true;
	    } catch (ConfigException e) {}
	    if (xfail) throw new Exception("Expected ConfigException");
	    
	    thisCase = 19;
	    bvals = conf.getBooleans("boolean.d");
	    if (bvals.length != 2) throw new Exception("Wrong count");
	    if (!(bvals[0])) throw new Exception("Wrong value");
	    if (bvals[1]) throw new Exception("Wrong value");
	    
	    thisCase = 20;
	    xfail = false;
	    try {
		bval = conf.getBoolean("boolean.d");
		xfail = true;
	    } catch (ConfigException e) {}
	    if (xfail) throw new Exception("Expected ConfigException");
	    
	    thisCase = 21;
	    sval = conf.getString("string.a");
	    if (!sval.equals("Hello World"))
		throw new Exception("Wrong value");

	    thisCase = 22;
	    svals = conf.getStrings("string.a");
	    if (svals.length != 1) throw new Exception("Wrong count");
	    if (!svals[0].equals("Hello World"))
		throw new Exception("Wrong value");

	    thisCase = 23;
	    sval = conf.getString("string.b");
	    if (!sval.equals("Hello-World"))
		throw new Exception("Wrong value");

	    thisCase = 24;
	    svals = conf.getStrings("string.b");
	    if (svals.length != 1) throw new Exception("Wrong count");
	    if (!svals[0].equals("Hello-World"))
		throw new Exception("Wrong value");
	    
	    thisCase = 25;
	    svals = conf.getStrings("string.c");
	    if (svals.length != 3) throw new Exception("Wrong count");
	    if (!svals[0].equals("Hello"))
		throw new Exception("Wrong value");
	    if (!svals[1].equals("World"))
		throw new Exception("Wrong value");
	    if (!svals[2].equals(""))
		throw new Exception("Wrong value");

	    thisCase = 26;
	    xfail = false;
	    try {
		sval = conf.getString("string.c");
		xfail = true;
	    } catch (ConfigException e) {}
	    if (xfail) throw new Exception("Expected ConfigException");

	    thisCase = 27;
	    sval = conf.getString("string.d");
	    if (!sval.equals("\007"))
		throw new Exception("Wrong value");

	    thisCase = 28;
	    svals = conf.getStrings("string.d");
	    if (svals.length != 1) throw new Exception("Wrong count");
	    if (!svals[0].equals("\007"))
		throw new Exception("Wrong value");

	    thisCase = 29;
	    sval = conf.getString("string.e");
	    if (!sval.equals("There is a newline in here."))
		throw new Exception("Wrong value");

	    thisCase = 30;
	    svals = conf.getStrings("string.e");
	    if (svals.length != 1) throw new Exception("Wrong count");
	    if (!svals[0].equals("There is a newline in here."))
		throw new Exception("Wrong value");

	    thisCase = 31;
	    sval = conf.getString("string.f");
	    if (!sval.equals("Another" + NL + " newline."))
		throw new Exception("Wrong value");

	    thisCase = 32;
	    svals = conf.getStrings("string.f");
	    if (svals.length != 1) throw new Exception("Wrong count");
	    if (!svals[0].equals("Another" + NL + " newline."))
		throw new Exception("Wrong value");

	    thisCase = 33;
	    sval = conf.getString("string.g");
	    if (!sval.equals("PartAPartBPartC"))
		throw new Exception("Wrong value");

	    thisCase = 34;
	    svals = conf.getStrings("string.g");
	    if (svals.length != 1) throw new Exception("Wrong count");
	    if (!svals[0].equals("PartAPartBPartC"))
		throw new Exception("Wrong value");

	    //
	    // Doubles
	    // 
	    thisCase = 35;
	    dval = conf.getDouble("double.a");
	    if (dval != Double.valueOf("1.000000000000001").doubleValue())
		throw new Exception("Wrong value");

	    thisCase = 36;
	    dvals = conf.getDoubles("double.a");
	    if (dvals.length != 1) throw new Exception("Wrong count");
	    if (dvals[0] != Double.valueOf("1.000000000000001").doubleValue())
		throw new Exception("Wrong value");

	    thisCase = 37;
	    dvals = conf.getDoubles("double.b");
	    if (dvals.length != 4) throw new Exception("Wrong count");
	    if (dvals[0] != Double.valueOf("0").doubleValue())
		throw new Exception("Wrong value");
	    if (dvals[1] != Double.valueOf("200000000000.0002").doubleValue())
		throw new Exception("Wrong value");
	    if (dvals[2] != Double.valueOf("3.1415927").doubleValue())
		throw new Exception("Wrong value");
	    if (dvals[3] != Double.valueOf("-2.2E+12").doubleValue())
		throw new Exception("Wrong value");

	    thisCase = 38;
	    xfail = false;
	    try {
		dval = conf.getDouble("double.b");
		xfail = true;
	    } catch (ConfigException e) {}
	    if (xfail) throw new Exception("Expected ConfigException");

	    thisCase = 39;
	    dval = conf.getDouble("double.c");
	    if (dval != Double.valueOf("3000000.000000003").doubleValue())
		throw new Exception("Wrong value");

	    thisCase = 40;
	    dvals = conf.getDoubles("double.c");
	    if (dvals.length != 1) throw new Exception("Wrong count");
	    if (dvals[0] != Double.valueOf("3000000.000000003").doubleValue())
		throw new Exception("Wrong value");
	    
	    //
	    // Misc.
	    //
	    thisCase = 41;
	    sval = conf.getString("comment.a");
	    if (!sval.equals("Hello world")) 
		throw new Exception("Wrong value");
	    
	    thisCase = 42;
	    bval = conf.containsKey("comment.a");
	    if (!bval) throw new Exception("Wrong value");
	    
	    thisCase = 43;
	    bval = conf.containsKey("IDontExist");
	    if (bval) throw new Exception("Wrong value");
	    
	    thisCase = 44;
	    ival = conf.containsCount("int.a");
	    if (ival != 1) throw new Exception("Wrong value");

	    thisCase = 45;
	    ival = conf.containsCount("double.b");
	    if (ival != 4) throw new Exception("Wrong value");

	    thisCase = 46;
	    ival = conf.containsCount("NotHere");
	    if (ival >= 0) throw new Exception("Wrong value");
	}
	catch (Exception e) {
	    failTest("testGoodInput (case " + thisCase + ")", e.toString());
	}
	goodTest("testGoodInput");
    }

    public static void runTests()
    {
	int i;
	String comma = "";
	try {
	    testGoodFileRead();
	    testGoodStreamRead();
	    testGoodInput();
	}
	catch (Exception e) {
	    System.out.println("Test FAILED: " + e);
	    e.printStackTrace(System.out);

	    System.out.println("ival=" + ival);
	    System.out.print("ivals=");
	    if (ivals == null) System.out.println("null");
	    else {
		System.out.print("{");
		comma = "";
		for (i=0; i<ivals.length; i++) {
		    System.out.print(comma + ivals[i]);
		    comma = ", ";
		}
		System.out.println("}");
	    }

	    System.out.println("lval=" + lval);
	    System.out.print("lvals=");
	    if (lvals == null) System.out.println("null");
	    else {
		System.out.print("{");
		comma = "";
		for (i=0; i<lvals.length; i++) {
		    System.out.print(comma + lvals[i]);
		    comma = ", ";
		}
		System.out.println("}");
	    }

	    System.out.println("sval=" + sval);
	    System.out.print("svals=");
	    if (svals == null) System.out.println("null");
	    else {
		System.out.print("{");
		comma = "";
		for (i=0; i<svals.length; i++) {
		    System.out.print(comma + svals[i]);
		    comma = ", ";
		}
		System.out.println("}");
	    }

	    System.out.println("bval=" + bval);
	    System.out.print("bvals=");
	    if (bvals == null) System.out.println("null");
	    else {
		System.out.print("{");
		comma = "";
		for (i=0; i<bvals.length; i++) {
		    System.out.print(comma + bvals[i]);
		    comma = ", ";
		}
		System.out.println("}");
	    }

	    System.out.println("dval=" + dval);
	    System.out.print("dvals=");
	    if (dvals == null) System.out.println("null");
	    else {
		System.out.print("{");
		comma = "";
		for (i=0; i<dvals.length; i++) {
		    System.out.print(comma + dvals[i]);
		    comma = ", ";
		}
		System.out.println("}");
	    }
	    return;
	}
	System.out.println("Test OK!");
    }

    public static void main (String [] args)
    {
        runTests();
    }
}
