/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: GeneratorInfo.java,v 1.10.12.1 2000/10/05 00:56:03 jay Exp $
 */



package com.lutris.dods.builder.engine;

import com.lutris.dods.store.doa.DataObject;
import com.lutris.dods.store.doa.Attribute;

import  com.lutris.util.Config;

import java.lang.reflect.*;
import java.io.*;
import java.io.FileNotFoundException;
import java.util.*;

public class GeneratorInfo {

    public Config config;
    public boolean runForAbstract;
    public boolean handlesArrays;
    public String outputDir;
    public String outputFileFlavor;
    public String outputFileSuffix;
    public String objectSuffix;
    public String outputClassParent;
    public String extraClasspath;
    public Class  processor;
    public String templateDir;
    public TemplateReader[] templates;

    public static String WRITE_FILE_HEADER    = "writeFileHeader";
    public static String WRITE_FILE_FOOTER    = "writeFileFooter";
    public static String WRITE_MEMBER_STUFF   = "writeMemberStuff";

    private String[] methodNames =
	{ WRITE_FILE_HEADER, WRITE_FILE_FOOTER, WRITE_MEMBER_STUFF };

    public TemplateReader getTemplateReader( String key ) {
	for ( int i = 0; i < methodNames.length; i++ ) {
	    if ( methodNames[i].equals( key ) ) {
/* 		return templates[i]; */
		/* I'd like to reuse the 3 main TemplateReaders.
		   I wanted to open them only once, and reset() them
		   for each DataObject.
		   But for some reason, reset does not work for them.
		   Reset() does work for the 2 extra templates used
		   by SourceGenerator_DO, so maybe I just need to
		   figure out what makes them work correctly.
		   FIX remove the 'return' below in favor of the one above.
		*/
		try {
	    	return new TemplateReader(
	    	templateDir + File.separator + methodNames[i] + ".template" );
		} catch ( Exception e ) {
		}
	    }
	}
	return null;
    }

    public GeneratorInfo(
				Config cfg,
				boolean genAbstract,
				boolean doArrays,
				String outDir,
				String flavor,
				String suffix,
				String parent,
				String extraCP,
				Class  generator,
				String tempDir,
				String objSuffix )

    throws Exception {
	config            = cfg;
	runForAbstract    = genAbstract;
	handlesArrays     = doArrays;
	outputDir         = outDir;
	outputFileFlavor  = flavor;
	outputFileSuffix  = suffix;
	outputClassParent = parent;
	extraClasspath    = extraCP;
	templateDir       = tempDir;
	objectSuffix      = objSuffix;
	processor         = generator;
	Method m[]        = null;
	try {
	    m = generator.getMethods();
	} catch ( Exception e ) {
	    throw e;
	}
	int found = 0;
	for ( int i = 0; i < m.length; i++ ) {
//System.err.println("m[i].getName()="+m[i].getName());
	    Class[] parms = m[i].getParameterTypes();
	    if ( 	m[i].getName().equals( WRITE_FILE_HEADER ) ||
	    	 	m[i].getName().equals( WRITE_FILE_FOOTER ) ) {
	    	Class[] expect = {DataObject.class};
		checkTypeOfParms( processor, m[i], parms, expect );
		found++;
	    } else if (	m[i].getName().equals( WRITE_MEMBER_STUFF ) ) {
	    	Class[] expect = {Attribute.class};
		checkTypeOfParms( processor, m[i], parms, expect );
		found++;
	    }
	}
//System.err.println("methodNames.length="+methodNames.length);
	if ( found != methodNames.length )
	    throw new Exception( "Class " + generator.getName() +
		", is missing at least 1 method." );

	/* see FIX above
	templates = new TemplateReader[methodNames.length];
	for ( int i = 0; i < methodNames.length; i++ ) {
	    templates[i] = new TemplateReader(
	    	templateDir + File.separator + methodNames[i] + ".template" );
	}
	*/
    }

    private void checkTypeOfParms( Class c, Method m, Class[] parms,
					Class[] correctTypes )
    throws Exception {
	if ( parms.length != correctTypes.length )
	    throw new Exception( "Class " + c.getName() +
		", method " + m.getName() +
		" has " + parms.length +
		" parameters, but should take " + correctTypes.length );

	for ( int i = 0; i < parms.length; i++ ) {
	    if ( parms[i] != correctTypes[i] )
		throw new Exception( "Class " + c.getName() +
		    ", method " + m.getName() +
		    ", parm #" + i +
		    " is type " + parms[i].getName() +
		    " but should be type " + correctTypes[i].getName() );
    	}
    }
}
