/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 */

package com.lutris.dods.gui;

import com.lutris.dods.builder.engine.Builder;
import com.lutris.dods.util.Util;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.ResourceBundle;

public class BuildMonitorDialog extends JDialog {
  static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    private JPanel panelMain = new JPanel();
  private GridBagLayout layoutMain = new GridBagLayout();
  private JScrollPane scrollText = new JScrollPane();
  private JTextArea textArea = new JTextArea();
  private JButton buttonClose = new JButton();
  private JLabel labelWatch = new JLabel();

  public static void main(String[] args) {
    BuildMonitorDialog d = new BuildMonitorDialog();
    d.show();
  }
  public BuildMonitorDialog(Frame frame, boolean modal) {
    super(frame, modal);
    try  {
      jbInit();
      pmInit();
      pack();
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
  }

  public BuildMonitorDialog() {
    this(null, false);
  }

  private void pmInit() {
    Builder.setTextArea( textArea );
    textArea.append( res.getString("Generating_java_and") );
  }

  private void jbInit() throws Exception {
    labelWatch.setText(res.getString("Please_check_for"));

    textArea.setEditable( false );
    textArea.setFont( new Font( "Courier", Font.PLAIN, 12 ) );
    panelMain.setPreferredSize(new Dimension(500, 300));
    scrollText.getViewport().add(textArea, null);

    buttonClose.setText(res.getString("Close1"));
    buttonClose.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        closeDialog();
      }
    });


    panelMain.setLayout(layoutMain);
    panelMain.add(scrollText, new GridBagConstraintsX(0, 1, 1, 1, 0.8, 0.8
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
    panelMain.add(buttonClose, new GridBagConstraintsX(0, 3, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 5, 5));
    panelMain.add(labelWatch, new GridBagConstraintsX(0, 0, 1, 1, 0.1, 0.1
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 5, 5));


    getContentPane().add(panelMain);
    setResizable(true);
    setTitle( res.getString("Generating_source") );
  }

  public void show() {
    Point centeringPoint;
    centeringPoint = Util.getCenteringPoint(getSize());
    setLocation(centeringPoint);
    buttonClose.requestFocus();
    super.show();
  }


  private void closeDialog() {
    this.setVisible(false);
    this.dispose();
  }
}

