/*
 * -----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 * must display the following acknowledgement:
 * This product includes Enhydra software developed by Lutris
 * Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * -----------------------------------------------------------------------------
 * $Id
 * -----------------------------------------------------------------------------
 */
package com.lutris.dods.gui;
import com.lutris.dods.gui.importer.*;
import com.lutris.dods.gui.tree.*;
import com.lutris.dods.gui.table.*;
import com.lutris.dods.gui.editors.*;
import com.lutris.dods.store.doa.*;
import com.lutris.dods.store.*;
import com.lutris.dods.exceptions.*;
import com.lutris.dods.util.Util;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.awt.*;
import java.awt.event.*;
import java.text.MessageFormat;
import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.util.Vector;
import com.lutris.dods.builder.engine.Builder;
import java.util.ResourceBundle;

/**
 * This class is the heart of the Data Object Design Studio.  This frame
 * contains all of the panels and menus that make up DODS.
 *
 * @author  Philip Martin
 * @version $Revision: 1.30.2.1.4.1 $
 */
public final class DataObjectApplication extends JFrame {
    static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    private static final String appPrimary = res.getString("Enhydra");
    private static final String appSecondary = res.getString("Data_Object_Design");
    private static final String appVersion = res.getString("Version_2_3");
    private Store		store = new Store();

    /**
     * This constructor initializes all of the visible parts of the Frame.
     */
    public DataObjectApplication() {
	pmInit();
	jbInit();
    }

    /**
     * Initialize non UI elements.
     */
    private void pmInit() {
	store = new Store();

	store.create();
	store.getProject().setDatabase(res.getString("Standard"));
	setTitle();
    }

    /**
     * Setup user Interface
     */
    private void jbInit() {

	// General Frame initial values.
	setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	getContentPane().setLayout(new BorderLayout(0, 0));
	setSize(800, 600);
	setBackground(Color.lightGray);

	// Toolbar Initializations.
	toolBar = new JToolBar();

	toolBar.setForeground(new Color(0));
	toolBar.setBackground(Color.lightGray);

	newButton = new JButton(Icons.getIcon("new.gif"));

	newButton.setMargin(new Insets(0, 0, 0, 0));
	newButton.setToolTipText(res.getString("New"));
	newButton.setFocusPainted(false);
	toolBar.add(newButton);

	openButton = new JButton(Icons.getIcon("open.gif"));

	openButton.setMargin(new Insets(0, 0, 0, 0));
	openButton.setToolTipText(res.getString("Open"));
	openButton.setFocusPainted(false);
	toolBar.add(openButton);

	saveButton = new JButton(Icons.getIcon("save.gif"));

	saveButton.setDisabledIcon(Icons.getIcon("disabled_save.gif"));
	saveButton.setMargin(new Insets(0, 0, 0, 0));
	saveButton.setToolTipText(res.getString("Save"));
	saveButton.setFocusPainted(false);
	toolBar.add(saveButton);
	toolBar.addSeparator();

	insertPackageButton =
	    new JButton(Icons.getIcon("insert_package.gif"));

	insertPackageButton.setDisabledIcon(Icons.getIcon("disabled_insert_package.gif"));
	insertPackageButton.setMargin(new Insets(0, 0, 0, 0));
	insertPackageButton.setToolTipText(res.getString("Insert_Package"));
	insertPackageButton.setFocusPainted(false);
	toolBar.add(insertPackageButton);

	insertDataObjectButton =
	    new JButton(Icons.getIcon("insert_dataobject.gif"));

	insertDataObjectButton.setDisabledIcon(Icons.getIcon("disabled_insert_dataobject.gif"));
	insertDataObjectButton.setMargin(new Insets(0, 0, 0, 0));
	insertDataObjectButton.setToolTipText(res.getString("Insert_Data_Object"));
	insertDataObjectButton.setFocusPainted(false);
	toolBar.add(insertDataObjectButton);

	// ****opensource contribution by Larry D'Wolcoto
	insertImportedObjectButton =
	    new JButton(Icons.getIcon("import_schema.gif"));

	insertImportedObjectButton.setDisabledIcon(Icons.getIcon("disabled_insert_dataobject.gif"));
	insertImportedObjectButton.setMargin(new Insets(0, 0, 0, 0));
	insertImportedObjectButton.setToolTipText(res.getString("Insert_Imported"));
	insertImportedObjectButton.setFocusPainted(false);
	toolBar.add(insertImportedObjectButton);

	// ****end
	insertAttributeButton =
	    new JButton(Icons.getIcon("insert_attribute.gif"));

	insertAttributeButton.setDisabledIcon(Icons.getIcon("disabled_insert_attribute.gif"));
	insertAttributeButton.setMargin(new Insets(0, 0, 0, 0));
	insertAttributeButton.setToolTipText(res.getString("Insert_Attribute"));
	insertAttributeButton.setFocusPainted(false);
	toolBar.add(insertAttributeButton);
	toolBar.addSeparator();

	editPackageButton = new JButton(Icons.getIcon("edit_package.gif"));

	editPackageButton.setDisabledIcon(Icons.getIcon("disabled_edit_package.gif"));
	editPackageButton.setMargin(new Insets(0, 0, 0, 0));
	editPackageButton.setToolTipText(res.getString("Edit_Package"));
	editPackageButton.setFocusPainted(false);
	toolBar.add(editPackageButton);

	editDataObjectButton =
	    new JButton(Icons.getIcon("edit_dataobject.gif"));

	editDataObjectButton.setDisabledIcon(Icons.getIcon("disabled_edit_dataobject.gif"));
	editDataObjectButton.setMargin(new Insets(0, 0, 0, 0));
	editDataObjectButton.setToolTipText(res.getString("Edit_Data_Object"));
	editDataObjectButton.setFocusPainted(false);
	toolBar.add(editDataObjectButton);

	editAttributeButton =
	    new JButton(Icons.getIcon("edit_attribute.gif"));

	editAttributeButton.setDisabledIcon(Icons.getIcon("disabled_edit_attribute.gif"));
	editAttributeButton.setMargin(new Insets(0, 0, 0, 0));
	editAttributeButton.setToolTipText(res.getString("Edit_Attribute"));
	editAttributeButton.setFocusPainted(false);
	toolBar.add(editAttributeButton);
	toolBar.addSeparator();

	findErrorsButton = new JButton(Icons.getIcon("error_button.gif"));

	findErrorsButton.setDisabledIcon(Icons.getIcon("disabled_error_button.gif"));
	findErrorsButton.setMargin(new Insets(0, 0, 0, 0));
	findErrorsButton.setToolTipText(res.getString("Find_An_Error"));
	findErrorsButton.setFocusPainted(false);
	toolBar.add(findErrorsButton);
	toolBar.addSeparator();

	buildAllButton = new JButton(Icons.getIcon("build_all.gif"));

	buildAllButton.setDisabledIcon(Icons.getIcon("disabled_build_all.gif"));
	buildAllButton.setMargin(new Insets(0, 0, 0, 0));
	buildAllButton.setToolTipText(res.getString("Build_All"));
	buildAllButton.setFocusPainted(false);
	toolBar.add(buildAllButton);
	toolBar.setFloatable(false);
	toolBar.setBorder(new EmptyBorder(3, 7, 3, 0));
	toolBar.setBorderPainted(true);
	getContentPane().add("North", toolBar);

	// Initializing the tree elements;
	tree = new JTree();

	DOTreeMouseListener dotreeml = new DOTreeMouseListener(this, table);

	tree.addMouseListener(dotreeml);
	tree.addMouseMotionListener(dotreeml);
	tree.addTreeExpansionListener(new DOTreeExpansionListener());

	ToolTipListener tipListener = new ToolTipListener();

	tree.setToolTipText(res.getString("Package_Tree"));

	treeRoot = new DOTreeNode(store.getProject(), true);

	updateTree();

	// Initializing the Graphical Pane.
	graphicalPanel = new GraphicalPanel(store.getProject(), tree);

	graphicalPanel.setBackground(Color.white);

	top = new JScrollPane();

	top.setBackground(Color.white);
	top.setViewport(graphicalPanel);
	top.setMinimumSize(new Dimension(0, 0));

	doTreeSelectionListener = new DOTreeSelectionListener(this, tree,
		graphicalPanel);

	tree.addTreeSelectionListener(doTreeSelectionListener);

	TreeSelectionModel selModel = tree.getSelectionModel();

	selModel.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	tree.setEditable(false);

	DOTreeRenderer dotr = new DOTreeRenderer();

	tree.setCellRenderer(dotr);

	JScrollPane left = new JScrollPane(tree);

	left.setMinimumSize(new Dimension(0, 0));
	left.setPreferredSize(new Dimension(150, 0));

	// Initializing the table.
	table = new JTable();
	doTableModel = new DOTableModel(new DataObject(), this, table);

	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	table.setBackground(Color.white);
	table.setModel(doTableModel);
	table.getTableHeader().setReorderingAllowed(false);

	TableMouseListener tableMouseListener = new TableMouseListener(table,
		this);

	table.addMouseListener(tableMouseListener);
	table.addMouseMotionListener(tipListener);
	table.addMouseListener(tipListener);
	table.getTableHeader().addMouseListener(new TableHeaderListener(this));

	TableColumnModel tcm = table.getColumnModel();
	JScrollPane      right = new JScrollPane(table);

	right.setViewportBorder(new LineBorder(Color.gray, 1));
	right.setMinimumSize(new Dimension(0, 0));

	// Initializing the TableColumns.
	for (int i = 0; i < tcm.getColumnCount(); i++) {
	    TableColumn tc = tcm.getColumn(i);

	    switch (i) {

	    case 0:
		tc.setMaxWidth(16);
		tc.setResizable(false);
		tc.setCellRenderer(new ErrorColumnRenderer(false));

		ErrorColumnRenderer ecr = new ErrorColumnRenderer(true);

		ecr.setToolTipText(res.getString("Errors"));
		tc.setHeaderRenderer(ecr);

		break;

	    case 1:
		tc.setMaxWidth(16);
		tc.setResizable(false);
		tc.setCellRenderer(new ObjectRefColumnRenderer(false));

		ObjectRefColumnRenderer orcr =
		    new ObjectRefColumnRenderer(true);

		orcr.setToolTipText(res.getString("Is_an_Object"));
		tc.setHeaderRenderer(orcr);

		break;

	    case 2:
		tc.setMaxWidth(16);
		tc.setResizable(false);
		tc.setCellRenderer(new CanBeNullColumnRenderer(false));

		CanBeNullColumnRenderer cbncr =
		    new CanBeNullColumnRenderer(true);

		cbncr.setToolTipText(res.getString("Can_be_Null"));
		tc.setHeaderRenderer(cbncr);

		break;

	    case 3:
		tc.setMaxWidth(16);
		tc.setResizable(false);
		tc.setCellRenderer(new IndexColumnRenderer(false));

		IndexColumnRenderer icr = new IndexColumnRenderer(true);

		icr.setToolTipText(res.getString("Is_an_Index"));
		tc.setHeaderRenderer(icr);

		break;

	    case 4:
		tc.setMaxWidth(16);
		tc.setResizable(false);
		tc.setCellRenderer(new QueryColumnRenderer(false));

		QueryColumnRenderer qcr = new QueryColumnRenderer(true);

		qcr.setToolTipText(res.getString("Can_be_used_in_a"));
		tc.setHeaderRenderer(qcr);

		break;

	    case 5:
		tc.setMaxWidth(16);
		tc.setResizable(false);
		tc.setCellRenderer(new RefConstraintColumnRenderer(false));

		RefConstraintColumnRenderer rccr =
		    new RefConstraintColumnRenderer(true);

		rccr.setToolTipText(res.getString("Is_a_referential"));
		tc.setHeaderRenderer(rccr);

		break;

	    case 6:
		tc.setMaxWidth(16);
		tc.setResizable(false);
		tc.setCellRenderer(new ConstantColumnRenderer(false));

		ConstantColumnRenderer ccr = new ConstantColumnRenderer(true);

		ccr.setToolTipText(res.getString("Has_a_constant_value"));
		tc.setHeaderRenderer(ccr);

		break;

	    case 7:
		tc.setWidth(150);

		break;

	    case 8:
		tc.setWidth(150);
		tc.setCellEditor(null);

		break;

	    case 9:
		break;

	    case 10:
		tc.setWidth(50);
		tc.setCellEditor(null);

		break;

	    case 11:
		tc.setWidth(100);

		break;

	    default:
		break;
	    }
	}

	// Initializing the SplitPanes.
	verticalBar = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, left,
				     right);
	horizontalBar = new JSplitPane(JSplitPane.VERTICAL_SPLIT, top,
				       verticalBar);

	verticalBar.setBackground(Color.white);
	horizontalBar.setBackground(Color.white);
	getContentPane().add(res.getString("Center"), horizontalBar);

	// Initializing the Menus.
	dodsMenuBar = new JMenuBar();
	fileMenu = new JMenu(res.getString("File"));
	newMenuItem = new JMenuItem(res.getString("New_"));

	fileMenu.add(newMenuItem);

	openMenuItem = new JMenuItem(res.getString("Open"));

	fileMenu.add(openMenuItem);

	closeMenuItem = new JMenuItem(res.getString("Close"));

	fileMenu.add(closeMenuItem);
	fileMenu.addSeparator();

	buildAllMenuItem = new JMenuItem(res.getString("Build_All"));

	fileMenu.add(buildAllMenuItem);
	fileMenu.addSeparator();

	saveMenuItem = new JMenuItem(res.getString("Save"));

	fileMenu.add(saveMenuItem);

	saveAsMenuItem = new JMenuItem(res.getString("Save_As_"));

	fileMenu.add(saveAsMenuItem);
	fileMenu.addSeparator();

	exitMenuItem = new JMenuItem(res.getString("Exit"));

	fileMenu.add(exitMenuItem);
	dodsMenuBar.add(fileMenu);

	editMenu = new JMenu(res.getString("Edit"));
	undoMenuItem = new JMenuItem(res.getString("Undo_Delete"));

	editMenu.add(undoMenuItem);
	editMenu.addSeparator();

	deleteMenu = new JMenu(res.getString("Delete_"));
	deletePackageMenuItem = new JMenuItem(res.getString("Package"));

	deleteMenu.add(deletePackageMenuItem);

	deleteDataObjectMenuItem = new JMenuItem(res.getString("Data_Object"));

	deleteMenu.add(deleteDataObjectMenuItem);

	deleteAttributeMenuItem = new JMenuItem(res.getString("Attribute_s_"));

	deleteMenu.add(deleteAttributeMenuItem);
	editMenu.add(deleteMenu);
	editMenu.addSeparator();

	findErrorsMenuItem = new JMenuItem(res.getString("Find_Next_Error"));

	editMenu.add(findErrorsMenuItem);
	editMenu.addSeparator();

	editPackageMenuItem = new JMenuItem(res.getString("Package_"));

	editMenu.add(editPackageMenuItem);

	editDOMenuItem = new JMenuItem(res.getString("Data_Object_"));

	editMenu.add(editDOMenuItem);

	editAttributeMenuItem = new JMenuItem(res.getString("Attribute_"));

	editMenu.add(editAttributeMenuItem);
	dodsMenuBar.add(editMenu);

	viewMenu = new JMenu(res.getString("View"));
	toolbarCheckBoxMenuItem = new JCheckBoxMenuItem(res.getString("Toolbar"));

	toolbarCheckBoxMenuItem.setState(true);
	viewMenu.add(toolbarCheckBoxMenuItem);
	viewMenu.addSeparator();

	refreshMenuItem = new JMenuItem(res.getString("Refresh"));

	viewMenu.add(refreshMenuItem);
	dodsMenuBar.add(viewMenu);

	insertMenu = new JMenu(res.getString("Insert"));
	insertPackageMenuItem = new JMenuItem(res.getString("Package_"));

	insertMenu.add(insertPackageMenuItem);

	insertDOMenuItem = new JMenuItem(res.getString("Data_Object_"));

	insertMenu.add(insertDOMenuItem);

	// ****opensource contribution by Larry D'Wolcoto
	insertImportedObjectMenuItem = new JMenuItem(res.getString("Imported_Object_"));

	insertMenu.add(insertImportedObjectMenuItem);

	// ****end
	insertAttributeMenuItem = new JMenuItem(res.getString("Attribute_"));

	insertMenu.add(insertAttributeMenuItem);
	dodsMenuBar.add(insertMenu);

	databaseMenu = new JMenu(res.getString("Database"));
	standardJDBCCheckBoxMenuItem = new JCheckBoxMenuItem("Standard JDBC");

	standardJDBCCheckBoxMenuItem.setState(true);
	databaseMenu.add(standardJDBCCheckBoxMenuItem);
        databaseMenu.addSeparator();

	informixCheckBoxMenuItem = new JCheckBoxMenuItem("Informix");
        databaseMenu.add(informixCheckBoxMenuItem);

	msqlCheckBoxMenuItem = new JCheckBoxMenuItem("Microsoft SQL");
	databaseMenu.add(msqlCheckBoxMenuItem);

	oracleCheckBoxMenuItem = new JCheckBoxMenuItem("Oracle");
	databaseMenu.add(oracleCheckBoxMenuItem);

	postCheckBoxMenuItem = new JCheckBoxMenuItem("PostgreSQL");
	databaseMenu.add(postCheckBoxMenuItem);

	sybaseCheckBoxMenuItem = new JCheckBoxMenuItem("Sybase");
	databaseMenu.add(sybaseCheckBoxMenuItem);

	dodsMenuBar.add(databaseMenu);

	helpMenu = new JMenu(res.getString("Help"));
	aboutMenuItem = new JMenuItem(res.getString("About_DODS"));

	helpMenu.add(aboutMenuItem);
	dodsMenuBar.add(helpMenu);
	setJMenuBar(dodsMenuBar);

	// Setting up the DataBase MenuGroup.
	menuGroup = new ButtonGroup();

	menuGroup.add(standardJDBCCheckBoxMenuItem);
	menuGroup.add(informixCheckBoxMenuItem);
	menuGroup.add(msqlCheckBoxMenuItem);
	menuGroup.add(oracleCheckBoxMenuItem);
      	menuGroup.add(postCheckBoxMenuItem);
	menuGroup.add(sybaseCheckBoxMenuItem);

	// Initialize Listeners and add them where appropriate.
	SymWindow aSymWindow = new SymWindow();

	this.addWindowListener(aSymWindow);

	SymAction lSymAction = new SymAction();

	exitMenuItem.addActionListener(lSymAction);
	insertAttributeMenuItem.addActionListener(lSymAction);
	insertDOMenuItem.addActionListener(lSymAction);

	// ****opensource contribution by Larry D'Wolcoto
	insertImportedObjectMenuItem.addActionListener(lSymAction);

	// ****end
	insertPackageMenuItem.addActionListener(lSymAction);
	saveMenuItem.addActionListener(lSymAction);
	openMenuItem.addActionListener(lSymAction);
	editDOMenuItem.addActionListener(lSymAction);
	saveAsMenuItem.addActionListener(lSymAction);
	newMenuItem.addActionListener(lSymAction);
	closeMenuItem.addActionListener(lSymAction);
	editPackageMenuItem.addActionListener(lSymAction);
	deletePackageMenuItem.addActionListener(lSymAction);
	deleteDataObjectMenuItem.addActionListener(lSymAction);
	deleteAttributeMenuItem.addActionListener(lSymAction);
	undoMenuItem.addActionListener(lSymAction);

	SymItem lSymItem = new SymItem();

	toolbarCheckBoxMenuItem.addItemListener(lSymItem);
	standardJDBCCheckBoxMenuItem.addItemListener(lSymItem);
	informixCheckBoxMenuItem.addItemListener(lSymItem);
	msqlCheckBoxMenuItem.addItemListener(lSymItem);
	oracleCheckBoxMenuItem.addItemListener(lSymItem);
      	postCheckBoxMenuItem.addItemListener(lSymItem);

	sybaseCheckBoxMenuItem.addItemListener(lSymItem);
	refreshMenuItem.addActionListener(lSymAction);
	editAttributeMenuItem.addActionListener(lSymAction);
	newButton.addActionListener(lSymAction);
	openButton.addActionListener(lSymAction);
	saveButton.addActionListener(lSymAction);
	editPackageButton.addActionListener(lSymAction);
	editDataObjectButton.addActionListener(lSymAction);
	editAttributeButton.addActionListener(lSymAction);
	insertPackageButton.addActionListener(lSymAction);
	insertDataObjectButton.addActionListener(lSymAction);

	// ****opensource contribution by Larry D'Wolcoto
	insertImportedObjectButton.addActionListener(lSymAction);

	// ****end
	insertAttributeButton.addActionListener(lSymAction);
	buildAllButton.addActionListener(lSymAction);
	buildAllMenuItem.addActionListener(lSymAction);
	aboutMenuItem.addActionListener(lSymAction);
	findErrorsButton.addActionListener(lSymAction);
	findErrorsMenuItem.addActionListener(lSymAction);
	updateMenus();

	attributeEditor = new AttributeEditor(tree, this);
	tableBrowser = new TableBrowser(tree, this);

	errorsExist();
    }

    /**
     * The main method creates an instance of DataObjectApplication and shows it.
     */
    public static void main(String args[]) {
	System.out.println(res.getString("Starting_DODS_"));
	Util.setLookForWindowsOnly();

	DataObjectApplication app = new DataObjectApplication();
	boolean		      cool = true;

	if (args.length > 0) {
	    try {
		File file = new File(args[0]);

		if (null != file && file.isFile()) {
		    app.store.setInputFile(file);
		    app.readStore(null);
		    app.updateTree();

		    app.treeRoot = new DOTreeNode(app.store.getProject(),
						  true);

		    app.graphicalPanel.setDODSProject(app.store.getProject());
		    app.updateTree();
		    app.tree.setRootVisible(true);
		    app.setDataObjectInTable(null);
		    app.updateTable();
		    app.updateMenus();
		    app.setCursor(app.normal);
		} else {
		    System.err.println(args[0]
				       + res.getString("is_not_a_project_file"));

		    cool = false;
		}
	    } catch (Exception e) {
		System.err.println(res.getString("First_argument_error_")
				   + e.getMessage());

		cool = false;

		e.printStackTrace();
	    }

	    if (args.length > 1) {
		try {
		    File outputDir = new File(args[1]);

		    if (null != outputDir
			    && ((!outputDir.exists())
				|| outputDir.isDirectory())) {
			app.targetDir = outputDir;
		    } else {
			System.err.println(args[1]
					   + res.getString("cannot_be_the_output"));

			cool = false;
		    }
		} catch (Exception e) {
		    System.err.println(res.getString("Second_argument_error")
				       + e.getMessage());

		    cool = false;

		    // FIX remove next line
		    e.printStackTrace();
		}

		if (args.length > 2) {
		    int exit_code = 0;

		    if (args[2].equals("regen")) {

			// If 3rd arg is "regen", we just regenerate the code.
			// we do not show the gui.
			//
			// Useful for testing changes to .template files.
			//
			// Also useful for rebuilding an entire application
			// when a DODS project file is included in the
			// source code that is put under version control.
			// The "regen" method of invoking DODS would be
			// used in the application's makefile.
			//
			// NOTE:  use with caution!  Directory specified
			// in 2nd arg is deleted and rewritten!
			try {
			    if (cool) {
				Builder.run(args[1], app.store.getProject());
			    }
			} catch (Exception e) {
			    exit_code = 2;

			    System.err.println(res.getString("Code_Generator_failed")
					       + e.getMessage());
			    e.printStackTrace();
			}
		    } else {
			System.err.println(res.getString("Third_argument_was"));

			exit_code = 1;
		    }

		    System.exit(exit_code);
		}
	    }
	}

	app.setVisible(true);
    }

    // All of the attributes for the DataObjectApplication.
    private AttributeEditor	    attributeEditor;

    // ****opensource contribution by Larry D'Wolcoto
    private TableBrowser	    tableBrowser;

    // ****end
    private BuildMonitorDialog      bmd = null;
    private File		    targetDir = null;
    private JMenuBar		    dodsMenuBar;
    private JSplitPane		    verticalBar;
    private JSplitPane		    horizontalBar;
    private DOTableModel	    doTableModel;
    private DOTreeModel		    doTreeModel;
    private JTree		    tree;
    private JTable		    table;
    private JScrollPane		    top;
    private GraphicalPanel	    graphicalPanel;
    private DOTreeSelectionListener doTreeSelectionListener;
    private DOTreeNode		    treeRoot;
    private JToolBar		    toolBar;

    // The holding spots for deleted objects in case of undo.
    private Vector		    deletedNodeExtendedBy;
    private DataObject		    deletedNodeExtensionOf;
    private DOTreeNode		    deletedNode;
    private DOTreeNode		    deletedNodeParent;
    private Vector		    deletedAttributes = new Vector();

    // The cursors
    private Cursor		    wait =
	Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
    private Cursor		    normal =
	Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

    // All of the Buttons
    private JButton		    newButton;
    private JButton		    openButton;
    private JButton		    saveButton;
    private JButton		    editPackageButton;
    private JButton		    editDataObjectButton;
    private JButton		    editAttributeButton;
    private JButton		    insertPackageButton;

    // ****opensource contribution by Larry D'Wolcoto
    private JButton		    insertImportedObjectButton;

    // ****end
    private JButton		    insertDataObjectButton;
    private JButton		    insertAttributeButton;
    private JButton		    buildAllButton;
    private JButton		    findErrorsButton;

    // All of the Menus.
    private JMenu		    fileMenu;
    private JMenuItem		    newMenuItem;
    private JMenuItem		    openMenuItem;
    private JMenuItem		    closeMenuItem;
    private JMenuItem		    saveMenuItem;
    private JMenuItem		    saveAsMenuItem;
    private JMenuItem		    exitMenuItem;
    private JMenu		    editMenu;
    private JMenuItem		    undoMenuItem;
    private JMenu		    deleteMenu;
    private JMenuItem		    deletePackageMenuItem;
    private JMenuItem		    deleteDataObjectMenuItem;
    private JMenuItem		    deleteAttributeMenuItem;
    private JMenuItem		    findErrorsMenuItem;
    private JMenuItem		    editPackageMenuItem;
    private JMenuItem		    editDOMenuItem;
    private JMenuItem		    editAttributeMenuItem;
    private JMenu		    viewMenu;
    private JCheckBoxMenuItem       toolbarCheckBoxMenuItem;
    private JMenuItem		    refreshMenuItem;
    private JMenu		    insertMenu;
    private JMenuItem		    insertPackageMenuItem;
    private JMenuItem		    insertDOMenuItem;

    // ****opensource contribution by Larry D'Wolcoto
    private JMenuItem		    insertImportedObjectMenuItem;

    // ****end
    private JMenuItem		    insertAttributeMenuItem;
    private JMenuItem		    buildAllMenuItem;
    private JMenu		    databaseMenu;
    private JCheckBoxMenuItem       standardJDBCCheckBoxMenuItem;
    private JCheckBoxMenuItem       informixCheckBoxMenuItem;
    private JCheckBoxMenuItem       msqlCheckBoxMenuItem;
    private JCheckBoxMenuItem       oracleCheckBoxMenuItem;
    private JCheckBoxMenuItem       postCheckBoxMenuItem;
    private JCheckBoxMenuItem       sybaseCheckBoxMenuItem;
    private JMenu		    helpMenu;
    private ButtonGroup		    menuGroup;
    private JMenuItem		    aboutMenuItem;

    /**
     * This method changes the DataObject that has its Attributes displayed in the Table.
     * @param DO The new DataObject in the table.
     */
    public void setDataObjectInTable(DataObject DO) {
	if (doTableModel == null) {
	    return;
	} else {
	    doTableModel.setDataObject(DO);
	}

	boolean errors = errorsExist();

	table.revalidate();
	table.repaint();
	updateMenus();
    }

    /*
     * This method is called when the window receives a closing event.
     */

    /**
     * Method declaration
     *
     *
     * @param event
     */
    private void DataObjectApplication_WindowClosing(WindowEvent event) {
	if (showSaveOptionDialog() == 2) {

	    // do nothing because exiting was canceled.
	} else {
	    setVisible(false);
	    dispose();
	    System.exit(0);
	}
    }

    /*
     * This method is called when the Menu receives a exit event.
     */

    /**
     * Method declaration
     *
     *
     * @param event
     *
     */
    private void exitMenuItem_ActionPerformed(ActionEvent event) {
	if (showSaveOptionDialog() == 2) {

	    // do nothing because the exiting was canceled.
	} else {
	    setVisible(false);
	    dispose();
	    System.exit(0);
	}
    }

    /*
     * This Method saves the Data Object tree.
     */

    /**
     * Method declaration
     *
     *
     * @throws Exception
     *
     */
    private void saveTree() throws Exception {
	File file = store.getProject().getFile();

	if (file == null) {
	    saveAs();
	} else {
	    saveFile();
	}
    }

    /*
     * This Method saves the Data Object tree when asked to.
     */

    /**
     * Method declaration
     *
     *
     */
    private void saveFile() {

	// When an exception is caught, the SaveAs.. Dialog is called so
	// the file can be renamed.
	try {
	    store.write();
	} catch (Exception e) {
	    saveAs();

	    return;
	}

	boolean errors = errorsExist();

	updateMenus();
    }

    /*
     * This method opens a saved DODSProject and throws away the old one,
     * but first it gives the option to save.
     */

    /**
     * Method declaration
     *
     *
     */
    private void openFile() {
	DODSFileChooser fileChooser = new DODSFileChooser();

	fileChooser.setCurrentDirectory(store.getCurrentDirectory());

	if (showSaveOptionDialog() != 2) {
	    int choice = fileChooser.showOpenDialog(this);

	    if (choice != DODSFileChooser.CANCEL_OPTION) {
		setCursor(wait);

		File file = fileChooser.getSelectedFile();

		store.setInputFile(file);

		try {
		    readStore(this);

		    if (store.getProject() == null) {
			store.create();
			JOptionPane.showMessageDialog(null,
						      res.getString("Unable_to_load_file_")
						      + file, appSecondary,
							      JOptionPane.WARNING_MESSAGE);
		    }
		} catch (dods.doml.UpException e) {
		    store.create();
            Object[] args = { file };
            MessageFormat formatter = new MessageFormat(res.getString("Unable_to_convert_DOML_format"));
            String msg = formatter.format(args);
		    JOptionPane.showMessageDialog(null,
                          msg,
						  appSecondary,
						  JOptionPane.WARNING_MESSAGE);
		}

		treeRoot = new DOTreeNode(store.getProject(), true);

		graphicalPanel.setDODSProject(store.getProject());
		updateTree();
		tree.setRootVisible(true);

		boolean errors = errorsExist();

		setDataObjectInTable(null);
		updateTable();
		updateMenus();
		setCursor(normal);
	    }
	}
    }

    /**
     * this Method updates the table's comboBox Editor to include the appropriate options.
     */
    public void updateTable() {
	boolean errors = errorsExist();

	updateMenus();
	table.revalidate();
	table.repaint();
    }

    /**
     * This method resets the tree when the set of DataObjects and packages changes,
     * while ensuring that the open and selected nodes remain open and selected.
     */
    public void updateTree() {
	TreePath selectionPath = tree.getSelectionPath();

	treeRoot.updateChildrenOrder();

	Vector openPaths = new Vector();

	for (int i = 0; i < tree.getRowCount(); i++) {
	    if (tree.isExpanded(tree.getPathForRow(i))) {
		openPaths.addElement(tree.getPathForRow(i));
	    }
	}

	doTreeModel = new DOTreeModel(treeRoot, this);

	tree.setModel(doTreeModel);

	for (int i = 0; i < openPaths.size(); i++) {
	    tree.expandPath((TreePath) openPaths.elementAt(i));
	}

	setDataObjectInTable(null);
	tree.setSelectionPath(selectionPath);
	store.getProject().updatePackageName();
	updateGraphicalPanel();
    }

    /**
     * This method will update the graphical View when needed.
     */
    public void updateGraphicalPanel() {
	if (graphicalPanel == null) {
	    return;
	}

	setCursor(wait);
	graphicalPanel.setDODSProject(store.getProject());
	graphicalPanel.updateBranches();
	graphicalPanel.getPreferredSize();
	graphicalPanel.repaint();
	setCursor(normal);
    }

    /*
     * This method is called when a new Data object is to be inserted.
     */

    /**
     * Method declaration
     *
     *
     * @param event
     *
     */
    private void insertDOMenuItem_ActionPerformed(ActionEvent event) {
	DataObject DO = new DataObject();

	editDataObject(DO);
    }

    // ****opensource contribution by Larry D'Wolcoto

    /**
     * This method is called when a new Imported Table object is to be inserted.
     */
    private void insertImportedObjectMenuItem_ActionPerformed(ActionEvent event) {
	DataObject DO = new DataObject();

	ImportObject(DO);
    }

    /**
     * This method is called when a new Package is to be inserted.
     */
    private void insertPackageMenuItem_ActionPerformed(ActionEvent event) {
	DOPackage DOP = new DOPackage();

	editDOPackage(DOP);
    }

    /**
     * This Method shows the Attribute Editor with a new Attribute which.
     * is then created.
     */
    private void insertAttributeMenuItem_ActionPerformed(ActionEvent event) {
	Attribute att = new Attribute();

	editAttribute(att);
	table.setRowSelectionInterval(doTableModel.getRowCount() - 1,
				      doTableModel.getRowCount() - 1);
	table.repaint();
    }

    /*
     */

    /**
     * This method is called when a Data object is to be edited.
     */
    private void editDOMenuItem_ActionPerformed(ActionEvent event) {
	TreePath   path = tree.getSelectionPath();
	DOTreeNode node = (DOTreeNode) path.getLastPathComponent();
	DataObject DO = (DataObject) node.getDataObject();
	Object[]   newpath = new Object[path.getPath().length - 1];

	for (int i = 0; i < path.getPath().length - 1; i++) {
	    newpath[i] = path.getPath()[i];
	}

	// This is done so the dataObjectEditor will set up the correct
	// path for the PackageName.
	tree.setSelectionPath(new TreePath(newpath));
	editDataObject(DO);
    }

    /**
     * This method is called when a Package is to be edited.
     */
    private void editPackageMenuItem_ActionPerformed(ActionEvent event) {
	TreePath   path = tree.getSelectionPath();
	DOTreeNode node = (DOTreeNode) path.getLastPathComponent();
	DOPackage  DOP = (DOPackage) node.getDataObject();

	if (DOP instanceof DODSProject) {
	    editRoot(DOP);
	} else {
	    Object[] newpath = new Object[path.getPath().length - 1];

	    for (int i = 0; i < path.getPath().length - 1; i++) {
		newpath[i] = path.getPath()[i];
	    }

	    // This is done so the packageEditor will set up the correct
	    // path for the PackageName.
	    tree.setSelectionPath(new TreePath(newpath));
	    editDOPackage(DOP);
	}
    }

    /**
     * This method is called when an Attribute is to be edited.
     */
    private void editAttributeMenuItem_ActionPerformed(ActionEvent event) {
	Attribute att =
	    (Attribute) (doTableModel.getDataObject().getAttributes().elementAt(table.getSelectedRow()));

	editAttribute(att);
    }

    /**
     * This method takes care of the initialization of dataObjbectEditor and shows it.
     * @param DO The DataObject to be edited.
     */
    public void editDataObject(DataObject DO) {
	DataObjectEditor dataObjectEditor;

	setCursor(wait);

	dataObjectEditor = new DataObjectEditor(tree, this);

	dataObjectEditor.setDataObject(DO);
	dataObjectEditor.setVisible(true);
	updateGraphicalPanel();

	boolean errors = errorsExist();

	updateMenus();
	setCursor(normal);
    }

    // ****opensource contribution by Larry D'Wolcoto

    /**
     * This method takes care of the initialization of tableBrowser and shows it.
     * @param DO The DataObject to be edited.
     */
    public void ImportObject(DataObject DO) {
	setCursor(wait);
	tableBrowser.setDataObject(DO);
	tableBrowser.setVisible(true);
	updateGraphicalPanel();

	boolean errors = errorsExist();

	updateMenus();
	setCursor(normal);
    }

    // ****end

    /**
     * This method takes care of the initialization of packageEditor and shows it.
     * @param DOP The DOPackage to be edited.
     */
    public void editDOPackage(DOPackage DOP) {
	setCursor(wait);

	PackageEditor packageEditor;

	packageEditor = new PackageEditor(tree, this);

	packageEditor.setDOPackage(DOP);
	setCursor(normal);
	packageEditor.setVisible(true);

	boolean errors = errorsExist();

	updateMenus();
    }

    /**
     * This method takes care of the initialization of attributeEditor and shows it.
     * @param att The Attribute to be edited.
     */
    public void editAttribute(Attribute att) {
	setCursor(wait);
	attributeEditor.setAttribute(att, doTableModel.getDataObject());
	attributeEditor.setVisible(true);

	boolean errors = errorsExist();

	updateMenus();
	doTableModel.resetRows();
	table.repaint();
	updateGraphicalPanel();
	setCursor(normal);
    }

    /**
     * This method allows you to change the name of the root package.
     */
    public void editRoot(DOPackage DOP) {
	String newName = JOptionPane.showInputDialog(this,
		res.getString("Please_enter_the_new_name"),
		res.getString("Root_Editor"),
        JOptionPane.QUESTION_MESSAGE);

	if (newName != null) {
	    try {
		store.getProject().setName(newName);
	    } catch (DataObjectNameException e) {
		namingExceptionOccurred(e, this);
		editRoot(DOP);
	    } catch (NullPointerException e) {
		e.printStackTrace();
	    }

	    boolean errors = errorsExist();

	    updateTree();
	    updateMenus();
	}
    }

    /**
     * This method opens a save File Dialog and saves the project.
     */
    private void saveAs() {
	DODSFileChooser fileChooser = new DODSFileChooser();

	fileChooser.setCurrentDirectory(store.getCurrentDirectory());

	if (store.getOutputFile() == null) {
	    fileChooser.setSelectedFile(new File(res.getString("untitled_doml")));
	} else {
	    fileChooser.setSelectedFile(store.getOutputFile());
	}

	int choice = fileChooser.showSaveDialog(this);

	if (choice != DODSFileChooser.CANCEL_OPTION) {
	    try {
		File file = fileChooser.getSelectedFile();

		store.setOutputFile(file);

		// If the file exists, confirm that we want to overwrite it.
		if (file.exists()) {
		    if (JOptionPane.YES_OPTION
			    == JOptionPane.showConfirmDialog(this,
							     res.getString("That_file_exists"),
							     res.getString("Confirm_File_Save"),
							     JOptionPane.YES_NO_CANCEL_OPTION)) {
			store.write();
		    }
		} else {

		    // Otherwise, just save it.
		    store.write();
		}
	    } catch (Exception e) {
		System.err.println(e.toString());
		JOptionPane.showMessageDialog(this, res.getString("Error_saving_file_"),
					      res.getString("Error_"),
					      JOptionPane.ERROR_MESSAGE);
	    }

	    boolean errors = errorsExist();

	    updateMenus();
	}
    }

    /**
     * This method removes the currentproject(with an option to save) and
     * creates a new one.
     */
    private void newFile() {
	if (showSaveOptionDialog() == 2) {
	    return;
	}

	store.create();

	treeRoot = new DOTreeNode(store.getProject(), true);

	updateTree();
	tree.setRootVisible(true);
	setDataObjectInTable(null);
	updateTable();

	boolean errors = errorsExist();

	updateMenus();
    }

    /**
     * This Method closes the current project but leaves the program open.
     */
    private void closeFile() {
	if (showSaveOptionDialog() == 2) {
	    return;
	}

	setDataObjectInTable(null);
	table.repaint();
	updateGraphicalPanel();
	store.create();

	treeRoot = new DOTreeNode(store.getProject(), true);

	updateTree();
	tree.setRootVisible(false);

	boolean errors = errorsExist();

	updateMenus();
    }

    /**
     * This method is called frequently to update which menus are enabled and disabled.
     * A menuItem will be disabled if It cannot currently be activated.
     */
    public void updateMenus() {
	updateDatabaseMenu();    // Sets up which database is selected.

	if (tree.isRootVisible()) {
	    saveAsMenuItem.setEnabled(true);
	} else {
	    saveAsMenuItem.setEnabled(false);
	}

	if (isSaved()) {
	    saveMenuItem.setEnabled(false);
	    saveButton.setEnabled(false);
	} else {
	    saveMenuItem.setEnabled(true);
	    saveButton.setEnabled(true);
	}

	if (tree.getSelectionPath() != null) {
	    TreePath   path = tree.getSelectionPath();
	    DOTreeNode node = (DOTreeNode) path.getLastPathComponent();

	    if (node.getDataObject() instanceof DOPackage) {
		deletePackageMenuItem.setEnabled(true);
		deleteDataObjectMenuItem.setEnabled(false);
	    } else {
		deleteDataObjectMenuItem.setEnabled(true);
		deletePackageMenuItem.setEnabled(false);
	    }

	    if (node.isLeaf()) {
		editPackageMenuItem.setEnabled(false);
		editPackageButton.setEnabled(false);
		editDOMenuItem.setEnabled(true);
		editDataObjectButton.setEnabled(true);
	    } else {
		editPackageMenuItem.setEnabled(true);
		editPackageButton.setEnabled(true);
		editDOMenuItem.setEnabled(false);
		editDataObjectButton.setEnabled(false);
	    }

	    if (path.getPath().length == 1) {
		deletePackageMenuItem.setEnabled(false);
		deleteDataObjectMenuItem.setEnabled(false);
	    }
	} else {
	    editPackageMenuItem.setEnabled(false);
	    editPackageButton.setEnabled(false);
	    editDOMenuItem.setEnabled(false);
	    editDataObjectButton.setEnabled(false);
	    deletePackageMenuItem.setEnabled(false);
	    deleteDataObjectMenuItem.setEnabled(false);
	}

	if (tree.isRootVisible()) {
	    closeMenuItem.setEnabled(true);
	    insertPackageMenuItem.setEnabled(true);
	    insertDOMenuItem.setEnabled(true);
	    insertPackageButton.setEnabled(true);
	    insertDataObjectButton.setEnabled(true);
	} else {
	    closeMenuItem.setEnabled(false);
	    insertPackageMenuItem.setEnabled(false);
	    insertDOMenuItem.setEnabled(false);
	    insertDataObjectButton.setEnabled(false);
	    insertPackageButton.setEnabled(false);
	}

	if (deletedNode != null) {
	    undoMenuItem.setEnabled(true);
	} else {
	    undoMenuItem.setEnabled(false);
	}

	Vector     allDOS = new Vector();
	DataObject DO = treeRoot.getDataObject();

	((DODSProject) DO).getAllDataObjects(allDOS);

	if (allDOS.size() == 0) {
	    insertAttributeMenuItem.setEnabled(false);
	    insertAttributeButton.setEnabled(false);
	} else {
	    insertAttributeMenuItem.setEnabled(true);
	    insertAttributeButton.setEnabled(true);
	}

	if (table.getSelectedRow() < 0 || table.getRowCount() == 0
		|| table.getSelectedRow()
		   > doTableModel.getDataObject().getAttributes().size()
		     - 1) {
	    editAttributeMenuItem.setEnabled(false);
	    editAttributeButton.setEnabled(false);
	    deleteAttributeMenuItem.setEnabled(false);
	} else {
	    editAttributeMenuItem.setEnabled(true);
	    editAttributeButton.setEnabled(true);
	    deleteAttributeMenuItem.setEnabled(true);
	}

	if (errorsExist()) {
	    buildAllMenuItem.setEnabled(false);
	    buildAllButton.setEnabled(false);

	    if (allDOS.size() != 0) {
		findErrorsMenuItem.setEnabled(true);
		findErrorsButton.setEnabled(true);
	    } else {
		findErrorsMenuItem.setEnabled(false);
		findErrorsButton.setEnabled(false);
	    }
	} else {
	    buildAllMenuItem.setEnabled(true);
	    buildAllButton.setEnabled(true);
	    findErrorsMenuItem.setEnabled(false);
	    findErrorsButton.setEnabled(false);
	}

	setTitle();
    }

    /**
     * This method compares the current project to it's saved version.
     * @return True when the two are equal, meaning it has been saved.
     */
    public boolean isSaved() {
	boolean saved = false;

	if (treeRoot.getChildren().size() == 0) {
	    saved = true;
	} else {
	    if (store.isSaved()) {
		saved = true;
	    } else {
		saved = (!tree.isRootVisible());
	    }
	}

	return saved;
    }

    /**
     * This method is called when the current project is being closed.  It
     * presents a dialog asking if the user wants to save changes to the
     * current project. If they do, then the save action takes place in this method.
     * @return 0 if the user wants to save. <br> 1 if the user doesn't want
     * to save. <br> 2 if the user wants to cancel their action.
     */
    public int showSaveOptionDialog() {
	if (!isSaved()) {
	    String message;
	    try {
           Object[] args = { store.getProject().getFile().getName() };
           MessageFormat formatter = new MessageFormat(res.getString("Save_Changes_to"));
           message = formatter.format(args);
	    } catch (NullPointerException x) {
		   message = res.getString("Save_Changes_to1");
	    }

	    JOptionPane saveOptionPane = new JOptionPane(message,
		    JOptionPane.QUESTION_MESSAGE,
		    JOptionPane.YES_NO_CANCEL_OPTION);
	    JDialog     saveOptionDialog = saveOptionPane.createDialog(this,
		    res.getString("DataObject"));

	    saveOptionDialog.setVisible(true);

	    if (saveOptionPane.getValue().equals(new Integer(2))) {
		return 2;
	    }

	    if (saveOptionPane.getValue().equals(new Integer(0))) {
		saveFile();
	    }

	    return 0;
	}

	return 1;
    }

    /**
     * When a naming exception occurs, this method is called, and an appropriate
     * warning dialog is displayed.
     * @param e The Name Exception which contains the message.
     * @param par The Component where the error occurred.
     */
    public void namingExceptionOccurred(DataObjectNameException e,
					Component par) {
	JOptionPane.showMessageDialog(par, e.getMessage(), res.getString("Naming_Error"),
				      JOptionPane.WARNING_MESSAGE);
    }

    /**
     * This method deletes a package or DataObject.
     */
    private void deleteMenuItem_ActionPerformed(ActionEvent event) {
	setCursor(wait);
	deletedAttributes.removeAllElements();

	TreePath   path = tree.getSelectionPath();
	Object[]   pathComponents = path.getPath();
	DOTreeNode node =
	    (DOTreeNode) pathComponents[pathComponents.length - 1];
	DOTreeNode parentNode =
	    (DOTreeNode) pathComponents[pathComponents.length - 2];

	if (node.isLeaf()) {
	    int result;

	    if (node.getDataObject().getDescendents().size() != 0) {
		setCursor(normal);

		result =
		    JOptionPane.showConfirmDialog(this,
						  res.getString("This_Data_Object_is"),
						  res.getString("Delete_Object"),
						  JOptionPane.YES_NO_OPTION);
	    } else {
		result = JOptionPane.YES_OPTION;
	    }

	    switch (result) {

	    case JOptionPane.YES_OPTION:
		setCursor(wait);

		// FIX?  This is not undoable.
		deleteReferringAttributes(node);

		deletedNodeExtensionOf =
		    node.getDataObject().getExtensionOf();
		deletedNodeExtendedBy = new Vector();

		for (int i = 0;
			i < node.getDataObject().getExtendedBy().size();
			i++) {
		    deletedNodeExtendedBy.addElement(node.getDataObject().getExtendedBy().elementAt(i));
		}

		node.getDataObject().setExtensionOf(null);

		for (int i = 0;
			i < node.getDataObject().getExtendedBy().size();
			i += 0) {
		    DataObject descend =
			(DataObject) node.getDataObject().getExtendedBy().elementAt(i);

		    descend.setExtensionOf(null);
		}

		((DOPackage) parentNode.getDataObject()).getDataObjects().removeElement(node.getDataObject());
		parentNode.getChildren().removeElement(node);

		deletedNode = node;
		deletedNodeParent = parentNode;

		break;

	    case JOptionPane.NO_OPTION:
		setCursor(normal);

		return;

	    default:
		setCursor(normal);

		return;
	    }
	} else {
	    if (node.getChildren().size() != 0) {
		setCursor(normal);
		JOptionPane.showMessageDialog(this,
					      res.getString("This_Package_cannot"),
					      res.getString("Delete_Package"),
					      JOptionPane.WARNING_MESSAGE);

		return;
	    } else {
		((DOPackage) parentNode.getDataObject()).getDataObjects().removeElement(node.getDataObject());
		parentNode.getChildren().removeElement(node);

		deletedNode = node;
		deletedNodeParent = parentNode;
	    }
	}

	boolean errors = errorsExist();

	updateTree();
	updateMenus();
	setCursor(normal);
    }

    /**
     * Search all DataObjects for Attributes that refer to this DataObject,
     * and remove those Attributes.
     * Not (currently) undoable.
     */
    private void deleteReferringAttributes(DOTreeNode deletedNode) {
	if (null == deletedNode) {
	    return;
	}

	DataObject deletedDO = deletedNode.getDataObject();
	DOTreeNode treeRoot = new DOTreeNode(store.getProject(), true);

	deleteReferringAttributes(treeRoot, deletedDO);
	refreshDODS();
    }

    /**
     * Method declaration
     *
     *
     * @param checkNode
     * @param deletedDO
     *
     */
    private void deleteReferringAttributes(DOTreeNode checkNode,
					   DataObject deletedDO) {
	DataObject DO = checkNode.getDataObject();

	if (DO instanceof DataObject) {
	    Vector attrs = DO.getAttributes();

	    for (int j = 0; j < attrs.size(); j++) {
		Attribute  attr = (Attribute) attrs.elementAt(j);
		DataObject ref = attr.getObjectRefDO();

		if (null != ref && deletedDO == ref) {
		    attrs.removeElement(attr);

		    j--;    // otherwise we'd skip the next one
		}
	    }
	}

	Vector children = checkNode.getChildren();

	if (null != children) {
	    for (int i = 0; i < children.size(); i++) {
		DOTreeNode node = (DOTreeNode) children.elementAt(i);

		deleteReferringAttributes(node, deletedDO);
	    }
	}
    }

    /**
     * This reverses the most recent delete action.
     */
    private void undoMenuItem_ActionPerformed(ActionEvent event) {
	setCursor(wait);

	if (deletedAttributes.size() == 0) {
	    if (deletedNode.getDataObject() instanceof DOPackage) {
		((DOPackage) deletedNodeParent.getDataObject()).addDOPackage((DOPackage) deletedNode.getDataObject());
	    } else {
		((DOPackage) deletedNodeParent.getDataObject()).addDataObject(deletedNode.getDataObject());

		if (deletedNodeExtensionOf != null) {
		    deletedNode.getDataObject().setExtensionOf(deletedNodeExtensionOf);
		}

		if (deletedNodeExtendedBy != null) {
		    for (int i = 0; i < deletedNodeExtendedBy.size(); i++) {
			((DataObject) deletedNodeExtendedBy.elementAt(i)).setExtensionOf(deletedNode.getDataObject());
		    }
		}
	    }

	    deletedNodeParent.getChildren().addElement(deletedNode);

	    deletedNodeExtensionOf = null;
	    deletedNodeExtendedBy = null;
	    deletedNode = null;
	    deletedNodeParent = null;

	    boolean errors = errorsExist();

	    updateTree();
	    updateMenus();
	} else {
	    for (int i = 0; i < deletedAttributes.size(); i++) {
		deletedNode.getDataObject().addAttribute((Attribute) deletedAttributes.elementAt(i));
	    }

	    deletedAttributes.removeAllElements();

	    deletedNode = null;

	    boolean errors = errorsExist();

	    doTableModel.resetRows();
	    table.setRowSelectionInterval(-1, -1);

	    errors = errorsExist();

	    table.repaint();
	    updateGraphicalPanel();
	    updateMenus();
	}

	setCursor(normal);
    }

    /**
     * This method adds or removes the toolbar depending on the state.
     */
    private void toolbarCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
	if (!toolbarCheckBoxMenuItem.getState()) {
	    getContentPane().remove(toolBar);
	    SwingUtilities.updateComponentTreeUI(this);
	} else {
	    getContentPane().add("North", toolBar);
	    SwingUtilities.updateComponentTreeUI(this);
	}
    }

    /**
     * The refresh menu item is for when inexplicable painting errors occur
     * and the user wants to repaint everything to remove them.
     */
    private void refreshMenuItem_ActionPerformed(ActionEvent event) {
	refreshDODS();
    }

    /**
     * This function will refresh the entire DODS application.
     */
    public void refreshDODS() {

	// errorsExist();
	updateTree();
	updateMenus();
	repaint();
    }

    /**
     * This method checks all of the Attributes and if they have errors, then
     * this method takes the appropriate action.
     */
    public void checkForErrors() {
	DataObject dataobject = doTableModel.getDataObject();
	Vector     atts = dataobject.getAttributes();

	for (int i = 0; i < atts.size(); i++) {
	    Attribute att = (Attribute) atts.elementAt(i);
	    boolean   hasError = false;
	    String    errorText = new String();
	    String    javatype = att.getJavaType();
	    String    iValue = att.getInitialValue();

	    // Most of the error checking involves valid initial values.
	    if (javatype.equals("String")) {
		hasError = false;    // any string is valid.
	    } else if (javatype.equals("java.math.BigDecimal")) {
		try {
		    java.math.BigDecimal bd =
			new java.math.BigDecimal(iValue);

		    hasError = false;
		} catch (NumberFormatException e) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		}
	    } else if (javatype.equals("boolean")) {
		if (!iValue.equals("true") &&!iValue.equals("false")) {
		    hasError = true;
		    errorText = res.getString("Must_be_true_or_false");
		} else {
		    hasError = false;
		}
	    } else if (javatype.equals("byte[]")) {
		if (!iValue.equals("")) {
		    hasError = true;
		    errorText =
			res.getString("No_Initial_Value");

		    // FIX: Should you allow "{byte1, byte2, byte3}"?
		} else {
		    hasError = false;
		}
	    } else if (javatype.equals("byte")) {
		try {
		    Byte bt = new Byte(iValue);

		    hasError = false;
		} catch (NumberFormatException e) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		}
	    } else if (javatype.equals("short")) {
		try {
		    Short st = new Short(iValue);

		    hasError = false;
		} catch (NumberFormatException e) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		}
	    } else if (javatype.equals("int")) {
		try {
		    Integer inte = new Integer(iValue);

		    hasError = false;
		} catch (NumberFormatException e) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		}
	    } else if (javatype.equals("long")) {
		try {
		    Long lo = new Long(iValue);

		    hasError = false;
		} catch (NumberFormatException e) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		}
	    } else if (javatype.equals("float")) {
		try {
		    Float lo = new Float(iValue);

		    hasError = false;
		} catch (NumberFormatException e) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		}
	    } else if (javatype.equals("double")) {
		try {
		    Double lo = new Double(iValue);

		    hasError = false;
		} catch (NumberFormatException e) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		}
	    } else if (javatype.equals("java.sql.Date")) {
		try {
		    String  s = new String(iValue);
		    boolean b =
			(java.sql.Date.valueOf(s)).toString().equals(iValue);

		    hasError = false;
		} catch (Exception e) {
		    hasError = true;
		    errorText =
			res.getString("Initial_Value_must_be");
		}
	    } else if (javatype.equals("java.sql.Time")) {
		try {
		    String  s = new String(iValue);
		    boolean b =
			(java.sql.Time.valueOf(s)).toString().equals(iValue);

		    hasError = false;
		} catch (Exception e) {
		    hasError = true;
		    errorText =
			res.getString("Initial_Value_must_be1");
		}
	    } else if (javatype.equals("java.sql.Timestamp")) {
		try {
		    String  s = new String(iValue);
		    boolean b =
			(java.sql.Timestamp.valueOf(s)).toString().equals(iValue);

		    hasError = false;
		} catch (Exception e) {
		    hasError = true;
		    errorText =
			res.getString("Initial_Value_must_be2");
		}
	    } else {		     // for all Object References.
		if (!iValue.equals("")) {
		    hasError = true;
		    errorText = res.getString("Invalid_Initial_Value");
		} else {
		    hasError = false;
		}
	    }

	    if (iValue.equals("")) {
		hasError =
		    false;    // if there is no initial value then it can't be invalid.
	    }

	    if (sizeHasError(att)) {    // If the attribute has an invalid size.
		hasError = true;

		if (!errorText.equals("")) {
		    errorText += "\n";
		}

		errorText = res.getString("Invalid_size_for_this");
	    }

	    // Duplicated names in an Attribute cause an error.
	    for (int j = 0; j < atts.size(); j++) {
		Attribute at = (Attribute) atts.elementAt(j);

		if (at != att) {
		    if (at.getName().equals(att.getName())) {
			hasError = true;

			if (!errorText.equals("")) {
			    errorText += " \n ";
			}

			errorText += res.getString("Duplicate_Attribute");
		    }
		}
	    }

	    if (att.isObjectRef()) {
		if (att.getObjectRefDO().isAbstract()) {
		    hasError = true;

		    if (!errorText.equals("")) {
			errorText += " \n ";
		    }

		    errorText += res.getString("Referencing_an");
		}
	    }

	    att.setError(hasError);
	    att.setErrorText(errorText);
	}
    }

    /**
     * Currently always returns false, which should be changed.
     */
    private boolean sizeHasError(Attribute att) {
	String db = att.getDBType();

	// FIX: shouldn't always return false.
	return false;
    }

    /**
     * called when Standard is chosen as the Database.
     */
    private void standardJDBCCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
	if (standardJDBCCheckBoxMenuItem.getState()) {
	    store.getProject().setDatabase("Standard");
	}
    }

    /**
     * called when Oracle is chosen as the Database.
     */
    private void oracleCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
	if (oracleCheckBoxMenuItem.getState()) {
	    store.getProject().setDatabase("Oracle");
	}
    }

    /**
     * called when Informix is chosen as the Database.
     */
    private void informixCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
	if (informixCheckBoxMenuItem.getState()) {
	    store.getProject().setDatabase("Informix");
	}
    }

    /**
     * called when MSQL is chosen as the Database.
     */
    private void msqlCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
	if (msqlCheckBoxMenuItem.getState()) {
	    store.getProject().setDatabase("MSQL");
	}
    }

    /**
     * called when PostgresSQL is chosen as the Database.
     */
    private void postCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
	if (postCheckBoxMenuItem.getState()) {
	    store.getProject().setDatabase("PostgreSQL");
	}
    }


    /**
     * called when Sybase is chosen as the Database.
     */
    private void sybaseCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
	if (sybaseCheckBoxMenuItem.getState()) {
	    store.getProject().setDatabase("Sybase");
	}
    }

    /**
     * @return True if any of the attributes have errors in them.
     */
    public boolean errorsExist() {

	// System.err.println("errorsExist() was called!");
	checkForErrors();

	Vector allDOS = new Vector();

	store.getProject().getAllDataObjects(allDOS);

	for (int i = 0; i < allDOS.size(); i++) {
	    DataObject DO = (DataObject) allDOS.elementAt(i);

	    for (int j = 0; j < DO.getAttributes().size(); j++) {
		if (((Attribute) DO.getAttributes().elementAt(j)).hasError()) {
		    return true;
		}
	    }
	}

	if (allDOS.size() == 0) {
	    return true;    // no DataObjects existing counts as an error.
	}

	return false;
    }

    /**
     * Launch Builder to generate and compile source code.
     */
    private void buildAll() {
	File targetDirectory;

	saveFile();

	DODSDirectoryChooser chooser = new DODSDirectoryChooser();

	if (targetDir != null) {
	    chooser.setCurrentDirectory(targetDir);
	} else {
	    chooser.setCurrentDirectory(store.getCurrentDirectory());
	}

	int choice = chooser.showDialog(this, res.getString("Re_Create_Directory"));

	if (choice == chooser.CANCEL_OPTION) {
	    return;
	}

	targetDirectory = chooser.getSelectedFile();

	// System.err.println("targetDirectory ="+targetDirectory );
	// System.err.println("targetDirectory.getAbsolutePath()="+targetDirectory.getAbsolutePath());
	if (targetDirectory == null) {
	    JOptionPane.showMessageDialog(this,
					  res.getString("You_mush_choose_a"),
					  res.getString("Choose_Directory"),
					  JOptionPane.WARNING_MESSAGE);

	    return;
	}

	if (targetDirectory.exists()) {
        Object[] args;
        MessageFormat formatter;
        String msg;

	    if (!targetDirectory.isDirectory()) {
		JOptionPane.showMessageDialog(this,
					      res.getString("Your_selection_was"),
					      res.getString("Bad_Directory"),
					      JOptionPane.WARNING_MESSAGE);

		return;
	    }
        args = new Object[] { targetDirectory.getAbsolutePath() };
        formatter = new MessageFormat(res.getString("Are_you_sure"));
        msg = formatter.format(args);

	    int result =
		JOptionPane.showConfirmDialog(this,
					      msg,
					      res.getString("Directory_contents"),
					      JOptionPane.YES_NO_OPTION,
					      JOptionPane.QUESTION_MESSAGE);

	    if (result == JOptionPane.NO_OPTION) {
		return;
	    }

	    String stuck = FileSystem.rmdir(targetDirectory);

	    if (null != stuck) {

        args = new Object[] { stuck };
        formatter = new MessageFormat(res.getString("Unable_to_delete_dir"));
        msg = formatter.format(args);

		JOptionPane.showMessageDialog(this,
					      msg,
                          res.getString("Bad_Directory"),
					      JOptionPane.WARNING_MESSAGE);
		return;
	    }
	}

	if (!targetDirectory.mkdirs()) {

        Object[] args = new Object[] { targetDirectory.getAbsolutePath() };
        MessageFormat formatter = new MessageFormat(res.getString("Unable_to_mkdir"));
        String msg = formatter.format(args);

	    JOptionPane.showMessageDialog(this,
					  msg,
					  res.getString("Bad_Directory"),
					  JOptionPane.WARNING_MESSAGE);

	    return;
	}

	targetDir = targetDirectory;

	/*
	 * if ( null == targetDir || 0 == targetDir.length() )
	 * targetDir = System.getProperty( "DODS_OUTPUT_DIR" );
	 * DirectoryChooser chooser = new DirectoryChooser( this, targetDir );
	 * //System.err.println("chooser.dir="+chooser.dir);
	 * if ( null == chooser.dir )
	 * return;
	 * targetDir = chooser.dir.getAbsolutePath();
	 *
	 */

	/*
	 * We must remove the target directory (and everything under it)
	 * because it was likely used for a previous Build run
	 * and could contain source files for DataObjects that no longer
	 * exist in the project.
	 * This means that the user should never add any files to
	 * a directory tree generated by DODS.
	 * No check for user files is currently performed.
	 */
	try {

	    // System.err.println(res.getString("BEFORE_saveTree"));
	    saveTree();

	    // System.err.println(res.getString("BEFORE_Builder"));
	    if (null != bmd) {
		bmd.dispose();
	    }

	    bmd = new BuildMonitorDialog();

	    bmd.setModal(false);
	    bmd.setResizable(true);
	    bmd.setVisible(true);
	    (new Thread(new BuildRunner(targetDirectory.getAbsolutePath(),
					store.getProject()))).start();

	    /* Builder.run( targetDirectory.getAbsolutePath(), dodsProject ); */
	} catch (Exception e) {
	    e.printStackTrace();
	    JOptionPane.showMessageDialog(this, res.getString("ERROR_") + e.getMessage(),
					  res.getString("Building_Data_Objects"),
					  JOptionPane.WARNING_MESSAGE);
	}
    }

    /**
     * This method deletes any selected attributes in the table.
     */
    private void deleteAttributeMenuItem_ActionPerformed(ActionEvent event) {
	int[] selrows = table.getSelectedRows();
	int[] rows = new int[selrows.length + 1];

	rows[0] = -1;

	for (int j = 1; j < rows.length; j++) {
	    int row = rows[j - 1];

	    for (int i = 0; i < selrows.length; i++) {
		if (selrows[i] > row) {
		    row = selrows[i];
		}
	    }

	    if (row != rows[j - 1]) {
		rows[j] = row;
	    }
	}

	DataObject dataObject = doTableModel.getDataObject();

	deletedAttributes.removeAllElements();

	for (int i = 1; i < rows.length; i++) {
	    Attribute att =
		(Attribute) ((DOTableModel) table.getModel()).getDataObject().getAttributes().elementAt(rows[i]);

	    deletedAttributes.addElement(att);
	    dataObject.getAttributes().removeElement(att);
	}

	if (deletedAttributes.size() != 0) {
	    TreePath path = tree.getSelectionPath();

	    deletedNode = (DOTreeNode) tree.getLastSelectedPathComponent();
	}

	table.setRowSelectionInterval(-1, -1);
	refreshDODS();
    }

    /**
     * This method makes sure that the Database for this project is correct.
     */
    private void updateDatabaseMenu() {
        String dbkey = store.getProject().getDatabase().toLowerCase().trim();
	if (dbkey.startsWith("informix")) {
	    informixCheckBoxMenuItem.doClick();
	} else if (dbkey.startsWith("msql")) {
	    msqlCheckBoxMenuItem.doClick();
	} else if (dbkey.startsWith("oracle")) {
	    oracleCheckBoxMenuItem.doClick();
	} else if (dbkey.startsWith("post")) {
	    postCheckBoxMenuItem.doClick();
	} else if (dbkey.startsWith("sybase")) {
	    sybaseCheckBoxMenuItem.doClick();
	} else {
            standardJDBCCheckBoxMenuItem.doClick();
        }
    }

    /**
     * Displays a Dialog which should display the appropriate information.
     */
    private void showAboutBox() {
	AboutBox dlg = new AboutBox(this);

	dlg.setPrimary(appPrimary);
	dlg.setSecondary(appSecondary);
	dlg.setVersion(appVersion);
	dlg.show();
    }

    /**
     * This method finds the first attribute with an error and takes you there.
     */
    private void findErrorsMenuItem_ActionPerformed(ActionEvent event) {
	for (int i = 0; i < tree.getRowCount(); i++) {
	    TreePath path = tree.getPathForRow(i);

	    if (((DOTreeNode) path.getLastPathComponent()).isLeaf()) {
		if (((DOTreeNode) path.getLastPathComponent()).getDataObject().hasAttributeErrors()) {
		    tree.setSelectionPath(path);

		    return;
		}
	    }
	}
    }

    /**
     * For debugging purposes
     */
    public void checkDOS() {
	Vector allDOS = new Vector();

	store.getProject().getAllDataObjects(allDOS);

	for (int i = 0; i < allDOS.size(); i++) {
	    DataObject DO = (DataObject) allDOS.elementAt(i);

	    System.out.println();
	    System.out.println(DO.getName() + res.getString("KEY1"));
	    System.out.println(res.getString(" ") + res.getString("ExtensionOf_")
			       + DO.getExtensionOf());
	    System.out.println(res.getString(" ") + res.getString("ExtendedBy_") + DO.getExtendedBy());
	    System.out.println(res.getString(" ") + res.getString("Ancestors_") + DO.getAncestors());
	    System.out.println(res.getString(" ") + res.getString("Descendents_")
			       + DO.getDescendents());
	}
    }

    /**
     * Method declaration
     */
    private void setTitle() {
	String current = res.getString("_untitled_");

	if (store.getOutputFile() != null) {
	    current = store.getOutputFile().getAbsolutePath();
	}

	setTitle(appSecondary + " - " + current);
    }

    /**
     * Method declaration
     *
     *
     * @param owner
     *
     * @throws dods.doml.UpException
     */
    private void readStore(Component owner) throws dods.doml.UpException {
	boolean OK = true;

	if (store.isInputDOA()) {
	    if (store.getOutputFile().exists()) {
		if (JOptionPane.showConfirmDialog(owner, store.getOutputFile()
						  + res.getString("already_exists_Would_you_like_to_overwrite")
						  , appSecondary, JOptionPane.ERROR_MESSAGE) == JOptionPane.CANCEL_OPTION) {
		    OK = false;
		}
	    }
	}

	if (OK) {
	    store.read();
	} else {
	    store.create();
	}
    }

    /*
     *
     *
     * The rest are all internal listener classes which call the above
     * methods when appropriate events occur.
     *
     *
     */

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class SymWindow extends WindowAdapter {

	/**
	 * Method declaration
	 *
	 *
	 * @param event
	 */
	public void windowClosing(WindowEvent event) {
	    Object object = event.getSource();

	    if (object == DataObjectApplication.this) {
		DataObjectApplication_WindowClosing(event);
	    }
	}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class SymAction implements ActionListener {

	/**
	 * Method declaration
	 *
	 *
	 * @param event
	 */
	public void actionPerformed(ActionEvent event) {
	    Object object = event.getSource();

	    if (object == exitMenuItem) {
		exitMenuItem_ActionPerformed(event);
	    } else if (object == insertAttributeMenuItem) {
		insertAttributeMenuItem_ActionPerformed(event);
	    } else if (object == insertDOMenuItem) {
		insertDOMenuItem_ActionPerformed(event);

		// ****opensource contribution by Larry D'Wolcoto
	    } else if (object == insertImportedObjectMenuItem) {
		insertImportedObjectMenuItem_ActionPerformed(event);

		// ****end
	    } else if (object == insertPackageMenuItem) {
		insertPackageMenuItem_ActionPerformed(event);
	    } else if (object == saveMenuItem) {
		saveAs();
	    } else if (object == openMenuItem) {
		openFile();
	    } else if (object == editDOMenuItem) {
		editDOMenuItem_ActionPerformed(event);
	    } else if (object == saveAsMenuItem) {
		saveAs();
	    } else if (object == newMenuItem) {
		newFile();
	    } else if (object == closeMenuItem) {
		closeFile();
	    } else if (object == editPackageMenuItem) {
		editPackageMenuItem_ActionPerformed(event);
	    } else if (object == deletePackageMenuItem) {
		deleteMenuItem_ActionPerformed(event);
	    } else if (object == deleteDataObjectMenuItem) {
		deleteMenuItem_ActionPerformed(event);
	    } else if (object == deleteAttributeMenuItem) {
		deleteAttributeMenuItem_ActionPerformed(event);
	    } else if (object == undoMenuItem) {
		undoMenuItem_ActionPerformed(event);
	    } else if (object == refreshMenuItem) {
		refreshMenuItem_ActionPerformed(event);
	    } else if (object == editAttributeMenuItem) {
		editAttributeMenuItem_ActionPerformed(event);
	    } else if (object == newButton) {
		newFile();
	    } else if (object == openButton) {
		openFile();
	    } else if (object == saveButton) {
		saveAs();
	    } else if (object == editDataObjectButton) {
		editDOMenuItem_ActionPerformed(event);
	    } else if (object == editPackageButton) {
		editPackageMenuItem_ActionPerformed(event);
	    } else if (object == editAttributeButton) {
		editAttributeMenuItem_ActionPerformed(event);
	    } else if (object == insertDataObjectButton) {
		insertDOMenuItem_ActionPerformed(event);

		// ****opensource contribution by Larry D'Wolcoto
	    } else if (object == insertImportedObjectButton) {
		insertImportedObjectMenuItem_ActionPerformed(event);

		// ****end
	    } else if (object == insertPackageButton) {
		insertPackageMenuItem_ActionPerformed(event);
	    } else if (object == insertAttributeButton) {
		insertAttributeMenuItem_ActionPerformed(event);
	    } else if (object == buildAllMenuItem) {
		buildAll();
	    } else if (object == buildAllButton) {
		buildAll();
	    } else if (object == aboutMenuItem) {
		showAboutBox();
	    } else if (object == findErrorsButton) {
		findErrorsMenuItem_ActionPerformed(event);
	    } else if (object == findErrorsMenuItem) {
		findErrorsMenuItem_ActionPerformed(event);
	    }
	}

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class SymItem implements ItemListener {

	/**
	 * Method declaration
	 *
	 *
	 * @param event
	 */
	public void itemStateChanged(ItemEvent event) {
	    Object object = event.getSource();

	    if (object == toolbarCheckBoxMenuItem) {
		toolbarCheckBoxMenuItem_ItemStateChanged(event);
	    }

	    if (object == standardJDBCCheckBoxMenuItem) {
		standardJDBCCheckBoxMenuItem_ItemStateChanged(event);
	    }

	    if (object == informixCheckBoxMenuItem) {
		informixCheckBoxMenuItem_ItemStateChanged(event);
	    }

	    if (object == msqlCheckBoxMenuItem) {
		msqlCheckBoxMenuItem_ItemStateChanged(event);
	    }

	    if (object == oracleCheckBoxMenuItem) {
		oracleCheckBoxMenuItem_ItemStateChanged(event);
	    }

       	    if (object == postCheckBoxMenuItem) {
		postCheckBoxMenuItem_ItemStateChanged(event);
	    }


	    if (object == sybaseCheckBoxMenuItem) {
		sybaseCheckBoxMenuItem_ItemStateChanged(event);
	    }
	}

    }

}

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
class BuildRunner implements Runnable {
    static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    String      dir;
    DODSProject project;

    /**
     * Constructor declaration
     *
     *
     * @param s
     * @param p
     */
    public BuildRunner(String s, DODSProject p) {
	dir = s;
	project = p;
    }

    /**
     * Method declaration
     *
     *
     */
    public void run() {
	try {
	    Builder.run(dir, project);
	} catch (Exception e) {
	    Builder.print(res.getString("ERROR_") + e.getMessage());
	    e.printStackTrace();
	}
    }

}

