/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: ToolTipListener.java,v 1.10.14.1 2000/12/01 22:27:39 chip Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.gui;

import com.lutris.dods.gui.*;
import com.lutris.dods.gui.table.*;
import com.lutris.dods.store.doa.*;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.JOptionPane;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;
/**
 * This class changes Tooltips depending on where the mouse is.
 */
public class ToolTipListener extends MouseAdapter implements MouseMotionListener {
    static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    /**
     * When entering a place where there is a tooltip to display, this defines
     * that tooltip.
     */
    public void mouseMoved(MouseEvent e) {
        if(e.getSource() instanceof JTable) {
            JTable table = (JTable)e.getSource();
            Point point = new Point(e.getX(), e.getY());
            int row = table.rowAtPoint(point);
            int column = table.columnAtPoint(point);
            if(row == -1) {
                table.setToolTipText(null);
                return;
            }
            DataObject DO = ((DOTableModel)table.getModel()).getDataObject();
            Attribute att = (Attribute)DO.getAttributes().elementAt(row);
            switch(column) {
                case 0:
                    if(att.hasError()) {
                        table.setToolTipText(att.getErrorText());
                    }
                    break;
                case 1:
                    if(att.isObjectRef()) {
                        table.setToolTipText(res.getString("Reference_to_") + att.getJavaType());
                    }
                    break;
                case 2:
		    /*
                    if(att.isArray()) {
                        table.setToolTipText(res.getString("Size_1") + att.getArraySize());
                    }
		    */
                    break;
                default:
                    break;
            }
        }
    }
    /**
     * Does nothing.
     */
    public void mouseDragged(MouseEvent e) {
    }
    /**
     * If you click the error column and there is an error, a dialog is displayed
     * explaining the error.
     */
    public void mouseClicked(MouseEvent e) {
        if(e.getSource() instanceof JTable) {
            JTable table = (JTable)e.getSource();
	    /*
            DOTableComboEditor six = (DOTableComboEditor)table.getColumnModel().getColumn(8).getCellEditor();
            six.stopCellEditing();
	    */
            Point point = new Point(e.getX(), e.getY());
            int row = table.rowAtPoint(point);
            int column = table.columnAtPoint(point);
            if(row == -1) {
                return;
            }
            DataObject DO = ((DOTableModel)table.getModel()).getDataObject();
            Attribute att = (Attribute)DO.getAttributes().elementAt(row);
            switch(column) {
                case 0:
                    if(att.hasError()) {
                        JOptionPane.showMessageDialog(table,
                                      att.getErrorText(),
                                      res.getString("Error_Explaination"),
                                      JOptionPane.ERROR_MESSAGE);
                    }
                    break;
                default:
                    break;
            }
        }
    }
}
