
   package com.lutris.dods.gui.importer;

   import com.lutris.dods.gui.*;
   import com.lutris.dods.store.doa.*;
   import com.lutris.dods.gui.tree.*;
   import com.lutris.dods.exceptions.*;
   import javax.swing.JFrame;
   import javax.swing.ButtonGroup;
   import javax.swing.JDialog;
   import javax.swing.JTabbedPane;
   import javax.swing.JOptionPane;
   import javax.swing.JTree;
   import javax.swing.JScrollPane;
   import javax.swing.JPanel;
   import javax.swing.JPasswordField;
   import javax.swing.JTextField;
   import javax.swing.JTextArea;
   import javax.swing.DefaultListModel;
   import javax.swing.JButton;
   import javax.swing.JLabel;
   import javax.swing.JComboBox;
   import javax.swing.JSplitPane;
   import javax.swing.JList;
   import javax.swing.JCheckBox;
   import javax.swing.JRadioButton;
   import javax.swing.tree.DefaultMutableTreeNode;
   import javax.swing.event.TreeSelectionListener;
   import javax.swing.tree.TreeSelectionModel;
   import javax.swing.tree.DefaultTreeModel;
   import javax.swing.tree.MutableTreeNode;
   import javax.swing.border.TitledBorder;
   import javax.swing.event.TreeSelectionEvent;
   import javax.swing.event.ChangeListener;
   import javax.swing.event.ChangeEvent;
   import javax.swing.tree.TreePath;
   import javax.swing.BoxLayout;
   import java.awt.GridLayout;
   import java.awt.Cursor;
   import java.awt.Color;
   import java.awt.BorderLayout;
   import java.awt.GridBagLayout;
   import java.awt.GridBagConstraints;
   import java.awt.Dimension;
   import java.awt.Component;
   import java.awt.FlowLayout;
   import java.awt.Insets;
   import java.awt.event.ActionListener;
   import java.awt.event.ActionEvent;
   import java.util.Vector;
   import java.sql.*;
import java.util.ResourceBundle;
/**
 * This class is a pop-up window which imports database tables
 *  and attributes.
 *
 * @author  Larry Wolcott, and Philip Martin
 * @version $Revision: 1.7.14.1 $
 */
   public final class TableBrowser extends JDialog {

      static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    private boolean isNotBusy;
      private String defaultURL = "jdbc:odbc:yourDataSource";
      private String defaultDriver = "sun.jdbc.odbc.JdbcOdbcDriver";
      private DefaultListModel tableListModel;
      private JList tableList;
      private Vector tempListOfAttribs;
      private Attribute tempAttrib;
      private DefaultMutableTreeNode top;
      private DefaultTreeModel schemaModel;
      private JTree schemaTree;
      private Vector dataCatalogs;
      private Vector dataTables;
      private DatabaseAdapter database = null;
      private DataObject editObject;
      private PackageTree tree;
      private JTree structTree;
      private DataObjectApplication doa;
      private JTabbedPane jtp;
      private PackageTreeSelectionListener listener;
      private JPanel scrollPanePanel;
      private JScrollPane treeView;
      private JTextField packageName;
      private JLabel packageLabel;
      private JPanel dbPanel;
      private JLabel tableNameLabel;
      private JTextField tableName;
      private JPanel dbTypePanel;
      private JRadioButton isEveryClass;
      private JRadioButton isConcreteClass;
      private JRadioButton isHierarchy;
      private JRadioButton isLazyLoading;
      private JRadioButton isCaching;
      private JRadioButton isFullCaching;
      private JPanel buttonPanel;
      private JButton okButton;
      private JButton cancelButton;
      private JButton helpButton;
      private JPanel classPanel;
      private JLabel nameLabel;
      private JLabel extendsLabel;
      private JTextField textName;
      private JComboBox extensionOf;
      private JPanel typePanel;
      private JCheckBox isAbstract;
      private JCheckBox isFinal;
      private JTextField driverField, userNameField, serverField;
      private JPasswordField passwordField;
      private static boolean DEBUG = true;
      private String dataSourceName = "";
      private JLabel label4, label5, label6;
      private JButton datasourceButton;
      private Cursor wait = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
      private Cursor normal = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

    /**
     * This Constructor lays out all of the input devices.
     * @param tre The JTree from the DataObjectApplication.
     * @param da The Parent DataObjectApplication.
     */
      public TableBrowser(JTree tre, DataObjectApplication da) {
         super(da);
        //Initializing the tree for the package selection.
         structTree = tre;
         doa = da;
         tree = new PackageTree();

         DOTreeNode       oldRoot   = null;
         DOPackage        rootPack  = null;
         PackageTreeNode  packRoot  = null;
         PackageTreeModel treeModel = null;
         oldRoot   = (DOTreeNode)structTree.getModel().getRoot();
         rootPack  = (DOPackage) oldRoot.getDataObject();
         packRoot  = new PackageTreeNode(rootPack);
         treeModel = new PackageTreeModel(packRoot);

         tree.setModel(treeModel);
         tree.setMinimumSize(new Dimension(0,0));
        //Initializing the Frame.
         setTitle(res.getString("Import_Schema"));
         setResizable(true);
         setModal(true);
         setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
         setBackground(Color.lightGray);
         getContentPane().setLayout(new GridBagLayout());
         GridBagConstraints gbc = new GridBagConstraints();
         setSize(600,500);
         JPanel jp = new JPanel();
         jp.setLayout(new GridBagLayout());
        //ScrollPanePanel initialization
         scrollPanePanel = new JPanel();
         scrollPanePanel.setLayout(new GridBagLayout());
         treeView = new JScrollPane();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 50.0;
         gbc.anchor = GridBagConstraints.NORTH;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(2,20,0,20);
         ((GridBagLayout)scrollPanePanel.getLayout()).setConstraints(treeView, gbc);
         scrollPanePanel.add(treeView);
         packageName = new JTextField();
         packageName.setMargin(new java.awt.Insets(0,0,0,0));
         packageName.setEditable(false);
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 3;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTH;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(10,20,20,20);
         ((GridBagLayout)scrollPanePanel.getLayout()).setConstraints(packageName, gbc);
         scrollPanePanel.add(packageName);
         packageName.setEnabled(false);
         packageLabel = new JLabel();
         packageLabel.setText(res.getString("Place_In_1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(10,20,0,0);
         ((GridBagLayout)scrollPanePanel.getLayout()).setConstraints(packageLabel, gbc);
         scrollPanePanel.add(packageLabel);
        //DBPanel Initialization.
         dbPanel = new JPanel();
         dbPanel.setLayout(new GridBagLayout());
         tableNameLabel = new JLabel();
         tableNameLabel.setText(res.getString("db_Table_Name_1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 0.5;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,0,0);
         ((GridBagLayout)dbPanel.getLayout()).setConstraints(tableNameLabel, gbc);
         dbPanel.add(tableNameLabel);
         tableName = new JTextField();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 20.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(2,20,0,20);
         ((GridBagLayout)dbPanel.getLayout()).setConstraints(tableName, gbc);
         dbPanel.add(tableName);
         dbTypePanel = new JPanel();
         dbTypePanel.setLayout(new GridBagLayout());
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.gridwidth = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 2.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,20,20);
         ((GridBagLayout)dbPanel.getLayout()).setConstraints(dbTypePanel, gbc);
         dbPanel.add(dbTypePanel);
         isEveryClass = new JRadioButton();
         isEveryClass.setText(res.getString("Every_Class1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,10,0,10);
         ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isEveryClass, gbc);
         dbTypePanel.add(isEveryClass);
         isEveryClass.setEnabled(false);
         isConcreteClass = new JRadioButton();
         isConcreteClass.setText(res.getString("Every_Concrete_Class1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,10,0,10);
         ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isConcreteClass, gbc);
         dbTypePanel.add(isConcreteClass);
         isHierarchy = new JRadioButton();
         isHierarchy.setText(res.getString("Entire_Hierarchy1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,10,0,10);
         ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isHierarchy, gbc);
         dbTypePanel.add(isHierarchy);

         isLazyLoading = new JRadioButton();
         isLazyLoading.setText(res.getString("Lazy_Loading1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 3;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,10,0,10);
         ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isLazyLoading, gbc);
         dbTypePanel.add(isLazyLoading);

         isCaching = new JRadioButton();
         isCaching.setText(res.getString("Cached1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 4;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,10,0,10);
         ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isCaching, gbc);
         dbTypePanel.add(isCaching);

         isFullCaching = new JRadioButton();
         isFullCaching.setText(res.getString("Fully_Cached1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 5;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,10,0,10);
         ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isFullCaching, gbc);
         dbTypePanel.add(isFullCaching);

         isLazyLoading.setEnabled(true);
         isCaching.setEnabled(true);
         isFullCaching.setEnabled(true);

         isEveryClass.setEnabled(false);
         isHierarchy.setEnabled(false);
        //ButtonPanel initialization.
         buttonPanel = new JPanel();
         buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
         okButton = new JButton();
         okButton.setText(res.getString("OK"));
         buttonPanel.add(okButton);
         cancelButton = new JButton();
         cancelButton.setText(res.getString("Cancel1"));
         buttonPanel.add(cancelButton);
         helpButton = new JButton();
         helpButton.setText(res.getString("Help"));
         buttonPanel.add(helpButton);
         helpButton.setEnabled(false);
        //ClassPanel initialization.
         classPanel = new JPanel();
         classPanel.setLayout(new GridBagLayout());
         nameLabel = new JLabel();
         nameLabel.setText(res.getString("Name_1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,0,0);
         ((GridBagLayout)classPanel.getLayout()).setConstraints(nameLabel, gbc);
         classPanel.add(nameLabel);
         extendsLabel = new JLabel();
         extendsLabel.setText(res.getString("Extends_1"));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,0,0);
         ((GridBagLayout)classPanel.getLayout()).setConstraints(extendsLabel, gbc);
         classPanel.add(extendsLabel);
         textName = new JTextField();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(2,20,0,20);
         ((GridBagLayout)classPanel.getLayout()).setConstraints(textName, gbc);
         classPanel.add(textName);
         extensionOf = new JComboBox();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 3;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(2,20,0,20);
         ((GridBagLayout)classPanel.getLayout()).setConstraints(extensionOf, gbc);
         classPanel.add(extensionOf);
         typePanel = new JPanel();
         typePanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 4;
         gbc.gridwidth = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 0.5;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,20,20,20);
         ((GridBagLayout)classPanel.getLayout()).setConstraints(typePanel, gbc);
         classPanel.add(typePanel);
         isAbstract = new JCheckBox();
         isAbstract.setText(res.getString("Abstract1"));
         typePanel.add(isAbstract);
         isFinal = new JCheckBox();
         isFinal.setText(res.getString("Final1"));
         typePanel.add(isFinal);
        //Initializing borders.
         TitledBorder border1 = new TitledBorder(res.getString("Modifiers1"));
         border1.setTitlePosition(TitledBorder.TOP);
         border1.setTitleJustification(TitledBorder.LEFT);
         typePanel.setBorder(border1);
         TitledBorder border2 = new TitledBorder(res.getString("DataBase_Mapping_Type1"));
         border2.setTitlePosition(TitledBorder.TOP);
         border2.setTitleJustification(TitledBorder.LEFT);
         dbTypePanel.setBorder(border2);
        //setting up the treeView.
         listener = new PackageTreeSelectionListener(tree, packageName);
         tree.addTreeSelectionListener(listener);
         tree.setMinimumSize(new Dimension(0,0));
         treeView.setViewportView(tree);
         treeView.setMinimumSize(new Dimension(0,0));
        //Initializing button group.
         ButtonGroup bg = new ButtonGroup();
         bg.add(isEveryClass);
         bg.add(isConcreteClass);
         bg.add(isHierarchy);

        //Initializing listeners.
         SymAction lSymAction = new SymAction();
         cancelButton.addActionListener(lSymAction);
         okButton.addActionListener(lSymAction);
         textName.addActionListener(lSymAction);
         tableName.addActionListener(lSymAction);
         SymChange lSymChange = new SymChange();
         isEveryClass.addChangeListener(lSymChange);
         isConcreteClass.addChangeListener(lSymChange);
         isHierarchy.addChangeListener(lSymChange);

         isLazyLoading.addChangeListener(lSymChange);
         isCaching.addChangeListener(lSymChange);
         isFullCaching.addChangeListener(lSymChange);

         isAbstract.addChangeListener(lSymChange);
         extensionOf.addActionListener(lSymAction);
        //adding buttonPanel
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 0.3;
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,0,0,5);
         ((GridBagLayout)jp.getLayout()).setConstraints(buttonPanel, gbc);
         jp.add(buttonPanel);
        //initializing the TabbedPane
         jtp = new JTabbedPane();
         jtp.addTab(res.getString("Import_Tables"), getTreePanel());
         jtp.addTab(res.getString("Package"), scrollPanePanel);
         jtp.addTab(res.getString("DataBase1"), dbPanel);
        //Adding the TabbedPane
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(10,10,0,10);
         ((GridBagLayout)jp.getLayout()).setConstraints(jtp, gbc);
         jp.add(jtp);
        //Adding the Panel.
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,0,0,0);
         ((GridBagLayout)getContentPane().getLayout()).setConstraints(jp, gbc);
         getContentPane().add(jp);
      }

    /*
     * This method hides the Dialog when cancel is pressed.
     */
      private void cancelButton_actionPerformed(ActionEvent event) {
         isAbstract.setEnabled(true);
         setVisible(false);
      }
    /*
     * This method shows the datasource connection dialog when
     * the datasource button is pressed
     */
      private void datasourceButton_actionPerformed(ActionEvent event) {
         setStatus(res.getString("Connecting_to"));
         activateConnectionDialog();
      }
    /*
     * This method changes all of the DataObject's attributes and hides the dialog.
     */
      private void okButton_actionPerformed(ActionEvent event) {
      	if (isUpdateValid()) {
	    try {
		updateDataObject();
                tree.updateStructure(structTree, editObject);
	    } catch (DataObjectNameException e) {
		e.printStackTrace();
		doa.namingExceptionOccurred(e, this);
	    } catch (DataObjectTypeException e) {
		e.printStackTrace();
	    }
	}

	doa.updateTree();
	doa.updateTable();
	isAbstract.setEnabled(true);
	setVisible(false);
      }

    private boolean isUpdateValid() {
	boolean valid = true;

	if (!tree.isNameUniqueForSelection(textName.getText().trim(),
					   editObject)) {
	    JOptionPane.showMessageDialog(this,
					  "An object with that Name already exists in this package.",
					  res.getString("Error_editing_Data1"),
					  JOptionPane.WARNING_MESSAGE);
	    jtp.setSelectedIndex(0);

	    valid = false;
	} else if (!isTableUnique()) {
	    JOptionPane.showMessageDialog(this,
					  res.getString("An_object_with_that4"),
					  res.getString("Error_editing_Data1"),
					  JOptionPane.WARNING_MESSAGE);
	    jtp.setSelectedIndex(2);

	    valid = false;
	}

	return valid;
    }

    private void updateDataObject()
	    throws DataObjectNameException, DataObjectTypeException {
	if (tableName.isEnabled()) {
	    editObject.setDBTableName(tableName.getText());
	}
        // Add Attributes from tempListOfAttribs into the DO
        for (int i = 0; i < tempListOfAttribs.size(); i++){
            editObject.addAttribute((Attribute)tempListOfAttribs.elementAt(i));
         }

	editObject.setDBTableType(DataObject.CONCRETE);
	editObject.setName(textName.getText());

	// sets what DataObject this extends.
	if (extensionOf.getSelectedItem() instanceof String) {
	    editObject.setExtensionOf(null);
	} else {
	    editObject.setExtensionOf((DataObject) extensionOf.getSelectedItem());
	    ((DataObject) extensionOf.getSelectedItem()).setAbstract(true);
	}

	editObject.setLazyLoading(isLazyLoading.isSelected());
	editObject.setCaching(isCaching.isSelected());
	editObject.setFullCaching(isFullCaching.isSelected());
	editObject.setAbstract(isAbstract.isSelected());
	editObject.setFinal(isFinal.isSelected());
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private boolean isTableUnique() {
	boolean unique = true;
	Vector  allDOS = new Vector();

	((DOPackage) ((DOTreeNode) structTree.getModel().getRoot()).getDataObject()).getAllDataObjects(allDOS);

	for (int i = 0; i < allDOS.size(); i++) {
	    DataObject dobject = (DataObject) allDOS.elementAt(i);

	    if (dobject.getDBTableName().equals(tableName.getText())
		    && dobject != editObject
		    &&!(editObject.isAbstract() || dobject.isAbstract())) {
		unique = false;
	    }
	}

	return unique;
    }


    /*
     * When you hit enter in the name Field, it's as if the okButton was pressed.
     */
      private void name_actionPerformed(ActionEvent event) {
         okButton.doClick();
      }
    /*
     * When you hit enter in the table name Field, it's as if the okButton was pressed.
     */
      private void tableName_actionPerformed(ActionEvent event) {
         okButton.doClick();
      }
    /*
     * This method checks the mapping type, and depending on what type of
     * mapping is chosen, will enable or disable the option to set the table name.
     */
      private void checkEnabledMapping() {
         if(isEveryClass.isSelected()){
            tableName.setEnabled(true);
            tableName.setBackground(Color.white);
            tableNameLabel.setForeground(Color.black);
         }
         else if(isConcreteClass.isSelected()){
            if(isAbstract.isSelected()){
               tableName.setEnabled(false);
               tableName.setBackground(Color.lightGray);
               tableNameLabel.setForeground(Color.gray);
            }
            else {
               tableName.setEnabled(true);
               tableName.setBackground(Color.white);
               tableNameLabel.setForeground(Color.black);
            }
         }
         else if(isHierarchy.isSelected()){
            if(extensionOf.getSelectedItem() instanceof String) {
               tableName.setEnabled(true);
               tableName.setBackground(Color.white);
               tableNameLabel.setForeground(Color.black);
            }
            else {
               tableName.setEnabled(false);
               tableName.setBackground(Color.lightGray);
               tableNameLabel.setForeground(Color.gray);
            }
         }

         if(isLazyLoading.isSelected()){
         }
         if(isCaching.isSelected()){
            isFullCaching.setEnabled(true);
         }
         else {
            isFullCaching.setEnabled(false);
         /*             isFullCaching.setSelected(false); */
         }
         if(isFullCaching.isSelected()){
         // FIX recursion if either of these is uncommented
         /*             isCaching.setSelected(true); */
         }
         else {
         /*             isCaching.setSelected(false); */
         }
      }
    /**
     * This method sets up the DataObject which will be edited by the DOE.
     * It also initializes all of the fields to what the value is in the editObject.
     * @param Do The new DataObject.
     */
      public void setDataObject(DataObject Do) {
         jtp.setSelectedIndex(0);
         editObject = Do;
         tableName.setText(editObject.getDBTableName());
         tableName.setEnabled(!editObject.isAbstract());
         isConcreteClass.setSelected(true);//FIX:if multiple mapping types
                                          //    are allowed, this will need
                                          //    to be changed.
         textName.setText(editObject.getName());
         isAbstract.setSelected(editObject.isAbstract());
         if(editObject.getExtendedBy().size() != 0) {
            isAbstract.setEnabled(false);
         }
         isFinal.setSelected(editObject.isFinal());
         try {
            extensionOf.removeAllItems();
         }
            catch(IndexOutOfBoundsException e){
            }
         extensionOf.addItem(res.getString("Nothing1"));
         Vector DOS = new Vector();

         DOTreeNode       oldRoot   = null;
         DOPackage        rootPack  = null;
         PackageTreeNode  packRoot  = null;
         PackageTreeModel treeModel = null;
         oldRoot   = (DOTreeNode)structTree.getModel().getRoot();
         rootPack  = (DOPackage) oldRoot.getDataObject();
         packRoot  = new PackageTreeNode(rootPack);
         treeModel = new PackageTreeModel(packRoot);

         tree.setModel(treeModel);
         rootPack.getAllDataObjects(DOS);
         for(int i = 0; i < DOS.size(); i++) {
            DataObject data = (DataObject)DOS.elementAt(i);
            if(!editObject.getDescendents().contains(data) && !editObject.equals(data) && !data.isFinal()) {
               extensionOf.addItem(data);
            }
         }
         Vector extOf = new Vector();
         for(int i = 0; i < extensionOf.getItemCount(); i++){
            extOf.addElement(extensionOf.getItemAt(i));
         }
         extOf = DataObject.alphabetizeVector(extOf);
         try {
            extensionOf.removeAllItems();
         }
            catch(IndexOutOfBoundsException e){
            }
         for(int i = 0; i < extOf.size(); i++) {
            extensionOf.addItem(extOf.elementAt(i));
         }
         if(editObject.getExtensionOf()!=null) {
            extensionOf.setSelectedItem(editObject.getExtensionOf());
         }
         tree.synchSelectionWithStuctureTree(structTree);
         listener.valueChanged(new TreeSelectionEvent(this, null, null, null, null));
      }
    /*
     *
     * All of the following private methods simply call checkEnabledMapping
     * it might have changed.
     *
     */
      private void isEveryClass_stateChanged(ChangeEvent event) {
         checkEnabledMapping();
      }
      private void isConcreteClass_stateChanged(ChangeEvent event) {
         checkEnabledMapping();
      }
      private void isHierarchy_stateChanged(ChangeEvent event) {
         checkEnabledMapping();
      }
      private void isLazyLoading_stateChanged(ChangeEvent event) {
         checkEnabledMapping();
      }
      private void isCaching_stateChanged(ChangeEvent event) {
         checkEnabledMapping();
      }
      private void isFullCaching_stateChanged(ChangeEvent event) {
         checkEnabledMapping();
      }
      private void isAbstract_stateChanged(ChangeEvent event) {
         checkEnabledMapping();
      }
      private void extensionOf_actionPerformed(ActionEvent event) {
         checkEnabledMapping();
      }
    /*
     *
     * The inner classes are just listeners for events.
     *
     */
      class SymAction implements ActionListener {
         public void actionPerformed(ActionEvent event) {
            Object object = event.getSource();
            if (object == cancelButton)
               cancelButton_actionPerformed(event);
            if (object == datasourceButton)
               datasourceButton_actionPerformed(event);
            else if (object == okButton)
               okButton_actionPerformed(event);
            else if (object == textName)
               name_actionPerformed(event);
            else if (object == tableName)
               tableName_actionPerformed(event);
            else if (object == extensionOf)
               extensionOf_actionPerformed(event);
         }
      }
      class SymChange implements ChangeListener {
         public void stateChanged(ChangeEvent event) {
            Object object = event.getSource();
            if (object == isEveryClass)
               isEveryClass_stateChanged(event);
            else if (object == isConcreteClass)
               isConcreteClass_stateChanged(event);
            else if (object == isHierarchy)
               isHierarchy_stateChanged(event);
            else if (object == isAbstract)
               isAbstract_stateChanged(event);
            else if (object == isLazyLoading)
               isLazyLoading_stateChanged(event);
            else if (object == isCaching)
               isCaching_stateChanged(event);
            else if (object == isFullCaching)
               isFullCaching_stateChanged(event);
         }
      }
    /*
     *
     * All of the following private methods creates the initial
     * node structure for the JTree
     *
     */
      private void createNodes(DefaultMutableTreeNode top, Vector catalogs, String[][] tables) {
         DefaultMutableTreeNode catalog = null;
         DefaultMutableTreeNode table = null;

         for (int i = 0; i < catalogs.size(); i++){
            catalog = new DefaultMutableTreeNode((String)catalogs.elementAt(i));
            for (int t = 0; t < tables[i].length; t++){
               table = new DefaultMutableTreeNode(tables[i][t]);
               catalog.add(table);
            }
            if (tables[i].length == 0) {
               table = new DefaultMutableTreeNode(res.getString("Empty_Collection"));
               catalog.add(table);
            }
            top.add(catalog);
         }
      }
    /*
     *
     * All of the following private method passes the database catalogs to the
     * JTree window
     *
     */
      private void createCatalogs(DefaultMutableTreeNode top, Vector catalogs) {
         DefaultMutableTreeNode catalog = null;
         DefaultMutableTreeNode table = null;
         if (catalogs != null){
            for (int i = 0; i < catalogs.size(); i++){
               catalog = new DefaultMutableTreeNode((String)catalogs.elementAt(i));
               DefaultMutableTreeNode dummyTable = new DefaultMutableTreeNode(res.getString("Loading_Please_Wait_"));
               catalog.add(dummyTable);
               top.add(catalog);
            }
            if (catalogs.size() == 0){
               catalog = new DefaultMutableTreeNode(res.getString("Default_Collection"));
               catalog.setAllowsChildren(true);
               DefaultMutableTreeNode dummyTable = new DefaultMutableTreeNode(res.getString("Loading_Please_Wait_"));
               catalog.add(dummyTable);
               top.add(catalog);
            }
         }
         else {
            catalog = new DefaultMutableTreeNode(res.getString("Default_Collection"));
            catalog.setAllowsChildren(true);
            top.add(catalog);
         }
      }
    /*
     *
     * All of the following private method passes the catalog tables to the
     * JTree window
     *
     */
      private void createTables(int insertAtLevel, Vector theseTables) {
         DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)top.getChildAt(insertAtLevel);
         if (parentNode == null) { parentNode = (DefaultMutableTreeNode)top; }

         for (int i = 0; i < theseTables.size(); i++){
            DefaultMutableTreeNode tableToInsert = new DefaultMutableTreeNode((String)theseTables.elementAt(i));
            schemaModel.insertNodeInto( tableToInsert, parentNode, 1) ;
         }
         if (parentNode.getChildCount() == 1){
            DefaultMutableTreeNode tableToInsert = new DefaultMutableTreeNode(res.getString("Empty_Collection"));
            schemaModel.insertNodeInto( tableToInsert, parentNode, 1) ;
         }
         parentNode.remove(0);
      }

    /**
     * This method creates and returns a fully initialized schema tree.
     */
      public JPanel getTreePanel(){
      //Initialize test stuff
         String[][] tables = { };
         Vector catalogs = new Vector(0);
         label4 = new JLabel(res.getString("_No_Datasource"), JLabel.LEFT );
         label5 = new JLabel("", JLabel.LEFT );
         label6 = new JLabel("", JLabel.LEFT );

        //Create the nodes.
         top = new DefaultMutableTreeNode(res.getString("Datasource_") + dataSourceName);
         top.setAllowsChildren(true);
         createNodes(top, catalogs, tables);
         schemaModel = new DefaultTreeModel(top);

        //Create a tree
         schemaTree = new JTree(schemaModel);
         schemaTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        //Listen for tree node selection changes.
         schemaTree.addTreeSelectionListener(
               new TreeSelectionListener() {
                  public void valueChanged(TreeSelectionEvent e) {
                     if (isNotBusy){
                        setCursor(wait);
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                        (e.getPath().getLastPathComponent());
                        String selectedTable = (String)node.getUserObject();
                        if (node.isLeaf()) {
                           if (selectedTable != res.getString("Empty_Collection")) {
                              label6.setText(selectedTable); label5.setText(node.getParent().toString());
                              getAttributes();
                              textName.setText(label6.getText().trim());
                              tableName.setText(label6.getText().trim());
                              tableListModel.clear();
                              for (int i = 0; i < tempListOfAttribs.size(); i++){
                                 tableListModel.addElement(tempListOfAttribs.elementAt(i));
                              }
                              okButton.setEnabled(true);
                              setCursor(normal);
                              setStatus(res.getString("press_OK_to_import"));
                              repaint();
                           }
                           else { okButton.setEnabled(false); }
                        }
                        else {
                           tableListModel.clear();
                           TreePath currentSelection = schemaTree.getSelectionPath();
                           DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());
                           if (currentNode.getChildAt(0).toString() == res.getString("Loading_Please_Wait_")) {
                              DefaultMutableTreeNode thisParent = (DefaultMutableTreeNode)currentNode.getParent();
                              int curIndex = thisParent.getIndex(currentNode);
                              setStatus(res.getString("Fetching_tables_in") + (String)dataCatalogs.elementAt(curIndex));
                              try {	Vector gotTheseTables = database.getTables(dataCatalogs, curIndex);
                                 tableList = new JList();
                                 createTables(curIndex , gotTheseTables);}
                                 catch (SQLException ex) {
                                    showDBErrors(ex); };
                              repaint();
                           }
                           label6.setText(""); label5.setText(selectedTable);
                           okButton.setEnabled(false);
                           setCursor(normal);
                           setStatus(res.getString("Select_table_to"));
                        }
                     }}
               });

        //Create the scroll pane and add the tree to it.
         JScrollPane treeView = new JScrollPane(schemaTree);

        //Create the table viewing pane.
         tableListModel = new DefaultListModel();
         tableList = new JList(tableListModel);
         JScrollPane tableView = new JScrollPane(tableList);
         JPanel infoPanel = new JPanel();
         JPanel gridPanel = new JPanel();
         infoPanel.setLayout( new BorderLayout());
         infoPanel.add(gridPanel, res.getString("Center1"));
         SymAction lSymAction = new SymAction();
         datasourceButton = new JButton(res.getString("Datasource"));
         datasourceButton.addActionListener(lSymAction);
         JLabel label1 = new JLabel(res.getString("Status_"), JLabel.RIGHT);
         JLabel label2 = new JLabel(res.getString("Collection_"), JLabel.RIGHT);
         JLabel label3 = new JLabel(res.getString("Table_"), JLabel.RIGHT);
         gridPanel.add(label1); gridPanel.add(label2); gridPanel.add(label3); gridPanel.add(label4);
         gridPanel.add(label5); gridPanel.add(label6);
         gridPanel.add(datasourceButton);
        //FIX
        //Yes, yes, I know! It's 11:30 PM, and my head hurts. No GridBag for me right now  :-)
         gridPanel.setLayout(null);  //Poor form... uncouth.  ha.
         label1.setBounds(10, 00, 80, 20); label4.setBounds(105, 00, 200, 20);
         label2.setBounds(10, 20, 80, 20); label5.setBounds(105, 20, 200, 20);
         label3.setBounds(10, 40, 80, 20); label6.setBounds(105, 40, 200, 20);
         datasourceButton.setBounds(415, 20, 130, 28);

        //Add the scroll panes to a split pane.
         JSplitPane horizPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
         JSplitPane vertPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
         horizPane.setLeftComponent(treeView);
         horizPane.setRightComponent(tableView);
         vertPane.setTopComponent(horizPane);
         vertPane.setBottomComponent(infoPanel);

         tableView.setMinimumSize(new Dimension(100, 100));
         treeView.setMinimumSize(new Dimension(100, 100));
         vertPane.setPreferredSize(new Dimension(570, 352));

        //Add the split pane to this frame
         JPanel thisPanel = new JPanel();
         thisPanel.add(vertPane);
         vertPane.setDividerLocation(270);
         vertPane.setEnabled(false);
         horizPane.setDividerLocation(250);
         okButton.setEnabled(false);
         return thisPanel;
      }


    /**
     * Creates the connectionPanel, which will contain all the fields for
     * the connection information.
     */
      public JPanel getConnectionPanel() {
      // Create the labels and text fields.
         JLabel userNameLabel = new JLabel(res.getString("User_name_"), JLabel.RIGHT);
         userNameField = new JTextField("");

         JLabel passwordLabel = new JLabel(res.getString("Password_"), JLabel.RIGHT);
         passwordField = new JPasswordField("");

         JLabel serverLabel = new JLabel(res.getString("Database_URL_"), JLabel.RIGHT);
         serverField = new JTextField(defaultURL);

         JLabel driverLabel = new JLabel(res.getString("Driver_"), JLabel.RIGHT);
         driverField = new JTextField(defaultDriver);

         JPanel connectionPanel = new JPanel(false);
         connectionPanel.setLayout(new BoxLayout(connectionPanel,
            BoxLayout.X_AXIS));

         JPanel namePanel = new JPanel(false);
         namePanel.setLayout(new GridLayout(0, 1));
         namePanel.add(userNameLabel);
         namePanel.add(passwordLabel);
         namePanel.add(serverLabel);
         namePanel.add(driverLabel);

         JPanel fieldPanel = new JPanel(false);
         fieldPanel.setLayout(new GridLayout(0, 1));
         fieldPanel.add(userNameField);
         fieldPanel.add(passwordField);
         fieldPanel.add(serverField);
         fieldPanel.add(driverField);

         connectionPanel.add(namePanel);
         connectionPanel.add(fieldPanel);
         return connectionPanel;
      }

    /**
     * Brigs up a JDialog using JOptionPane containing the connectionPanel.
     * If the user clicks on the 'Connect' button the connection is reset.
     */
      void activateConnectionDialog() {
         setCursor(wait);
         isNotBusy = false;
         String[] ConnectOptionNames = { res.getString("Connect") , res.getString("Cancel1") };
         if(JOptionPane.showOptionDialog(this, getConnectionPanel(), res.getString("Connection_Info_"), JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null, ConnectOptionNames, ConnectOptionNames[0]) == 0) {
            if ( (serverField.getText() != "") & (driverField.getText() != "") & (userNameField.getText() != "")){
               if ((database != null) && (database.isConnected())){
                  try { database.closeConnection(); }
                     catch (SQLException ex) {
                        showDBErrors(ex); }
               }
            //clearTree(null);
               database.setSystemName(serverField.getText());
               defaultURL = serverField.getText();
               database.setDriverName(driverField.getText());
               defaultDriver = driverField.getText();
               database.setPassword(new String(passwordField.getPassword()));
               database.setUserID(userNameField.getText());
               database = connectDatabase(database);
               if (database != null){
                  setStatus(res.getString("Connecting_to1") + serverField.getText());
                  displayDatasource(database);
               }
               else { setStatus(res.getString("Error_connecting_to")); clearTree(null); }
            }
         }
         setCursor(normal);
         isNotBusy = true;
      }
    /**
     * This method checks the state of the DatabaseAdapter and
     * connects if needed
     * @param thisDatabase The DatabaseAdapter
     */
      public DatabaseAdapter connectDatabase(DatabaseAdapter thisDatabase){
         if (thisDatabase == null) {
            thisDatabase = new DatabaseAdapter();
         }
         try { thisDatabase.makeConnection();}
            catch (SQLException ex) {
               showDBErrors(ex); }
            catch (ClassNotFoundException ex) {
               showDBErrors(ex); }
            finally {
               return thisDatabase; }
      }

    /**
     * This method takes the newly connected DatabaseAdapter and
     * displays the results into the various windows panes.
     * @param thisDatabase The DatabaseAdapter
     */
      public void displayDatasource(DatabaseAdapter thisDatabase){
         if (thisDatabase.getLastError() == null) {
            setStatus(res.getString("Receiving_list_of") + serverField.getText());
            this.repaint();
            try { dataCatalogs = thisDatabase.getCatalogs(); }
               catch (SQLException ex) {
                  showDBErrors(ex); }
            if (dataCatalogs.size() == 0){
            }

         //Create the catalog nodes.
          //change the tree
            clearTree(dataCatalogs);
            Vector gotTheseTables = new Vector(0);
            try {
               if (dataCatalogs.size() > 0){
                  setStatus(res.getString("Fetching_tables_in") + (String)dataCatalogs.elementAt(0));
                  this.repaint();
                  gotTheseTables = database.getTables(dataCatalogs, 0);
                  createTables(0 , gotTheseTables);
               }
               else {
                  setStatus(res.getString("Fetching_tables_in1") );
                  this.repaint();
                  gotTheseTables = database.getTables(null, 0);
                  createTables(0 , gotTheseTables);
               }
            }
               catch (SQLException ex) {
                  showDBErrors(ex);}
            schemaModel.reload();

         }
         else { thisDatabase.setLastError(null); }
      }

    /**
     * This method pulls the attributes from the table that is currently selected
     */
      public void getAttributes(){
         try {
            String getThisTable = null;
            if (label5.getText() == res.getString("Default_Collection")){
               getThisTable = label6.getText().trim();
            }
            else {
               getThisTable = label5.getText() + "." + label6.getText().trim();
            }
            ResultSet emptyQuery = database.executeQuery("SELECT * FROM " + getThisTable.trim());
            ResultSetMetaData queryMeta = emptyQuery.getMetaData();
            String colData = ""; Integer thisSize;
            tempListOfAttribs = new Vector(0);
            for (int i = 1; i < queryMeta.getColumnCount() + 1; i++){
               tempAttrib = new Attribute(queryMeta.getColumnName(i));
               String origTableName = label5.getText() + "." + label6.getText();
               boolean acceptsNulls = false;
               if (queryMeta.isNullable(i) == ResultSetMetaData.columnNullable) { acceptsNulls = true; }
            //Map all of the attrib features from SQL to DOA
               tempAttrib.setJavaType(getMapType(queryMeta.getColumnTypeName(i).trim()));
               thisSize = new Integer(queryMeta.getColumnDisplaySize(i));
               tempAttrib.setSize(thisSize.toString());
               tempAttrib.setJavadoc(res.getString("_n_Attribute_imported") + origTableName + res.getString("_n_"));
               tempAttrib.setCanBeNull(acceptsNulls);
               tempAttrib.setUsedForQuery(queryMeta.isSearchable(i));
               tempAttrib.setDBType(queryMeta.getColumnTypeName(i).trim());
               tempListOfAttribs.addElement(tempAttrib);
            }

         }
            catch (SQLException e) {
            }

      }

    /*
     * Resets the TableTree view to default
     */
      public void clearTree(Vector useTheseCatalogs){
         top.removeAllChildren();
         top.setUserObject(res.getString("Datasource_") + serverField.getText());
         setStatus(res.getString("Cataloging") + serverField.getText());
         createCatalogs(top, useTheseCatalogs);
         tableListModel.clear();
         setStatus(res.getString("Select_collection_to"));
         this.repaint();
      }

    /*
     * displays the SQLExceptionError
     */
      public void showDBErrors(SQLException e){
         String showError = res.getString("SQL_ERROR_") + e.getErrorCode() + res.getString("STATE_2") + e.getSQLState();
         JOptionPane.showMessageDialog(this, showError, res.getString("Datasource_connection"), JOptionPane.WARNING_MESSAGE);
      }
    /*
     * displays the ClassNotFoundError
     */
      public void showDBErrors(ClassNotFoundException e){
         String showError = res.getString("DRIVER_NOT_FOUND");
         JOptionPane.showMessageDialog(this, showError, res.getString("Datasource_connection"), JOptionPane.WARNING_MESSAGE);
      }
    /**
     * This method takes a string message and displays
     * it in the status label.
     * @param statString status message
     */
      public void setStatus(String statString){
         label4.setText(statString);
      }

    /**
     * This method takes the SQL type name
     * and returns the JavaType
     * @param thisType SQL data type
     */
      public String getMapType(String thisType){
         String retString = "String";// set default to String
         if (thisType.equalsIgnoreCase("STRING")) { retString = "String"; }
         if (thisType.equalsIgnoreCase("CHAR")) { retString = "String"; }
         if (thisType.equalsIgnoreCase("VARCHAR")) { retString = "String"; }
         if (thisType.equalsIgnoreCase("LONGVARCHAR")) { retString = "String"; }
         if (thisType.equalsIgnoreCase("TEXT")) { retString = "String"; }
         if (thisType.equalsIgnoreCase("BIGDECIMAL")) { retString = "java.math.BigDecimal"; }
         if (thisType.equalsIgnoreCase("NUMERIC")) { retString = "java.math.BigDecimal"; }
         if (thisType.equalsIgnoreCase("DECIMAL")) { retString = "java.math.BigDecimal"; }
         if (thisType.equalsIgnoreCase("INTEGER")) { retString = "int"; }
         if (thisType.equalsIgnoreCase("INT")) { retString = "int"; }
         if (thisType.equalsIgnoreCase("BIT")) { retString = "boolean"; }
         if (thisType.equalsIgnoreCase("BINARY[]")) { retString = "byte[]"; }
         if (thisType.equalsIgnoreCase("VARBINARY[]")) { retString = "byte[]"; }
         if (thisType.equalsIgnoreCase("LONGVARBINARY[]")) { retString = "byte[]"; }
         if (thisType.equalsIgnoreCase("TINYINT")) { retString = "byte"; }
         if (thisType.equalsIgnoreCase("SMALLINT")) { retString = "short"; }
         if (thisType.equalsIgnoreCase("BIGINT")) { retString = "long"; }
         if (thisType.equalsIgnoreCase("LONG")) { retString = "long"; }
         if (thisType.equalsIgnoreCase("REAL")) { retString = "float"; }
         if (thisType.equalsIgnoreCase("DATETIME")) { retString = "java.sql.Date"; }
         if (thisType.equalsIgnoreCase("DATE")) { retString = "java.sql.Date"; }
         if (thisType.equalsIgnoreCase("TIME")) { retString = "java.sql.Time"; }
         if (thisType.equalsIgnoreCase("TIMESTAMP")) { retString = "java.sql.Timestamp"; }
         if (thisType.equalsIgnoreCase("NONE")) { retString = "none"; }
         if (thisType.equalsIgnoreCase("DOUBLE")) { retString = "double"; }
         return retString;
      }

   }



