/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOTableModel.java,v 1.12.14.1 2000/12/01 22:27:39 chip Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.gui.table;

import com.lutris.dods.store.doa.*;
import com.lutris.dods.gui.*;
import javax.swing.JTable;
import javax.swing.JComboBox;
import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import java.util.Vector;
import java.util.ResourceBundle;
/**
 * This Table Model organizes everything for the table in how it is layed out
 * and how you edit it.
 *
 * @author  Philip Martin
 * @version $Revision: 1.12.14.1 $
 */
public class DOTableModel implements TableModel {
    /**
     * Any Listeners for this TableModel.
     */
    static ResourceBundle res = ResourceBundle.getBundle("com.lutris.dods.gui.StringResources");
    protected Vector listeners;
    /**
     * The DataObject whose attributes are being displayed.
     */
    protected DataObject DO;
    /**
     * The number of rows in the table.
     */
    protected int rows;
    /**
     * The parent DataObjectApplication.
     */
    protected DataObjectApplication doa;
    /**
     * True when a value has changed and the table needs to be updated.
     */
    protected boolean hasBeenEdited;
    /**
     * The table for this model.
     */
    protected JTable table;
    /**
     * Constructor which initializes all of the relevant variables.
     */
    public DOTableModel(DataObject Do, DataObjectApplication da, JTable tab) {
        listeners = new Vector();
        doa = da;
        setDataObject(Do);
        hasBeenEdited = false;
        table = tab;
    }
    /**
     * This takes the new value for a cell, and updates the appropriate Attribute.
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Attribute att = (Attribute)DO.getAttributes().elementAt(rowIndex);
        switch(columnIndex) {
            //The editing for the first five columns is handled elsewhere.
            case 0:
                break;
            case 1:
                break;
            case 2:
                break;
	    /*
            case 3:
                break;
		*/
            case 3:
                break;
            case 4:
                break;
            case 5:
                break;
            case 6:
                break;
            case 7:
                att.setName(aValue.toString());
                break;
            case 8:
                att.setJavaType(aValue.toString());
                updateDBType(att, aValue.toString());
                break;
            case 9:
                break;
            case 10:
                att.setSize(aValue.toString());
                break;
            case 11:
                att.setInitialValue(aValue.toString());
                break;
            default:
                break;
        }
        doa.checkForErrors();
    }
    /**
     * Removes the given listener.
     */
    public void removeTableModelListener(TableModelListener l) {
        listeners.removeElement(l);
    }
    /**
     * Returns true if the given cell is editable.
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        if(columnIndex > 4 && columnIndex < 10 && columnIndex != 7) {
            return true;
        }
        return false;
    }
    /**
     * Returns the Object contained in the given cell.
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        Attribute att = (Attribute)DO.getAttributes().elementAt(rowIndex);
        switch(columnIndex) {
            case 0:
                return att;
            case 1:
                return new Boolean(att.isObjectRef());
	    /*
            case 2:
                return att;
		*/
            case 2:
                return new Boolean(att.canBeNull());
            case 3:
                return new Boolean(att.isIndex());
            case 4:
                return new Boolean(att.isUsedForQuery());
            case 5:
                return new Boolean(att.isRefConstraint());
            case 6:
                return new Boolean(att.isConstant());
            case 7:
                return att.getName();
            case 8:
		// FIX
		// I really want to change the Attribute.setObjectRef() method
		// to call Attribute.setJavaType().  But that would cause
		// existing saved project (.doa) files to become invalid.
		// So instead we have this patch code.
		if ( att.isObjectRef() ) {
			DataObject o = att.getObjectRefDO();
			return o.getPackageName() + "." + o.getName() + "DO";
		}
		// END PATCH
                return att.getJavaType();
            case 9:
                return att.getDBType();
            case 10:
                return att.getSize();
            case 11:
                return att.getInitialValue();
            default:
                return "";
        }
    }
    /**
     * Returns the number of rows in the table.
     */
    public int getRowCount() {
        return rows;
    }
    /**
     * Returns the name that is meant for each column header.
     */
    public String getColumnName(int columnIndex) {
        switch(columnIndex) {
            case 7:
                return res.getString("Name");
            case 8:
                return res.getString("Java_Type");
            case 9:
                return res.getString("db_Type");
            case 10:
                return res.getString("Size");
            case 11:
                return res.getString("Initial_Value");
            default:
                return "";
        }
    }
    /**
     * Returns the number of Columns in the table.
     */
    public int getColumnCount() {
        return 12;
    }
    /**
     * Returns the class of Objects in a given column.
     */
    public Class getColumnClass(int columnIndex) {
        if(columnIndex < 7) {
            return Boolean.class;
        }
	/*
        if(columnIndex == 8) {
            return JComboBox.class;
        }
	*/
        return String.class;
    }
    /**
     * Adds a new Listener.
     */
    public void addTableModelListener(TableModelListener l) {
        if(!listeners.contains(l)) {
            listeners.addElement(l);
        }
    }
    /**
     * Sets the DataObject being displayed in a given table.
     */
    public void setDataObject(DataObject Do) {
        if(Do != null) {
            DO = Do;
            rows = DO.getAttributes().size();
        }
        else {
            rows = 0;
        }
    }
    /**
     * resets the row count when it has changed.
     */
    public void resetRows() {
        rows = DO.getAttributes().size();
    }
    /**
     * Returns the DataObject Being displayed in the table.
     */
    public DataObject getDataObject() {
        return DO;
    }
    /**
     * Resets the DBType cell when the Javatype is changed.
     */
    public void updateDBType(Attribute att, String javatype){
        att.setObjectRef(false);
        if(javatype.equals("String")) {
            att.setDBType("VARCHAR");
        }
        else if(javatype.equals("java.math.BigDecimal")) {
            att.setDBType("DECIMAL");
        }
        else if(javatype.equals("boolean")) {
            att.setDBType("BIT");
        }
        else if(javatype.equals("byte[]")) {
            att.setDBType("VARBINARY");
        }
        else if(javatype.equals("byte")) {
            att.setDBType("TINYINT");
        }
        else if(javatype.equals("short")) {
            att.setDBType("SMALLINT");
        }
        else if(javatype.equals("int")) {
            att.setDBType("INTEGER");
        }
        else if(javatype.equals("long")) {
            att.setDBType("BIGINT");
        }
        else if(javatype.equals("float")) {
            att.setDBType("REAL");
        }
        else if(javatype.equals("double")) {
            att.setDBType("DOUBLE");
        }
        else if(javatype.equals("java.sql.Date")) {
            att.setDBType("DATE");
        }
        else if(javatype.equals("java.sql.Time")) {
            att.setDBType("TIME");
        }
        else if(javatype.equals("java.sql.Timestamp")) {
            att.setDBType("TIMESTAMP");
        }
        else {
            att.setDBType("none");
            att.setObjectRef(true);
        }
        table.repaint();
    }
}
