/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: ErrorColumnRenderer.java,v 1.11 2000/01/29 07:01:42 markd Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.gui.table;

import com.lutris.dods.gui.*;
import com.lutris.dods.store.doa.*;

import javax.swing.ImageIcon;
import javax.swing.JTable;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
/**
 * An extension of DOTableColumnRenderer, this class is meant for displaying
 * an Icon for an error.
 *
 * @author  Philip Martin
 * @version $Revision: 1.11 $
 */
public final class ErrorColumnRenderer extends DOTableColumnRenderer {

    private boolean toShow = false;

    /**
     * Use "error.gif".
     */
    public ErrorColumnRenderer(boolean sh) {
        super(sh);
        toShow = sh;
        icon = Icons.getIcon("error.gif");
    }
    /**
     * Shows the icon when the attribute is an Error, and is blank when it is not.
     * @param value This is an Attribute as opposed to a Boolean as in the super
     *  class.
     */
    public Component getTableCellRendererComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   boolean hasFocus,
                                                   int row,
                                                   int column) {
        super.setForeground(table.getForeground());
        super.setBackground(Color.lightGray);
        if(toShow) {
            setIcon(icon);
            return this;
	}
        else {
            setIcon(null);
	}
        setFont(table.getFont());
        if(value!=null) {
            if(((Attribute)value).hasError()) {
                setIcon(icon);
            }           
        }
        return this;
    }
}
