/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOEditorTreeModel.java,v 1.8 1999/12/09 23:13:10 markd Exp $
 *-----------------------------------------------------------------------------
 */



package com.lutris.dods.gui.tree;

import com.lutris.dods.store.doa.*;
import com.lutris.dods.gui.*;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.util.Vector;
/**
 * This is virtually the same as the DOTreeModel except nodes aren't shown.
 *
 * @author  Philip Martin
 * @version $Revision: 1.8 $
 */
public final class DOEditorTreeModel extends DOTreeModel {
    private DOTreeNode root;
    private Vector listeners;
    private int in= 0;
    private Object par;
    private DOPackage dop;
    /**
     * This constructor calls the constructor for DOTreeModel.
     */
    public DOEditorTreeModel(DOTreeNode r, DataObjectApplication da, DOPackage dp) {
        super(r, da);
        par = r;
        dop = dp;
    }
    /**
     * The Tree cannot be edited so this does nothing.
     */
    public void valueForPathChanged(TreePath path, Object newValue) {
    }
    /**
     * Returns the number of children who are not Leaves.
     */
    public int getChildCount(Object parent) {
        int ret = 0;
        try {
            for(int i = 0; i < ((DOTreeNode)parent).getChildren().size(); i++) {
                if(!((DOTreeNode)((DOTreeNode)parent).getChildren().elementAt(i)).isLeaf() &&
                   !(((DOPackage)((DOTreeNode)((DOTreeNode)parent).getChildren().elementAt(i)).getDataObject()) == dop) ) {
                    ret++;
                }
            }
        }
        catch(Exception e) {}
        return ret;
    }
    /**
     * Returns the child at the given index but ignores leaves.
     */
    public Object getChild(Object parent, int index) {
        if(parent != par) {
            in = 0;
            par = parent;
        }
        if(index < in ) {
            index = in+1;
        }
        if(parent instanceof DOTreeNode) {
            try {
                if(((DOTreeNode)((DOTreeNode)parent).getChildren().elementAt(index)).isLeaf() ||
                   ((DOPackage)((DOTreeNode)((DOTreeNode)parent).getChildren().elementAt(index)).getDataObject()) == (dop)) {
                    for(int i = index+1; i < ((DOTreeNode)parent).getChildren().size(); i++) {
                        if(!((DOTreeNode)((DOTreeNode)parent).getChildren().elementAt(i)).isLeaf() &&
                           !(((DOPackage)((DOTreeNode)((DOTreeNode)parent).getChildren().elementAt(i)).getDataObject()) == dop)) {
                            in = i;
                            return ((DOTreeNode)parent).getChildren().elementAt(i);
                        }
                    }
                }
                else {
                    in = index;
                    return ((DOTreeNode)parent).getChildren().elementAt(index);
                }
            }
            catch(Exception x) {
                System.out.println(x.toString());
            }
        }
        return null;
    }
}
