/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 */

package com.lutris.dods.store.doml;

import java.io.*;
import org.w3c.dom.*;

//FIXME: Use Apache serialize code...

/**
 * This class is a standby tool used to print DODS XML DODML files to some
 * output stream. It is mostly a direct lifting of the Xerces sample code
 * dom/DOMWriter.java, with some minor modifications. This file should be
 * junked and re-written as soon as possible. As soon as that happens, the
 * following Apache text need not apply to it...
 * <p>
 * @author Jason Abbott (jason@lutris.com)
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
public class DOMLWriter {
  protected static String defaultEncoding = "UTF-8";

  protected String      encoding = null;
  protected PrintWriter out = null;
  protected boolean     canonical = true;
  protected boolean     indent = false;

  private int     indentLevel = 0;
  private boolean indentNeedsNewLine = false;
  private boolean includeDocType     = false;

  /**
   * Constructor.
   *
   * @param outputStream
   *          The OutputStream that this writer should be writing to.
   * @param canonical
   *          This boolean will be true if the writer is to strip all macros
   *          and abbreviations out of this document.
   * @param indent
   *          This boolean will be true to indicate that this document should
   *          be indented in human readable form. It should be noted that
   *          this will introduce bogus text elements to the XML structure
   *          that the reader will have to understand how and when to ignore.
   */
  public DOMLWriter(OutputStream outputStream,
    boolean      canonical,
    boolean      indent, 
    boolean      docType) {
    
    OutputStreamWriter osw = null;
    try {
      osw = new OutputStreamWriter(outputStream, defaultEncoding);
      encoding = defaultEncoding;
    } catch (IllegalArgumentException iae) {
      osw = new OutputStreamWriter(outputStream);
      encoding = null;
    } catch (UnsupportedEncodingException uee) {
      osw = new OutputStreamWriter(outputStream);
      encoding = null;
    }

    out = new PrintWriter(osw);
    
    this.canonical      = canonical;
    this.indent         = indent;
    this.includeDocType = docType;
  }
  
  private void increaseIndent() {
    indentLevel += 2;
  }

  private void decreaseIndent() {
    indentLevel -= 2;
  }

  private void initIndentLine() {
    indentNeedsNewLine = true;
  }

  private void uninitIndentLine() {
    indentNeedsNewLine = false;
  }

  private void indent() {
    if (indent == false) {
      return;
    }
    if (indentNeedsNewLine == false) {
      return;
    }
    out.println();
    indentNeedsNewLine = false;
    if (indentLevel > 0) {
      for (int i = 0; i<indentLevel; i++) {
         out.print(' ');
      }
    }
  }

  /**
   * Dump the given argument and all it's children. The given object will
   * have a recursive traversal run on it, and its entire structure will be
   * sent to the <b>OutputStream</b> specified in the constructor.
   *
   * @param node
   *          This Node should specify the root of whatever structure should
   *          be printed.
   */
  public void print(Node node) {
    // is there anything to do?
    if (node == null) {
      return;
    }

    int type = node.getNodeType();

    switch (type) {
    case Node.DOCUMENT_NODE:
      {
        // print document
        indent();
        if (canonical == false) {
          out.print("<?xml version=\"1.0\"");
          if (encoding != null) {
            out.print(" encoding=\""+ encoding + "\"");
          }
          out.println("?>");
        }
        Document     doc = (Document) node;
        DocumentType docType = doc.getDoctype();
        if (docType != null && includeDocType) {
          out.print("<!DOCTYPE " + docType.getName());
          String publicId = docType.getPublicId();
          String systemId = docType.getSystemId();
          if (publicId != null && publicId.trim().length() > 0) {
              out.print(" PUBLIC \"" + publicId + "\"");
          }
          if (systemId != null && systemId.trim().length() > 0) {
              out.print(" SYSTEM \"" + systemId + "\"");
          }
          out.println(">");
        }
        print(doc.getDocumentElement());
        out.flush();
        indent();
        break;
      }

    case Node.ELEMENT_NODE:
      {
        // print element with attributes
        indent();
        initIndentLine();
        out.print('<');
        out.print(node.getNodeName());
        Attr attrs[] = sortAttributeArray(node.getAttributes());
        for (int i = 0; i < attrs.length; i++) {
            Attr attr = attrs[i];
            out.print(' ');
            out.print(attr.getNodeName());
            out.print("=\"");
            out.print(normalize(attr.getNodeValue()));
            out.print('"');
        }

        NodeList children = node.getChildNodes();

        if (children == null || children.getLength() == 0) {
          out.print("/>");
        } else {
          out.print('>');
          increaseIndent();
          if (children != null) {
            int len = children.getLength();
            for (int i = 0; i < len; i++) {
              print(children.item(i));
            }
          }
          decreaseIndent();
          indent();
          out.print("</");
          out.print(node.getNodeName());
          out.print('>');
          initIndentLine();
        }
        break;
      }

    case Node.ENTITY_REFERENCE_NODE:
      {
        // handle entity reference nodes
        if (canonical) {
          NodeList children = node.getChildNodes();
          if (children != null) {
            int len = children.getLength();
            for ( int i = 0; i < len; i++ ) {
              print(children.item(i));
            }
          }
        } else {
          out.print('&');
          out.print(node.getNodeName());
          out.print(';');
          uninitIndentLine();
        }
        break;
      }

    case Node.CDATA_SECTION_NODE:
      {
        // print cdata sections
        if (canonical) {
          out.print(normalize(node.getNodeValue()));
          uninitIndentLine();
        } else {
          indent();
          out.print("<![CDATA[");
          out.print(node.getNodeValue());
          out.print("]]>");
        }
        break;
      }

    case Node.TEXT_NODE:
      {
        // print text
        out.print(normalize(node.getNodeValue()));
        uninitIndentLine();
        break;
      }

    case Node.PROCESSING_INSTRUCTION_NODE:
      {
        // print processing instruction
        indent();
        out.print("<?");
        out.print(node.getNodeName());
        String data = node.getNodeValue();
        if ( data != null && data.length() > 0 ) {
          out.print(' ');
          out.print(data);
        }
        out.print("?>");
        break;
      }
    }

    out.flush();
  }

  /**
   * Normalizes the given string. Turn all XML sensistive characters into
   * escaped character sequences.
   */
  private String normalize(String s) {
    StringBuffer str = new StringBuffer();
    int          len = (s != null) ? s.length() : 0;

    for (int i = 0; i < len; i++) {
        char ch = s.charAt(i);
        switch (ch) {
        case '<':
            {
                str.append("&lt;");
                break;
            }
        case '>':
            {
                str.append("&gt;");
                break;
            }
        case '&':
            {
                str.append("&amp;");
                break;
            }
        case '"':
            {
                str.append("&quot;");
                break;
            }
        case '\r':
        case '\n':
            {
                if (canonical) {
                    str.append("&#");
                    str.append(Integer.toString(ch));
                    str.append(';');
                    break;
                }
                // else, default append char
            }
        default:
            {
                str.append(ch);
            }
        }
    }

    return (str.toString());
  }

  private Attr[] sortAttributeArray(NamedNodeMap attrs)  {
    int  len = (attrs != null) ? attrs.getLength() : 0;
    Attr array[] = new Attr[len];
    for (int i = 0; i < len; i++) {
        array[i] = (Attr)attrs.item(i);
    }
    for (int i = 0; i < len - 1; i++) {
        String name  = array[i].getNodeName().toLowerCase();
        int    index = i;
        for (int j = i + 1; j < len; j++) {
            String curName = array[j].getNodeName().toLowerCase();
            if (curName.compareTo(name) < 0 ||
                    curName.compareTo("id") == 0) {
                name  = curName;
                index = j;
            }
        }
        if (index != i) {
            Attr temp    = array[i];
            array[i]     = array[index];
            array[index] = temp;
        }
    }

    return (array);
  }
}
