/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOPackage.java,v 1.9 2000/01/29 07:01:45 markd Exp $
 *-----------------------------------------------------------------------------
 */



package dods.data;

import com.lutris.dods.exceptions.*;
import java.io.Serializable;
import java.util.Vector;

/**
 * A holder for all the information describing a DOPackage.
 *
 * @author  Philip Martin
 * @version $Revision: 1.9 $
 */
public class DOPackage extends DataObject implements Serializable {
    /**
     * This is the Vector which holds all of the DataObjects and Packages that
     * are in this package.
     */
    protected Vector dataObjects;
    /**
     * This constructor creates a DOPackage with name "NewPackage"
     */
    public DOPackage() {
        this("NewPackage");
    }
    /**
     * This constructor creates a DOPackage with the specified name.
     */
    public DOPackage(String name) {
        super(name);
        dataObjects = new Vector();
    }
    /**
     * @return The DataObjects in this Package.
     */
    public Vector getDataObjects() {
        return dataObjects;
    }
    /**
     * This method adds a DataObject to this Package.
     */
    public void addDataObject(DataObject DO) {
        dataObjects.addElement(DO);
        try {
            if(this.getPackageName().equals("")){
                DO.setPackageName(this.getName());
            }
            else {
                DO.setPackageName(this.getPackageName()+ "." + this.getName());
            }
        }
        catch(DataObjectNameException e){}
    }
    /**
     * This method adds a Package into this Package.
     */
    public void addDOPackage(DOPackage DOP) {
        dataObjects.addElement(DOP);
        try {
            if(this.getPackageName().equals("")){
                DOP.setPackageName(this.getName());
            }
            else {
                DOP.setPackageName(this.getPackageName()+ "." + this.getName());
            }
        }
        catch(DataObjectNameException e){
        }
    }
    /**
     * This method is used when a package name is changed, so all of the Objects
     * in that Package need to have thier packageNames updated.
     */
    public void updatePackageName() {
        for(int i = 0; i < dataObjects.size(); i++) {
            try {
                ((DataObject)dataObjects.elementAt(i)).setPackageName(this.getPackageName() + "." + this.getName());
                if(this.getPackageName().equals("")) {
                    ((DataObject)dataObjects.elementAt(i)).setPackageName(this.getName());
                }
                if((DataObject)dataObjects.elementAt(i) instanceof DOPackage) {
                    ((DOPackage)dataObjects.elementAt(i)).updatePackageName();
                }
            }
            catch(DataObjectNameException e) {}
        }
    }
    /**
     * Returns a String representation of the form:
     * "<I>packageName</I>.<I>name</I>"
     */
    public String toString() {
        if(packageName.equals("")){
            return new String(name + "");
        }
        return new String(packageName + "." + name);
    }
    /**
     * This will return true if the two names and all of the dataObjects are equal.
     */
    public boolean equals(DOPackage dop) {
        try {
            if(dataObjects.size()!=dop.getDataObjects().size()) {
                return false;
            }
            for(int i = 0; i < dataObjects.size(); i++) {
                DataObject dataObject = (DataObject)dataObjects.elementAt(i);
                if(dataObject instanceof DOPackage) {
                    DOPackage doPackage = (DOPackage)dataObject;
                    if(!((DOPackage)dop.getDataObjects().elementAt(i)).equals(doPackage)) {
                        return false;
                    }
                }
                else {
                    if(!((DataObject)dop.getDataObjects().elementAt(i)).equals(dataObject)) {
                        return false;
                    }
                }
            }
            if(!name.equals(dop.getName()))
                return false;
            if(!packageName.equals(dop.getPackageName()))
                return false;
        }
        catch(Exception e) {
            return false;
        }
        return true;
    }
    /**
     * This method adds all of the DataObjects in this package and any subPackages
     * into the given vector.
     * @param The vector to add the DataObjects to.
     */
    public void getAllDataObjects(Vector allDOS) {
        for(int i = 0; i < dataObjects.size(); i++) {
            if(dataObjects.elementAt(i) instanceof DOPackage) {
                ((DOPackage)dataObjects.elementAt(i)).getAllDataObjects(allDOS);
            }
            else {
                allDOS.addElement(dataObjects.elementAt(i));
            }
        }
    }
}
