/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Upgrade.java,v 1.3 2000/01/29 07:01:45 markd Exp $
 *-----------------------------------------------------------------------------
 */

package dods.doml;

import com.lutris.dods.store.Store;
import com.lutris.dods.store.doml.DOMLException;
import com.lutris.dods.store.doml.DOMLWriter;
import org.w3c.dom.Document;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

public class Upgrade {
  private File input  = null;
  private File output = null;
  private dods.data.DODSProject project = null;

  public static void main(String[] args) {
    if (args == null || args.length < 1) {
      System.out.println("Missing doa file argument");
    } else {
      Upgrade up = new Upgrade();
      File f = new File(args[0]);
      try {
        up.convert(f);
        System.out.println("Upgrade complete: " + up.getOutput().getAbsolutePath());
      } catch (UpException e) {
        System.out.println(e.getMessage());
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  public File convert(File f) throws UpException {
    Store store = null;
    store = new Store();
    store.setInputFile(f);

    input  = f;
    output = store.getOutputFile();
    checkDOA();
    readDOA();
    writeDOML(project);
    return output;
  }

  public File getOutput() {
    return output;
  }

  private void checkDOA() throws UpException {
    if ((! input.exists()) || (! input.isFile())) {
      throw new UpException("DOA file not found: " + input.getName());
    }

    Store store = null;
    store = new Store();
    store.setInputFile(input);
    if (! store.isInputDOA()) {
      throw new UpException("Not a DOA file: " + input.getName());
    }
  }


  private void readDOA() throws UpException {
    Object objectIn = null;

    try {
      FileInputStream   finput = new FileInputStream(input);
      ObjectInputStream oInput = new ObjectInputStream(finput);

      objectIn = oInput.readObject();

      if (objectIn instanceof dods.data.DODSProject) {
        project = (dods.data.DODSProject) objectIn;
      }
    } catch (Exception e) {
      throw new UpException("Error: Unable to read DOA file: " + e.toString());
    }
  }


  private void writeDOML(dods.data.DODSProject project) throws UpException {
    DOMLUpgradeFactory docFactory = new DOMLUpgradeFactory();
    Document                  doc = null;

    try {
      doc = docFactory.createDocument(project);
    } catch (UpException e) {
      throw new UpException("Unable to ccreate DOML file");
    }

    try {
      FileOutputStream out = new FileOutputStream(output);
      DOMLWriter      writer = new DOMLWriter(out, false, true, false);
      writer.print(doc);
    } catch (IOException e) {
      throw new UpException("Unable to create DOML file: " +
                 output.getAbsolutePath());

    }
  }

}
