/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Test4.java,v 1.1.2.2 2000/12/07 22:27:27 jay Exp $
 */


package tests.org.enhydra.dods.test4;

import tests.org.enhydra.dods.TestHarness;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

import com.lutris.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;

import tests.org.enhydra.dods.test4.data.*;

public class Test4 extends TestHarness {

    /*
       TestHarness can find these method names via reflection,
       but they must be run in a certain order.
       This list also makes it easier to turn tests on/off.
    */
    public static String[] tests = { 
	"dropAndRecreateTables",
	"deleteAllData",
	"createData",
	"checkData1",
	"printAllHandles",
	"selfRefCaching",
	"selfRefNullParentInCache",
	"selfRefNullParentInDB",
	null
    };

    public static void main( String args[] ) {
	run( new Test4(), tests, args );
    }

    private Test4() {
    }

    public static boolean dropAndRecreateTables() throws Exception {
	print( TITLE, 
	    "dropAndRecreateTables: exercise drop and create table commands.");
	runSqlFromFile( "data/drop_tables.sql",   true, true, false );
	runSqlFromFile( "data/create_tables.sql", false, true, true );
	return true;
    }

    public static boolean printAllHandles() throws Exception {
	print( TITLE, 
	    "printAllHandles: Verify all classes and tables are present.");
	if ( ! opDOArray( Test4ProjectQuery.class,		PRINT_OID ) )
	    return false;
	return true;
    }

    public static boolean deleteAllData() throws Exception {
	print( TITLE, 
	    "deleteAllData: delete all rows of test data.");
	if ( ! opDOArray( Test4ProjectQuery.class,		DELETE_DO ) )
	    return false;
	return true;
    }

    public static int numProjs = 4;
    public static Test4ProjectDO[] projs = new Test4ProjectDO[ numProjs ];
    public static int[] parentIndexForProject = { -1, 0, 0, 2 };
    public static Hashtable projHandlesByName = new Hashtable();

    /*
       totalRows is a Hashtable in TestHarness.
       TestHarness.opDOArray uses totalRows to perform query validation.
    */
    static {
	setTotalRows( Test4ProjectQuery.class,              numProjs );
    }

    public static boolean createData() throws Exception {
	print( TITLE, "createData: create initial data set");
        for ( int i = 0; i < numProjs; i++ ) {
	    Test4ProjectDO p = projs[ i ] = Test4ProjectDO.createVirgin();
	    String name = "p" + i;
	    p.setName( name );
	    int n = parentIndexForProject[ i ];
	    if ( -1 == n )
		p.setParentProject( null );
	    else
		p.setParentProject( projs[ n ] );
	    p.commit();
	    projHandlesByName.put( name, p.getHandle() );
        }
	Test4ProjectDO.refreshCache();
	return true;
    }

    private static Test4ProjectDO[] chk_projs = null;

    public static boolean checkData1() throws Exception {
	print( TITLE, "checkData: verify initial data set was written to db");
	Test4ProjectQuery pq = new Test4ProjectQuery();
	chk_projs = pq.getDOArray();
        for ( int i = 0; i < chk_projs.length; i++ ) {
	    String name = chk_projs[ i ].getName();
	    String handle = chk_projs[ i ].getHandle();
	    String phbn = (String) projHandlesByName.get( name );
            if ( ! phbn.equals( handle ) ) {
		print( PASS_FAIL, 
		    "Proj named " + name + " did not have handle " + handle );
		return false;
	    }
        }
	return true;
    }

    public static boolean selfRefCaching() throws Exception {
	print( TITLE, "selfRefCaching: check self reference with cache");
	Test4ProjectQuery q = new Test4ProjectQuery();
	q.setQueryHandle( projs[ 3 ].getHandle() );
	Test4ProjectDO child = q.getNextDO();
	String childHandle = child.getHandle();
	print( DEBUG3, "Child project handle " + childHandle );
	q.reset();
	print( DEBUG3, "Child project parent handle " + 
			child.getParentProject().getHandle() );
	q.setQueryHandle( child.getParentProject().getHandle() );
	Test4ProjectDO parent = q.getNextDO();
	if ( null == parent ) {
	    print( PASS_FAIL,
		"Child project " + childHandle + " has null parent." );
	    print( DEBUG2, "Child project " + child );
	    return false;
	}
	String parentHandle = parent.getHandle();
	print( DEBUG2, "parent = " + parent );
	Test4ProjectDO expectedParent = projs[ 2 ];
	String expectedHandle = expectedParent.getHandle();
	print( DEBUG3, "Expected parent project handle " + expectedHandle );
	if ( ! parentHandle.equals( expectedHandle ) ) {
	    print( PASS_FAIL,
		"Child project " + childHandle +
		" has incorrect parent " + parent.getHandle() +
		", expected parent to be " + expectedHandle );
	    print( DEBUG2, "Child project " + child );
	    return false;
	}

	String name = "p2";
	q.reset();
	q.setQueryName( name );
	child = q.getNextDO();
	if ( ! child.getName().equals( name ) )
	    return false;
	return true;
    }

    public static boolean selfRefNullParentInCache() throws Exception {
	print( TITLE, "selfRef: check self reference of parent");
	Test4ProjectQuery q = new Test4ProjectQuery();
	q.setQueryParentProject( null );
	Test4ProjectDO[] a = q.getDOArray();
	return ( 1 == a.length );
    }

    public static boolean selfRefNullParentInDB() throws Exception {
	print( TITLE, "selfRef: check self reference of parent");
	Test4ProjectQuery q = new Test4ProjectQuery();
	q.hitDatabase();
	q.setQueryParentProject( null );
	Test4ProjectDO[] a = q.getDOArray();
	return ( 1 == a.length );
    }


}
