/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Test5.java,v 1.1.2.1 2000/11/18 03:41:06 jay Exp $
 */


package tests.org.enhydra.dods.test5;

import tests.org.enhydra.dods.TestHarness;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.lang.reflect.*;

import com.lutris.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;

import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;

import tests.org.enhydra.dods.test5.data.*;

public class Test5 extends TestHarness {

    /*
       TestHarness can find these method names via reflection,
       but they must be run in a certain order.
       This list also makes it easier to turn tests on/off.
    */
    public static String[] tests = { 
	"dropAndRecreateTables",
	"deleteAllData",
	"createData",
	"checkDataStandard",
	"checkDataLongSmall",
	"checkDataLongLarge",
	"printAllHandles",
	null
    };

    public static void main( String args[] ) {
	run( new Test5(), tests, args );
    }

    private Test5() {
    }

    public static boolean dropAndRecreateTables() throws Exception {
	print( TITLE, 
	    "dropAndRecreateTables: exercise drop and create table commands.");
	runSqlFromFile( "data/drop_tables.sql",   true, true, false );
	runSqlFromFile( "data/create_tables.sql", false, true, true );
	return true;
    }

    public static boolean printAllHandles() throws Exception {
	print( TITLE, 
	    "printAllHandles: Verify all classes and tables are present.");
	if ( ! opDOArray( ZQuery.class,			PRINT_OID ) )
	    return false;
	return true;
    }

    public static boolean deleteAllData() throws Exception {
	print( TITLE, 
	    "deleteAllData: delete all rows of test data.");
	if ( ! opDOArray( ZQuery.class,			DELETE_DO ) )
	    return false;
	return true;
    }

    public static int numZ = 2;
    public static ZDO[] xs = new ZDO[ numZ ];

    /*
       totalRows is a Hashtable in TestHarness.
       TestHarness.opDOArray uses totalRows to perform query validation.
    */
    static {
	setTotalRows( ZQuery.class,             numZ );
    }

    private static String[] longText = { "", "" };
    // Oracle JDBC driver setString() throws SQLException when >4000 bytes 
    // are stored in a LONG column.  This is bad driver behavior.
    // longText[0] will be 4000 chars long.
    // longText[1] will be 5000 chars long.
    static {
        for ( int j = 0; j < 2; j++ ) {
	    for ( int i = 0; i < 1000; i++ ) {
		longText[j] += "S";
	    }
	    if ( 1 == j ) {
		for ( int k = 0; k < 1000; k++ ) {
		    longText[j] += "L";
		}
	    }
	}
    }

    private static String makeString( int i ) {
	return "a" + i + "b";
    }

    public static boolean createData() throws Exception {
	print( TITLE, "createData: create initial data set");
        for ( int i = 0; i < numZ; i++ ) {
	    ZDO x = xs[i] = ZDO.createVirgin();
	    String s = makeString( i );
	    x.setCHAR_String ( 			s );
	    x.setLONGVARCHAR_String ( 		longText[0] );
	    x.commit();

	    if ( 1 == i ) {
                try {
		    x.setLONGVARCHAR_String ( 	longText[1] );
		    x.commit();
                } catch ( Exception e ) { 
		    print( REPORT1, "createData: failed to set long value for LONG column (known Oracle problem)" );
                }
	    }
        }
	return true;
    }

    private static ZDO[] chk_x = null;

    public static boolean checkDataStandard() throws Exception {
	print( TITLE, "checkDataStandard: verify initial data set was written to db");
	ZQuery xq = new ZQuery();
	chk_x = xq.getDOArray();
        for ( int i = 0; i < chk_x.length; i++ ) {
//System.err.println("TTTTTTTTTTTTTTTTTT");
//System.err.println("chk_x["+i+"].getHandle()="+chk_x[i].getHandle());
//System.err.println("i ="+i );
	    ZDO x = chk_x[i];
	    String s = makeString( i );
	    String getter = "???";
	    String failedGetters = "";
            try {
		getter = "getCHAR_String";
	        if ( 0 != x.getCHAR_String().indexOf( s ) ) 
		    failedGetters += getter + " ";

		getter = "DONE";
            } catch ( Exception e ) { 
		print( ERROR, test + 
			": got exception after calling get method: " + getter +
			": Exception=" + e.getMessage() );
		return false;
            }
	    if ( 0 != failedGetters.length() ) {
		print( REPORT1, test + 
			": get methods failed: " + failedGetters + "." );
		return false;
	    }
        }
	return true;
    }

    public static boolean checkDataLongSmall() throws Exception {
	return checkDataLong( "small", longText[0] );
    }
    public static boolean checkDataLongLarge() throws Exception {
	return checkDataLong( "large", longText[1] );
    }
    public static boolean checkDataLong( String smallOrLarge, String cmpText )
    throws Exception {
	print( TITLE, "checkDataLONG: verify " + smallOrLarge +
		"amount of LONG text data was written to db");
        for ( int i = 0; i < chk_x.length; i++ ) {
	    ZDO x = chk_x[i];
	    String getter = "???";
	    String failedGetters = "";
            try {

		getter = "getLONGVARCHAR_String";
/*
String xs = x.getLONGVARCHAR_String();
System.err.println("xs ="+xs );
System.err.println("xs.length="+xs.length());
*/
/*
System.err.println("x.getLONGVARCHAR_String()="+xs+".");
char[] cs = xs.toCharArray();
for ( int k = 0; k < cs.length; k++ ) {
System.err.println("cs["+k+"]="+((int)cs[k]));
}
System.err.println("s="+s+".");
*/
	        if ( ! cmpText.equals( x.getLONGVARCHAR_String() ) ) 
		    failedGetters += getter + " ";

		getter = "DONE";
            } catch ( Exception e ) { 
		print( ERROR, test + 
			": got exception after calling get method: " + getter +
			": Exception=" + e.getMessage() );
		return false;
            }
	    if ( 0 != failedGetters.length() ) {
		print( DEBUG1, test + 
			": get methods failed: " + failedGetters + "." );
		return false;
	    }
        }
	return true;
    }

}
