/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PostgreSQLConnectionAllocator.java,v 1.2.12.1 2000/10/19 17:59:06 jasona Exp $
 */

package	com.lutris.appserver.server.sql.postgresql;

import java.sql.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.sql.standard.*;
import com.lutris.util.*;

/**
 * An PostgreSQL database connection allocator used by the PostgreSQL logical
 * database.
 *
 * @see		StandardConnectionAllocator
 * @author	Ole Arndt (ole@opus5.de)
 * @since	Enhydra 2.1
 * @version	$Revision: 1.2.12.1 $
 */
public class PostgreSQLConnectionAllocator extends StandardConnectionAllocator {

    /**
     * Creates an PostgreSQL database connection allocator.
     *
     * @see StandardConnectionAllocator#StandardConnectionAllocator
     * @param logicalDatabase
     *   the logical PostgreSQL database for which connections are pooled.
     * @param connectionConfig
     *   configuration parameters for the database connections.  There
     *   are no Informix specific configuration data.
     * @exception ConfigException
     *   If bad configuration information is given in the config file.
     */
    protected PostgreSQLConnectionAllocator(
        PostgreSQLLogicalDatabase logicalDatabase,
        Config connectionConfig) throws ConfigException {
        super(logicalDatabase, connectionConfig);
    }


    /**
     * Creates a new PostgreSQL connection to be added to the connection pool.
     * 
     * @exception SQLException If a SQL error occures.
     */
    protected DBConnection createConnection () throws SQLException {
        return new PostgreSQLDBConnection (this, url, user, password,
                                           maxPreparedStatements,
                                           sqlLogging, generation);
    }
}
