/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SybaseLogicalDatabase.java,v 1.6.12.1 2000/10/19 17:59:06 jasona Exp $
 */





package	com.lutris.appserver.server.sql.sybase;

import com.lutris.appserver.server.sql.standard.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.util.*;
import java.util.Date;	// Must be before java.sql.Date
import java.sql.*;


/**
 * Represents a logical database with Sybase specific capabilities.
 *
 * @see		StandardLogicalDatabase
 * @author	Paul Morgan
 * @since	LBS1.8
 * @version	$Revision: 1.6.12.1 $
 */
public class SybaseLogicalDatabase extends StandardLogicalDatabase {

    /**
     * Creates and configures a single logical database with sybase
     * specific enhancements.
     *
     * @see StandardLogicalDatabase#StandardLogicalDatabase
     * @param dbName
     *   The logical name of the database.
     * @param dbConfig
     *   The configuration object for this logical database.
     * @exception ConfigException
     *   If there is an error in the configuration file.
     * @exception SQLException
     *   If a SQL error occurs.
     */
    public SybaseLogicalDatabase (String dbName, Config dbConfig)
        throws ConfigException, SQLException {
	    
	    super.init(dbName, dbConfig);
    }
    

    /**
     * Default constructor to configure a single logical database.
     * Note that the <CODE>init()</CODE> method must be called to
     * configure the database.
     * 
     * @see StandardLogicalDatabase#StandardLogicalDatabase
     */
    public SybaseLogicalDatabase () {
        super();
    }
    

    /**
     * Return the connection allocator.
     *
     * @param connectionConfig
     *   The configuration object for the connection allocator.
     * @exception ConfigException
     *   If there is an error in the configuration file.
     * @return
     *   The connection allocator.
     */
    public ConnectionAllocator
        loadConnectionAllocator(Config connectionConfig)
        throws ConfigException {

	return (ConnectionAllocator)
	    new SybaseConnectionAllocator (this, connectionConfig);
    }


    /**
     * Return the object id allocator.
     *
     * @param objIdConfig
     *   The configuration object for the object id allocator.
     * @exception ConfigException
     *   If there is an error in the configuration file.
     * @return
     *   The object id allocator.
     */
    public ObjectIdAllocator
        loadObjectIdAllocator (Config objIdConfig)
        throws ConfigException {

	return (ObjectIdAllocator)
	    new SybaseObjectIdAllocator (this, objIdConfig);
    }


    /**
     * Return a transaction for use on this Sybase logical database.
     *
     * @return The transaction object.
     * @exception SQLException
     *   if a SQL error occurs.
     */
    public DBTransaction createTransaction ()
	throws SQLException {

	return new SybaseDBTransaction
	    ((SybaseDBConnection)connectionAllocator.allocate());
    }


    /**
     * Return a query for use on this logical database.
     *
     * @return The query object.
     * @exception SQLException
     *   if a SQL error occurs.
     */
    public DBQuery createQuery ()
	throws SQLException {

	return new SybaseDBQuery (connectionAllocator.allocate());
    }


    /**
     * Return a description of the logical database type.
     *
     * @return The type.
     */
    public String getType () {
	return "Sybase JDBC";
    }
}
