/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Main.java,v 1.8.8.1 2000/10/19 17:59:06 jasona Exp $
 */





package	com.lutris.appserver.server.sql.tests;

import com.lutris.appserver.server.sql.*;
import java.sql.*;

public class Main {
    
    private static final String DBNAME = "kyle";
    private static final String DRIVER =
        "intersolv.jdbc.sequelink.SequeLinkDriver";
    private static final String CONNECTION =
        "jdbc:sequelink://ra:4004/[Informix];Database=" + DBNAME;
    private static final String USER = "informix";
    private static final String PASSWORD = "irilab!";
    private static final boolean LOGGING = false;
    private static final int MAXSIZE = 2;

    private static String [] product = {
        "toothbrush", "2.75",
        "underware", "4.50",
        "wax", "2.75",
        "ear plugs", "2.75",
        "surfboard", "475.00" };

    /**
     * Main entry point to tests.
     */
    public static void main (String [] args)
        throws Exception
    {
        DBConnection conn;
        String dropTable =
            "drop table PRODUCTS";
        String createTable =
            "create table PRODUCTS " +
            "(PROD_NAME varchar(32), " +
            "PRICE float)";

        ConnectionManager.initializeDatabase(DBNAME,
                                             DRIVER,
                                             CONNECTION,
                                             USER,
                                             PASSWORD,
                                             LOGGING,
                                             MAXSIZE);

        conn = ConnectionManager.allocate(DBNAME);

        //
        // Clean up existing tables from DB.
        //
        try {
            conn.execute(dropTable);
        }
        catch (SQLException ignore) {
            System.out.println("Table doesn't exist - OK");
        }

        //
        // Create a new table
        //
        conn.execute(createTable);

        DBQuery query = new DBQuery(DBNAME);
        DBTransaction transaction;

        //
        // Populate the table w/ data.
        //
        System.out.println();
        transaction = new DBTransaction(DBNAME);
        try {
            System.out.println("Populating table:");
            Product p = new Product();
            for (int idx=0; idx<product.length; idx+=2) {
                p.setName(product[idx]);
                p.setPrice(Float.valueOf(product[idx+1]).floatValue());
                transaction.insert(p);
                System.out.println("adding product: " +
                                   product[idx] + "\t" +
                                   product[idx + 1]);
            }
            transaction.commit();
        }
        catch (SQLException e) {
            System.out.println(e.toString());
            System.out.println("rolling back changes");
        }
        finally {
            transaction.release();
        }
        
        //
        // List the all products in table.
        //
        System.out.println("\nTEST: query all");
        try {
            int total = 0;
            System.out.println("Querying all products from table:");
            ProductQuery pQuery = new ProductQuery();
            query.query(pQuery);
            Product p;
            while ((p = (Product)query.next()) != null) {
                total++;
                System.out.println(p.toString());
            }
            if (total < product.length/2) {
                System.out.println("\n<<<<FAILURE>>>>> ** " +
                                   "unable to query all objects" +
                                   " ** <<<<</FAILURE>>>>>\n");
            }
            query.release();
        }
        catch (SQLException e) {
            System.out.println(e.toString());
        }
        finally {
            query.release();
        }

        //
        // List a subset of products.
        //
        System.out.println("\nTEST: query");
        try {
            boolean ok = false;
            System.out.println("Querying objects of price $2.75:");
            ProductQuery pQuery = new ProductQuery();
            pQuery.setQueryPrice((float)2.75);
            query.query(pQuery);
            Product p;
            while ((p = (Product)query.next()) != null) {
                ok = true;
                System.out.println(p.toString());
            }
            query.release();
            if (!ok) {
                System.out.println("\n<<<<FAILURE>>>>> ** " +
                                   "unable to query objects" +
                                   " ** <<<<</FAILURE>>>>>\n");
            }
        }
        catch (SQLException e) {
            System.out.println(e.toString());
        }
        finally {
            query.release();
        }

        
        //
        // Update fields in the table.
        //
        System.out.println("\nTEST: transaction commit");
        transaction = new DBTransaction(DBNAME);
        try {
            System.out.println("Update all products of price $2.75 " +
                               "to $3.25 - you know, inflation...");
            ProductQuery pQuery = new ProductQuery();
            pQuery.setQueryPrice((float)2.75);
            query.query(pQuery);
            Product p;
            while ((p = (Product)query.next()) != null) {
                System.out.println("Updating: " + p.getName());
                p.setPrice((float)3.25);
                transaction.update(p);
            }
            transaction.commit();

            // Now query the results.
            pQuery.reset();
            pQuery.setQueryPrice((float)2.75);
            query.query(pQuery);
            if (query.next() != null)
                System.out.println("\n<<<<FAILURE>>>>> ** " +
                                   "transaction did not commit" +
                                   " ** <<<<</FAILURE>>>>>\n");
        }
        catch (SQLException e) {
            System.out.println(e.toString());
        }
        finally {
            transaction.release();
            query.release();
        }
        
        //
        // Update fields in the table but rollback the transaction.
        //
        System.out.println("\nTEST: transactio rollback");
        transaction = new DBTransaction(DBNAME);
        try {
            System.out.println("Update all products of price $3.25 " +
                               "to $2.75 but rollback the transaction ...");
            System.out.println("you know, prices never drop.");
            ProductQuery pQuery = new ProductQuery();
            pQuery.setQueryPrice((float)3.25);
            query.query(pQuery);
            Product p;
            while ((p = (Product)query.next()) != null) {
                System.out.println("Updating: " + p.getName());
                p.setPrice((float)2.75);
                transaction.update(p);
            }
            transaction.rollback();

            // Now query the results.
            pQuery.reset();
            pQuery.setQueryPrice((float)2.75);
            query.query(pQuery);
            if (query.next() != null)
                System.out.println("\n<<<<FAILURE>>>>> ** " +
                                   "transaction did not roll back" +
                                   " ** <<<<</FAILURE>>>>>\n");
        }
        catch (SQLException e) {
            System.out.println(e.toString());
        }
        finally {
            transaction.release();
            query.release();
        }

        //
        // Drop the table.
        //
        conn.execute(dropTable);        
        conn.release();
    }

}
