/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ProductQuery.java,v 1.7.8.1 2000/10/19 17:59:06 jasona Exp $
 */





package	com.lutris.appserver.server.sql.tests;

import com.lutris.appserver.server.sql.*;
import java.sql.*;

public class ProductQuery implements Query {
    
    private final String TABLE = "PRODUCTS";
    private final String ALL = "*";
    private boolean qName = false;
    private boolean qPrice = false;
    private String name = null;
    private float price = 0;

    public String getQueryName()
    {
        return name;
    }

    public float getQueryPrice()
    {
        return price;
    }

    public void setQueryName(String name)
    {
        this.name = name;
        this.qName = true;
    }

    public void setQueryPrice(float price)
    {
        this.price = price;
        this.qPrice = true;
    }

    public void reset()
    {
        this.qName = false;
        this.qPrice = false;
    }

    public ResultSet executeQuery(DBConnection conn)
        throws SQLException
    {
        PreparedStatement stmt;

        if (qName && qPrice) {
            String sql =
                "select * from " + TABLE + " " +
                "where PROD_NAME = ? and PRICE = ?";
            stmt = conn.prepareStatement(sql);
            stmt.setString(1, getQueryName());
            stmt.setFloat(2, getQueryPrice());
        }
        else if (qName) {
            String sql =
                "select * from " + TABLE + " " +
                "where PROD_NAME = ?";
            stmt = conn.prepareStatement(sql);
            stmt.setString(1, getQueryName());
        }
        else if (qPrice) {
            String sql =
                "select * from " + TABLE + " " +
                "where PRICE = ?";
            stmt = conn.prepareStatement(sql);
            stmt.setFloat(1, getQueryPrice());
        }
        else {
            String sql =
                "select * from " + TABLE;
            stmt = conn.prepareStatement(sql);
        }
        return conn.executeQuery(stmt, "product query");
    }

    public Object next(ResultSet rs)
        throws SQLException
    {
        if (rs.next()) {
            Product p = new Product();
            p.setName(rs.getString(1));
            p.setPrice(rs.getFloat(2));
            return (Object)p;
        }
        return null;
    }

}
