/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: CachedFilePresentation.java,v 1.10.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation;

import java.io.*;
import com.lutris.util.*;

/**
 * Presentation for file data that is cached in memory.
 */
class CachedFilePresentation implements HttpPresentation {
    private String mimeType;
    private int contentLen = 0;
    private ByteArrayOutputStream fileData;

    private static final int READ_BUFFER_SIZE = 4096;

    /**
     * Construct a new static file presentation, given an input stream.
     */
    protected CachedFilePresentation(ClassLoader classLoader,
                                     String urlPath,
                                     String fileMimeType) 
            throws FilePresentationException {
        mimeType = fileMimeType;
        InputStream input = classLoader.getResourceAsStream(urlPath);
        if (input == null) {
            throw new FilePresentationException
                (new FileNotFoundException ("File \"" + urlPath
                                            + "\" not found on application class path"));
        }
            
        try {
            // Read in file.
            try {
                byte [] buffer = new byte[READ_BUFFER_SIZE];
                fileData = new ByteArrayOutputStream();
                while (true) {
                    int readLen = input.read(buffer, 0, READ_BUFFER_SIZE);
                    if (readLen < 0)
                        break;
                    fileData.write(buffer, 0, readLen);
                    contentLen += readLen;
                }
            } finally {
                input.close();
            }
        } catch (Exception except) {
            throw new FilePresentationException(except);
        }
    }
    
    /**
     * Entry point for static file presenation.
     */
    public void run(HttpPresentationComms comms)
            throws FilePresentationException {
        try {
            comms.response.setContentLength(contentLen);
            comms.response.setContentType(mimeType);

            if (!comms.request.getMethod().equals("HEAD")) {
                fileData.writeTo(comms.response.getOutputStream());
            }
        } catch (Exception except) {
            throw new FilePresentationException(except);
        }
    }
}
