/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HttpPresentationComms.java,v 1.21.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.session.*;
import org.enhydra.xml.xmlc.XMLCFactory;


/**
 * Object passed to presentation objects that contains HTTP and Presentation
 * Manager access and control objects.
 */
public class HttpPresentationComms {
    /**
     * Object used to access HTTP request data.
     */
    public final HttpPresentationRequest request;

    /**
     * Object used to generate HTTP responses.
     */
    public final HttpPresentationResponse response;

    /**
     * Application object that this presentation is associated with.
     * If null, no application is associated.
     */
    public final Application application;

    /**
     * Session object that this presentation is associated with.
     * If null, no session has been associated by the application.
     */
    public Session session;

    /**
     * Session data object.  This is a handy reference to 
     * <CODE>session.getSessionData()</CODE>
     */
    public SessionData sessionData;

    /**
     * If an exception/error occured, this field is set to the exception
     *  object. This is only set for and used by `ErrorHandler' presentations.
     */
    public Throwable exception;

    /**
     * Reference to the XMLC factory object contained in the application.
     */
    public XMLCFactory xmlcFactory;

    /**
     * Construct an object, setting all of its fields.
     *
     * @param request Object used to access HTTP request data.
     * @param response Object used to generate HTTP responses.
     * @param application Application object that this presentation is
     * associated with or null if no application is associated.
     */
    protected HttpPresentationComms (HttpPresentationRequest request,
                                     HttpPresentationResponse response,
                                     Application application) {
        this.request = request;
        this.response = response;
        this.application = application;
        this.xmlcFactory = application.getXMLCFactory();
    }

}
