/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HttpPresentationOutputStream.java,v 1.13.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation;

import java.io.IOException;
import java.io.OutputStream;


/**
 * HTTP request output stream writter.
 */
public abstract class HttpPresentationOutputStream extends OutputStream {

    //FIX:  Need MIME-write functions.

    /**
     * Prints the string provided.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void print(String s) throws IOException;

    /**
     * Prints the boolean provided.
     *
     * @exception IOException if an I/O error has occurred.
     */
    public abstract void print(boolean b) throws IOException;

    /*
     * Prints the character provided.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void print(char c) throws IOException;

    /**
     * Prints the integer provided.
     *
     * @exception IOException if an I/O error has occurred
     */  
    public abstract void print(int i) throws IOException;
 
    /**
     * Prints the long provided.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void print(long l) throws IOException;

    /**
     * Prints the float provided.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void print(float f) throws IOException;

    /**
     * Prints the double provided.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void print(double d) throws IOException;

    /**
     * Prints a CRLF.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void println() throws IOException;

    /**
     * Prints the string provided, followed by a CRLF.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void println(String s) throws IOException;

    /**
     * Prints the boolean provided, followed by a CRLF.
     *
     * @exception IOException if an I/O error has occurred.
     */
    public abstract void println(boolean b) throws IOException;

    /**
     * Prints the character provided, followed by a CRLF.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void println(char c) throws IOException;

    /**
     * Prints the integer provided, followed by a CRLF.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void println(int i) throws IOException;

    /**  
     * Prints the long provided, followed by a CRLF.
     *
     * @exception IOException if an I/O error has occurred
     */  
    public abstract void println(long l) throws IOException;

    /**
     * Prints the float provided, followed by a CRLF.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void println(float f) throws IOException;

    /**
     * Prints the double provided, followed by a CRLF.
     *
     * @exception IOException if an I/O error has occurred
     */
    public abstract void println(double d) throws IOException;

    /*
     * We disallow closing the output stream, but don't export the
     * doc on this function.
     */
    public void close() throws IOException {
        throw new IOException("HttpPresentationOutputStream may not be closed");
    }
}
