/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HttpUtil.java,v 1.16.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation;

import com.lutris.util.*;
import com.lutris.http.*;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;


/**
 * Class containing various static methods that are useful in implementing
 * presentations.
 */
public class HttpUtil {

    /**
     * Private constructor, prevent instantiation.
     */
    private HttpUtil() {}

    /**
     * Convert a HTTP query string object to a keyword/value table.
     * 
     * @param query Hash table containing the parsed key/value pairs.
     * @return A keyword/value table with the parsed string.
     */
    public static KeywordValueTable buildQueryTable(Hashtable query)
        throws KeywordValueException {
        Enumeration arguments = query.keys();
        KeywordValueTable table = new KeywordValueTable();
        while (arguments.hasMoreElements()) {
            String key = (String)(arguments.nextElement());
            table.set(key, query.get(key));
        }
        return table;
    }

    /**
     * Convert a HTTP query string to a keyword/value table.
     * 
     * @param queryString Query string.
     * @return A keyword/value table with the parsed string.
     * FIX: better description of format.
     */
    public static KeywordValueTable buildQueryTable(String queryString)
        throws KeywordValueException {
	return buildQueryTable(HttpUtils.parseQueryString(queryString));
    }

    /**
     * Read in post data return it as a string.  If content is not
     * "application/x-www-form-urlencoded", return an empty string.  FIX:
     * change mime type handling as part of form upload, should generate an
     * error if we get something we can't handle.  this should be part of a
     * module to handle input restructured.
     */
    private static String getPostData(HttpPresentationRequest httpRequest)
            throws HttpPresentationException {
        String cType = httpRequest.getContentType().toLowerCase();
        if (cType.indexOf("application/x-www-form-urlencoded") < 0) {
            return "";
        } else {
            byte[] buffer = new byte[httpRequest.getContentLength()];
            try {
		int count = 0;
		int c = 0;
		boolean done = false;
		while ((count < buffer.length) && !done) {
		    c = httpRequest.getInputStream().read(buffer,
				    count, buffer.length - count);
                    if (c == -1) {
                        done = true;
                    } else {
                        count += c;
                    }
		}
            } catch (IOException except) {
                throw new HttpPresentationException(except);
            }
	    return new String(buffer);
        }
    }

    /**
     * Get the query string for the request and save it in the page data.
     * If this is a post, read the query string first.
     *
     * @param request HTTP request data.
     * @exception HttpPresentationException
     *   If an error occurs accessing the request.
     */
    public static String getQueryFromRequest(HttpPresentationRequest httpRequest)
        throws HttpPresentationException {

        String queryString;
        if (httpRequest.getMethod().equals("POST")) {
            // POST method
            queryString = getPostData(httpRequest);
        } else {
            // GET method
            queryString = httpRequest.getQueryString();
        }
        if (queryString == null) {
            queryString = "";
        }
        return queryString;
    }
}
