/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: PageRedirect.java,v 1.19.2.3.4.2 2000/11/07 22:51:27 peterj Exp $
 *-----------------------------------------------------------------------------
 */





package com.lutris.appserver.server.httpPresentation;

import java.net.URL;
import java.net.URLEncoder;
import java.net.MalformedURLException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.*;
import javax.servlet.http.*;
import javax.servlet.*;

import com.lutris.logging.*;
import com.lutris.appserver.server.StandardAppUtil;

/**
 * Static methods for processing a page redirect.
 *
 * @version	$Revision: 1.19.2.3.4.2 $
 * @author	Mark Diekhans
 * @since	Jolt1.3
 */
class PageRedirect {
    /**
     * Process a page redirection.
     *
     * @return Path of the presentation object to invoke for a server-side
     *  redirect or null if a redirect response has been sent to the
     *  client. NOTE: Currently always returns null.
     */
    protected static String handler(HttpPresentationComms comms,
                                    String presObjPath,
                                    PageRedirectException redirectExcept,
                                    LogChannel logChannel,
                                    long requestId)
            throws MalformedURLException,
		   HttpPresentationException,
		   IOException {

        boolean debug = logChannel.isEnabled(Logger.DEBUG);

        //FIX: Break into two functions; one for client., one for server.
        if (debug)
            logChannel.write(Logger.DEBUG,  "RID:" + requestId + ": "
                         + redirectExcept.getClass().getName()
                         + ": " + presObjPath + " => "
                         + redirectExcept.getUrl());
        String url = redirectExcept.getUrl();
        if (redirectExcept instanceof ClientPageRedirectException) {
            return clientSideRedirect(url,comms,logChannel,requestId);
        } else {
            RequestDispatcher rd = comms.request.getHttpServletRequest().getRequestDispatcher(url);
            try {
                rd.forward(comms.request.getHttpServletRequest(),
                        comms.response.getHttpServletResponse());

            } catch (Exception ex) {
                logChannel.write(Logger.WARNING,  "Server redirect to \"" +
                        url + "\" failed. Reason: \""
                        + ex.getLocalizedMessage() + "\" reattempt as client redirect.");

                 try {
                    clientSideRedirect(url,comms,logChannel,requestId);
                 } catch (Exception e) {
                    throw new HttpPresentationException(e);
                 }
            }
            return null;

        }
    }



    private static String clientSideRedirect(String url,
                                            HttpPresentationComms comms,
                                            LogChannel logChannel,
                                            long requestId)
        throws 	   HttpPresentationException {

            boolean debug = logChannel.isEnabled(Logger.DEBUG);
            url = makeAbsolute(url,comms.request.getHttpServletRequest());
            if (comms.request.isRequestedSessionIdFromUrl() &&
                    StandardAppUtil.pointsToPO(url)) {
                url = StandardAppUtil.encodeUrl(url, comms.session.getSessionKey());

                if (debug)
                    logChannel.write(Logger.DEBUG,  "RID:" + requestId + ": "
                                 + "Adding session ID to new URL: "
                                 + url);
            } else {
                logChannel.write(Logger.DEBUG,  "RID:" + requestId + ": "
                                 + "No session ID for URL: "
                                 + url);
            }
            comms.response.setHeader("Location", url);
            comms.response.setStatus(HttpPresentationResponse.SC_MOVED_TEMPORARILY,
                                     "Redirected to new location.");
            comms.response.flush();
            return null;

    }

    private static String makeAbsolute(String location,
            HttpServletRequest request) {
        URL url = null;
        try {
	    // Try making a URL out of the location
	    // Throws an exception if the location is relative
            url = new URL(location);
	}
	catch (MalformedURLException e) {
	    String requrl = HttpUtils.getRequestURL(request).toString();
	    try {
	        url = new URL(new URL(requrl), location);
	    }
	    catch (MalformedURLException ignored) {
	        // Give up
	        return location;
	    }
	}
        return url.toString();
    }
}
