/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PageRedirectException.java,v 1.20.14.1 2000/10/19 17:59:06 jasona Exp $
 */





package com.lutris.appserver.server.httpPresentation;
import com.lutris.http.*;
import com.lutris.util.*;
import java.net.URLEncoder;


/**
 * Base class of page redirection classes.  It is a subclass of Error
 * rather than exception, as we only want the presentation manager
 * to catch it.
 */
abstract public class PageRedirectException extends Error {
    /**
     * URL that is the target of the redirect.
     */
    private String url;

    /**
     * Are there any arguments associated with the URL?
     */
    private boolean urlHasArgs = false;

    /**
     * Constructor a redirect, saving the URL..
     *
     * @param url The url where the request should be redirected.
     *  It may have arguments encoded and/or they may be added with
     *  <CODE>addArgument</CODE>.
     * @see PageRedirectException#addArgument
     */
    protected PageRedirectException(String url) {
        this.url = url;
        urlHasArgs = (url.lastIndexOf('?') >= 0);
    }
 
    /**
     * Add an argument to the URL.
     *
     * @param name Argument name.
     * @param value Argument value.
     */
    public void addArgument(String name,
                            String value) {
        StringBuffer argStr = new StringBuffer();
        if (urlHasArgs) {
            argStr.append("&");
        } else {
            argStr.append("?");
        }
        argStr.append(URLEncoder.encode(name));
        if (value != null) {
            argStr.append("=");
            argStr.append(URLEncoder.encode(value));
        }
        url += argStr;
        urlHasArgs = true;
    }
 
   /**
     * Get the redirection URL associated with this redirect.
     *
     * @return The URL string. 
     */
    public String getUrl() {
        return url;
    }
}
