#!/usr/local/bin/tcl

#
# http-get url outbase ?method?
#
# Simple script for evaluating the results of an HTTP request
# Will create files $outbase.head $outbase.body.  Method defaults
# to get.
#

# Taken from Tcl 8.0 http.tcl
proc parseUrl url {
    if {! [regexp -nocase {^(http://)?([^/:]+)(:([0-9]+))?(/.*)?$} $url \
            {} proto host y port srvurl]} {
        error "Unsupported URL: $url"
    }
    return [list $host $port $srvurl]
}

proc httpGet {url outbase {method GET}} {
    lassign [parseUrl $url] host port file
    if [lempty $port] {
        set port http
    }
    set s [socket $host $port]
    fconfigure $s -translation binary
    
    puts $s "$method $file HTTP/1.0\r\n\r"
    flush $s
    set outhead [open $outbase.head w]
    fconfigure $outhead -translation binary
    while {[gets $s line] >= 0} {
        puts $outhead $line
        if [lempty $line] break
    }
    close $outhead
    set outbody [open $outbase.body w]
    fconfigure $outbody -translation binary
    fcopy $s $outbody
    close $outbody
    close $s
}


if {$argc < 2 || $argc > 3} {
    puts stderr {wrong # args: http-get url outbase ?method?}
    exit 
}

lassign $argv url outbase method
if [lempty $method] {
    set method GET
}
set method [string toupper $method]

httpGet $url $outbase $method
