/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PagedSession.java,v 1.3.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session;

import java.net.*;
import com.lutris.util.*;
import com.lutris.appserver.server.user.User;
import com.lutris.appserver.server.*;
import java.io.Serializable;

/**
 * PagedSession requires that all session data and the user
 * associated with the session be serializable.
 *
 * @version $Revision: 1.3.14.1 $
 * @author  Kyle Clark
 * @see PagedSessionHome
 */
public class PagedSession extends BasicSession {

    // Need the following constructor in order to serialize.
    public PagedSession() {
    }
    
    /**
     * Construct a new session.  Only called by
     * <CODE>PagedSessionHome</CODE>.
     *
     * @param sessionManager The session manager that will manage this session.
     * @param sessionKey The unique session key associated with the session.
     */
    protected PagedSession(StandardSessionManager sessionManager,
                           String sessionKey) {
        super(sessionManager, sessionKey, new PagedSessionData());
    }

    /**
     * Ensures that the user associated with the session is serializable.<p>
     * 
     * @param user
     *   the user object to associate with the session.
     * @exception SessionException
     *   if the user cannot be associated with the session.
     */
    public void setUser(User user) throws SessionException {
        if (!(user instanceof Serializable)) {
            throw new SessionException("Paged session user must be serializable.");
        }
        super.setUser(user);
    }

    /**
     * Called before the session is paged to disk.
     * Returns the transient data that should be restored
     * when the session is read from disk.
     *
     * @param data 
     *   array of transient data objects.
     * @see #restoreTransientData
     */
    Object [] getTransientData() {
        Object [] transientData = new Object[1];
        transientData[0] = sessionManager;
        return transientData;
    }

    /**
     * Called when the session is paged back into memory.
     * Allows it to restore references to the transient
     * data associated with the session.
     *
     * @param data 
     *   array of transient data objects.
     * @see #getTransientData
     */
    void restoreTransientData(Object [] data) {
        this.sessionManager = (StandardSessionManager)data[0];
    }

}




