/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PagedSessionThresholdTimer.java,v 1.2.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session;

import com.lutris.appserver.server.*;
import com.lutris.logging.*;

/**
 * This thread sleeps in the background, waking up periodically to
 * check for inactive sessions and to page out any session that
 * has been inactive for too long.
 *
 * @version	$Revision: 1.2.14.1 $
 * @author	Kyle Clark
 */
public class PagedSessionThresholdTimer extends Thread {

    private PagedSessionHome home;
    private Application app;    // The application context.
    private boolean done = false;
    
    /*
     * @param home the paged session home.
     * @param app the application context.
     * @param scanIntervalSec Time interval, in seconds, to scan for idle
     *          session.
     */ 
    public PagedSessionThresholdTimer(PagedSessionHome home,
                                      Application app) {
        this.home = home;
        this.app = app;
	setDaemon(true);
    }
    
    /**
     * Main loop.
     */
    public void run() {
        Enhydra.register(app);
        long scanInterval = 0;
	while (!done) {
            try {
                scanInterval = home.checkPassiveSessions();
            } catch (Exception e) {
                Enhydra.getLogChannel().write(Logger.ALERT,
                                              "Failed to check idle sessions: " + e, e);
            }
	    try {
		sleep(scanInterval);
	    } catch (InterruptedException e) {
                // Ignore.
            }
	}
        Enhydra.unRegister();
    }

    /**
     * Shutdown the thread associated with this object.
     */
    public void shutdown() {
        done = true;
    }
}

