/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SessionThread.java,v 1.2.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session;


/**
 * Represents a combination of a thread and a session key.
 * A session must be associated with a thread of execution
 * before a reference to the session can be obtained by the
 * application.  This object represents the existing thread
 * to session association.
 *
 * @version $Revision: 1.2.14.1 $
 * @author  Kyle Clark
 */
public class SessionThread {

    public Thread thread;
    public String sessionKey;

    /**
     * Creates an instance of a session-thread association.
     *
     * @param thread the thread to associate with a session.
     * @param sessionKey the session key.
     */
    public SessionThread(Thread thread, String sessionKey) {
        this.thread = thread;
        this.sessionKey = sessionKey;
    }

    /**
     * Compares equality of two objects.
     *
     * @param sessionThread the reference object with which
     *   compare.
     */
    public boolean equals(Object sessionThread) {
        if (!(sessionThread instanceof SessionThread)) {
            return false;
        }
        SessionThread s = (SessionThread)sessionThread;
        return (thread.equals(s.thread)
                && sessionKey.equals(s.sessionKey));
    }

    /**
     * Returns string representation.
     */
    public String toString() {
        return thread.toString() + "-" + sessionKey;
    }

    /**
     * Returns the hashcode for this object.
     */
    public int hashCode() {
        return sessionKey.hashCode();
    }
}

