/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: StandardSession.java,v 1.26.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.user.*;
import java.net.*;
import javax.servlet.http.HttpSession;

/**
 * Defines the interface for the Session object expected
 * by the StandardSessionManager.
 * 
 * @version	$Revision: 1.26.14.1 $
 * @author	Kyle Clark
 */
public interface StandardSession extends Session {

    /**
     * Set the "last used" timestamp to the current time, resetting the
     * idle period.
     */
    public void touch();

    /**
     * Obtain the creation time for this object.  The time is in
     * milliseconds since Midnight, Jan 1, 1970 (epoch).
     *
     * @return The creation time since epoch for this object.
     */
    public long getTimeCreated();

    /**
     * Obtain the time of last use for this object.  The time is in
     * milliseconds since Midnight, Jan 1, 1970 (epoch).
     *
     * @return The time of last use since epoch for this object.
     */
    public long getTimeLastUsed();

    /**
     * Obtain the time of expiry for this object.  The time is in
     * milliseconds since Midnight, Jan 1, 1970 (epoch).
     * Returns zero (or negative) if there is no expiration date.
     * Note: this is a maximum lifespan for the session, regardless of activity.
     *
     * @return The time of expiry since epoch for this object.
     */
    public long getTimeExpires();

    /**
     * Set the time of expiry for this object.  The time is in
     * milliseconds since Midnight, Jan 1, 1970 (epoch).
     * Set this value to zero (or less) to indicate that the session does not
     * have a maximum age.
     *
     * @param timeLastUsed The time of expiry since epoch.
     */
    public void setTimeExpires(long timeExpires);

    /**
     * Obtain the maximum idle time for this object. Zero (or negative)
     * indicates that sessions may be idle indefinetly.
     *
     * @return The maximum number of milliseconds this session
     *      may be idle.
     */
    public long getMaxIdleTime();

    /**
     * Set the maximum idle time for this object. Set this to zero
     * (or negative) to disable idle checking.
     *
     * @param maxIdleTime The maximum number of milliseconds this
     *  session may be idle, or zero (or negative) to allow sessions to be idle
     *  indefinetly.
     */
    public void setMaxIdleTime(long maxIdleTime);

    /**
     * Obtain the maximum idle time when a <CODE>User</CODE> object
     * is not associated with the session. Zero (or negative) indicates that
     * sessions may be idle indefinetly.
     *
     * @return The maximum number of milliseconds this session
     *      may be idle.
     */
    public long getMaxNoUserIdleTime();

    /**
     * set the maximum idle time when a <CODE>User</CODE> object
     * is not associated with the session. Set this to zero (or negative) to
     * disable idle checking.
     *
     * @param maxIdleTime The maximum number of milliseconds this
     *      session may be idle.
     */
    public void setMaxNoUserIdleTime(long maxIdleTime);

    /**
     * Expose the HttpSession interface
     */
    public HttpSession  getHttpSession() ;


}



