/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: StandardSessionIdleTimer.java,v 1.19.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session;

import com.lutris.appserver.server.*;
import com.lutris.logging.*;
import java.security.*;
import java.io.*;
import java.util.*;
import com.lutris.util.*;

/**
 * The idle timer thread for <CODE>StandardSessionManager</CODE>.
 * This thread sleeps in the background, waking up periodically to
 * check for inactive sessions and to terminate any session that has been
 * inactive for too long.
 *
 * @version	$Revision: 1.19.14.1 $
 * @author	John Marco
 * @author	Shawn McMurdo
 */
public class StandardSessionIdleTimer extends Thread {

    private StandardSessionIdleHandler idleHandler;
    private long scanInterval;   // In milliseconds.
    private Application app;    // The application context.

    /*
     * Constructor an idle timer object.  Sets the
     * internal session manager reference and check period,
     * but doesn't start the thread.
     *
     * @param manager The session manager to be checked periodically for
     * 		idle sessions.
     * @param app the application context.
     * @param scanIntervalSec Time interval, in seconds, to scan for idle
     *          session.
     */
    public StandardSessionIdleTimer(StandardSessionIdleHandler manager,
                                    Application app,
                                    long scanIntervalSec) {
        idleHandler = manager;
        this.app = app;
	// Convert idle timer scanning interval milliseconds.
	scanInterval = scanIntervalSec * 1000;
        setPriority(MIN_PRIORITY); // For performance
	setDaemon(true); // Don't require this thread to exit.
    }

    /**
     * The main code body of the Idle Timer Thread.  Enters an endless
     * loop that sleeps for a configurable period, periodically waking
     * up to check the Session Manager for idle sessions.
     */
    public void run() {
        if (app != null) {
          Enhydra.register(app);
        }
	while (true) {
	    try {
		sleep(scanInterval);
	    } catch (InterruptedException e) {
                // Ignore.
            }
            try {
                idleHandler.cleanUpIdleSessions();
            } catch (Exception e) {
                LogChannel log = Enhydra.getLogChannel();
                log.write(Logger.ALERT, "Failed to clean up idle sessions: " + e);
                // TODO - print to log
                e.printStackTrace();
            }
	}
        // TODO - Enhydra.unregister();
    }

    /**
     * Shutdown the thread associated with this object.
     */
    public void shutdown() {
        stop();
    }
}

