/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PersistentSession.java,v 1.4.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session.persistent;

import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.user.User;
import com.lutris.appserver.server.*;
import java.io.Serializable;

/**
 * A serializable version of BasicSession that
 * can be written to persistent store.
 *
 * @version $Revision: 1.4.14.1 $
 * @author  Kyle Clark
 */
public class PersistentSession extends PagedSession {

    transient PersistentSessionHome sessionHome;

    /**
     * Need the following constructor in order to serialize?
     */
    public PersistentSession() {
    }
    
    /**
     * Construct a new session.  Only called by <CODE>PersistentSessionHome</CODE>.
     *
     * @param sessionManager The session manager that will manager this session.
     * @param sessionKey The unique session key associated with the session.
     */
    protected PersistentSession(StandardSessionManager sessionManager,
                                String sessionKey,
                                PersistentSessionHome sessionHome) {
        super(sessionManager, sessionKey);
        this.sessionHome = sessionHome;
    }

    /**
     * Ensures that the user associated with the session is serializable.<p>
     * 
     * @param user
     *   the user object to associate with the session.
     * @exception SessionException
     *   if the user cannot be associated with the session.
     */
    public void setUser(User user) throws SessionException {
        if (!(user instanceof Serializable)) {
            throw new SessionException("Persistent session user must be serializable.");
        }
        super.setUser(user);
        // Need to synchronize any changes to user since
        // the session user table is maintained in the database.
        DBUtil.dbUpdate(this, sessionHome.getDatabaseName());
    }

    /**
     * Called when the session is read back into memory.
     * Restores the (transient) session manager.
     *
     * @param mgr
     *   the session manager.
     */
    void restoreSessionManager(StandardSessionManager mgr) {
        this.sessionManager = mgr;
    }

    /**
     * Called when the session is read back into memory.
     * Restores the (transient) session manager.
     *
     * @param mgr
     *   the session manager.
     */
    void restoreSessionHome(PersistentSessionHome home) {
        this.sessionHome = home;
    }

}




