/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: SizeQuery.java,v 1.2.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session.persistent;

import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.session.*;
import java.util.*;
import java.sql.*;

/**
 * Queries the total number of sessions in the database.
 *
 * @version $Revision: 1.2.14.1 $
 * @author  Kyle Clark
 */
class SizeQuery implements Query {

    private String sessionKey;
    private static final Integer zero = new Integer(0);

    SizeQuery() {
    }

    /**
     * Method to query objects from the database.
     *
     * @param conn Handle to database connection.
     * @exception java.sql.SQLException If a database access error occurs. 
     */
    public ResultSet executeQuery(DBConnection conn)
        throws SQLException {
        String sql = "select count (*) totalSessions from "
            + PersistentSessionHome.dbTableName;
        PreparedStatement stmt = conn.prepareStatement(sql);
        return conn.executeQuery(stmt, sql);
    }

    /**
     * Method that returns the number of sessions.
     *
     * @param rs
     *   result set from which the number of sessions
     *   will be instantiated.
     * @return an Integer representing the number of sessions.
     * @exception SQLException If a database access error occurs. 
     */
    public Object next(ResultSet rs) throws SQLException {
        if (rs.next()) {
            return new Integer(rs.getInt("totalSessions"));
        }
        return zero;
    }

}
