/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: UserSessionsQuery.java,v 1.3.14.1 2000/10/19 17:59:06 jasona Exp $
 */

package com.lutris.appserver.server.session.persistent;

import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.user.User;
import java.util.*;
import java.sql.*;

/**
 * Queries the sessions associated with a user.
 *
 * @version $Revision: 1.3.14.1 $
 * @author  Kyle Clark
 */
class UserSessionsQuery implements Query {

    private User user;

    /**
     * @param sessionKey
     *   the key for the session that should be retrieved.
     */
    UserSessionsQuery(User user) {
        this.user = user;
    }

    /**
     * Method to query the user's session keys.
     *
     * @param conn Handle to database connection.
     * @exception java.sql.SQLException If a database access error occurs.
     */
    public ResultSet executeQuery(DBConnection conn)
        throws SQLException {
        String sql = "select sessionKey from "
            + PersistentSessionHome.dbTableName
            + " where userName = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setString(1, user.getName());
        return conn.executeQuery(stmt, sql);
    }

    /**
     * Returns an enumeration of the session keys associated with
     * the user.
     *
     * @return Enumeration of session keys.
     * @param rs result set from which sesison keys enumeration
     *   will be instantiated.
     * @return an Enumration of session keys.
     * @exception SQLException If a database access error occurs.
     */
    public Object next(ResultSet rs) throws SQLException {
        Vector v = new Vector();
        while (rs.next()) {
            v.addElement(rs.getString("sessionKey"));
        }
        return v.elements();
    }

}
