/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Welcome.java,v 1.1.14.1 2000/10/19 17:59:07 jasona Exp $
 */

package com.lutris.appserver.welcome.presentation;

import com.lutris.xml.xmlc.*;
import com.lutris.xml.xmlc.html.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.lutris.util.*;
import java.io.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

/**
 * Output the welcome page.
 */
public class Welcome implements HttpPresentation {
    /*
     * Used to keep track of the total number of times this page has been
     * served.
     */
    private static int counter = 0;

    /*
     * Set counters in the page.
     */
    public void setCounters(Session session,
                            WelcomeHTML htmlObj)
        throws KeywordValueException {
        /*
         * Fill the tag that holds the total number of times this page has
         * been served, to all users. We use a static variable to hold this
         * information.
         */
        counter++;
        htmlObj.setTextTotalHits(Integer.toString(counter));

        /*
         * Fill in the tag that holds the number of times this user
         * has been served this page. Since this is a per-user thing, 
         * we use session data. The first time a user loads this page,
         * the session data will not contain the counter, treat this
         * as if the counter was zero.
         */
        SessionData sessionData = session.getSessionData();
        String userCounterString = sessionData.getString("counter");
        int userCounter = 0;
        if (userCounterString != null) {
            userCounter = Integer.parseInt(userCounterString);
        }
        userCounter++;
        userCounterString = Integer.toString(userCounter);
        sessionData.set("counter", userCounterString);

        htmlObj.setTextUserHits(userCounterString);
    }

    /**
     * Entry.
     */
    public void run(HttpPresentationComms comms)
        throws HttpPresentationException {
        try {
            WelcomeHTML htmlObj
                = (WelcomeHTML)comms.xmlcFactory.create(WelcomeHTML.class);
            setCounters(comms.session, htmlObj);
            comms.response.writeHTML(htmlObj);
        } catch (KeywordValueException except) {
            throw new HttpPresentationException(except);
        }
    }   
}
