/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HttpUtilsTests.java,v 1.6.10.1 2000/10/19 17:59:07 jasona Exp $
 */





package com.lutris.http.tests;
import com.lutris.http.*;
import java.io.*;
import java.util.*;

/**
 * Class that tests the HttpUtilsTests object.
 * TODO: more robust tests are needed.
 *
 * @author John Marco
 */
public class HttpUtilsTests {
    
    private static final String TestQuery1 =
	"aaa=Hello&bbb=world&ccc=%21";
    
    private static final String TestQuery2 =
	"sp+ace=has+space&field2=field%20two";

    private static final String TestQuery3 =
	"field1=good&field2=&field3=alsogood&field3=dupfield";
    
    private static final String EmptyQuery = 
	"";

    private static final String BadQuery1 =
	"&foo=bar";

    private static final String BadQuery2 =
	"foo=bar&bad&oof=rab";

    private static final String BadQuery3 =
	"foo=bar&&oof=rab";
    
    private static final String URLPrefix = 
	"http://www.foobar.com/cgi-bin/foo.po?";
    
    static
    Hashtable testGood(String note, String qs, boolean asStream)
    throws Exception
    {
	beginTest(note, qs);
	try {
	    if (asStream) {
		int i;
		char[] chars = qs.toCharArray();
		byte[] bytes = new byte[qs.length()];
		for (i=0; i<qs.length(); i++) {
		    int ch = chars[i];
		    if (ch >= 128) ch -= 256;
		    bytes[i] = (byte) ch;
		}
		InputStream in = new ByteArrayInputStream(bytes);
		return HttpUtils.parsePostData(qs.length(), in);
	    } else {
		return HttpUtils.parseQueryString(qs);
	    }
	}
	catch (IllegalArgumentException e) {
	    failTest(note, e.toString());
	}
	failTest(note, "????");
	return null;
    }

    static
    void testBad(String note, String qs, boolean asStream)
    throws Exception
    {
	boolean failed = false; // We expect exception here.
	beginTest(note, qs);
	Hashtable ht;
	try {
	    if (asStream) {
		int i;
		char[] chars = qs.toCharArray();
		byte[] bytes = new byte[qs.length()];
		for (i=0; i<qs.length(); i++) {
		    int ch = chars[i];
		    if (ch >= 128) ch -= 256;
		    bytes[i] = (byte) ch;
		}
		InputStream in = new ByteArrayInputStream(bytes);
		ht = HttpUtils.parsePostData(qs.length(), in);
	    } else {
		ht = HttpUtils.parseQueryString(qs);
	    }
	}
	catch (IllegalArgumentException e) {
	    goodTest(note);
	    return;
	}
	failTest(note, "Expected IllegalArgumentException");
    }
    
    static
    void testFind(String note, Hashtable ht,
			String key, Object expectedValue)
    throws Exception
    {
	beginTest(note, "Find '"+key+"'");
	if (expectedValue == null) {
	    Object gotValue = ht.get(key);
	    if (gotValue != null) {
		failTest(note, "Expected (null) for '" + key + "'");
	    }
	} else if (expectedValue instanceof String[]) {
	    String[] evs = (String[]) expectedValue;
	    Object gotValue = ht.get(key);
	    if (gotValue == null) {
		failTest(note, "UnExpected (null) for '" + key + "'");
	    }
	    if (gotValue instanceof String[]) {
		String[] gvs = (String[]) gotValue;
		if (evs.length != gvs.length) {
		    failTest(note, "Array length wrong for '"+key+"'");
		}
		for (int i=0; i<evs.length; i++) {
		    if ((evs[i] == null)||(gvs[i] == null)) {
			if (evs[i] != gvs[i]) {
			    failTest(note, "Array element "+i+" mismatch.");
			}
		    } else if (!(evs[i].equals(gvs[i]))) {
			failTest(note, "Array element "+i+" mismatch.");
		    }
		}
	    } else {
		failTest(note, "Expected String[] for '" + key + "'" +
				"Got '" + gotValue.getClass().getName() + "'");
	    }
	} else if (expectedValue instanceof String) {
	    String ev = (String)expectedValue;
	    Object gval = ht.get(key);
	    if (gval instanceof String) {
		String gv = (String) gval;
		if ((gv == null)||(ev==null)) {
		    if (gv != ev) {
			failTest(note, "Expected both null for '"+key+"'");
		    }
		} else if (!(ev.equals(gv))) {
		    failTest(note, "Expected '"+ev+"' got '"+gv+"'");
		}
	    } else {
		failTest(note, "Expected type String for '"+key+"'" +
				"Got '" + gval.getClass().getName() + "'");
	    }
	} else {
	    failTest("TEST BUG", "Unknown expected value type.");
	}
	goodTest(note);
    }
    
    static
    void beginTest(String name, String reason)
    {
	System.out.println("TEST: " + name + "  --  " + reason);
    }

    private static
    void goodTest(String name)
    {
	System.out.println("SUCCEED: " + name);
    }

    private static
    void failTest(String name, String reason)
    throws Exception
    {
	Exception e = new Exception("FAIL: " + name + ": " + reason);
	throw (Exception)(e.fillInStackTrace());
    }
    
    private static final String[] strings = { "alsogood", "dupfield" };

    static
    void testGoodInput(boolean isStr, boolean url)
    throws Exception
    {
	Hashtable ht;
	String urlStr = url ? "-url" : "";
	String strStr = isStr ? "-str" : "";
	String tn = "Good" + urlStr + strStr;
	String urlPrefix = url ? URLPrefix : "";

	ht = testGood(tn+"1", urlPrefix + TestQuery1, isStr);
        testFind(tn+"1a", ht, "aaa", "Hello");	
        testFind(tn+"1b", ht, "bbb", "world");	
        testFind(tn+"1c", ht, "ccc", "!");	
        testFind(tn+"1d", ht, "Idontexist", null);	
	
	ht = testGood(tn+"2", urlPrefix + TestQuery2, isStr);
        testFind(tn+"2a", ht, "sp ace", "has space");	
        testFind(tn+"2b", ht, "field2", "field two");	
        testFind(tn+"2c", ht, "nobody", null);	

	ht = testGood(tn+"3", urlPrefix + TestQuery3, isStr);
        testFind(tn+"3a", ht, "field1", "good");	
        testFind(tn+"3b", ht, "field2", "");	
        testFind(tn+"3c", ht, "field3", strings);	
        testFind(tn+"3d", ht, "nobody3k", null);	
	
	ht = testGood(tn+"4", urlPrefix + EmptyQuery, isStr);
        testFind(tn+"4a", ht, "nobodyX", null);	
    }

    static
    void testBadInput(boolean isStr, boolean url)
    throws Exception
    {
	String urlStr = url ? "-url" : "";
	String strStr = isStr ? "-str" : "";
	String tn = "Bad" + urlStr + strStr;
	String urlPrefix = url ? URLPrefix : "";

	testBad(tn+"1", urlPrefix + BadQuery1, isStr);
	
	testBad(tn+"2", urlPrefix + BadQuery2, isStr);

	//testBad(tn+"3", urlPrefix + BadQuery3, isStr);
	System.out.println("WARNING: '&&' is accepted due to bug in StringTokenizer");
    }
    
    static
    void goodInputTests()
    throws Exception
    {
	testGoodInput(true, true);
	testGoodInput(true, false);
	testGoodInput(false, true);
	testGoodInput(false, false);
    }

    static
    void badInputTests()
    throws Exception
    {
	testBadInput(true, true);
	testBadInput(true, false);
	testBadInput(false, true);
	testBadInput(false, false);
    }
    
    
    public static void runTests()
    {
	boolean testfailed = false;
	try {
	    goodInputTests();
	    badInputTests();
	}
	catch (Exception e) {
	    System.out.println("Test FAILED: " + e);
	    e.printStackTrace(System.out);
	    testfailed = true;
	}
	if (testfailed)
	    System.out.println("Test FAILED!");
	else
	    System.out.println("Test OK!");
    }

    public static void main (String [] args)
    {
        runTests();
    }
}
