/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Main.java,v 1.2.2.1.2.1 2000/10/19 17:59:08 jasona Exp $
 */

package org.enhydra.httpServerTest.client;

import java.util.Vector;

/**
 * This class provides the static main function that launches the sims-
 * computing test bed.  The reason for this 'wrapper' class is to parse
 * some command line parameters used by the client and server sides of 
 * the HttpServerTest module.  The parameters passed in are held in public
 * static variables and are accessed by the test module for information
 * such as the current configuration file to be used.
 *
 * @see com.simscomputing.testbed.Main
 * @see com.simscomputing.testbed.gui.Main;
 * 
 * @author Mike Ward
 **/
public class Main {

    /**
     * This is the reference to the current configuration file.  This value
     * is parsed from the command line arguments.
     **/
    public static String USER_DEFINED_CONFIG_FILE = null;

    /**
     * This is the reference to the current directory used for placing
     * log files.  Unique files will be created for each test case and 
     * placed under this location.
     **/
    public static String USER_DEFINED_LOG_DIR = null;

    public static void printUsageMessage() {
	System.out.println("Usage: Main -config <path to configuration file>");
	System.out.println("            -log <location to place log files>");
	System.out.println("            -d <path to directory of TestCaseCollections>");
	System.out.println("       Main -config <path to configuration file>");
	System.out.println("            -log <location to place log files>");
	System.out.println("            -r <path to recursive directory of TestCaseCollections>");
	System.out.println("       Main -config <path to configuration file>");
	System.out.println("            -log <location to place log files>");
	System.out.println("            -gui");
    }

    public static void main(final String[] args) {
	if (args.length < 5 ||
	    args.length > 6) {
	    printUsageMessage();
	    return;
	}

	boolean guiFlagFound = false;
	Vector testBedArgsVector = new Vector();
	for (int i = 0; i < args.length; i++) {
	    if (args[i].equals("-config")) {
		USER_DEFINED_CONFIG_FILE = args[++i];
	    } else if (args[i].equals("-log")) {
		USER_DEFINED_LOG_DIR = args[++i];
	    }
	    else if (args[i].equals("-gui")) {
		guiFlagFound = true;
	    } else if (args[i].equals("-d")) {
		testBedArgsVector.addElement(args[i]);
		testBedArgsVector.addElement(args[++i]);
	    } else if (args[i].equals("-r")) {
		testBedArgsVector.addElement(args[i]);
		testBedArgsVector.addElement(args[++i]);
	    }
	}

	if (guiFlagFound) {
	    if (testBedArgsVector.size() > 0) {
		printUsageMessage();
		return;
	    }
	    com.simscomputing.testbed.gui.Main.main(new String[0]);
	} else {
	    if (testBedArgsVector.size() < 1 ||
		testBedArgsVector.size() > 2) {
		printUsageMessage();
		return;
	    }
	    String[] testBedArgsArray = new String[testBedArgsVector.size()];
	    testBedArgsVector.copyInto(testBedArgsArray);	    
	    com.simscomputing.testbed.Main.main(testBedArgsArray);
	}
    }
}
