/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Javac.java,v 1.9.12.1 2000/10/19 17:59:08 jasona Exp $
 */





package com.lutris.appserver.server.jolt.joltc;

import java.io.*;

/**
 * Run the Java compiler as a child process and allow 
 * the filtering of stderr.  This is somewhat crudely
 * done.
 */
class Javac {
    String javacProg = "javac";

    /**
     * Construct a javac runner.  If JAVAC property is defined
     * it is used as the java compiler, otherwise "javac" is used
     */
    Javac() {
        String prog = System.getProperty("JAVAC");
        if ((prog != null) && (prog.length() > 0)) {
            javacProg = prog;
        }
    }
    
    /**
     * Monitor the running process, doing I/O filtering.
     */
    private void monProc(Process jproc,
                         OutputFilter stderrFilter) 
        throws IOException, InterruptedException {
        OutputFilterThread outMon =
            new OutputFilterThread(jproc.getInputStream(),
                                   System.out, null);
        OutputFilterThread errMon =
            new OutputFilterThread(jproc.getErrorStream(),
                                   System.err, stderrFilter);
        try {
            outMon.start();
            errMon.start();
            jproc.waitFor();
        } catch (InterruptedException except) {
            try {
                // Last ditch effort to clean up proc
                jproc.destroy();
                jproc.waitFor();
            } catch (InterruptedException drop) {
                // Already have an error...
            }
            outMon.stop();
            errMon.stop();
        } finally {
            outMon.join();
            errMon.join();
        }
        if (outMon.getException() != null) {
            throw outMon.getException();
        }
        if (errMon.getException() != null) {
            throw errMon.getException();
        }
    }

    /**
     * Run a javac process.
     *
     * @param args Arguments to pass to Javac.
     * @return Exit code of javac; or 1 if an error was detected
     *  internally.
     */
    int javac(String[] args,
              OutputFilter stderrFilter) {
        String [] cmd = new String[args.length+1];
        cmd[0] = javacProg;
        System.arraycopy(args, 0, cmd, 1, args.length);
        
        Process jproc;
        try {
            jproc = Runtime.getRuntime().exec(cmd);
            monProc(jproc, stderrFilter);
        } catch (IOException except) {
            except.printStackTrace();
            return 1;
        } catch (InterruptedException except) {
            except.printStackTrace();
            return 1;
        }
        return jproc.exitValue();
    }

    // FIX: for debugging
    public static void main(String[] args) {
        Javac javac = new Javac();
        
        int code = javac.javac(args, null);
        System.exit(code);
    }

}
