/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: OutputFilterThread.java,v 1.6.12.1 2000/10/19 17:59:08 jasona Exp $
 */





package com.lutris.appserver.server.jolt.joltc;

import java.io.*;

/**
 * Output filtering thread.
 */
class OutputFilterThread extends Thread {
    private final BufferedReader input;
    private final BufferedWriter output;
    private final OutputFilter filter;
    private IOException ioException = null;
    
    /**
     * Construct an filter thread object.
     * @param input Stream to read from.
     * @param output Stream to write to.
     * @param filter Filter to call on each line.
     *  If null, this thread just copies the streams.
     */
    OutputFilterThread(InputStream input,
                       OutputStream output,
                       OutputFilter filter) {
        this.input = new BufferedReader(new InputStreamReader(input));
        this.output = new BufferedWriter(new OutputStreamWriter(output));
        this.filter = filter;
    }
    
    /**
     * Get an exception, if one terminated the thread, otherwise
     * null indicates no errors.
     */
    IOException getException() {
        return ioException;
    }


    /**
     * Filter the stream until eof.
     */
    private void filterStream() throws IOException {
        String line;
        while ((line = input.readLine()) != null) {
            if (filter != null) {
                line = filter.filterLine(line);
            }
            if (line != null) {
                output.write(line, 0, line.length());
                output.write('\n');
                output.flush();
            }
        }
    }

    /**
     * Run the filter thread.
     */
    public void run() {
        try {
            filterStream();
        } catch (IOException except) {
            ioException = except;
        }
        stop();
    }
}
