/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JoltDebug.java,v 1.19.12.1 2000/10/19 17:59:08 jasona Exp $
 */





package com.lutris.appserver.server.jolt.joltpo;

import com.lutris.http.debug.*;

/**
 * This class generates html formatted output that
 * can be used for debugging purposes in a JOLT (.jhtml)
 * page. It displays either the request, response or both.
 * <P>
 * Insert the line:
 *
 * <pre>
 *     &lt;JOLT JAVACALL=JoltDebug.displayRequest&gt;&lt;/JOLT&gt;
 *     &lt;JOLT JAVACALL=JoltDebug.displayResponse&gt;&lt;/JOLT&gt;
 *     &lt;JOLT JAVACALL=JoltDebug.displayAll&gt;&lt;/JOLT&gt;
 * </pre>
 *
 * in a jolt page causes the appropriate information to be
 * displayed within the page.
 *
 * @see lutris.jolt.HttpDebug
 * @author Kyle Clark
 */
public class JoltDebug {
    /**
     * Display the httpRequest object.
     *
     * @param page Page to which data should be written.
     * @exception Exception Throws all exceptions.
     */
    public static void displayRequest(JoltPage page)
        throws Exception {

        page.append("<HR>\n");
        page.append(HttpDebug.getRequestDebugInfo(
			(DebugRequest)page.presentationComms.request));
    }

    /**
     * Display the httpResponse object.
     *
     * @param page Page to which data should be written.
     * @exception Exception Throws all exceptions.
     */
    public static void displayResponse(JoltPage page)
        throws Exception {

        page.append("<HR>\n");
        page.append(HttpDebug.getResponseDebugInfo(
			(DebugResponse)page.presentationComms.response));
    }

    /**
     * Display the httpRequest object.
     *
     * @param page Page to which data should be written.
     * @exception Exception Throws all exceptions.
     */
    public static void displayAll(JoltPage page)
	throws Exception {

	displayRequest(page);
	displayResponse(page);
    }
}
