/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common;

// ToolBox imports
import org.enhydra.tool.common.event.CancelEvent;
import org.enhydra.tool.common.event.CancelListener;
import org.enhydra.tool.common.event.ProgressEvent;
import org.enhydra.tool.common.event.ProgressListener;

// Standard imports
import java.util.Vector;

//
public class KelpTool extends Thread implements CancelListener {
    private Vector  progVector = new Vector();
    private boolean fresh = true;

    public KelpTool() {
        setPriority((int) ((Thread.MIN_PRIORITY + Thread.NORM_PRIORITY) / 2));
    }

    /**
     * Make the thread stale so it stops running gracefully.
     */
    public void setFresh(boolean b) {
        fresh = b;
    }

    public boolean isFresh() {
        return fresh;
    }

    public ProgressListener[] getProgressListeners() {
        ProgressListener[] progListeners = new ProgressListener[0];

        progListeners = new ProgressListener[progVector.size()];
        progListeners =
            (ProgressListener[]) progVector.toArray(progListeners);
        return progListeners;
    }

    public void addProgressListener(ProgressListener l) {
        if (!progVector.contains(l)) {
            progVector.addElement(l);
        }
    }

    public void removeProgressListener(ProgressListener l) {
        if (progVector.contains(l)) {
            progVector.removeElement(l);
        }
    }

    public void refreshProgress(int progress, String message) {
        ProgressListener[] progListeners = getProgressListeners();

        if (progListeners.length == 0) {

            // done
        } else {
            ProgressEvent event = null;

            event = new ProgressEvent(this, progress, message);
            for (int i = 0; i < progListeners.length; i++) {
                progListeners[i].onProgress(event);
            }
        }
    }

    public void onCancel(CancelEvent event) {
        setFresh(false);
    }

}
