/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common;

// Kelp imports
import org.enhydra.kelp.common.node.OtterNode;

// Standard imports
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class PropUtil {
    /**
     * Constructor declaration
     *
     */
    public PropUtil() {}

    /**
     * Method declaration
     *
     *
     * @param s
     * @param def
     *
     * @return
     */
    public static int stringToInt(String s, int def) {
        int i = def;

        if (s != null) {
            if (s.trim().length() > 0) {
                try {
                    i = Integer.parseInt(s.trim());
                } catch (Exception e) {
                    i = def;
                }
            }
        }
        return i;
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @return
     */
    public static String booleanToString(boolean b) {
        String flag = b ? Boolean.TRUE.toString() : Boolean.FALSE.toString();

        return flag;
    }

    /**
     * Method declaration
     *
     *
     * @param s
     * @param def
     *
     * @return
     */
    public static boolean stringToBoolean(String s, boolean def) {
        boolean b = def;

        if (s != null) {
            if (s.trim().length() > 0) {
                b = s.trim().equalsIgnoreCase(Boolean.TRUE.toString());
            }
        }
        return b;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param lengthName
     * @param columnName1
     * @param columnName2
     *
     * @return
     */
    public static String[][] getArrayProperty(OtterNode node,
                                              String lengthName,
                                              String columnName1,
                                              String columnName2) {
        String[][] array = new String[0][0];
        String     columnValue1 = null;
        String     columnValue2 = null;
        Vector     valueVector = new Vector();
        int        length = 0;

        length = stringToInt(node.getProperty(lengthName), 100);
        for (int i = 0; i < length; i++) {
            columnValue1 = node.getProperty(columnName1 + '.' + i);
            columnValue2 = node.getProperty(columnName2 + '.' + i);
            if (columnValue1 != null && columnValue2 != null) {
                if (columnValue1.trim().length() > 0
                        && columnValue2.trim().length() > 0) {

                    // note: new 'row' object is needed for each call to addElement
                    String[] row = new String[2];

                    row[0] = columnValue1;
                    row[1] = columnValue2;
                    valueVector.addElement(row);
                }
            }
        }
        length = valueVector.size();
        node.setProperty(lengthName, length);    // update to valid length
        String[] row = new String[2];

        array = new String[length][2];
        for (int i = 0; i < length; i++) {
            row = (String[]) valueVector.elementAt(i);
            array[i][0] = row[0];
            array[i][1] = row[1];
        }
        return array;
    }

    /**
     * Method declaration
     *
     *
     * @param map
     */
    public static void putArrayProperty(OtterNode node, String lengthName,
                                        String columnName1,
                                        String columnName2,
                                        String[][] array) {
        int    rawLength = 0;
        int    newLength = 0;
        String columnValue1 = new String();
        String columnValue2 = new String();

        rawLength = stringToInt(node.getProperty(lengthName), 100);
        for (int i = 0; i < rawLength; i++) {
            node.setProperty(columnName1 + '.' + i, null);
            node.setProperty(columnName2 + '.' + i, null);
        }
        rawLength = array.length;
        for (int i = 0; i < rawLength; i++) {
            columnValue1 = array[i][0];
            columnValue2 = array[i][1];
            if (columnValue1.trim().length() > 0
                    && columnValue2.trim().length() > 0) {
                node.setProperty(columnName1 + '.' + newLength, columnValue1);
                node.setProperty(columnName2 + '.' + newLength, columnValue2);
                newLength++;
            }
        }
        node.setProperty(lengthName, newLength);    // update to valid length
    }

    public static String arrayToList(String[] array) {
        StringBuffer list = new StringBuffer();

        for (int i = 0; i < array.length; i++) {
            if (i > 0) {
                list.append(';');
            }
            list.append(array[i]);
        }
        return list.toString();
    }

    public static String[] listToArray(String list, String[] defaultArray) {
        String[]        array = defaultArray;
        StringTokenizer tokenizer = null;

        if (list == null || list.trim().length() == 0) {}
        else {
            // string not to be resourced
            final String DELIM = ";"; // nores
            tokenizer = new StringTokenizer(list, DELIM);
            array = new String[tokenizer.countTokens()];
            int i = 0;

            while (tokenizer.hasMoreTokens()) {
                array[i] = tokenizer.nextToken();
                i++;
            }
        }
        return array;
    }

    public static String[] XMLCParametersToArray(String in) {
        StringTokenizer tokenizer = null;
        StringBuffer    buf = null;
        String          cursor = null;
        String[]        out = new String[0];
        Vector          paramVector = new Vector();
        int             count = -1;

        buf = new StringBuffer();
        if (in == null) {
            in = new String();
        }
        tokenizer = new StringTokenizer(in);
        while (tokenizer.hasMoreTokens()) {
            cursor = tokenizer.nextToken().trim();
            if ((cursor.length() > 0) && (cursor.charAt(0) == '-')
                    && (buf.length() > 0)) {
                paramVector.addElement(buf.toString().trim());
                buf.setLength(0);
            }
            buf.append(' ');
            buf.append(cursor);
        }
        if (buf.length() > 0) {
            paramVector.addElement(buf.toString().trim());
        }
        paramVector.trimToSize();
        out = new String[paramVector.size()];
        out = (String[]) paramVector.toArray(out);
        return out;
    }

    public static String XMLCParametersToString(String[] in) {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < in.length; i++) {
            buf.append(in[i]);
            buf.append(Constants.TAB1);
        }
        return buf.toString().trim();
    }

    public static String cleanXMLCParameters(String in) {
        String[] clean = new String[0];
        String   out = new String();

        clean = PropUtil.XMLCParametersToArray(in);
        clean = PropUtil.cleanXMLCParameters(clean);
        out = PropUtil.XMLCParametersToString(clean);
        return out;
    }

    public static String[] cleanXMLCParameters(String[] paramIn) {
        String[]  clean = new String[0];
        String[]  exclude = Constants.XMLC_NOOPS;
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(paramIn));
        for (int i = 0; i < exclude.length; i++) {
            for (int j = 0; j < list.size(); j++) {
                String cursor = null;

                cursor = list.get(j).toString().toLowerCase();
                if (cursor.startsWith(exclude[i])) {
                    list.remove(j);
                    list.trimToSize();
                    j--;
                }
            }
        }
        clean = new String[list.size()];
        clean = (String[]) list.toArray(clean);
        list.clear();
        return clean;
    }

}
