/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.node;

// Kelp imports
import org.enhydra.kelp.common.importer.ImportTool;
import org.enhydra.kelp.common.importer.ImportPaths;

// Standard imports
import java.io.File;

/**
 * Interface declaration
 *
 *
 * @author Paul Mahar
 */
public abstract class OtterNodeFactory {

    /**
     * Method declaration
     *
     *
     * @param node
     * @param name
     *
     * @return
     *
     */
    abstract public OtterFolderNode createFolderNode(OtterNode node,
            String name);

    /**
     * Method declaration
     *
     *
     * @param node
     * @param file
     *
     * @return
     */
    abstract public OtterDocumentNode createDocumentNode(OtterNode node,
            String file);

    /**
     * Method declaration
     *
     *
     * @param node
     * @param file
     *
     * @return
     */
    abstract public OtterImageFileNode createImageFileNode(OtterNode node,
            String file);

    /**
     * Method declaration
     *
     *
     * @param node
     * @param file
     *
     * @return
     */
    abstract public OtterJavaFileNode createJavaFileNode(OtterNode node,
            String file);

    /**
     * Method declaration
     *
     *
     * @param node
     * @param file
     *
     * @return
     */
    abstract public OtterTextFileNode createTextFileNode(OtterNode node,
            String file);

    /**
     * Method declaration
     *
     *
     * @param node
     * @param file
     *
     * @return
     */
    abstract public OtterTemplateNode createTemplateNode(OtterNode node,
            String file);

    abstract public OtterTemplateNode getTemplateNode(OtterNode node);


    /**
     * Method declaration
     *
     *
     * @param project
     * @param node
     * @param file
     *
     * @return
     */
    abstract public OtterJavaFileNode[] replaceGeneratedSource(OtterProject project,
            OtterNode node, String[] files);

    //
    //
    public void importFile(OtterProject project, File newFile) {
        File        source = null;
        File[]      files = new File[1];
        ImportPaths paths = null;
        ImportTool  tool = null;

        files[0] = newFile;
        source = new File(project.getSourcePathArray()[0]);
        if (!source.exists()) {
            source = (new File(project.getFilePath())).getParentFile();
        }
        paths = new ImportPaths();
        tool = new ImportTool();
        try {
            paths.setSourcePath(source.getAbsolutePath());
        } catch (Exception e) {
            e.printStackTrace();
        }
        paths.setDeployRootPath(project.getDeployRootPath());
        paths.setProject(project);
        tool.setPaths(paths);
        tool.addFiles(files);
    }

}
