/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.AboutBox;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ProgressMeter;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.capture.Writer;
import org.enhydra.kelp.common.capture.WriteEvent;
import org.enhydra.kelp.common.capture.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;

// Standard imports
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.util.Vector;
import java.util.ResourceBundle;

/**
 * This class defines the main dialog for the XMLC Wizard.
 *
 * @author Paul Mahar
 */
abstract public class AbstractBuildDialog extends JDialog
    implements SelectionPanelListener {

    //
    public static ResourceBundle  res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    //
    private SelectionPanel        selectionPanel = null;
    private boolean               standalone = false;
    private OtterNode[]           nodes = null;
    private OtterProject          project = null;
    private LocalButtonListener   buttonListener = null;
    private JPanel                panelMain = null;
    private JPanel                panelTab = null;
    private JPanel                panelButton = null;
    private JPanel                panelBottom = null;
    private JPanel                panelFiller2 = null;
    private JPanel                panelOptionFiller0 = null;
    private JPanel                panelOptionFiller1 = null;
    private JPanel                panelOptionFiller2 = null;
    private GridBagLayout         layoutMain = null;
    private GridBagLayout         layoutOption = null;
    private GridBagLayout         layoutButton = null;
    private GridBagLayout         layoutBottom = null;
    private JPanel                propertyPanelHolder = null;
    private AbstractPropertyPanel propertyPanel = null;
    private OutputPanel           outputPanel = null;
    private JTabbedPane           tab = null;
    private JButton               buttonClose = null;
    private JButton               buttonAbout = null;
    private Vector                writeListeners = new Vector();
    private ProgressMeter         progress = null;
    public JButton                buttonMake = null;

    public AbstractBuildDialog(Frame owner) {
        super(owner);
    }

    public SelectionPanel getSelectionPanel() {
        return selectionPanel;
    }

    /**
     * Create the dialog as a child of a frame.
     *
     * @param owner
     * The parent Frame used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public AbstractBuildDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
    }

    /**
     * Create the dialog as a child of another dialog.
     *
     * @param owner
     * The parent Dialog used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public AbstractBuildDialog(Dialog owner, String title, boolean modal) {
        super(owner, title, modal);
    }

    abstract protected String getProgressTitle();

    /**
     * Method declaration
     *
     *
     * @param e
     */
    public void onSelection(SelectionPanelEvent e) {
        boolean enable = e.getSelected().size() > 0;

        buttonMake.setEnabled(enable);
    }

    /**
     * Overloaded method that will center the dialog prior to making it
     * visible.
     */
    public void show() {

        // setResizable(false);  May cause crash in Win98
        Point cPoint;

        invalidate();
        doLayout();
        pack();
        cPoint = SwingUtil.getCenteringPoint(getSize());
        setLocation(cPoint);
        progress = new ProgressMeter();
        progress.setDisposeOnClose(false);
        super.show();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public synchronized Vector getWriteListeners() {
        return writeListeners;
    }

    /**
     * Method declaration
     *
     *
     * @param l
     */
    public synchronized void addWriteListener(WriteListener l) {
        writeListeners.addElement(l);
    }

    /**
     * Method declaration
     *
     *
     * @param l
     */
    public synchronized void removeWriteListener(WriteListener l) {
        if (writeListeners.contains(l)) {
            writeListeners.removeElement(l);
        }
    }

    /**
     * Remove all object references.
     */
    public void clearAll() {
        layoutMain.removeLayoutComponent(panelTab);
        panelTab.removeAll();
        panelMain.removeAll();
        layoutOption = null;
        layoutMain = null;
        panelTab = null;
        panelMain = null;
        setTitle(null);
    }

    /**
     * Get source files that can be selected for compilation.
     */
    public OtterNode[] getNodes() {
        return nodes;
    }

    /**
     * Set the source files that can be selected for compilation
     */
    public void setNodes(OtterNode[] n) {
        nodes = n;
        selectionPanel.setNodes(nodes);
    }

    /**
     * Get the IDE project containing the HTML files.
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Set the IDE project containing the HTML files.
     */
    public void setProject(OtterProject p) {
        project = p;
        propertyPanel.setProject(project);
        propertyPanel.readProperties();
        outputPanel.setProject(project);
    }

    //
    // PROTECTED
    //

    /**
     * Close the dialog and exit the VM if this wizard
     * was run as a standalone application.
     */
    protected void closeDialog() {
        setVisible(false);
        removeAll();
        dispose();
        if (standalone) {
            System.exit(0);
        }
    }

    protected ProgressMeter getProgressMeter() {
        return progress;
    }

    //
    // PRIVATE
    //

    /**
     * Select the output to show compilation progress. This
     * is called when XMLC is invoked.
     */
    protected void selectOutputTab() {
        selectTab(outputPanel);
    }

    protected void selectOptionTab() {
        selectTab(panelOptionFiller0);
    }

    private void selectTab(final JPanel goPanel) {
        SelectTab runSwing = null;

        runSwing = new SelectTab(tab, goPanel);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                runSwing.run();
            } else {
                SwingUtilities.invokeAndWait(runSwing);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {

            //
        }
    }

    /**
     * Construct the object. This is called from both constructors.
     */
    public void init(AbstractPropertyPanel p) {
        propertyPanel = p;
        try {
            jbInit();
            pmInit();
            pack();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
        propertyPanelHolder.add(propertyPanel, BorderLayout.CENTER);
        tab.add(res.getString("Selections"), selectionPanel);
        tab.add(res.getString("Options"), panelOptionFiller0);
        tab.add(res.getString("Output"), outputPanel);
        addWriteListener(outputPanel);
        buttonListener = new LocalButtonListener();
        buttonMake.addActionListener(buttonListener);
        buttonClose.addActionListener(buttonListener);
        buttonAbout.addActionListener(buttonListener);
        selectionPanel.addSelectionPanelListener(this);
    }

    private void showAbout() {
        StringBuffer textBuf = new StringBuffer();
        StringBuffer titleBuf = new StringBuffer();

        textBuf.append(Constants.ENHYDRA);
        textBuf.append(' ');
        textBuf.append(ToolBoxInfo.getEnhydraVersion());
        textBuf.append('\n');
        textBuf.append(Constants.XMLC);
        textBuf.append(' ');
        textBuf.append(ToolBoxInfo.getXMLCVersion());
        AboutBox about = new AboutBox(this);

        titleBuf.append(Constants.KELP);
        titleBuf.append(' ');
        titleBuf.append(KelpInfo.getKelpVersion());
        about.setTitleText(titleBuf.toString());
        about.setTextAreaText(textBuf.toString());
        about.setSmallText(res.getString("Copyright"));
        about.show();
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {
        panelMain = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelTab = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        panelBottom = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelButton = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelFiller2 = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        panelOptionFiller0 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        panelOptionFiller1 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        panelOptionFiller2 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutOption =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutButton =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutBottom =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        selectionPanel =
            (SelectionPanel) Beans.instantiate(getClass().getClassLoader(),
                                               SelectionPanel.class.getName());
        propertyPanelHolder =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        outputPanel =
            (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                            OutputPanel.class.getName());
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        buttonMake = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonClose = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonAbout = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonMake.setText(res.getString("Make"));
        buttonMake.setEnabled(false);
        buttonClose.setText(res.getString("Close"));
        panelTab.setLayout(layoutOption);
        buttonAbout.setText(res.getString("About_"));
        panelTab.add(tab,
                     new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.BOTH,
                                            new Insets(5, 5, 5, 5), 5, 5));
        panelButton.setLayout(layoutButton);
        panelButton.add(buttonMake,
                        new GridBagConstraints(1, 0, 1, 4, 0.5, 0.5,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(buttonClose,
                        new GridBagConstraints(2, 0, 1, 4, 0.5, 0.5,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(buttonAbout,
                        new GridBagConstraints(3, 3, 1, 1, 0.0, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.setLayout(new GridBagLayout());
        panelOptionFiller0.add(panelOptionFiller1,
                               new GridBagConstraints(0, 0, 1, 1, 0.15, 0.15,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.BOTH,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.add(propertyPanelHolder,
                               new GridBagConstraints(1, 0, 1, 4, 0.7, 0.7,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.NONE,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.add(panelOptionFiller2,
                               new GridBagConstraints(2, 0, 1, 1, 0.15, 0.15,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.BOTH,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelBottom.setLayout(layoutBottom);
        panelBottom.add(panelFiller2,
                        new GridBagConstraints(0, 0, 1, 1, 0.4, 0.4,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.BOTH,
                                               new Insets(0, 0, 0, 0), 0, 0));
        panelBottom.add(panelButton,
                        new GridBagConstraints(1, 0, 1, 4, 0.6, 0.6,
                                               GridBagConstraints.EAST,
                                               GridBagConstraints.NONE,
                                               new Insets(0, 0, 0, 0), 0, 0));
        panelMain.setLayout(layoutMain);
        panelMain.add(panelTab,
                      new GridBagConstraints(0, 0, 1, 1, 0.75, 0.75,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(0, 0, 0, 0), 0, 0));
        panelMain.add(panelBottom,
                      new GridBagConstraints(0, 1, 1, 1, 0.25, 0.25,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelMain);
    }

    /**
     * Method declaration
     */
    abstract public void make();

    //
    private Dialog getDialog() {
        return this;
    }

    private void openProgress() {
        getProgressMeter().setModal(true);
        getProgressMeter().setTitle(getProgressTitle());
        getProgressMeter().setOwner(getDialog());
        getProgressMeter().startDialogThread();
    }

    /**
     * This inner class defines the actions for buttons in the wizard.
     */
    class LocalButtonListener implements ActionListener {

        /**
         * ActionListener implementation.
         *
         * @param event
         * Event that triggered this listener.
         *
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonMake) {
                selectOutputTab();
                propertyPanel.writeProperties();
                outputPanel.clearOutput();
                openProgress();
                make();
            } else if (source == buttonClose) {
                propertyPanel.writeProperties();
                closeDialog();
            } else if (source == buttonAbout) {
                showAbout();
            }
        }

    }
    private class SelectTab implements Runnable {
        private JTabbedPane tp = null;
        private JPanel      go = null;

        protected SelectTab(JTabbedPane t, JPanel g) {
            tp = t;
            go = g;
        }

        synchronized public void run() {
            tp.setSelectedComponent(go);
            SwingUtilities.windowForComponent(tp).repaint();
        }

    }
}
