/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard;

// ToolBox imports
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.capture.WriteEvent;
import org.enhydra.kelp.common.capture.WriteListener;

// Standard imports
import javax.swing.*;
import javax.swing.text.BadLocationException;
import java.awt.*;
import java.awt.event.*;
import java.beans.Beans;
import java.io.File;
import java.io.FileWriter;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class OutputPanel extends JPanel implements WriteListener {
    private LocalButtonListener buttonListener = null;
    private LocalCheckListener  checkListener = null;
    private OtterProject        project = null;
    private JPanel              panelFile;
    private JScrollPane         scrollOutput;
    private JCheckBox           checkSave;
    private JTextArea           textAreaOutput;
    private JTextField          textFile;
    private JButton             buttonBrowse;
    private GridBagLayout       layoutFile;
    private GridBagLayout       layoutMain;
    private int                 waitCount = 0;

    /**
     * Constructor declaration
     *
     */
    public OutputPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param event
     */
    public void onWrite(WriteEvent event) {
        String out = event.getString();

        if (event.getType() == WriteEvent.CLEAR) {
            clearOutput();
        } else {
            if (File.separatorChar == '\\') {

                // string not to be resourced
                if (out.indexOf(":\\") > 0) {
                    out = out.replace('\\', '/');
                }
            }
            addOutput(out);
        }
    }

    //
    // PROTECTED
    //

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     */
    protected void setProject(OtterProject p) {
        project = p;
        String outPath = project.getOutputFilename();

        if ((outPath == null) || (outPath.trim().length() == 0)) {
            checkSave.setSelected(false);
            textFile.setText(new String());
        } else {
            textFile.setText(PathHandle.createPathString(outPath));
            checkSave.setSelected(true);
        }
    }

    /**
     * Method declaration
     *
     */
    protected void scrollToBottom() {
        OutputScroll runSwing = null;

        runSwing = new OutputScroll(scrollOutput);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                runSwing.run();
            } else {
                SwingUtilities.invokeAndWait(runSwing);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {

            //
        }
    }

    /**
     * Method declaration
     *
     *
     * @param output
     */
    protected void addOutput(final String output) {
        OutputAdd adder = null;

        adder = new OutputAdd(textAreaOutput, scrollOutput, output);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                adder.run();
            } else {
                waitCount++;
                if (waitCount == 10) {
                    waitCount = 0;
                    SwingUtilities.invokeAndWait(adder);
                } else {
                    SwingUtilities.invokeLater(adder);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (textFile.getText().trim().length() > 0) {
            try {
                FileWriter writer = null;

                writer = new FileWriter(project.getOutputFilename(), true);
                writer.write(output);    // line + new Character((char)13) + new Character((char)10));
                writer.close();
            } catch (java.io.IOException e) {
                e.printStackTrace();
            }
        }
    }

    protected void clearOutput() {
        OutputClear clearRun = null;

        clearRun = new OutputClear(textAreaOutput, scrollOutput);
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                clearRun.run();
            } else {
                SwingUtilities.invokeAndWait(clearRun);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //
    // PRIVATE
    //
    private void buttonFileAction() {
        JFileChooser    chooser = null;
        ExtensionFilter filter = null;
        PathHandle      path = null;
        File            file = null;
        String          fileDir = new String();
        int             choice = -1;

        //
        filter = new ExtensionFilter();
        filter.addExtension(Constants.TYPE_TXT);
        filter.setDescriptionTitle("Kelp log file");

        //
        chooser = new JFileChooser();
        if (textFile.getText().trim().length() == 0) {
            file = new File(project.getFilePath());
            file = new File(file.getParentFile(), Constants.FILE_KELP_TXT);
        } else {
            file = new File(textFile.getText());
        }
        chooser.setSelectedFile(file);
        chooser.setFileFilter(filter);
        chooser.setDialogTitle("Select a log file");
        chooser.setApproveButtonText("OK");
        choice = chooser.showOpenDialog(this);

        //
        this.requestFocus();
        buttonBrowse.requestFocus();

        //
        if (choice == JFileChooser.APPROVE_OPTION
                && chooser.getSelectedFile() != null) {
            path = PathHandle.createPathHandle(chooser.getSelectedFile());
            textFile.setText(path.getPath());
            project.setOutputFilename(path.getPath());
        }

        //
        chooser.removeAll();
        chooser = null;
    }

    /**
     * Method declaration
     *
     */
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonBrowse.addActionListener(buttonListener);
        buttonBrowse.setEnabled(false);
        checkListener = new LocalCheckListener();
        checkSave.addItemListener(checkListener);
        checkSave.setSelected(false);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        panelFile = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        textAreaOutput =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());
        textAreaOutput.setEditable(false);
        textAreaOutput.setRows(15);
        scrollOutput = new JScrollPane(textAreaOutput);
        checkSave = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        textFile = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonBrowse =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutFile =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonBrowse.setText("Browse...");
        panelFile.setLayout(layoutFile);
        checkSave.setText("Output to log file");
        textFile.setBackground(SystemColor.controlLtHighlight);
        textFile.setEnabled(false);
        textFile.setPreferredSize(new Dimension(150, 21));
        panelFile.add(checkSave,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.BOTH,
                                             new Insets(5, 5, 5, 5), 14, 0));
        panelFile.add(textFile,
                      new GridBagConstraints(1, 0, 1, 1, 0.5, 0.5,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 5, 5), 14, 0));
        panelFile.add(buttonBrowse,
                      new GridBagConstraints(2, 0, 1, 1, 0.5, 0.5,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.NONE,
                                             new Insets(5, 5, 5, 5), 6, 0));
        this.setLayout(layoutMain);
        this.add(scrollOutput,
                 new GridBagConstraints(0, 0, 1, 1, 0.75, 0.75,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.BOTH,
                                        new Insets(8, 12, 0, 0), 0, 0));
        this.add(panelFile,
                 new GridBagConstraints(0, 1, 1, 1, 0.25, 0.25,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 12, 0, 0), 0, 0));
    }

    /**
     */
    private class LocalCheckListener implements ItemListener {
        public void itemStateChanged(ItemEvent event) {
            Object source = event.getSource();

            if (source == checkSave) {
                if (checkSave.isSelected()) {
                    buttonBrowse.setEnabled(true);
                } else {
                    buttonBrowse.setEnabled(false);
                    textFile.setText(new String());
                }
            }
        }

    }

    /**
     */
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonBrowse) {
                buttonFileAction();
            }
        }

    }

    //
    private class OutputAdd implements Runnable {
        private JTextArea   textArea = null;
        private JScrollPane scroll = null;
        private String      out = null;

        protected OutputAdd(JTextArea ta, JScrollPane sc, String o) {
            textArea = ta;
            scroll = sc;
            out = o;
        }

        synchronized public void run() {
            int start = 0;
            int end = 0;

            if (textArea.getLineCount() > 2000) {
                try {
                    start = textArea.getLineStartOffset(0);
                    end = textArea.getLineEndOffset(0);
                } catch (BadLocationException e) {
                    start = 0;
                    end = 0;
                }
                textArea.replaceRange(null, start, end);
            }
            textArea.append(out);
            scroll.getVerticalScrollBar().setValue(scroll.getVerticalScrollBar().getMaximum());
            textArea.repaint();
            repaint();
        }

    }

    //
    private class OutputClear implements Runnable {
        private JTextArea   textArea = null;
        private JScrollPane scroll = null;

        protected OutputClear(JTextArea ta, JScrollPane sc) {
            textArea = ta;
            scroll = sc;
        }

        synchronized public void run() {
            textArea.setText(new String());
            scroll.getVerticalScrollBar().setValue(scroll.getVerticalScrollBar().getMaximum());
            textArea.repaint();
            repaint();
        }

    }
    //
    private class OutputScroll implements Runnable {
        private JScrollPane scroll = null;

        protected OutputScroll(JScrollPane sc) {
            scroll = sc;
        }

        synchronized public void run() {
            scroll.getVerticalScrollBar().setValue(scroll.getVerticalScrollBar().getMaximum());
        }

    }
}
