/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.codegen;

// ToolBox imports
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.codegen.AppGenerator;
import org.enhydra.tool.codegen.AppOptionSet;
import org.enhydra.tool.codegen.CodeGen;
import org.enhydra.tool.codegen.OptionSet;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.wizard.CodeGenDialog;
import org.enhydra.tool.codegen.wizard.CodeGenPanel;
import org.enhydra.tool.codegen.webapp.WebAppGenerator;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.map.Mapper;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.importer.ImportPaths;
import org.enhydra.kelp.common.importer.ImportTool;
import org.enhydra.kelp.common.ValidationException;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
abstract public class AppGenUtil {

    // strings not to be resourced
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private final static String EXCEPTION_CREATE_GENERATOR =
        "CodeGenUtil.createGenerator()";                            // nores

    //
    public final static int     CHOICE_CANCEL = CodeGenDialog.CHOICE_CANCEL;
    public final static int     CHOICE_FINISH = CodeGenDialog.CHOICE_FINISH;

    //
    private OtterProject        project = null;
    private CodeGen             codeGen = null;
    private Mapper              mapper = null;
    private ExtensionFilter     optionFilter = null;
    private ExtensionFilter     javaFilter = null;
    private int                 choice = AppGenUtil.CHOICE_CANCEL;

    public static AppGenerator createGenerator() throws CodeGenException {
        throw new CodeGenException(AppGenUtil.EXCEPTION_CREATE_GENERATOR);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public static String getDisplayName() {
        String displayName = res.getString("Web_Application");

        if (KelpInfo.isClassPathComplete()) {
            displayName = WebAppGenerator.displayName;
        }
        return displayName;
    }

    /**
     * Constructor declaration
     *
     *
     */
    public AppGenUtil(AppGenerator gen) throws CodeGenException {
        try {
            codeGen = new CodeGen(gen);
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }

    public AppGenUtil(OtterProject proj,
                      AppGenerator gen) throws CodeGenException {
        this(gen);
        init(proj);
    }

    public void init(OtterProject proj) throws CodeGenException {
        project = proj;
        try {
            optionFilter = new ExtensionFilter();
            optionFilter.setDirectoryValid(false);
            optionFilter.addExtension(Constants.TYPE_XMLC);
            javaFilter = new ExtensionFilter();
            javaFilter.setDirectoryValid(false);
            javaFilter.addExtension(Constants.TYPE_JAVA);
            initOptions();
            initPanels();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     */
    public CodeGenPanel[] getWizardPanels() throws CodeGenException {
        CodeGenPanel[] panels = null;

        try {
            panels = getGenerator().getWizardPanels();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        return panels;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     */
    public File[] generate() throws CodeGenException {
        File[] files = new File[0];

        try {
            for (int i = 0; i < getGenerator().getWizardPanels().length;
                    i++) {
                getGenerator().getWizardPanels()[i].writeOptionSet();
            }
            getGenerator().setEcho(false);
            files = getGenerator().generate();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        setupProject(files);
        return files;
    }

    public AppGenerator getGenerator() {
        AppGenerator generator = null;

        generator = (AppGenerator) codeGen.getSelection();
        return generator;
    }

    /**
     * Method declaration
     *
     *
     * @param owner
     *
     * @return
     */
    public File[] invokeWizard(Component owner) throws CodeGenException {
        File[] files = new File[0];

        files = codeGen.invokeWizard(owner);
        choice = codeGen.getChoice();
        if (choice == AppGenUtil.CHOICE_CANCEL) {

            // done
        } else if (files != null) {
            setupProject(files);
        }
        return files;
    }

    /**
     * Method declaration
     *
     * @return
     */
    public int getChoice() {
        return choice;
    }

    //
    //

    /**
     * Method declaration
     *
     * @return
     */
    private String getProjectSourcePath() throws CodeGenException {
        String path = new String();

        try {
            path = getGenerator().getProjectSourcePath();
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
        return path;
    }

    /**
     * Method declaration
     *
     * @param files
     */
    private void setupProject(File[] files) throws CodeGenException {
        ImportTool  tool = null;
        ImportPaths paths = null;
        String      pack = new String();
        String[][]  map = new String[0][2];

        tool = new ImportTool();
        paths = new ImportPaths();
        try {
            pack = getPackage();
            map = getGenerator().expandKelpPackageMap();

            //
            paths.setRootPath(getGenerator().getDestinationPath());
            paths.setPackage(pack);
            paths.setPackageMap(map);
            paths.setSourcePath(getProjectSourcePath());
            paths.setProject(project);
            tool.setPaths(paths);
            tool.importFiles(files);
            String[] path = new String[1];

            path[0] = paths.getSourcePath();
            project.setSourcePathArray(path);
        } catch (ValidationException e) {
            throw new CodeGenException(e);
        } catch (GeneratorException e) {
            throw new CodeGenException(e);
        }
    }

    /**
     * Method declaration
     *
     *
     *
     * @exception GeneratorException
     */
    private void initPanels() throws GeneratorException, CodeGenException {}

    /**
     * Method declaration
     *
     *
     * @param p
     */
    private void initOptions() throws GeneratorException {
        File   file = null;
        String dest = new String();
        String projectName = new String();
        String pack = new String();
        int    index = -1;

        dest = project.getCodeGenDefaultDestination();
        pack = project.getProjectName();
        file = new File(project.getFilePath());
        if (!file.getParent().equalsIgnoreCase(dest)) {
            projectName = file.getParentFile().getName();
        }
        if (projectName.trim().length() == 0) {
            projectName =
                file.getName().substring(0, file.getName().indexOf('.'));
        }
        getGenerator().getOptionSet().lookup(AppOptionSet.ROOT).setValue(dest);
        getGenerator().getOptionSet().lookup(AppOptionSet.PROJECT).setValue(projectName);
        getGenerator().getOptionSet().lookup(AppOptionSet.PACKAGE).setValue(pack.toLowerCase());
        getGenerator().getOptionSet().lookup(AppOptionSet.NOMAKE).setValue(true);
        getGenerator().getOptionSet().lookup(AppOptionSet.NOSCRIPT).setValue(true);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @exception GeneratorException
     */
    private String getPackage() throws GeneratorException {
        OptionSet optionSet;
        String    pack;

        optionSet = getGenerator().getOptionSet();
        pack = optionSet.lookup(AppOptionSet.PACKAGE).getValue();
        return pack;
    }

}
