/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.deploy;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.TemplateFile;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.configure.ConfigTool;

// Kelp imports
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.wizard.AbstractBuilder;

// Standard imports
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Vector;
import java.io.File;
import java.util.ResourceBundle;

//
public class InputBuilder extends AbstractBuilder {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private File[]        builtTemplates = new File[0];
    private File[]        builtPassthrough = new File[0];

    public InputBuilder() {}

    public File[] getBuiltTemplates() {
        return builtTemplates;
    }

    public File[] getBuiltPassthrough() {
        return builtPassthrough;
    }

    public File[] make() throws ToolException {
        File[]    out = new File[0];
        ArrayList outList = new ArrayList();

        builtTemplates = makeTemplates();
        if (!getProject().isDeployInputFilter()) {
            builtPassthrough = makePassthrough();
        }
        outList.addAll(Arrays.asList(builtTemplates));
        outList.addAll(Arrays.asList(builtPassthrough));
        out = (File[]) outList.toArray(out);
        outList.clear();
        return out;
    }

    //
    private File[] makePassthrough() throws ToolException {
        File[]      out = new File[0];
        OtterNode[] pass = new OtterNode[0];
        PathHandle  outPath = null;
        File        inFile = null;

        pass = PathUtil.getInputPassthrough(getProject());
        for (int i = 0; i < pass.length; i++) {
            if (pass[i].isSelected()) {
              outPath = getPassthroughPath(pass[i]);
              inFile = new File(pass[i].getFilePath());
              try {
                  FileUtil.copyFile(inFile, outPath.getFile());
                  echo(res.getString("Copying_input_to") + outPath.getPath());
              } catch (ToolException e) {
                  echo(e);
                  e.printStackTrace();
              }
            }
        }
        return out;
    }

    private File[] makeTemplates() throws ToolException {
        File[]     output = new File[0];
        ConfigTool tool = new ConfigTool();

        tool.setEcho(isEcho());
        tool.setEchoWriter(getEchoWriter());
        tool.setTransformPath(true);
        tool.setTemplates(getTemplateFiles());
        tool.setOverwrite(getProject().isDeployOverwrite());
        tool.initReplacements(PathUtil.expandReplacementTable(getProject(),
                getProject().getReplacementTable()));
        output = tool.createOutput();
        return output;
    }

    private TemplateFile[] getTemplateFiles() {
        TemplateFile[]      templateFiles = new TemplateFile[0];
        Vector              tempVector = new Vector();
        OtterTemplateNode[] templateNodes = new OtterTemplateNode[0];

        templateNodes = PathUtil.getInputTemplates(getProject());
        for (int i = 0; i < templateNodes.length; i++) {
            OtterTemplateNode cursor = null;
            TemplateFile      temp = null;

            cursor = templateNodes[i];
            if (cursor.isSelected()) {
                temp = new TemplateFile(cursor.getFilePath());
                temp.setOutput(cursor.getOutputFile());
                tempVector.addElement(temp);
            }
        }
        templateFiles = (TemplateFile[]) tempVector.toArray(templateFiles);
        tempVector.removeAllElements();
        return templateFiles;
    }

    private PathHandle getPassthroughPath(OtterNode node) {
        PathHandle   inputPath = null;
        PathHandle   filePath = null;
        PathHandle   rootPath = null;
        PathHandle   passPath = null;
        StringBuffer buf = new StringBuffer();
        String       suffix = new String();

        inputPath = PathHandle.createPathHandle(getProject().getDeployInputPath());
        filePath = PathHandle.createPathHandle(node.getFilePath());
        rootPath =
            PathHandle.createPathHandle(getProject().getDeployRootPath());
        buf.append(rootPath.getPath());
        if (inputPath.parentOf(filePath)) {
          suffix = filePath.getPath().substring(inputPath.getPath().length());
        } else {
          suffix = filePath.getFile().getName();
        }
        if ((suffix.length() == 0) || (suffix.charAt(0) != '/')) {
            buf.append('/');
        }
        buf.append(suffix);
        passPath = PathHandle.createPathHandle(buf.toString());
        return passPath;
    }

}
