/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.deploy;

// ToolBox imports
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.configure.ConfigTool;

// Kelp imports
import org.enhydra.kelp.common.Backward;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.SwingTableSelectionEvent;
import org.enhydra.kelp.common.swing.SwingTableSelectionListener;
import org.enhydra.kelp.common.wizard.AbstractPropertyPanel;

// Standard imports
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.*;
import java.io.File;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import java.util.ResourceBundle;

//
public class TemplateNodePropertyPanel extends JPanel
    implements ListSelectionListener, SwingTableSelectionListener {
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");  // nores
    private LocalRadioListener  radioListener = null;
    private OtterNode           node = null;
    private GridBagLayout       layoutMain;
    private JCheckBox           checkOutput;
    private JCheckBox           checkOverwrite;
    private JPanel              panelClassName;
    private GridBagLayout       layoutClassName;
    private LocalButtonListener buttonListener;
    private JTable              table;
    private JScrollPane         scrollTable;
    private LocalTableModel     tableModel = new LocalTableModel();
    private JButton             buttonAdd;
    private JButton             buttonEdit;
    private JButton             buttonRemove;
    private JButton             buttonReset;
    private int                 currentSelectionIndex = -1;
    private Vector              swingTableSelectionListeners = new Vector();
    private JButton             buttonUp;
    private JButton             buttonDown;
    private JCheckBox           checkBootstrap;
    private JPanel              panelDeploy;
    private JRadioButton        radioFilter;
    private JRadioButton        radioAll;
    private GridBagLayout       layoutDeploy;

    public TemplateNodePropertyPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * ListSelectionListener event
     */
    public void valueChanged(ListSelectionEvent e) {
        ListSelectionModel lsm = (ListSelectionModel) e.getSource();

        if (lsm.isSelectionEmpty()) {
            currentSelectionIndex = -1;
        } else {
            currentSelectionIndex = lsm.getMinSelectionIndex();
        }
        fireSwingTableSelectionEvent();
    }

    private void fireSwingTableSelectionEvent() {
        SwingTableSelectionListener listener = null;

        for (int i = 0; i < swingTableSelectionListeners.size(); i++) {
            listener =
                (SwingTableSelectionListener) swingTableSelectionListeners.elementAt(i);
            listener.onSwingTableSelection(new SwingTableSelectionEvent(this,
                    currentSelectionIndex));
        }
    }

    public synchronized void addSwingTableSelectionListener(SwingTableSelectionListener l) {
        if (!swingTableSelectionListeners.contains(l)) {
            swingTableSelectionListeners.addElement(l);
        }
    }

    public synchronized void removeSwingTableSelectionListener(SwingTableSelectionListener l) {
        if (swingTableSelectionListeners.contains(l)) {
            swingTableSelectionListeners.removeElement(l);
        }
    }

    public void onSwingTableSelection(SwingTableSelectionEvent event) {
        boolean selection = (!event.isSelectionNull());

        buttonEdit.setEnabled(selection);
        buttonRemove.setEnabled(selection);
        if (selection) {
            int     index = event.getSelectionIndex();
            boolean down = (index < tableModel.getRowCount() - 1);
            boolean up = (index > 0);

            buttonUp.setEnabled(up);
            buttonDown.setEnabled(down);
        } else {
            buttonUp.setEnabled(false);
            buttonDown.setEnabled(false);
        }
    }

    public OtterNode getNode() {
        return node;
    }

    public void setNode(OtterNode n) {
        node = n;
    }

    private void pmInit() throws Exception {
        ButtonGroup group = null;

        buttonEdit.setEnabled(false);
        buttonEdit.setToolTipText(res.getString("buttonEdit_ToolTipText"));
        buttonRemove.setEnabled(false);
        buttonRemove.setToolTipText(res.getString("buttonRemove_ToolTipText"));
        buttonListener = new LocalButtonListener();
        buttonAdd.addActionListener(buttonListener);
        buttonAdd.setToolTipText(res.getString("buttonAdd_ToolTipText"));
        buttonEdit.addActionListener(buttonListener);
        buttonReset.addActionListener(buttonListener);
        buttonReset.setToolTipText(res.getString("buttonReset_ToolTipText"));
        buttonRemove.addActionListener(buttonListener);
        buttonUp.addActionListener(buttonListener);
        buttonDown.addActionListener(buttonListener);
        tableModel = new LocalTableModel();
        table.setModel(tableModel);

        // table.setDefaultRenderer(String.class, new CellRendererWithToolTip());
        tableModel.addTableModelListener(table);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.getTableHeader().setUpdateTableInRealTime(false);
        table.getTableHeader().setReorderingAllowed(false);
        ListSelectionModel rowSM = table.getSelectionModel();

        rowSM.addListSelectionListener(this);
        addSwingTableSelectionListener(this);
        group = new ButtonGroup();
        group.add(radioAll);
        group.add(radioFilter);
        radioListener = new LocalRadioListener();
        radioAll.addItemListener(radioListener);
        radioFilter.addItemListener(radioListener);
    }

    private void jbInit() throws Exception {
        checkOutput =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkOverwrite =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        checkBootstrap =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelClassName =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutClassName =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        table = (JTable) Beans.instantiate(getClass().getClassLoader(),
                                           JTable.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonEdit = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonReset = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonUp = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                               JButton.class.getName());
        buttonDown = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        panelDeploy = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        radioFilter =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioAll =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        layoutDeploy =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkOutput.setText(res.getString("Deploy"));
        checkOutput.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                enableUI(checkOutput.isSelected());
            }

        });
        checkOverwrite.setText(res.getString("checkOverwrite_Text"));
        checkBootstrap.setText(res.getString("checkBootstrap_Text"));
        scrollTable = new JScrollPane(table);
        table.setToolTipText(new String());
        table.sizeColumnsToFit(JTable.AUTO_RESIZE_ALL_COLUMNS);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
        table.setColumnSelectionAllowed(false);
        scrollTable.setMinimumSize(new Dimension(300, 100));
        scrollTable.setPreferredSize(new Dimension(300, 100));
        buttonAdd.setText(res.getString("buttonAdd_Text"));
        buttonEdit.setText(res.getString("Edit"));
        buttonRemove.setText(res.getString("Remove"));
        buttonReset.setText(res.getString("buttonReset_Text"));
        buttonUp.setToolTipText(res.getString("buttonUp_ToolTipText"));
        buttonUp.setText(res.getString("buttonUp_Text"));
        buttonDown.setToolTipText(res.getString("buttonDown_ToolTipText"));
        buttonDown.setText(res.getString("buttonDown_Text"));
        radioFilter.setText(res.getString("radioFilter_Text"));
        radioAll.setSelected(true);
        radioAll.setText(res.getString("radioAll_Text"));
        scrollTable.getViewport().add(table, BorderLayout.CENTER);

        //
        panelClassName.setLayout(layoutClassName);
        panelClassName.setBorder(BorderFactory.createTitledBorder(res.getString("panelClassName_Border")));
        panelClassName.add(scrollTable,
                           new GridBagConstraints(3, 0, 6, 1, 0.8, 0.8,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.BOTH,
                                                  new Insets(0, 0, 0, 0), 0,
                                                  0));
        panelClassName.add(buttonAdd,
                           new GridBagConstraints(3, 1, 1, 1, 0.1, 0.1,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(2, 1, 2, 1), 0,
                                                  0));
        panelClassName.add(buttonEdit,
                           new GridBagConstraints(4, 1, 1, 1, 0.1, 0.1,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(2, 1, 2, 1), 0,
                                                  0));
        panelClassName.add(buttonRemove,
                           new GridBagConstraints(5, 1, 1, 1, 0.1, 0.1,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(2, 1, 2, 1), 0,
                                                  0));
        panelClassName.add(buttonReset,
                           new GridBagConstraints(8, 1, 1, 1, 0.1, 0.1,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(2, 1, 2, 1), 0,
                                                  0));
        panelClassName.add(buttonUp,
                           new GridBagConstraints(6, 1, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(2, 1, 2, 1), 0,
                                                  0));
        panelClassName.add(buttonDown,
                           new GridBagConstraints(7, 1, 1, 1, 0.0, 0.0,
                                                  GridBagConstraints.CENTER,
                                                  GridBagConstraints.NONE,
                                                  new Insets(2, 1, 2, 1), 0,
                                                  0));

        //
        panelDeploy.setBorder(BorderFactory.createTitledBorder(res.getString("panelDeploy_Border")));
        panelDeploy.setLayout(layoutDeploy);
        panelDeploy.add(radioFilter,
                        new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(1, 5, 1, 10), 10,
                                               0));
        panelDeploy.add(radioAll,
                        new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(1, 10, 1, 5), 10,
                                               0));

        //
        this.setLayout(layoutMain);
        this.add(checkOutput,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(10, 5, 5, 5), 0, 0));
        this.add(checkOverwrite,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 20, 2, 5), 0, 0));
        this.add(panelDeploy,
                 new GridBagConstraints(0, 2, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(4, 15, 4, 5), 0, 1));
        this.add(checkBootstrap,
                 new GridBagConstraints(0, 3, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 20, 3, 5), 0, 0));
        this.add(panelClassName,
                 new GridBagConstraints(0, 4, 1, 1, 0.4, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(3, 15, 3, 5), 10, 20));
    }

    public void readProperties() {
        String       stringIn = null;
        File         fileIn = null;
        OtterProject project = node.getProject();

        if (node instanceof OtterTemplateNode) {
            OtterTemplateNode template = (OtterTemplateNode) node;

            panelDeploy.setVisible(false);
            checkOutput.setSelected(template.isSelected());
            checkBootstrap.setSelected(template.isBootstrap());
            if (checkBootstrap.isSelected()) {
                checkBootstrap.setEnabled(false);
            }
            remove(panelDeploy);
            enableUI(checkOutput.isSelected());
        } else {
            panelDeploy.setVisible(true);
            checkOutput.setVisible(false);
            checkBootstrap.setVisible(false);
            remove(checkOutput);
            remove(checkBootstrap);
            readFilter();
            enableUI(true);
        }
        checkOverwrite.setSelected(project.isDeployOverwrite());
        setReplacementTable(project.getReplacementTable());
        invalidate();
    }

    protected void readFilter() {
        boolean inFilter = false;

        inFilter = PathUtil.isInputInSource(node.getProject());
        if (inFilter) {
            radioAll.setEnabled(false);
            radioFilter.setEnabled(false);
        } else {
            radioAll.setEnabled(true);
            radioFilter.setEnabled(true);
            inFilter = node.getProject().isDeployInputFilter();
        }
        if (inFilter) {
            radioAll.setSelected(false);
            radioFilter.setSelected(true);
        } else {
            radioAll.setSelected(true);
            radioFilter.setSelected(false);
        }
        updateUI();
    }

    public void writeProperties() {
        OtterProject project = node.getProject();

        if (node instanceof OtterTemplateNode) {
            OtterTemplateNode template = (OtterTemplateNode) node;

            template.setSelected(checkOutput.isSelected());
            if (checkBootstrap.isSelected()) {
                String path = template.getOutputFile().getAbsolutePath();

                path = path.replace('\\', '/');
                project.setDeployBootstrapPath(path);
            }
        } else {
            if (radioFilter.isEnabled()) {
                project.setDeployInputFilter(radioFilter.isSelected());
            }
        }
        project.setDeployOverwrite(checkOverwrite.isSelected());
        project.setReplacementTable(getReplacementTable());
    }

    private String[][] getReplacementTable() {
        String[][] repTable = null;
        LocalRow   row = null;
        int        length = tableModel.getRowCount();

        repTable = new String[length][2];
        for (int i = 0; i < length; i++) {
            row = tableModel.getRow(i);
            repTable[i][0] = row.getFind();
            repTable[i][1] = row.getReplaceWith();
        }
        return repTable;
    }

    private void setReplacementTable(String[][] repTable) {
        int length = tableModel.getRowCount();

        tableModel.removeAllRows();
        for (int i = 0; i < repTable.length; i++) {
            tableModel.addRow(repTable[i][0], repTable[i][1]);
        }
        removeTableSelections();
    }

    private void removeTableSelections() {
        if (table.getSelectedRow() > -1) {
            table.removeRowSelectionInterval(table.getSelectedRow(),
                                             table.getSelectedRow());
        }
        currentSelectionIndex = -1;
        fireSwingTableSelectionEvent();
        table.updateUI();
    }

    private void enableUI(boolean enable) {
        checkOverwrite.setEnabled(enable);
        table.setEnabled(enable);
        buttonAdd.setEnabled(enable);
        buttonEdit.setEnabled(currentSelectionIndex > -1);
        buttonRemove.setEnabled(currentSelectionIndex > -1);
        buttonReset.setEnabled(enable);
    }

    private void defaultTable() {
        int result = JOptionPane.YES_OPTION;

        if (tableModel.getRowCount() > 0) {
            result =
                JOptionPane.showConfirmDialog(this,
                                              res.getString("Replace_current_set"),
                                              res.getString("Switch_to_Default"),
                                              JOptionPane.YES_NO_OPTION);
        }
        if (result == JOptionPane.YES_OPTION) {
            OtterProject project = node.getProject();
            String[][]   def = new String[0][0];

            try {
                def =
                    ConfigTool.createReplacementStringArray(project.getDeployRootPath());
                def =
                    Backward.createReplacementTable(def,
                                                    PathUtil.getInputTemplates(project));
            } catch (ToolException e) {
                def = new String[0][0];
                e.printStackTrace();
            }
            setReplacementTable(def);
        }
    }

    private void addRow() {
        ReplaceEditorDialog d = null;

        if (getTopLevelAncestor() instanceof JDialog) {
            JDialog parentDialog = (JDialog) this.getTopLevelAncestor();

            d = new ReplaceEditorDialog(parentDialog,
                                        res.getString("Add_Replace_Text"),
                                        true);
            d.setFind(new String());
            d.setReplaceWith(new String());
            d.show();
            if (d.getOption() == ReplaceEditorDialog.ACTION_OK) {
                if (d.getFind().trim().length() > 0) {
                    tableModel.addRow(d.getFind(), d.getReplaceWith());
                    removeTableSelections();
                }
            }
        }
    }

    private void removeCurrent() {
        int result = 0;

        if (currentSelectionIndex > -1
                && currentSelectionIndex < tableModel.getRowCount()) {
            result =
                JOptionPane.showConfirmDialog(this,
                                              res.getString("Remove_current"),
                                              res.getString("Confirm_Remove"),
                                              JOptionPane.YES_NO_OPTION);
            if (result == JOptionPane.YES_OPTION) {
                tableModel.removeRow(currentSelectionIndex);
                removeTableSelections();
            }
        }
    }

    private void editCurrent() {
        ReplaceEditorDialog d = null;
        LocalRow            row = tableModel.getRow(currentSelectionIndex);

        if (getTopLevelAncestor() instanceof JDialog) {
            JDialog parentDialog = (JDialog) this.getTopLevelAncestor();

            d = new ReplaceEditorDialog(parentDialog,
                                        res.getString("Edit_Replace_Text"),
                                        true);
            d.setFind(row.getFind());
            d.setReplaceWith(row.getReplaceWith());
            d.show();
            if (d.getOption() == ReplaceEditorDialog.ACTION_OK) {
                if (d.getFind().trim().length() > 0) {
                    row.setFind(d.getFind());
                    row.setReplaceWith(d.getReplaceWith());
                    table.updateUI();
                }
            }
        }
    }

    private void moveRowUp() {
        tableModel.moveUp(currentSelectionIndex);
        table.updateUI();
    }

    private void moveRowDown() {
        tableModel.moveDown(currentSelectionIndex);
        table.updateUI();
    }

    private void refreshSelections() {
        DeployBuildDialog dialog = null;

        if (getTopLevelAncestor() instanceof DeployBuildDialog) {
            dialog = (DeployBuildDialog) getTopLevelAncestor();
        }
        if (dialog != null) {
            node.getProject().setDeployInputFilter(radioFilter.isSelected());
            dialog.getSelectionPanel().setNodes(node.getProject().getAllInput());
        }
    }

    //
    private class LocalRadioListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                refreshSelections();
            }
        }

    }
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();

            if (source == buttonReset) {
                defaultTable();
            } else if (source == buttonRemove) {
                removeCurrent();
            } else if (source == buttonAdd) {
                addRow();
            } else if (source == buttonEdit) {
                editCurrent();
            } else if (source == buttonUp) {
                moveRowUp();
            } else if (source == buttonDown) {
                moveRowDown();
            }
        }

    }

    //
    class LocalTableModel extends AbstractTableModel {
        private Vector        rowVector = new Vector();

        public LocalTableModel() {}

        public int getRowCount() {
            int count = 0;

            if (rowVector != null) {
                count = rowVector.size();
            }
            return count;
        }

        public int getColumnCount() {
            return 2;
        }

        public String getColumnName(int columnIndex) {
            String name = new String();

            switch (columnIndex) {
            case 0:
                name = res.getString("Text_to_find");
                break;
            case 1:
                name = res.getString("Replace_with");
                break;
            }
            return name;
        }

        public Class getColumnClass(int columnIndex) {
            Class  columnClass = null;
            Object value = getValueAt(0, columnIndex);

            if (value != null) {
                columnClass = value.getClass();
            }
            return columnClass;
        }

        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Object value = null;

            if (!isTableEmpty()) {
                LocalRow  row = (LocalRow) rowVector.elementAt(rowIndex);
                LocalCell cell = new LocalCell(columnIndex, row);

                value = cell.getValue();
            }
            return value;
        }

        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            if (!isTableEmpty()) {
                LocalRow  row = (LocalRow) rowVector.elementAt(rowIndex);
                LocalCell cell = new LocalCell(columnIndex, row);

                cell.setValue(aValue);
                fireTableCellUpdated(columnIndex, rowIndex);
            }
        }

        protected LocalRow getRow(int rowIndex) {
            LocalRow row = null;

            if (!isTableEmpty()) {
                if (rowVector.size() > rowIndex) {
                    row = (LocalRow) rowVector.elementAt(rowIndex);
                }
            }
            return row;
        }

        protected void saveMap() {
            OtterProject project = node.getProject();
            int          rowCount = rowVector.size();
            String[][]   map = new String[rowCount][2];
            LocalRow     row = null;

            for (int i = 0; i < rowCount; i++) {
                row = (LocalRow) rowVector.elementAt(i);
                map[i][0] = row.getFind();
                map[i][1] = row.getReplaceWith();
            }
            project.setPackageMap(map);
        }

        protected void populateModel() {
            OtterProject project = node.getProject();
            String[][]   map = project.getPackageMap();

            if (map != null) {
                int rowCount = map.length;

                for (int i = 0; i < rowCount; i++) {
                    addRow(map[i][0], map[i][1]);
                }
            }
        }

        // /
        // /
        private boolean isTableEmpty() {
            boolean empty = true;

            if (rowVector != null) {
                if (rowVector.size() > 0) {
                    empty = false;
                }
            }
            return empty;
        }

        private void addRow(String f, String p) {
            LocalRow newRow = null;

            newRow = new LocalRow(f, p);
            rowVector.addElement(newRow);
            int size = rowVector.size();

            fireTableDataChanged();
        }

        private void removeRow(int index) {
            if (index < rowVector.size()) {
                rowVector.removeElementAt(index);
                fireTableDataChanged();
            }
        }

        private void moveUp(int index) {
            if (index > 0 && index < rowVector.size()) {
                Object current = rowVector.elementAt(index);
                Object previous = rowVector.elementAt(index - 1);

                rowVector.setElementAt(current, index - 1);
                rowVector.setElementAt(previous, index);
                fireTableDataChanged();
            }
        }

        private void moveDown(int index) {
            if (index < (rowVector.size() - 1)) {
                Object current = rowVector.elementAt(index);
                Object next = rowVector.elementAt(index + 1);

                rowVector.setElementAt(current, index + 1);
                rowVector.setElementAt(next, index);
                fireTableDataChanged();
            }
        }

        private void removeAllRows() {
            rowVector.removeAllElements();
            fireTableDataChanged();
        }

    }

    //
    private class LocalRow {
        private String find = new String();
        private String replaceWith = new String();

        public LocalRow(String f, String rw) {
            find = f;
            replaceWith = rw;
        }

        public String getFind() {
            return find;
        }

        public void setFind(String f) {
            find = f;
            saveToProject();
        }

        public String getReplaceWith() {
            return replaceWith;
        }

        public void setReplaceWith(String rw) {
            replaceWith = rw;
            saveToProject();
        }

        private void saveToProject() {}

    }

    //
    private class LocalCell {
        private LocalRow row;
        private int      column;

        public LocalCell(int c, LocalRow r) {
            column = c;
            row = r;
        }

        public boolean setValue(Object value) {
            boolean set = true;

            switch (column) {
            case 0:
                row.setFind((String) value);
                break;
            case 1:
                row.setReplaceWith((String) value);
                break;
            default:
                set = false;
                break;
            }
            return set;
        }

        public Object getValue() {
            Object value = null;

            switch (column) {
            case 0:
                value = row.getFind();
                break;
            case 1:
                value = row.getReplaceWith();
                break;
            }
            return value;
        }

        // /
        // /
        protected LocalRow getRow() {
            return row;
        }

    }
}
