/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.wizard.xmlc;

// Kelp imports
import org.enhydra.kelp.common.swing.CellRendererWithToolTip;
import org.enhydra.kelp.common.swing.SwingTableSelectionEvent;
import org.enhydra.kelp.common.swing.SwingTableSelectionListener;

// Standard imports
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.beans.*;
import java.util.Vector;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahr
 */
public class MapPanel extends JPanel implements ListSelectionListener {

    //
    static ResourceBundle      res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    //
    protected static final int NO_SELECTION = -1;
    private JTable             table;
    private JScrollPane        scrollTable;
    private BorderLayout       layoutMain;
    private MapTableModel      tableModel = new MapTableModel();
    private int                currentSelectionIndex = NO_SELECTION;
    private Vector             swingTableSelectionListeners = new Vector();
    private String[][]         map = new String[0][2];

    /**
     * Constructor declaration
     *
     */
    public MapPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public synchronized void addSwingTableSelectionListener(SwingTableSelectionListener l) {
        if (!swingTableSelectionListeners.contains(l)) {
            swingTableSelectionListeners.addElement(l);
        }
    }

    public synchronized void removeSwingTableSelectionListener(SwingTableSelectionListener l) {
        if (swingTableSelectionListeners.contains(l)) {
            swingTableSelectionListeners.removeElement(l);
        }
    }

    /**
     * ListSelectionListener event
     */
    public void valueChanged(ListSelectionEvent e) {
        ListSelectionModel lsm = (ListSelectionModel) e.getSource();

        if (lsm.isSelectionEmpty()) {
            currentSelectionIndex = NO_SELECTION;
        } else {
            currentSelectionIndex = lsm.getMinSelectionIndex();
        }
        SwingTableSelectionListener listener = null;

        for (int i = 0; i < swingTableSelectionListeners.size(); i++) {
            listener =
                (SwingTableSelectionListener) swingTableSelectionListeners.elementAt(i);
            listener.onSwingTableSelection(new SwingTableSelectionEvent(this,
                    currentSelectionIndex));
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected int getCurrentSelectionIndex() {
        return currentSelectionIndex;
    }

    protected String getSelectedFolder() {
        String folder = null;

        if (currentSelectionIndex > -1) {
            folder = tableModel.getRow(currentSelectionIndex).getFolder();
        }
        return folder;
    }

    protected void setSelectedFolder(String folder) {
        if (currentSelectionIndex > -1) {
            tableModel.getRow(currentSelectionIndex).setFolder(folder);
            tableModel.fireTableDataChanged();
        }
    }

    protected String getSelectedPackageName() {
        String pack = null;

        if (currentSelectionIndex > -1) {
            pack = tableModel.getRow(currentSelectionIndex).getPackageName();
        }
        return pack;
    }

    protected void setSelectedPackageName(String pack) {
        if (currentSelectionIndex > -1) {
            tableModel.getRow(currentSelectionIndex).setPackageName(pack);
            tableModel.fireTableDataChanged();
        }
    }

    protected void updateSelectedRow(String folder, String pack) {
        if (currentSelectionIndex > -1) {
            tableModel.getRow(currentSelectionIndex).setFolder(folder);
            tableModel.getRow(currentSelectionIndex).setPackageName(pack);
            tableModel.fireTableDataChanged();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param f
     * @param p
     */
    protected void addRow(String f, String p) {
        tableModel.addRow(f, p);
    }

    /**
     * Method declaration
     *
     *
     * @param index
     */
    protected void removeRow(int index) {
        tableModel.removeRow(index);
    }

    /**
     * Method declaration
     *
     *
     * @throws Exception
     */
    private void jbInit() throws Exception {
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        table = (JTable) Beans.instantiate(getClass().getClassLoader(),
                                           JTable.class.getName());
        scrollTable = new JScrollPane(table);
        table.setToolTipText(new String());
        table.sizeColumnsToFit(JTable.AUTO_RESIZE_ALL_COLUMNS);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
        table.setColumnSelectionAllowed(false);
        scrollTable.getViewport().add(table, BorderLayout.CENTER);
        ListSelectionModel rowSM = table.getSelectionModel();

        rowSM.addListSelectionListener(this);
        this.setPreferredSize(new Dimension(620, 200));
        this.setLayout(layoutMain);
        this.add(scrollTable, BorderLayout.CENTER);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String[][] getMap() {
        tableModel.readMap();
        return map;
    }

    /**
     * Method declaration
     *
     *
     * @param m
     */
    public void setMap(String[][] m) {
        map = m;
        tableModel.populateModel();
        table.setModel(tableModel);
    }

    /**
     * Method declaration
     *
     */
    private void pmInit() {
        tableModel = new MapTableModel();
        table.setModel(tableModel);
        table.setDefaultRenderer(String.class, new CellRendererWithToolTip());
        tableModel.addTableModelListener(table);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.getTableHeader().setUpdateTableInRealTime(false);
        table.getTableHeader().setReorderingAllowed(false);
    }

    /**
     * Class declaration
     *
     *
     */
    class MapTableModel extends AbstractTableModel {
        private Vector rowVector = new Vector();

        public MapTableModel() {}

        public int getRowCount() {
            int count = 0;

            if (rowVector != null) {
                count = rowVector.size();
            }
            return count;
        }

        public int getColumnCount() {
            return 2;
        }

        /**
         * Method declaration
         *
         *
         * @param columnIndex
         *
         * @return
         */
        public String getColumnName(int columnIndex) {
            String name = new String();

            switch (columnIndex) {
            case 0:
                name = res.getString("Source_Directory");
                break;
            case 1:
                name = res.getString("Package");
                break;
            }
            return name;
        }

        /**
         * Method declaration
         *
         *
         * @param columnIndex
         *
         * @return
         */
        public Class getColumnClass(int columnIndex) {
            Class  columnClass = null;
            Object value = getValueAt(0, columnIndex);

            if (value != null) {
                columnClass = value.getClass();
            }
            return columnClass;
        }

        /**
         * Method declaration
         *
         *
         * @param rowIndex
         * @param columnIndex
         *
         * @return
         */
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        /**
         * Method declaration
         *
         *
         * @param rowIndex
         * @param columnIndex
         *
         * @return
         */
        public Object getValueAt(int rowIndex, int columnIndex) {
            Object value = null;

            if (!isTableEmpty()) {
                MapRow  row = (MapRow) rowVector.elementAt(rowIndex);
                MapCell cell = new MapCell(columnIndex, row);

                value = cell.getValue();
            }
            return value;
        }

        /**
         * Method declaration
         *
         *
         * @param aValue
         * @param rowIndex
         * @param columnIndex
         */
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            if (!isTableEmpty()) {
                MapRow  row = (MapRow) rowVector.elementAt(rowIndex);
                MapCell cell = new MapCell(columnIndex, row);

                cell.setValue(aValue);
                fireTableCellUpdated(columnIndex, rowIndex);
            }
        }

        /**
         * Method declaration
         *
         *
         * @param rowIndex
         *
         * @return
         */
        protected MapRow getRow(int rowIndex) {
            MapRow row = null;

            if (!isTableEmpty()) {
                if (rowVector.size() > rowIndex) {
                    row = (MapRow) rowVector.elementAt(rowIndex);
                }
            }
            return row;
        }

        protected void readMap() {
            int rowCount = rowVector.size();

            map = new String[rowCount][2];
            MapRow row = null;

            for (int i = 0; i < rowCount; i++) {
                row = (MapRow) rowVector.elementAt(i);
                map[i][0] = row.getFolder();
                map[i][1] = row.getPackageName();
            }
        }

        protected void populateModel() {
            while (getRowCount() > 0) {
                removeRow(0);
            }
            if (map != null) {
                int rowCount = map.length;

                for (int i = 0; i < rowCount; i++) {
                    addRow(map[i][0], map[i][1]);
                }
            }
        }

        // /
        // /
        private boolean isTableEmpty() {
            boolean empty = true;

            if (rowVector != null) {
                if (rowVector.size() > 0) {
                    empty = false;
                }
            }
            return empty;
        }

        private void addRow(String f, String p) {
            MapRow newRow = null;

            newRow = new MapRow(f, p);
            rowVector.addElement(newRow);
            int size = rowVector.size();

            fireTableDataChanged();
        }

        private void removeRow(int index) {
            rowVector.removeElementAt(index);
            fireTableDataChanged();
        }

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class MapRow {
        private String fromFolder = new String();
        private String toPackage = new String();

        /**
         * Constructor declaration
         *
         *
         * @param f
         * @param p
         */
        public MapRow(String f, String p) {
            fromFolder = f;
            toPackage = p;
        }

        /**
         * Method declaration
         *
         *
         * @return
         */
        public String getFolder() {
            return fromFolder;
        }

        /**
         * Method declaration
         *
         *
         * @param f
         */
        public void setFolder(String f) {
            fromFolder = f;
        }

        /**
         * Method declaration
         *
         *
         * @return
         */
        public String getPackageName() {
            return toPackage;
        }

        /**
         * Method declaration
         *
         *
         * @param p
         */
        public void setPackageName(String p) {
            toPackage = p;
        }

    }

    /**
     * Class declaration
     *
     *
     * @author
     * @version %I%, %G%
     */
    class MapCell {
        private MapRow row;
        private int    column;

        /**
         * Constructor declaration
         *
         *
         * @param c
         * @param r
         */
        public MapCell(int c, MapRow r) {
            column = c;
            row = r;
        }

        /**
         * Method declaration
         *
         *
         * @param value
         *
         * @return
         */
        public boolean setValue(Object value) {
            boolean set = true;

            switch (column) {
            case 0:
                row.setFolder((String) value);
                break;
            case 1:
                row.setPackageName((String) value);
                break;
            default:
                set = false;
                break;
            }
            return set;
        }

        /**
         * Method declaration
         *
         *
         * @return
         */
        public Object getValue() {
            Object value = null;

            switch (column) {
            case 0:
                value = row.getFolder();
                break;
            case 1:
                value = row.getPackageName();
                break;
            }
            return value;
        }

        //
        // PROTECTED
        //
        protected MapRow getRow() {
            return row;
        }

    }
}
