/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.properties;

// JBuilder imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.help.HelpTopic;
import com.borland.primetime.help.ZipHelpTopic;
import com.borland.primetime.properties.PropertyPage;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.wizard.xmlc.XMLCPropertyPanel;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// Standard imports
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.beans.Beans;

//
public class XMLCProjectPropertyPage extends PropertyPage {

    // string not to be resourced
    private final String      HELP_LINK = KelpInfo.HELP_DOC + "#xmlcproject"; // nores

    //
    private OtterProject      otterProject;
    private XMLCPropertyPanel propertyPanel;
    private BorderLayout      layoutMain;

    public XMLCProjectPropertyPage(Object topic) {
        if (topic instanceof JBProject) {
            PrimeProject primeProject = null;

            otterProject = new PrimeProject((JBProject) topic);
            try {
                jbInit();
                pmInit();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void readProperties() {
        propertyPanel.readProperties();
        enableUI(true);
    }

    public HelpTopic getHelpTopic() {
        ZipHelpTopic topic = null;

        topic = new ZipHelpTopic(null, HELP_LINK);
        return topic;
    }

    public void writeProperties() {
        propertyPanel.writeProperties();
    }

    public XMLCProjectPropertyPage() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //
    // PRIVATE
    //
    private void enableUI(boolean enabled) {}

    private void jbInit() throws Exception {
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        propertyPanel =
            (XMLCPropertyPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  XMLCPropertyPanel.class.getName());
        propertyPanel.setPreferredSize(new Dimension(0, 0));
        this.setLayout(layoutMain);
        this.add(propertyPanel, BorderLayout.CENTER);
    }

    private void pmInit() {
        propertyPanel.setProject(otterProject);
    }

}
