/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Devel doper of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.wizard;

// JBuilder imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.FileNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.ide.BrowserListener;
import com.borland.primetime.ide.NodeViewer;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.Project;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;
import com.borland.primetime.vfs.InvalidUrlException;

// Kelp imports
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.jbuilder.wizard.AbstractBasicWizard;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeTemplateNode;
import org.enhydra.kelp.common.wizard.deploy.DeployBuildDialog;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;

// Standard imports
import javax.swing.JOptionPane;
import java.io.File;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class DeployWizard extends AbstractBasicWizard {
    //
    public static WizardAction action = new DeployWizardAction();

    /**
     * Method declaration
     *
     *
     * @param majorVersion
     * @param minorVersion
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        WizardManager.registerWizardAction(action);
        Browser.addStaticBrowserListener(browserListener);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public static String getName() {
        return res.getString("Enhydra_Deployment");
    }

    //
    static class DeployWizardAction extends WizardAction {
        public DeployWizardAction() {
            super(ResUtil.format(res.getString("_0_dots"), getName()),
                  'X', res.getString("Deploy_content"),
                  IconSet.getSmallIcon(), IconSet.getLargeIcon(), false);
        }

        protected Wizard createWizard() {
            return new DeployWizard();
        }

        public void update(Object source) {
            setEnabled(AbstractBasicWizard.enableMenu);
        }

    }
    ;

    /**
     * Method declaration
     *
     *
     * @param host
     */
    public WizardPage invokeWizard(WizardHost host) {
        PrimeProject      primeProject = null;
        OtterNode[]       inputNodes = new OtterNode[0];
        DeployBuildDialog dlg = null;

        setNativeProject((JBProject) host.getBrowser().getProjectView().getActiveProject());
        if (getNativeProject() == null) {

            // do nothing
        } else {
            primeProject = new PrimeProject(getNativeProject());
            primeProject.save();
            inputNodes = primeProject.getAllInput();
            saveNodes(inputNodes);
            if (verifyAllClassPaths(host.getDialogParent())) {
                dlg = new DeployBuildDialog(host.getDialogParent(),
                                            getName(), true);
                dlg.setProject(primeProject);
                dlg.setNodes(inputNodes);
                dlg.show();
            }
        }
        return super.invokeWizard(host);
    }

    // recursively read through nodes.
    public String getHelpTopic() {
        return new String();
    }

}
