/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;

// JBuilder imports
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.wizard.codegen.WebAppUtil;
import org.enhydra.kelp.common.wizard.codegen.CodeGenException;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.wizard.AbstractCodeGenWizard;

// Standard imports
import java.util.Vector;
import javax.swing.JOptionPane;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class WebAppWizard extends AbstractCodeGenWizard {

    // string not to be resourced
    private final String       HELP_LINK = KelpInfo.HELP_DOC
                                           + "#appwizard";    // nores

    //
    public static WizardAction WIZARD_EnhydraAppWizard = new WizardAction(WebAppUtil.getDisplayName(),
            'X',
            ResUtil.format(res.getString("Creates_a_new_0_"), WebAppUtil.getDisplayName()),
            IconSet.getSmallIcon(), IconSet.getLargeIcon(), true,
            Constants.ENHYDRA) {

        /**
         * Method declaration
         *
         *
         * @return
         */
        protected Wizard createWizard() {
            WebAppWizard wizard = new WebAppWizard();

            return wizard;
        }

        /**
         * Method declaration
         *
         *
         * @param source
         */
        public void update(Object source) {
            setEnabled(true);
        }

    };

    /**
     * Method declaration
     *
     *
     * @param majorVersion
     * @param minorVersion
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        WizardManager.registerWizardAction(WIZARD_EnhydraAppWizard);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getDisplayName() {
        return WebAppUtil.getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getHelpTopic() {
        return HELP_LINK;
    }

    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @return
     */
    public WizardPage invokeWizard(WizardHost host) {
        WizardPage   page = null;
        OtterProject otterProject = null;
        WebAppUtil   util = null;

        if (KelpInfo.verifyIDEClassPath(host.getDialogParent())) {
            otterProject = initProject(host);
            if (otterProject != null) {
                try {
                    util = new WebAppUtil(otterProject);
                    page = invokeCodeGenWizard(host, util);
                } catch (CodeGenException e) {
                    JOptionPane.showMessageDialog(host.getDialogParent(),
                                                  e.getMessage(),
                                                  ResUtil.format(res.getString("_0_Wizard"), WebAppUtil.getDisplayName()),
                                                  JOptionPane.ERROR_MESSAGE);
                }
            }
        }
        return page;
    }

    /**
     *
     * Method declaration
     *
     *
     * @return
     */
    public String[] getSteps() {
        Vector   steps = new Vector();
        String[] array = new String[0];

        if (!ToolBoxInfo.isEnhydra3()) {
            if (System.getProperty(Constants.JAVA_VERSION).startsWith(Constants.JDK13)) {
                steps.addElement(res.getString("readmeWeb0"));
            }
        }
        steps.addElement(res.getString("readmeWeb1"));
        steps.addElement(res.getString("readmeWeb2"));
        steps.addElement(res.getString("readmeWeb3"));
        steps.addElement(res.getString("readmeWeb4"));
        steps.addElement(res.getString("readmeWeb5"));
        array = (String[]) steps.toArray(array);
        steps.removeAllElements();
        return array;
    }

}
