/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.node;

// ToolBox imports
import org.enhydra.tool.common.TemplateTool;
import org.enhydra.tool.common.TemplateFile;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.Replacement;

// JDevelopers imports
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Folder;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.Url;
import borland.jbuilder.addin.Workspace;
import borland.jbuilder.addin.WorkspaceEvent;
import borland.jbuilder.addin.WorkspaceListener;
import oracle.jdeveloper.addin.BaseNode;
import oracle.jdeveloper.addin.CustomContainer;
import oracle.jdeveloper.addin.NodeFactory;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * WinFolderNode represents a folder in a Win32 JBuilder project file.
 *
 * @author Paul Mahar
 */
public class OracleFolderNode extends OracleNode implements OtterFolderNode {

    // string not to be resourced
    private final String              LATTE_FOLDER = "latte:folder//";    // nores
    private static String[]           folders = new String[0];
    private static String[]           projects = new String[0];

    //
    private static WorkaroundFactory  workaroundFactory = null;
    private static WorkaroundListener workaroundListener = null;

    /**
     * Create a new folder in a project.
     *
     * @param otterProject
     * The Win32 JBuilder project.
     * @param otterNode
     * The parent node for the folder
     * @param name
     * The name for the new folder.
     *
     */
    public OracleFolderNode(OtterNode otterNode, String name) {
        Url     url = null;
        Project nativeProject = null;
        Node    nativeNode = null;

        name = fixName(name, otterNode.getProject());
        nativeProject = (Project) otterNode.getProject().getNativeProject();
        if (name.trim().length() == 0) {
            nativeNode = nativeProject;
        } else {
            url = new Url(LATTE_FOLDER + name);
            nativeNode = nativeProject.findNode(name);
            if (nativeNode == null) {
                url = new Url(name + '.' + WorkaroundFactory.EXT);
                loadWorkaround();
                nativeNode = nativeProject.createNode(url, true, false);
                nativeNode.BeginUpdate();
                nativeNode.save(true);
                nativeNode.rename(name);
                nativeNode.save(true);
                nativeNode.EndUpdate();
                unloadWorkaround(nativeNode);
            }
        }
        setNativeNode(nativeNode);
    }

    private void unloadWorkaround(Node newNode) {
        IdeEnvironment ide = null;

        OracleFolderNode.workaroundListener.addFolderNode(newNode);
        if (OracleFolderNode.workaroundFactory != null) {
            ide = IdeEnvironment.getInstance();
            ide.getProjectManager().removeNodeFactory(workaroundFactory);
        }
    }

    private String fixName(String inName, OtterProject otterProject) {
        String outName = new String();
        String projectName = new String();

        outName = new String(inName);
        outName = outName.replace('/', '.');
        outName = outName.replace('\\', '.');
        projectName = otterProject.getFilePath();
        projectName = projectName.substring(projectName.lastIndexOf('/') + 1);
        projectName = projectName.substring(0, projectName.length() - 4);
        if (outName.equalsIgnoreCase(projectName)) {
            outName = new String();
        } else if (outName.toLowerCase().startsWith(projectName.toLowerCase()
        + '.')) {
            outName = outName.substring(projectName.length() + 1);
        }
        outName = uniqueName(outName, otterProject.getFilePath());
        return outName;
    }

    private String uniqueName(String in, String projectPath) {
        String    out = new String(in);
        ArrayList folderList = null;
        ArrayList projectList = null;
        int       count = 1;
        boolean   update = true;

        folderList = new ArrayList(Arrays.asList(OracleFolderNode.folders));
        projectList = new ArrayList(Arrays.asList(OracleFolderNode.projects));
        while (folderList.contains(out)) {
            if (projectList.get(folderList.lastIndexOf(out)).equals(projectPath)) {
                update = false;
                break;
            }
            out = new String(in + count);
            count++;
        }
        if (update) {
            folderList.add(out);
            projectList.add(projectPath);
            folderList.trimToSize();
            projectList.trimToSize();
            OracleFolderNode.folders = new String[folderList.size()];
            OracleFolderNode.projects = new String[projectList.size()];
            OracleFolderNode.folders =
                (String[]) folderList.toArray(OracleFolderNode.folders);
            OracleFolderNode.projects =
                (String[]) projectList.toArray(OracleFolderNode.projects);
            folderList.clear();
            projectList.clear();
        }
        return out;
    }

    private void loadWorkaround() {
        IdeEnvironment ide = null;

        ide = IdeEnvironment.getInstance();
        if (OracleFolderNode.workaroundFactory == null) {
            OracleFolderNode.workaroundFactory = new WorkaroundFactory();
            OracleFolderNode.workaroundListener = new WorkaroundListener();
            ide.getWorkspace().addWorkspaceListener(OracleFolderNode.workaroundListener);
        }
        ide.getProjectManager().addNodeFactory(workaroundFactory, true);
    }

}

//
class WorkaroundFactory extends NodeFactory {

    // string not to be resouced
    public static final String EXT = "folder";    // nores

    public void registerViewers() {}

    public BaseNode createNode(String string) {
        WorkaroundFolder node = null;

        node = new WorkaroundFolder(this, string);
        return node;
    }

    public String likeURL(String string) {
        return string;    // WorkaroundFolder.fixUrl(string);
    }

    public String getFileExt() {
        return getContainerExt();
    }

    public String getContainerExt() {
        return WorkaroundFactory.EXT;
    }

    public String getCaption() {
        return null;
    }

    public String getURLforStorage(String string) {
        return null;
    }

}

//
class WorkaroundFolder extends CustomContainer {
    public WorkaroundFolder(NodeFactory nf, String newURL) {
        super(nf, newURL);
        setModified(true);
        save(true, true);
    }

    public boolean isContainer() {
        return true;
    }

    public String getType() {
        return Folder.FOLDER_TYPE;
    }

}
class WorkaroundListener implements WorkspaceListener {
    Vector projectPaths = new Vector();
    Vector folderNodes = new Vector();

    protected void addFolderNode(Node node) {
        if (!folderNodes.contains(node)) {
            folderNodes.addElement(node);
        }
    }

    public void workspaceClosing(WorkspaceEvent e) {
        Workspace workspace = e.getWorkspace();

        for (int i = 0; i < folderNodes.size(); i++) {
            Project cursor = null;
            Node    node = (Node) folderNodes.elementAt(i);

            cursor = workspace.getOwningProject(node);
            if ((cursor == null)
                    || projectPaths.contains(cursor.getFilename())) {}
            else {
                projectPaths.addElement(cursor.getFilename());
            }
        }
    }

    public void workspaceClosed(WorkspaceEvent e) {
        for (int i = 0; i < projectPaths.size(); i++) {
            String path = (String) projectPaths.elementAt(i);

            addFolderTypes(path);
        }
    }

    private void addFolderTypes(String path) {
        int[] parents = new int[0];

        parents = readParents(path);
        addTypesForParents(path, parents);
    }

    private void addTypesForParents(String path, int[] parents) {
        Vector         repVector = new Vector();
        TemplateTool   tool = null;
        TemplateFile[] template = new TemplateFile[0];

        if (parents.length >= 1) {
            tool = new TemplateTool();
            template = new TemplateFile[1];
            template[0] = new TemplateFile(path);
            template[0].setOutput(new File(path + '.' + Constants.TYPE_TMP));
            tool.setTemplates(template);
        }
        for (int i = 0; i < parents.length; i++) {
            Replacement rep = null;
            String      lookFor = new String();
            String[]    replaceWith = new String[2];

            // strings not to be resourced
            lookFor = "sys[" + parents[i] + "].Parent=0";    // nores
            replaceWith[0] = lookFor;
            replaceWith[1] = "sys[" + parents[i] + "].Type=Folder";    // nores
            try {
                rep = new Replacement(lookFor, replaceWith);
                repVector.addElement(rep);
            } catch (ToolException e) {
                e.printStackTrace();
            }
        }
        if (tool != null) {
            Replacement[] reps = new Replacement[repVector.size()];

            reps = (Replacement[]) repVector.toArray(reps);
            repVector.clear();
            tool.setReplacements(reps);
            try {
                File[] temp = tool.createOutput();
                File   jpr = new File(path);

                jpr.delete();
                temp[0].renameTo(jpr);
            } catch (ToolException e) {
                e.printStackTrace();
            }
        }
    }

    private int[] readParents(String path) {
        int[]          parents = new int[0];
        String         lineParent = null;
        Vector         parentVector = new Vector();
        BufferedReader reader = null;
        File           file = null;

        file = new File(path);
        try {
            reader = new BufferedReader(new FileReader(file));
            String line = reader.readLine();

            while (line != null) {
                lineParent = readParentFromLine(line);
                if ((lineParent == null)
                        || parentVector.contains(lineParent)) {}
                else {
                    int intParent = -1;

                    intParent = Integer.parseInt(lineParent);
                    if (intParent > 0) {
                        parentVector.addElement(lineParent);
                    }
                }
                line = reader.readLine();
            }
            reader.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        parents = new int[parentVector.size()];
        for (int i = 0; i < parentVector.size(); i++) {
            parents[i] = Integer.parseInt((String) parentVector.elementAt(i));
        }
        return parents;
    }

    private String readParentFromLine(String line) {
        String       lineParent = null;
        int          index = -1;

        // strings not to be resourced
        final String key = new String("].Parent=");    // nores

        index = line.indexOf(key);
        if (index > -1) {
            lineParent = line.substring(index + key.length());
        }
        return lineParent;
    }

}
