/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.wizard;

// JDeveloper imports
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.WizardManager;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.HtmlNode;
import borland.jbuilder.addin.Project;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.wizard.xmlc.XMLCBuildDialog;
import org.enhydra.kelp.jdeveloper.node.OracleProject;

// Standard imports
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Image;
import java.io.File;
import java.util.Vector;
import java.lang.System;

// /
import java.net.URLClassLoader;
import java.net.URL;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.security.*;

/**
 * This is the class that gets registered with the IDE through the
 * jbuilder.properties file.
 */
public class XMLCWizard extends AbstractBasicWizard {
    private String        title = null;
    private int           option = 0;

    /**
     * Constructor declaration
     *
     */
    public XMLCWizard() {}

    /**
     * Wizard interface implementation, run the wizard in response to this call.
     * @param ide Access to all AddIn interfaces
     * @param context Current node, project, browser
     */
    public void invoke(IdeEnvironment ide, Context context) {
        Frame frame = null;

        frame = ide.getIdeFrame();
        nativeProject = context.getProject();
        String ver = System.getProperty(Constants.JAVA_VERSION);

        if (ver.startsWith(Constants.JDK11)) {
            JOptionPane.showMessageDialog(frame,
                                          ResUtil.format(res.getString("Kelp_requires_Java_2"),
                                          ver));
            nativeProject = null;
        }
        if (verifyAllClassPaths(ide.getIdeFrame())) {
            nativeProject.save(false);    // save all file changes in project
            try {
                XMLCBuildDialog dlg = null;

                // Create Dialog
                dlg = new XMLCBuildDialog(frame);
                dlg.setModal(true);
                dlg.setTitle(getName());

                // Create Project Node
                OracleProject oracleProject = null;

                oracleProject = new OracleProject();
                oracleProject.setNativeNode(nativeProject);

                // show dialog
                dlg.setProject(oracleProject);
                dlg.show();
            } catch (Exception e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog(ide.getIdeFrame(),
                                              ResUtil.format(res.getString("Unable_to_load_wizard"),
                                              e.getMessage()));
            }
            frame.requestFocus();
        }
    }

    /**
     * Wizard interface method, returns the name of this wizard.
     * @return Name of the wizard
     */
    public String getName() {
        return res.getString("XMLC_Compiler");
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getTitle() {
        return this.title;
    }

    /**
     * Method declaration
     *
     *
     * @param newTitle
     */
    public void setTitle(String newTitle) {
        this.title = newTitle;
    }

    // read only properties

    /**
     * Method declaration
     *
     *
     * @return
     */
    public int getOption() {
        return option;
    }

    /**
     * Method declaration
     *
     *
     * @param newOption
     */
    private void setOption(int newOption) {
        option = newOption;
    }

}
