/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: KeyValuePair.java,v 1.7.14.1 2000/10/19 17:59:08 jasona Exp $
 */





package com.lutris.appserver.admin.business;


/**
 * A record that stores key/value pairs.
 * If the value of a key was origionally an array of strings, then
 * the array will be joined together into one string with newlines.
 * Any newlines in the origional strings will be turned into "\n".
 * If this has happened, the flag isArray will be set, and the
 * string should be shown to the user in a multi-line text editing box.
 * 
 * @version	$Revision: 1.7.14.1 $
 * @author	Andy John
 */
public class KeyValuePair {

    /**
     *  The name of the key. For example, if the line from the config
     *  file was "App.param1 = some_value", then this variable would
     *  be set to "App.param1". If the line was 
     *  "App.param1[] = 9.95, 14.99", then this variable would also
     *  be set to "App.param1".
     */    
    public String key;

    /**
     *  The value of the key. For example, if one line from a config
     *  file was "App.param1 = some_value", then this variable would
     *  be set to "some_value". If the line was 
     *  "App.param1[] = 9.95, 14.99", then this variable would be set
     *  to "9.95\n14.99" (where \n is a newline character).
     */    
    public String value;

    /**
     *  In the config file, was this origionally an array?
     *  This is only needed to know how to display the string. 
     *  The array will already have been turned into a single String.
     *  For example, if the line in the config file was
     *  "App.param1 = some_value" this would be false. If the line was 
     *  "App.param1[] = 9.95, 14.99", then this variable would be true.
     */
    public boolean isArray;



    /**
     *  Constructor. Only the class <CODE>ConfEditor</CODE> needs to
     *  call this.
     */
    protected KeyValuePair(String key, String value, boolean isArray) {
        this.key = key; 
        this.value = value;
        this.isArray = isArray;
    }

}



