/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ServletIOException.java,v 1.4.12.1 2000/10/19 17:59:09 jasona Exp $
 */

package org.enhydra.servlet;

import java.io.IOException;
import java.net.Socket;

/**
 * IOException derived class that is thrown when the servlet encounters
 * an I/O error when talking to a client.  This allows connection methods
 * to determine how the error should be handled, even though it was generated
 * by a call that occured in servlet code.
 */
public class ServletIOException extends IOException {
    /**
     * Static method that does the approriate conversion from an IOException
     * to a ServletIOException If the exception is already an
     * ServletIOException, it is simple rethrown.  If it is another type of
     * IOException, it is converted to a ServletIOException.
     *
     * @param except The exception to rethrow or encapsulate.
     * @return An exception ready to throw.
     */
    public static IOException rethrow(IOException except)
            throws IOException {
        if (except instanceof ServletIOException) {
            return except;
        } else {
            return new ServletIOException(except);
        }
    }


    /**
     * Flag that indicates that this came from an a I/O operation
     * rather than a generated exception.
     */
    private boolean trueIOException = false;

    /**
     * Construct a new exception from an existing IOException.
     *
     * @param except The exception to encapsulate.
     */
    public ServletIOException(IOException except) {
        super(except.getMessage());
        trueIOException = true;
    }

    /**
     * Construct a new exception with a specific message.
     *
     * @param msg The error message
     */
    public ServletIOException(String msg) {
        super(msg);
    }

    /**
     * Did this exception come from an a I/O operation
     * rather than a generated exception.
     * @return True if its an actual I/O exception. 
     */
    public boolean isTrueIOException() {
        return trueIOException;
    }

}
