/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EnhydraDirectorFunction.java,v 1.3.2.1.2.1 2000/10/19 17:59:09 jasona Exp $
 */

package org.enhydra.servlet.connectionMethods.EnhydraDirector;

/**
 * Interface for pluggable EnhydraDirector protocol functions.
 * <P>
 * EnhydraDirector function classes are only instantiated when their
 * function number is requested for a particular connection.  Each
 * function is permanently hard-coded into a dispatch method in
 * EnhydraDirectorProtocol.
 */
public interface EnhydraDirectorFunction
{
    /**
     * Generic run method for an EnhydraDirector protocol function.
     * The function argument data is retrieved from the connection
     * using <code>getFuncionArg()</code>.  The success/failure 
     * status is set using <code>conn.setFunctionStatus()</code>,
     * and the result data is returned via a call to
     * <code>conn.setFunctionData()</code>.
     * 
     * @param input Raw function argument data from the client.
     * @param conn Connection that received the function call.
     * @return True to return success to client, False to fail.
     */
    public void run(EnhydraDirectorProtocol conn);
}

