/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Queue.java,v 1.4.12.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.connectionMethods.https;

import com.lutris.util.*;

/*
 * Circular queue data structure implemented with an array of references
 * to objects.
 * It is the responsibility of the users to synchronize access to the queue.
 * The main functionality of this class was moved to
 * com.lutris.util.CircularQueue so it could be used by the apache
 * connection method.
 *
 * @version $Revision: 1.4.12.1 $
 * @auther Mark Sanguinetti
 * @see com.lutris.util.CircularQueue
 */

class Queue extends CircularQueue {

    /**
     * Creates a queue object of size s (s objects).
     *
     * @param s The maximum number of elements to be queued.
     */
    public Queue(int s) { 
	super(s);
    }

    /**
     * Checks to see if the queue is empty.
     *
     * @return true if queue is empty, false otherwise.
     */
    public boolean empty() {
        return isEmpty();
    }

    /*
     * DEBUG 
     * Prints the status (sIndex and  rIndex) and the contents of the 
     * queue to standard out.
     */
    private void dump() {
        System.out.println("Queue dump:");
        System.out.println("sIndex: " + sIndex + " Rindex: " + rIndex);
        for (int i = 0; i < qSize ; i++) {
            if (queue[i] != null)
                System.out.println("index: " + i 
                                   + " Object: " + queue[i].toString());
        }
    }
} 

