/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PrintTransactionRequest.java,v 1.4.12.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.filter;

import java.io.PrintWriter;
import java.io.IOException;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.ServletInputStream;
import org.enhydra.servlet.filter.*;


/*
    Print everything out.
*/
public class PrintTransactionRequest extends RequestFilter {
    
    private String          prefix;
    private PrintWriter     out;
    private boolean         logStream;

    public PrintTransactionRequest (HttpServletRequest req, 
                String prefix, PrintWriter out, boolean logStream) {
        super(req);
        this.prefix = prefix;
        this.out = out;
        this.logStream = logStream;
    }

    public int getContentLength() {
        int len = myRequest.getContentLength();
        out.println(prefix + "getContentLength() = " + len);
        out.flush();
        return len; 
    }

    public String getContentType() {
        String type = myRequest.getContentType();
        out.println(prefix + "getContentType() = " + type);
        out.flush();
        return type;
    }

    public String getProtocol() {
        String p = myRequest.getProtocol();
        out.println(prefix + "getProtocol() = " + p);
        out.flush();
        return p;
    }

    public String getScheme() {
        String s = myRequest.getScheme();
        out.println(prefix + "getScheme() = " + s);
        out.flush();
        return s;
    }

    public String getServerName() {
        String sn = myRequest.getServerName();
        out.println(prefix + "getServerName() = " + sn);
        out.flush();
        return sn;
    }

    public int getServerPort() {
        int port = myRequest.getServerPort();
        out.println(prefix + "getServerPort() = " + port);
        out.flush();
        return port;
    }

    public String getRemoteAddr() {
        String ra = myRequest.getRemoteAddr();
        out.println(prefix + "getRemoteAddr() = " + ra);
        out.flush();
        return ra;
    }

    public String getRemoteHost() {
        String rh = myRequest.getRemoteHost();
        out.println(prefix + "getRemoteHost() = " + rh);
        out.flush();
        return rh;
    }

    public String getRealPath(String path) {
        String rp = myRequest.getRealPath(path);
        out.println(prefix + "getRealPath(" + path + ") = " + path);
        out.flush();
        return rp;
    }

    public ServletInputStream getInputStream() throws IOException {
        ServletInputStream is;
        try {
            is = myRequest.getInputStream();
        } catch (IOException e) {
            out.println(prefix + "getInputStream() threw " + e);
            out.flush();
            throw(e);
        }
        out.println(prefix + "getInputStream() = " + is);
        out.flush();
        if (logStream) {
            String pre = prefix + "Input: ";
            return new PrintTransactionInputStream(pre, out, is);
        } else 
            return is;
    }

    public String getParameter(String name) {
        String p = myRequest.getParameter(name);
        out.println(prefix + "getParameter(" + name + ") = " + p);
        out.flush();
        return p;
    }

    public String[] getParameterValues(String name) {
        String[] pv = myRequest.getParameterValues(name);
        if (pv.length == 0) {
            out.println(prefix + "getParameterValues(" + name + ") = []");
            out.flush();
            return pv;
        } else {
            for (int i=0; i<pv.length; i++) 
                out.println(prefix + "getParameterValues(" + 
                            name + ") [" + i + "] = " + pv[i]);
            out.flush();
            return pv;
        }
    }

    public Enumeration getParameterNames() {
        String pre = prefix + "getParameterNames()";
        Enumeration enum = myRequest.getParameterNames();
        return new PrintTransactionStringEnum(pre, out, enum);
    }

    public Object getAttribute(String name) {
        Object o = myRequest.getAttribute(name);
        out.println(prefix + "getAttribute(" + name + ") = " + o);
        out.flush();
        return o;
    }

    //---------------------------------------------------------------

    public String getMethod() {
        String m = myRequest.getMethod();
        out.println(prefix + "getMethod() = " + m);
        out.flush();
        return m;
    }

    public String getRequestURI() {
        String u = myRequest.getRequestURI();
        out.println(prefix + "getRequestURI() = " + u);
        out.flush();
        return u;
    }

    public String getServletPath() {
        String sp = myRequest.getServletPath();
        out.println(prefix + "getServletPath() = " + sp);
        out.flush();
        return sp;
    }

    public String getPathInfo() {
        String pi = myRequest.getPathInfo();
        out.println(prefix + "getPathInfo() = " + pi);
        out.flush();
        return pi;
    }

    public String getPathTranslated() {
        String pt = myRequest.getPathTranslated();
        out.println(prefix + "getPathTranslated() = " + pt);
        out.flush();
        return pt;
    }

    public String getQueryString() {
        String qs = myRequest.getQueryString();
        out.println(prefix + "getQueryString() = " + qs);
        out.flush();
        return qs;
    }

    public String getRemoteUser() {
        String ru = myRequest.getRemoteUser();
        out.println(prefix + "getRemoteUser() = " + ru);
        out.flush();
        return ru;
    }

    public String getAuthType() {
        String at = myRequest.getAuthType();
        out.println(prefix + "getAuthType() = " + at);
        out.flush();
        return at;
    }

    public String getHeader(String name) {
        String h = myRequest.getHeader(name);
        out.println(prefix + "getHeader(" + name + ") = " + h);
        out.flush();
        return h;
    }

    public int getIntHeader(String name) {
        int h = myRequest.getIntHeader(name);
        out.println(prefix + "getIntHeader(" + name + ") = " + h);
        out.flush();
        return h;
    }

    public long getDateHeader(String name) {
        long dh = myRequest.getDateHeader(name);
        out.println(prefix + "getDateHeader(" + name + ") = " + dh);
        out.flush();
        return dh;
    }

    public Enumeration getHeaderNames() {
        String pre = prefix + "getHeaderNames()";
        Enumeration enum = myRequest.getHeaderNames();
        return new PrintTransactionStringEnum(pre, out, enum);
    }
}


