/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: RequestFilter.java,v 1.4.2.1.2.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.filter;

import java.io.IOException;
import java.io.BufferedReader;
import java.util.Enumeration;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletInputStream;
import javax.servlet.RequestDispatcher;
import org.apache.tomcat.core.RequestAdapter;
import org.apache.tomcat.util.MimeHeaders;


/**
 * This replaces the BasicTransactionRequest.
 *  This is part of the basic transaction filter. This is what wraps
 *  around the servlet request object. This class does nothing, every
 *  single call is a pass-through to the real request.
 *  This class exists so that it can be extended. You would then only
 *  need to override the methods you are interested in.
 *  All the other calls will be passed through by this class.
 *
 *  @author Andy John
 *  @author Shawn McMurdo
 */
public class RequestFilter implements HttpServletRequest, RequestAdapter {

    /**
     *  The original request. All calls will be passed through to
     *  this object. It is set by the constructor.
     */
    protected HttpServletRequest myRequest;

    /**
     * The RequestAdapter used by Tomcat.
     */
    protected RequestAdapter adapter;

    /**
     *  Constructor. You must pass in the original request. All the
     *  HttpServletRequest calls will be passed on to this object.
     *
     *  @param originalRequest The real HttpServletRequest, which all calls
     *  will be passed on to.
     *  @see javax.servlet.ServletRequest
     *  @see javax.servlet.http.HttpServletRequest
     */
    public RequestFilter(HttpServletRequest originalRequest) {
        myRequest = originalRequest;
	adapter = (RequestAdapter) originalRequest;
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public int getContentLength() {
        return myRequest.getContentLength();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getContentType() {
        return myRequest.getContentType();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getProtocol() {
        return myRequest.getProtocol();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getScheme() {
        return myRequest.getScheme();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getServerName() {
        return myRequest.getServerName();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public int getServerPort() {
        return myRequest.getServerPort();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getRemoteAddr() {
        return myRequest.getRemoteAddr();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getRemoteHost() {
        return myRequest.getRemoteHost();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getRealPath(String path) {
        return myRequest.getRealPath(path);
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public ServletInputStream getInputStream() throws IOException {
        return myRequest.getInputStream();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String getParameter(String name) {
        return myRequest.getParameter(name);
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public String[] getParameterValues(String name) {
        return myRequest.getParameterValues(name);
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public Enumeration getParameterNames() {
        return myRequest.getParameterNames();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the ServletRequest
     *  documentation.
     *
     *  @see javax.servlet.ServletRequest
     */
    public Object getAttribute(String name) {
        return myRequest.getAttribute(name);
    }

    //---------------------------------------------------------------


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getMethod() {
        return myRequest.getMethod();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getRequestURI() {
        return myRequest.getRequestURI();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getServletPath() {
        return myRequest.getServletPath();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getPathInfo() {
        return myRequest.getPathInfo();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getPathTranslated() {
        return myRequest.getPathTranslated();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getQueryString() {
        return myRequest.getQueryString();
    }

    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public void setQueryString(String queryString) {
        adapter.setQueryString(queryString);
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getRemoteUser() {
        return myRequest.getRemoteUser();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getAuthType() {
        return myRequest.getAuthType();
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public String getHeader(String name) {
        return myRequest.getHeader(name);
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public int getIntHeader(String name) {
        return myRequest.getIntHeader(name);
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public long getDateHeader(String name) {
        return myRequest.getDateHeader(name);
    }


    /**
     *  This call is passed on to the original HttpServletRequest object.
     *  For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     *  @see javax.servlet.http.HttpServletRequest
     */
    public Enumeration getHeaderNames() {
        return myRequest.getHeaderNames();
    }

    /**
     * This call is passed on to the original HttpServletRequest object.
     *
     * @see javax.servlet.http.HttpServletRequest#getCookies
     */
    public Cookie[] getCookies() {
        return myRequest.getCookies();
    }

    /**
     * This call is passed on to the original HttpServletRequest object.
     *
     * @see javax.servlet.http.HttpServletRequest#getSession
     */
    public HttpSession getSession(boolean create) {
        return myRequest.getSession(create);
    }

    /**
     * This call is passed on to the original HttpServletRequest object.
     * For a description of what this call does, see the HttpServletRequest
     *  documentation.
     *
     * @see javax.servlet.http.HttpServletRequest#getRequestedSessionId
     */
    public String getRequestedSessionId() {
        return myRequest.getRequestedSessionId();
    }

    /**
     * This call is passed on to the original HttpServletRequest object.
     *
     * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdValid
     */
    public boolean isRequestedSessionIdValid() {
        return myRequest.isRequestedSessionIdValid();
    }

    /**
     * This call is passed on to the original HttpServletRequest object.
     *
     * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdFromCookie
     */
    public boolean isRequestedSessionIdFromCookie() {
        return myRequest.isRequestedSessionIdFromCookie();
    }


    /**
     * This call is passed on to the original HttpServletRequest object.
     *
     * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdFromUrl
     */
    public boolean isRequestedSessionIdFromUrl() {
        return myRequest.isRequestedSessionIdFromUrl();
    }

    /**
     * This call is passed on to the original HttpServletRequest object.
     *
     * @see javax.servlet.http.HttpServletRequest#getReader
     * @exception IOException
     *   On other I/O related errors.
     */
    public BufferedReader getReader() throws IOException {
        return myRequest.getReader();
    }

    /**
     * This call is passed on to the original HttpServletRequest object.
     *
     * @see javax.servlet.http.HttpServletRequest#getCharacterEncoding
     */
    public String getCharacterEncoding() {
        return myRequest.getCharacterEncoding();
    }


    public boolean isUserInRole(String role) {
	// X22
	return myRequest.isUserInRole(role);
    }

    public Locale getLocale() {
	// X22
	return myRequest.getLocale();
    }

    public java.security.Principal getUserPrincipal() {
	// X22
	return myRequest.getUserPrincipal();
    }

    public Enumeration getLocales() {
	// X22
	return myRequest.getLocales();
    }

    public String getContextPath() {
	// X22
	return myRequest.getContextPath();
    }

    public Enumeration getHeaders(String name) {
	// X22
	return myRequest.getHeaders(name);
    }

    public void removeAttribute(String name) {
	// X22
	myRequest.removeAttribute(name);
    }

    public boolean isSecure() {
	// X22
	return myRequest.isSecure();
    }

    public boolean isRequestedSessionIdFromURL() {
	// X22
	return myRequest.isRequestedSessionIdFromURL();
    }

    public HttpSession getSession() {
	// X22
	return myRequest.getSession();
    }

    public Enumeration getAttributeNames() {
	// X22
	return myRequest.getAttributeNames();
    }

    public void setAttribute(String name, Object value) {
	// X22
	myRequest.setAttribute(name, value);
    }

    public RequestDispatcher getRequestDispatcher(String name) {
	// X22
	return myRequest.getRequestDispatcher(name);
    }

    // X22 for RequestAdapter
    public void recycle() {
	adapter.recycle();
    }

    // X22 for RequestAdapter
    public MimeHeaders getMimeHeaders() {
	return adapter.getMimeHeaders();
    }

    // X22 for RequestAdapter
    public String[] getCookieHeaders() {
	// X22 doesn't seem to be used
	throw new Error("not implemented");
    }

    // X22 for RequestAdapter
    public String getServletPrefix() {
	// X22 doesn't seem to be used
	throw new Error("not implemented");
    }

    // X22 for RequestAdapter
    public String getServletName() {
	// X22 doesn't seem to be used
	throw new Error("not implemented");
    }

    // X22 for RequestAdapter
    public int doRead() throws IOException {
	// X22 might need some checking here first
	return adapter.doRead();
    }

    // X22 for RequestAdapter
    public int doRead(byte[] b, int off, int len) throws IOException {
	// X22 might need some checking here first
	return adapter.doRead(b, off, len);
    }
}

