/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: StandardLoggingFilter.java,v 1.4.2.1.2.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.filter;

import java.io.*;
import com.lutris.logging.*;
import com.lutris.util.KeywordValueTable;
import com.lutris.util.KeywordValueException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 *  This is the standard logging filter that is used with the MultiServer.
 *  It uses the standard logging facility to log the basic information
 *  about each request and response.  The text is written to the MultiServer 
 *  facility, at the REQUEST level.
 *
 *  @see org.enhydra.servlet.filter.StandardLoggingServlet
 *  @see org.enhydra.servlet.filter.StandardLoggingResponse
 *  @author Andy John
 *  @author Kent Henneuse
 *  @author Shawn McMurdo
 */
public class StandardLoggingFilter extends MultiServerFilter {

    // Where to write the data to.
    private LogChannel logChannel;
    private int logLevel;
    private String logFile = null;

    /*
     * Used to give each request it's own id number. This is so requests
     * and responses can be matched up when looking at the log file.
     */
    protected static long transactionId = 0;

    /**
     *  Create a new filter. This filter does not use any initial
     *  arguments.
     * 
     *  @param description A human readable description of this filter.
     *  @param initArgs Unused by this filter.
     */
    public StandardLoggingFilter(String description, 
				 KeywordValueTable initArgs) 
	throws KeywordValueException {

        super(description, initArgs);
        logChannel = Logger.getCentralLogger().getChannel("Multiserver");
        logLevel = logChannel.getLevel("REQUEST");

	if (initArgs != null) {
	    logFile = (String)initArgs.get("logFile", null);
	}

	setServiceInterceptor(new StandardLoggingServiceInterceptor(logFile));
    }

    public ResponseFilter wrapResponse(HttpServletRequest request,
	    HttpServletResponse response) {
	StandardLoggingResponse res = new StandardLoggingResponse(response);
	request.setAttribute(
		"Enhydra.StandardLoggingFilter.standardLoggingResponse", res);
	return res;
    }

}

