/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EnhydraRequestDispatcher.java,v 1.4.4.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.servletManager;
 
import java.io.IOException;
import javax.servlet.*;
import javax.servlet.http.*;
import com.lutris.multiServer.MultiServer;

/**
 * Processes requests for resources at a particular path, including
 * Servlets, Enhydra Apps, and Web Apps.
 *
 * @author Shawn McMurdo
 * @version $Revision: 1.4.4.1 $
 *
 * @see javax.servlet.ServletContext#getRequestDispatcher(java.lang.String)
 */
public class EnhydraRequestDispatcher implements RequestDispatcher {

    private String urlPath;

    /**
     * Creates a new RequestDispatcher for the given url path.
     *
     * @param urlPath
     *		a String URL path that the dispatcher will handle request for.
     */
    public EnhydraRequestDispatcher(String urlPath) {
	this.urlPath = urlPath;
    }

    /**
     * Forwards a {@link ServletRequest} object from
     * this servlet to a resource (servlet, JSP file, or
     * HTML file) on the server. You can use this method
     * when one servlet does preliminary processing of
     * a request and lets another resource generate
     * the response.
     *
     * <p>The <code>ServletRequest</code> object has its
     * path and other parameters adjusted to be relative
     * to the path of the target resource.
     *
     * <p>You cannot use <code>forward</code> if the target
     * resource has already returned a <code>ServletOutputStream</code>
     * or <code>PrintWriter</code> object to the servlet.
     * In that situation, <code>forward</code> throws
     * an <code>IllegalStateException</code>.
     *
     * @param request		a {@link ServletRequest} object
     *				that represents the request the client
     * 				makes of the servlet
     *
     * @param response		a {@link ServletResponse} object
     *				that represents the response the servlet
     *				returns to the client
     *
     * @exception ServletException	if the target resource is a servlet and
     *				throws an exception
     *
     * @exception IOException	if an input or output exception occurs
     *
     * @exception IllegalStateException	if the target resource returned
     *					a <code>ServletOutputStream</code> or
     *					<code>PrintWriter</code> object before
     *					this method was called
     */
    public void forward(ServletRequest request, ServletResponse response)
	    throws ServletException, IOException {
	// X22
	throw new Error("not implemented");
    }




    /**
     * Includes the content of a resource in the response
     * of the calling servlet.  In essence, this method enables 
     * programmatic server-side includes.
     *
     * <p>The {@link ServletResponse} object passed to this
     * servlet has the caller's request path. The response
     * object only has access to this servlet's 
     * <code>ServletOutputStream</code> and <code>PrintWriter</code>
     * objects.
     *
     * <p>A servlet that calls <code>include</code> cannot
     * set response headers. If the servlet that calls
     * <code>include</code> also calls methods that set
     * headers, <code>include</code> is not guaranteed
     * to work.
     * 
     * @param request 			a {@link ServletRequest} object 
     *					that contains the client's request
     *
     * @param response 			a {@link ServletResponse} object 
     * 					that contains the servlet's response
     *
     * @exception ServletException 	if the target servlet throws a servlet
     *					exception
     *
     * @exception IOException 		if the servlet has already obtained
     *					a {@link ServletOutputStream} or
     *					print writer object from the
     *					<code>ServletResponse</code> object
     */
    public void include(ServletRequest request, ServletResponse response)
	    throws ServletException, IOException {
	// X22
	throw new Error("not implemented");
    }
}

