/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LutrisSnoopServlet.java,v 1.4.12.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.servlets;

import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;


/**
 * Snoop servlet. This servlet`echos back the request line, 
 * headers and posted content that were sent by the client
 * This servlet has been written using the Sun Snoop Servlet 
 * as model
 *
 * @version 	1.00, 06/30/97
 * @author 	Christophe Ney
 */
public class LutrisSnoopServlet extends HttpServlet {


    // when the url is requested with a POST method
    public void doPost (HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
        //value chosen to limit denial of service
        if (req.getContentLength() > 8*1024) {  
            res.setContentType("text/html");
            ServletOutputStream out = res.getOutputStream();
            out.println("<html><head><title>Too big</title></head>");
            out.println("<body><h1>Error - content length &gt;8k not ");
            out.println("</h1></body></html>");
        } else {
            doGet(req,res);
            ServletOutputStream out = res.getOutputStream();
            out.println("<html><body><h1>Posted data:</h1>");
            out.println("<pre>");
            printPostedData(out,req);
            out.println("</pre></body></html>");
        }
    }
    
    // when the url is requested with a GET method
    public void doGet (HttpServletRequest req, HttpServletResponse res)
        throws ServletException, IOException
    {
    res.setContentType("text/html");
    ServletOutputStream out = res.getOutputStream();
    out.println("<html>");
    out.println("<head><title>Snoop Servlet</title></head>");
    out.println("<body>");

    out.println("<h1>Requested URL:</h1>");
    out.println("<pre>");
    out.println (HttpUtils.getRequestURL (req).toString ());
    out.println("</pre>");

    Enumeration enum = getServletConfig().getInitParameterNames();
    if (enum != null) {
        boolean first = true;
        while (enum.hasMoreElements()) {
        if (first) {
            out.println("<h1>Init Parameters</h1>");
            out.println("<pre>");
            first = false;
                }
        String param = (String) enum.nextElement();
                out.println(" "+param+": "+getInitParameter(param));
        }
        out.println("</pre>");
    }

    out.println("<h1>Request information:</h1>");
    out.println("<pre>");
    print(out, "Request method", req.getMethod());
    print(out, "Request URI", req.getRequestURI());
    print(out, "Request protocol", req.getProtocol());
    print(out, "Servlet path", req.getServletPath());
    print(out, "Path info", req.getPathInfo());
    print(out, "Path translated", req.getPathTranslated());
    print(out, "Query string", req.getQueryString());
    print(out, "Content length", req.getContentLength());
    print(out, "Content type", req.getContentType());
    print(out, "Server name", req.getServerName());
    print(out, "Server port", req.getServerPort());
    print(out, "Remote user", req.getRemoteUser());
    print(out, "Remote address", req.getRemoteAddr());
    print(out, "Remote host", req.getRemoteHost());
    print(out, "Authorization scheme", req.getAuthType());
    out.println("</pre>");

    Enumeration e = req.getHeaderNames();
    if (e.hasMoreElements()) {
        out.println("<h1>Request headers:</h1>");
        out.println("<pre>");
        while (e.hasMoreElements()) {
            String name = (String)e.nextElement();
            out.println(" " + name + ": " + req.getHeader(name));
        }
        out.println("</pre>");
    }

    e = req.getParameterNames();
    if (e.hasMoreElements()) {
        out.println("<h1>Servlet parameters (Single Value style):</h1>");
        out.println("<pre>");
        while (e.hasMoreElements()) {
            String name = (String)e.nextElement();
            out.println(" " + name + " = " + req.getParameter(name));
        }
        out.println("</pre>");
    }

    e = req.getParameterNames();
    if (e.hasMoreElements()) {
        out.println("<h1>Servlet parameters (Multiple Value style):</h1>");
        out.println("<pre>");
        while (e.hasMoreElements()) {
            String name = (String)e.nextElement();
            String vals[] = (String []) req.getParameterValues(name);
            if (vals != null) {
                out.print("<b> " + name + " = </b>"); 
                out.println(vals[0]);
                for (int i = 1; i<vals.length; i++)
                out.println("           " + vals[i]);
            }
            out.println("<p>");
        }
        out.println("</pre>");
    }

    out.println("</body></html>");
    }

    private void print(ServletOutputStream out, String name, String value)
        throws IOException
    {
        out.print(" " + name + ": ");
        out.println(value == null ? "&lt;none&gt;" : value);
    }

    private void print(ServletOutputStream out, String name, int value)
        throws IOException
    {
        out.print(" " + name + ": ");
        if (value == -1) {
            out.println("&lt;none&gt;");
        } else {
            out.println(value);
        }
    }

    private void printPostedData(ServletOutputStream out, HttpServletRequest req)
        throws IOException
    {
        ServletInputStream in = req.getInputStream();
        out.print("\n  ");
        for (int i=0; i<req.getContentLength();i++) {
            int b = in.read();
            switch(b) {
                case '\n': out.print("\n  "); break;
                case '<': out.print("&lt;"); break;
                case '>': out.print("&gt;"); break;
                default:out.write(b);
            }
        }
        out.print("\n");
    }


    private static final String UNKNOWN = "&lt;unknown&gt;";

    public String getServletInfo() {
        return "A servlet that shows the data sent by the client";
    }

}
