/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HeaderConverter.java,v 1.5.12.1 2000/10/19 17:59:10 jasona Exp $
 */

package org.enhydra.servlet.tomcat;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import com.lutris.util.*;
import org.enhydra.servlet.connectionMethods.http.HttpHeaders;
import org.enhydra.servlet.connectionMethods.https.HttpsHeaders;
import org.apache.tomcat.util.*;

/*
 * Converts Enhydra HttpHeaders to Tomcat MimeHeaders and vice versa.
 * X22 This class should go away when we convert to using MimeHeaders
 * X22 since the conversion is a performance hit.
 *
 * @version $Revision: 1.5.12.1 $
 * @auther Shawn McMurdo
 */
public class HeaderConverter {

    /**
     * Use static methods.
     *
     */
    private HeaderConverter() { 
    }

    /**
     * Converts an Enhydra HttpHeaders to a Tomcat MimeHeaders.
     *
     * @param headers
     *		the HttpHeaders to be converted.
     * @return a MimeHeaders equivalent to the given headers.
     */
    public static MimeHeaders toMimeHeaders(HttpHeaders headers) {
	// X22
	MimeHeaders converted = new MimeHeaders();
	Enumeration nameEnum = headers.getNames();
	while (nameEnum.hasMoreElements()) {
	    String name = (String) nameEnum.nextElement();
	    String[] values = headers.getValues(name);
	    for (int i = 0; i < values.length; i++) {
		converted.addHeader(name, values[i]);
	    }
	}
	return converted;
    }

    /**
     * Converts a Tomcat MimeHeaders to an Enhydra HttpHeaders.
     *
     * @param headers
     *		the MimeHeaders to be converted.
     * @return a HttpHeaders equivalent to the given headers.
     */
    public static HttpHeaders toHttpHeaders(MimeHeaders headers) {
	// X22
	throw new Error("not implemented");
    }

    /**
     * Converts an Apache JServ headers Hashtable to a Tomcat MimeHeaders.
     * For the Apache Connection Method.
     *
     * @param headers
     *		the Hashtable to be converted.
     * @return a MimeHeaders equivalent to the given headers.
     */
    public static MimeHeaders toMimeHeaders(Hashtable headers) {
	// X22
	MimeHeaders converted = new MimeHeaders();
	Enumeration nameEnum = headers.keys();
	while (nameEnum.hasMoreElements()) {
	    String name = (String) nameEnum.nextElement();
	    Object value = headers.get(name);
            if (value instanceof String) {
                converted.addHeader(name, (String) value);
            } else if (value instanceof Vector) {
                Vector vec = (Vector) value;
                for (int i = 0; i < vec.size(); i++) {
                    converted.addHeader(name, (String) vec.elementAt(i));
                }
            }
	}
	return converted;
    }

    /**
     * Converts an Enhydra HttpsHeaders to a Tomcat MimeHeaders.
     * For the Https connection method.
     *
     * @param headers
     *		the HttpsHeaders to be converted.
     * @return a MimeHeaders equivalent to the given headers.
     */
    public static MimeHeaders toMimeHeaders(HttpsHeaders headers) {
	// X22
	MimeHeaders converted = new MimeHeaders();
	Enumeration nameEnum = headers.getNames();
	while (nameEnum.hasMoreElements()) {
	    String name = (String) nameEnum.nextElement();
	    String[] values = headers.getValues(name);
	    for (int i = 0; i < values.length; i++) {
		converted.addHeader(name, values[i]);
	    }
	}
	return converted;
    }
} 

