package com.simscomputing.swing;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;

import javax.swing.JDesktopPane;
import javax.swing.JFrame;

/**
Common Swing tasks.

@author <a href="http://www.simscomputing.com">Sims Computing</a>
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:37 $
*/
public class SwingUtilities {
  private static final Point ZERO_POINT = new Point();

  /**
  Centers the specified frame in the middle of the screen.
  */
  public static final void centerOnScreen(final JFrame frame) {
    final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    final Dimension frameSize = frame.getSize();
    center(frame, screenSize, ZERO_POINT);
  } // centerOnScreen()

  /**
  Centers the specified frame in the middle of the specified parent.
  */
  public static final void centerOnParent(final Container container,
                                          final Container parent) {
    center(container, parent.getSize(), parent.getLocation());
  } // centerOnParent()

  /**
  Centers the specified internal frame in the middle of the specified parent.
  */
  public static final void centerOnParent(final Container internalFrame,
                                          final JDesktopPane parent) {
    center(internalFrame, parent.getSize(), ZERO_POINT);//parent.getLocation());
  } // centerOnParent()


  private static void center(final Container inner, final Dimension outerSize,
                             final Point offset) {
    final Dimension innerSize = inner.getSize();

    // reset inner's width and height if they are too big
    if (innerSize.width > outerSize.width) {
      innerSize.width = outerSize.width;
    } // if
    if (innerSize.height > outerSize.height) {
      innerSize.height = outerSize.height;
    } // if

    // put the inner in the center of the desktop
    inner.setLocation((outerSize.width - innerSize.width) / 2 + offset.x,
                      (outerSize.height - innerSize.height) / 2 + offset.y);
  } // center()
} // class SwingUtilities
