// Java Test Bed: tools for testing Java classes
// Copyright 1999 Sims Computing
// This software is licensed under the GNU General Public License,
// available at http://www.gnu.org/copyleft/gpl.html.

package com.simscomputing.testbed;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;

/**


@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:33 $
*/
public class FileSystemTestCaseCollection extends TestCaseCollection {
  private File directory;

  public FileSystemTestCaseCollection(String directory) {
    this.directory = new File(directory);
  } // constructor

  /**
  Finds all class files that extend TestCaseCollection in the specified directory and all
  directories underneath. Then those classes are exercised and the collective results
	are returned.

  @return TestResultCollection the collective results of testing all classes
  */
  public TestResultCollection exercise() {
		ArrayList /* File */ testees = new ArrayList();

		findExercisableClasses(directory, new TestCaseCollectionFileFilter(), testees);

		for (int index = 0; index < testees.size(); ++index) {
			// this class can now be exercised
			File f = (File) testees.get(index);
//  			Class.forName
			System.out.println("ready for testing: " + f);
		} // for index

		return null;
  } // exercise()

	private void findExercisableClasses(File directory, FileFilter fileFilter,
																			ArrayList outputList) {
		File[] candidates = directory.listFiles(fileFilter);
		final int length = candidates.length;

		for (int index = 0; index < length; ++index) {
			if (candidates[index].isDirectory()) {
				findExercisableClasses(candidates[index], fileFilter, outputList);
			} // if
			else {
				outputList.add(candidates[index]);
			} // else
		} // for index
	} // findExercisableClasses()

	class TestCaseCollectionFileFilter implements FileFilter {
		/**
		Accept files that end in ".class" and implement TestCaseCollection
    */
		public boolean accept(File pathname) {
			if (pathname.isDirectory() || pathname.toString().endsWith(".class")) {
				// fixme: make sure it extends TestCaseCollections
				return true;
			} // if

			return false;
		} // accept()
	} // class TestCaseCollectionFileFilter

	public static void main(String[] args) {
		FileSystemTestCaseCollection fstcc = new FileSystemTestCaseCollection(".");
		fstcc.exercise();
	} // main()
} // class FileSystemTestCaseCollection
