package com.simscomputing.testbed;

import java.io.File;

/**
A simple command line interface for exercising TestCaseCollections.
The functionality mirrors the functionality of the GUI.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:33 $
*/
public class Main {
  public static void main(final String[] args) {
    if (args.length != 1 && args.length != 2) {
      System.out.println("usage: Main <path to class file>");
      System.out.println("       Main -d <path to directory of TestCaseCollections>");
      System.out.println("       Main -r <path to recursive directory of TestCaseCollections>");
      return;
    } // if

    try {
      TestResultCollection results;

      if (args[0].equals("-d")) {
        final DirectoryTester directoryTester = new DirectoryTester(new File(args[1]),
                                                                    false);
        results = directoryTester.exercise(true, null);
      } // if
      else if (args[0].equals("-r")) {
        final DirectoryTester directoryTester = new DirectoryTester(new File(args[1]),
                                                                    true);
        results = directoryTester.exercise(true, null);
      } // else if
      else {
        final FileTester fileTester = new FileTester(new File(args[0]));
        results = fileTester.exercise(true, null);
      } // else

      final DefaultReporter reporter = new DefaultReporter();
      reporter.report(results);
    } // try
    catch (TesterException e) {
      System.out.println("Test failed: " + e.getMessage());
    } // catch
  } // main()
} // class Main
