package com.simscomputing.testbed.gui;

import com.simscomputing.swing.SwingUtilities;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
Gathers a class name from the user.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:35 $
*/
public class ClassNameDialog extends JDialog {
  private String className;
  private final JTextField classNameField = new JTextField("");

  public ClassNameDialog(JFrame parent) {
    super(parent, true);

    setTitle("Enter class name");

    final Container root = getContentPane();
    root.setLayout(new GridBagLayout());

    final JLabel contents = new JLabel("Enter a class name to test (for example, com.foo.BarTest)");
    classNameField.addActionListener(new ActionListener() {
      public void actionPerformed(final ActionEvent e) {
        doOkAction();
      } // actionPerformed()
    });
		classNameField.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyPressed(final KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
          doCancelAction();
        } // if
			} // keyPressed()
		});

    JButton okButton = new JButton("OK");
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        doOkAction();
      }
    });
		okButton.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyPressed(final KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
          doCancelAction();
        } // if
			} // keyPressed()
		});

    JButton cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(final ActionEvent e) {
        doCancelAction();
      } // actionPerformed()
    });
		cancelButton.addKeyListener(new java.awt.event.KeyAdapter() {
			public void keyPressed(final KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
          doCancelAction();
        } // if
			} // keyPressed()
		});

    final int INSET = Resources.INSET;
    final Insets INSETS = Resources.INSETS;

    root.add(classNameField,
             new GridBagConstraints(0, 1, 2, 1, 1.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    INSETS,
                                    0, 0));

    root.add(contents,
             new GridBagConstraints(0, 0, 2, 1, 1.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    INSETS,
                                    0, 0));

    JPanel okCancelPanel = new JPanel();
    okCancelPanel.setLayout(new GridBagLayout());

    root.add(okCancelPanel,
             new GridBagConstraints(0, 2, 1, 1, 1.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    INSETS,
                                    0, 0));

    okCancelPanel.add(okButton,
             new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    INSETS,
                                    0, 0));
    okCancelPanel.add(cancelButton,
             new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                    GridBagConstraints.CENTER,
                                    GridBagConstraints.HORIZONTAL,
                                    INSETS,
                                    0, 0));

    pack();

    SwingUtilities.centerOnParent(this, parent);
  } // constructor

  private void doOkAction() {
    className = classNameField.getText();
    if (className.length() > 0) {
      dispose();
    } // if
  } // doOkAction()

  private void doCancelAction() {
    dispose();
  } // doOkAction()

  public String getClassName() {
    return className;
  } // getClassName()
} // class ClassNameDialog
