package com.simscomputing.testbed.gui;

import com.simscomputing.swing.WindowInternalFrame;
//import com.simscomputing.testbed.SelectiveTestCaseCollection;
import com.simscomputing.testbed.TestCaseCollection;
import com.simscomputing.testbed.Tester;
import com.simscomputing.testbed.TesterException;
import com.simscomputing.testbed.TesterListener;
import com.simscomputing.testbed.TesterProgress;
import com.simscomputing.testbed.TestResult;
import com.simscomputing.testbed.TestResultCollection;

import java.awt.Container;
import java.awt.Cursor;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Iterator;

import javax.swing.JTabbedPane;
import javax.swing.JButton;
import javax.swing.*;
import javax.swing.event.*;

/**
Graphically displays a TestResultCollection.

@author David Sims
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:35 $
*/
public class ReporterInternalFrame extends WindowInternalFrame {
  private TestResultCollection testResultCollection;
  private TestCaseCollection tester;

  private JTabbedPane testCaseResultsTabbedPane;
  private JTextArea testCaseMethodTextArea;
  private JTextArea testCaseOutTextArea;
  private JTextArea testCaseErrTextArea;

  // middle list 
  private TestCaseCollectionListModel testCaseListModel = 
    new TestCaseCollectionListModel();
  private JList testCaseList = new JList(testCaseListModel);

  // left list
  TestCaseCollectionListModel testCaseCollectionListModel =
    new TestCaseCollectionListModel();
  private JList testCaseCollectionList = new JList(testCaseCollectionListModel);

  private JTree testCaseCollectionTree;
  private JTabbedPane testCasesTabbedPane;
  private JSplitPane testCasesSplitPane;
  private JSplitPane methodsSplitPane;
  private JLabel summaryLabel = new JLabel();
  private final JPopupMenu popupMenu = new JPopupMenu();
  private final JMenuItem selectAllMenuItem = new JMenuItem("Select All");
  private final JMenuItem clearAllMenuItem = new JMenuItem("Clear All");
  private final JMenuItem selectiveExerciseMenuItem = new JMenuItem("Re-Exercise");

  public ReporterInternalFrame(final boolean runNow,
                               final TestCaseCollection tester)
      throws TesterException {
    super(Resources.getWindowMenu());

    selectAllMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        final int[] all = new int[testCaseList.getModel().getSize()];
        for (int index = 0; index < all.length; ++index) {
          all[index] = index;
        } // for index
        testCaseList.setSelectedIndices(all);
      } // actionPerformed()
    });

    clearAllMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        testCaseList.clearSelection();
      } // actionPerformed()
    });

    selectiveExerciseMenuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          exercise(true, true);
        } // try
        catch (TesterException ex) {
          // ok to ignore
        }  // catch
      } // actionPerformed()
    });


//    selectiveExerciseMenuItem.addActionListener();

    popupMenu.add(selectiveExerciseMenuItem);
    popupMenu.add(clearAllMenuItem);

    popupMenu.add(selectAllMenuItem);


    setTitle(tester.getName());

    this.tester = tester;

    setResizable(true);
    setClosable(true);
    setIconifiable(true);
    setMaximizable(true);

    Container root = getContentPane();

//      root.setLayout(new GridBagLayout());
    root.setLayout(new BorderLayout());

    testCaseResultsTabbedPane = new JTabbedPane();

    testCaseMethodTextArea = new JTextArea();
    testCaseMethodTextArea.setEditable(false);

    testCaseOutTextArea = new JTextArea();
    testCaseOutTextArea.setEditable(false);

    testCaseErrTextArea = new JTextArea();
    testCaseErrTextArea.setEditable(false);

    // make the summary label have a bold font
    final Font summaryFont = summaryLabel.getFont();
    final int newSize = (int) (summaryFont.getSize() * 1.25);
    summaryLabel.setFont(new Font(summaryFont.getName(), summaryFont.getStyle(), newSize));

    final int INSET = Resources.INSET;
    JPanel testCaseResultPanel = new JPanel();
    testCaseResultPanel.setLayout(new GridBagLayout());
    testCaseResultPanel.add(new ConstrainedJLabel("Test Case Result"), 
                            new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(INSET, INSET, INSET, INSET),
                                                   0, 0));

    testCaseResultPanel.add(testCaseResultsTabbedPane,
                            new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.BOTH,
                                                   new Insets(INSET, INSET, INSET, INSET),
                                                   0, 0));

    testCaseResultsTabbedPane.addTab("Result", new JScrollPane(testCaseMethodTextArea));
    testCaseResultsTabbedPane.addTab("Stdout", new JScrollPane(testCaseOutTextArea));
    testCaseResultsTabbedPane.addTab("Stderr", new JScrollPane(testCaseErrTextArea));

    testCaseList.setCellRenderer(new TestCaseRenderer(testCaseList.getCellRenderer()));
    JPanel testCaseListPanel = new JPanel();
    testCaseListPanel.setLayout(new GridBagLayout());
    testCaseListPanel.add(new ConstrainedJLabel("Test Cases"), 
                          new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.HORIZONTAL,
                                                 new Insets(INSET, INSET, INSET, INSET),
                                                 0, 0));

    testCaseListPanel.add(new JScrollPane(testCaseList), 
                          new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.BOTH,
                                                 new Insets(INSET, INSET, INSET, INSET),
                                                 0, 0));

    methodsSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    methodsSplitPane.add(testCaseListPanel);
    methodsSplitPane.add(testCaseResultPanel);
//    methodsSplitPane.setDividerLocation(0.5);
    methodsSplitPane.resetToPreferredSizes();

    testCasesTabbedPane = new JTabbedPane();
    testCaseCollectionTree = new JTree();
//      testCaseCollectionList = new JList();
    testCaseCollectionList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        int index = testCaseCollectionList.getSelectedIndex();
        if (index >= 0 && index < testCaseCollectionListModel.getSize()) {
          com.simscomputing.testbed.TestResult testResult =
            testCaseCollectionListModel.getTestResult(index);

          final TestResult[] testResults =
            ReporterInternalFrame.this.testResultCollection.findTestResults(testResult.getTestCaseCollectionClassName());

          testCaseListModel.clear();
          for (index = 0; index < testResults.length; ++index) {
            testCaseListModel.add(testResults[index].getTestCaseName(),
                                  testResults[index]);
          } // for index

          testCaseList.setSelectedIndex(0);
        } // if
        else {
          testCaseListModel.clear();
          testCaseOutTextArea.setText("");
          testCaseErrTextArea.setText("");
          testCaseMethodTextArea.setText("");
        } // else
      } // actionPerformed()
    });

    testCaseList.addMouseListener(new java.awt.event.MouseAdapter() {
  		public void mouseClicked(final MouseEvent e) {
				if (SwingUtilities.isRightMouseButton(e)) {
          final boolean enabled = (testCaseList.getSelectedIndex() >= 0);
          selectiveExerciseMenuItem.setEnabled(enabled);
					popupMenu.show(e.getComponent(), e.getX(), e.getY());
        } // if
      } // mouseClicked()
    } // subclass of MouseAdapter
    );

    testCaseList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        final int[] indices = testCaseList.getSelectedIndices();
        if (indices.length == 1 && indices[0] >= 0 && indices[0] < testCaseListModel.getSize()) {
          final TestResult testResult = testCaseListModel.getTestResult(indices[0]);
          if (!testResult.didTestCaseRun()) {
            testCaseOutTextArea.setText("");
            testCaseErrTextArea.setText("");
            testCaseMethodTextArea.setText("");

            return;
          } // if

          final StringBuffer result = new StringBuffer();
          if (testResult.didTestPass()) {
            result.append("Passed (all assertions passed)\n\n");
            final StringBuffer message = new StringBuffer("Execution time: ");
            final long time = testResult.getExecutionTimeInMilliseconds();
            if (time < 1) {
              message.append("< 1");
            } // if
            else {
              message.append(time);
            } // else
            message.append(" ms");
            result.append(message.toString() + "\n\n");
          } // if
          else if (testResult.didTestFailAbnormally()) {
            result.append("Failed abnormally (an exception was thrown out of the test case)\n\n");
            final Throwable exception = testResult.getTestFailureException();
            final StringWriter stringWriter = new StringWriter();
            final PrintWriter printWriter = new PrintWriter(stringWriter, false);
            exception.printStackTrace(printWriter);
            printWriter.flush();
            stringWriter.flush();
            result.append(stringWriter.toString() + "\n\n");
          } // else if
          else {
            result.append("Failed (an assertion did not pass)\n\n");
            result.append("Assertion message: " + testResult.getTestFailureDescription() + "\n\n");
          } // else

          testCaseOutTextArea.setText(testResult.getOutStreamData());
          // move caret to upperleft corner
          testCaseOutTextArea.setCaretPosition(0);

          testCaseErrTextArea.setText(testResult.getErrStreamData());
          // move caret to upperleft corner
          testCaseErrTextArea.setCaretPosition(0);

          testCaseMethodTextArea.setText(result.toString());
          // move caret to upperleft corner
          testCaseMethodTextArea.setCaretPosition(0);
        } // if index in range
        else {
          // there are either multiple selections or none or
          // the selection is out of range
          testCaseOutTextArea.setText("");
          testCaseErrTextArea.setText("");
          testCaseMethodTextArea.setText("");
        } // else
      } // actionPerformed()
    });

    // place these two components in the tabbed pane
    testCasesTabbedPane.addTab("Tree View", new JScrollPane(testCaseCollectionTree));
    testCasesTabbedPane.addTab("List View", new JScrollPane(testCaseCollectionList));
    testCasesTabbedPane.setSelectedIndex(1); // look at the List View

    testCasesSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);

    JPanel tabbedPanel = new JPanel();
    tabbedPanel.setLayout(new GridBagLayout());
    tabbedPanel.add(new ConstrainedJLabel("Test Case Collections"),
                          new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.HORIZONTAL,
                                                 new Insets(INSET, INSET, INSET, INSET),
                                                 0, 0));
    tabbedPanel.add(testCasesTabbedPane, 
                          new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.BOTH,
                                                 new Insets(INSET, INSET, INSET, INSET),
                                                 0, 0));

    testCasesSplitPane.add(tabbedPanel);
    testCasesSplitPane.add(methodsSplitPane);
    // note: argh: I can't get the split panes to divide evenly
    // across the screen. resetToPreferredSizes() gives more space
    // to the right anyway, which is what we really want.
//    testCasesSplitPane.setDividerLocation(0.85);
    testCasesSplitPane.resetToPreferredSizes();

    JPanel rootPanel = new JPanel();
    rootPanel.setLayout(new GridBagLayout());
    rootPanel.add(summaryLabel,
                  new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                         GridBagConstraints.CENTER,
                                         GridBagConstraints.NONE,
                                         new Insets(INSET, INSET, INSET, INSET),
                                         0, 0));

    rootPanel.add(testCasesSplitPane,
                            new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.BOTH,
                                                   new Insets(INSET, INSET, INSET, INSET),
                                                   0, 0));
    JButton reexerciseButton = new JButton("Re-Exercise");
    reexerciseButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          exercise(false, true);
        } // try 
        catch (TesterException ex) {
          // ok to ignore
        }  // catch
      } // actionPerformed()
    });
    rootPanel.add(reexerciseButton, 
                            new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.NONE,
                                                   new Insets(INSET, INSET, INSET, INSET),
                                                   0, 0));

    root.add(rootPanel, BorderLayout.CENTER);

    setSize(Resources.INTERNAL_FRAME_SIZE);

    exercise(false, runNow);

    setVisible(true);

//      try {
//        setMaximum(true);
//      } // try
//      catch (java.beans.PropertyVetoException e) {
//        // do nothing. ignore it.
//      } // catch

  } // constructor

  private void initializeTestResultCollection() {
    final Iterator iterator = testResultCollection.getIterator();
    TestResult testResult = null;
//      TestCaseCollectionListModel listModel = new TestCaseCollectionListModel();
    String testCaseCollectionName;

    int[] testCaseListSelections = testCaseList.getSelectedIndices();
    if (testCaseListSelections.length == 0) {
      testCaseListSelections = new int[1];
      testCaseListSelections[0] = 0;
    } // if
    testCaseList.clearSelection(); // unselect middle list

    int[] testCaseCollectionListSelections =
      testCaseCollectionList.getSelectedIndices();
    if (testCaseCollectionListSelections.length == 0) {
      testCaseCollectionListSelections = new int[1];
      testCaseCollectionListSelections[0] = 0;
    } // if
    testCaseCollectionList.clearSelection(); // unselect left list

    testCaseListModel.clear(); // clear the middle list
    testCaseCollectionListModel.clear(); // clear the left list
    testCaseMethodTextArea.setText(""); // clear the right text area

//      if (iterator.hasNext() == false) {
//        System.out.println("****************** there are no test results!!!!");
//      }
    int passedTests = 0;
    int totalTests = 0;
    int abnormallyFailedTests = 0;
    boolean atLeastOneRan = false;
    while (iterator.hasNext()) {
      testResult = (TestResult) iterator.next();

      if (testResult.didTestCaseRun()) {
        atLeastOneRan = true;
      } // if

      ++totalTests;
      if (testResult.didTestPass()) {
        ++passedTests;
      } // if
      else if (testResult.didTestFailAbnormally()) {
        ++abnormallyFailedTests;
      } // else if

      testCaseCollectionName = testResult.getTestCaseCollectionClassName();
      testCaseCollectionListModel.add(testCaseCollectionName, testResult);
    } // while

    if (atLeastOneRan) {
      StringBuffer summary = new StringBuffer(passedTests + " of " + totalTests + " test case");
      if (totalTests != 1) {
        summary.append("s");
      } // if
      summary.append(" passed");
      summaryLabel.setText(summary.toString());
      if (passedTests == totalTests) {
        summaryLabel.setForeground(Resources.GREEN);
      } // if
      else if (abnormallyFailedTests == 0 && totalTests > 0) {
        summaryLabel.setForeground(Resources.YELLOW);
      } // else if
      else {
        summaryLabel.setForeground(Resources.RED);
      } // else
    } // if
    else {
      summaryLabel.setText("No test cases have been exercised yet");
    } // else
//        System.out.println("added: " + testCaseCollectionName);

  // fixme: should try to reselect what was already selected

  // select the first element in the list of TestCaseCollections


  testCaseCollectionList.setSelectedIndices(testCaseCollectionListSelections);
//  testCaseList.setSelectedIndices(testCaseListSelections);




  /*
    testCaseCollection
  } // for index
  if (testCaseCollectionList.getModel().getSize() > 0) {
    testCaseCollectionList.setSelectedIndex(0);

    // select the first element in the list of test case results
    if (testCaseList.getModel().getSize() > 0) {
      testCaseList.setSelectedIndex(0);
    } // if
  } // if
  */

//      testCaseCollectionList.setModel(testCaseCollectionListModel);
  } // initializeTestResultCollection()

  public void exercise(final boolean selective,
                       final boolean runNow) throws TesterException {

    // turn on wait cursor
    final Cursor oldCursor = getCursor();

    try {
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));


//        testCaseListModel.clear(); // clear the middle list
//        testCaseCollectionListModel.clear(); // clear the left list
//        testCaseMethodTextArea.setText(""); // clear the right text area
//        this.testResultCollection = null;
//        System.gc();

      // non-modal progress dialog
      // fixme: would like to release ProgressDialog reference when done
//        final int testCount = tester.getNumberOfTestCases();
      ProgressInternalFrame progress = null;
      if (runNow) {
        progress = new ProgressInternalFrame(getTitle());
        Resources.getDesktop().add(progress, JLayeredPane.PALETTE_LAYER);
        progress.setVisible(true);
        com.simscomputing.swing.SwingUtilities.centerOnParent(progress, Resources.getDesktop());
      } // if

      TestResult[] selectedTestResults = null;
      if (selective) {
        final int[] indices = testCaseList.getSelectedIndices();
        selectedTestResults = new TestResult[indices.length];
        for (int j = 0; j < indices.length; ++j) {
          selectedTestResults[j] = testCaseListModel.getTestResult(indices[j]);
        } // for j
      } // if

//      testCaseList.clearSelection(); // unselect middle list
//      testCaseCollectionList.clearSelection(); // unselect left list

      final Thread t = new TesterThread(runNow, progress, selectedTestResults);

      if (progress != null) {
        progress.setTestCaseCollection(null);//tester);
      } // if

      // we want the event dispatching thread to have higher priority so it
      // gets a chance to paint the progress bar
      int newPriority = t.getPriority() + 2;
      if (newPriority > Thread.MAX_PRIORITY) {
        newPriority = Thread.MAX_PRIORITY;
      } // if
      t.setPriority(newPriority);
      // hmmm, trying raising the test thread Thread.currentThread().setPriority(newPriority);

      t.start();
    } // try
    finally {
      setCursor(oldCursor);
    } // finally
  } // exercise();

//    public void internalFrameOpened(InternalFrameEvent e) {
//      super.internalFrameOpened(e);

//      System.out.println("*************** RIF opened");

//      // position the progress frame over this one
//      progress.toFront();
//      com.simscomputing.swing.SwingUtilities.centerOnParent(progress, this);
//    } // internalFrameOpened()

  ///////////////////////////////////
  // TesterListener implementation //
  ///////////////////////////////////

//    public void progressMade(TesterProgress testerProgress) {
//      if (progress != null && testerProgress != null) {
//        System.out.println("Reporter: updating progress to: " + testerProgress);
//        progress.update(testerProgress);
//      } // if
//    } // progressMade()

  private class TestCaseRenderer extends JLabel implements ListCellRenderer {
    private Color originalColor;
    private ListCellRenderer prototype;

    public TestCaseRenderer(ListCellRenderer prototype) {
      this.prototype = prototype;
    } // constructor

    public Component getListCellRendererComponent(
                                                JList list,
       Object value,            // value to display
       int index,               // cell index
       boolean isSelected,      // is the cell selected
       boolean cellHasFocus)    // the list and the cell have the focus
     {

       Component result = prototype.getListCellRendererComponent(list, value, index,
                                                                 isSelected,
                                                                 cellHasFocus);

       if (originalColor == null) {
         originalColor = result.getForeground();
       } // if

       TestResult testResult = testCaseListModel.getTestResult(index);
       if (testResult.didTestPass()) {
         result.setForeground(Resources.GREEN);
       } // if
       else if (testResult.didTestFailAbnormally()) {
         result.setForeground(Resources.RED);
       } // else if
       else if (testResult.didTestCaseRun()) {
         result.setForeground(Resources.YELLOW);
       } // else

       return result;
     } // getListCellRendererComponent()
  } // class TestCaseRenderer

  private class TesterThread extends Thread {
    private final ProgressInternalFrame progress;
    private TestResult[] selectedTestResults;
    private final boolean runNow;

    TesterThread(final boolean runNow,
                 final ProgressInternalFrame newProgress,
                 final TestResult[] newSelectedTestResults) {
      this.runNow = runNow;
      progress = newProgress;
      selectedTestResults = newSelectedTestResults;
    } // constructor

    public void run() {
            TestResultCollection results = null;


            try {
              // fixme: temp take out: results = tester.exercise(progress);

              tester.clearMarkedMethods();
              if (selectedTestResults != null) {

                TestResult tr;
                for (int i = 0; i < selectedTestResults.length; ++i) {
                  tr = selectedTestResults[i];
                  tester.markForTesting(tr.getTestCaseCollectionClassName(),
                                        tr.getTestCaseName());
                } // for i

                results = ReporterInternalFrame.this.testResultCollection;
                results = tester.exercise(true, progress, results, selectedTestResults.length);//sttc.getNumberOfMarkedTestCases());

                /*
                SelectiveTestCaseCollection sttc =
                  new SelectiveTestCaseCollection(tester);

                TestResult tr;
                for (int i = 0; i < selectedTestResults.length; ++i) {
                  tr = selectedTestResults[i];
                  System.out.println("### selected: " + tr.getTestCaseName());
                  sttc.addTestCase(tr.getTestCaseCollectionClassName(),
                                   tr.getTestCaseName());
                } // for i
                selectedTestResults = null;
                System.gc();

//              System.out.println("### added test case? " +
//                sttc.addTestCase( (TestCaseCollection) tester.getRelatedTestCaseCollections().get(0),
//                                 "Trivial"));
                results = ReporterInternalFrame.this.testResultCollection;
                results = sttc.exercise(progress, results, sttc.getNumberOfMarkedTestCases());
                */
              } // if
              else {
                results = tester.exercise(runNow, progress);
              } // else


//                  tester.waitUntilFinished();
            } // try
            catch (final Throwable e) {
              SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                  JOptionPane.showMessageDialog(ReporterInternalFrame.this,
                                                "The test run failed: " + e.getMessage(),
                                                "Failed",
                                                JOptionPane.ERROR_MESSAGE);
                } // run()
              });
              return;
            } // catch

            if (tester.wasStopped()) {
              SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                  JOptionPane.showMessageDialog(ReporterInternalFrame.this,
                                                "The test run was canceled.",
                                                "Canceled",
                                                JOptionPane.ERROR_MESSAGE);
                } // run()
              });
              return;
            } // if

//              System.out.println("Reporter done exercising");
            ReporterInternalFrame.this.testResultCollection = results;//tester.getTestResults();
            initializeTestResultCollection();
//              } // try
//              catch (TesterException e) {
//                // fixme
//              } // catch
          } // run
  } // class TesterThread

} // class ReporterInternalFrame
