// Copyright 2000 Sims Computing, Inc.
// Licensed under the GNU Lesser General Public License.

package com.simscomputing.util;

import com.simscomputing.testbed.TestCaseCollection;
import com.simscomputing.testbed.ManualVerificationRequiredException;
import com.simscomputing.testbed.ManualVerification;
import java.io.UnsupportedEncodingException;
import java.util.Locale;

/**
The unit test for the MutableString class.

@author <a href="http://www.simscomputing.com/">Sims Computing</a>
@version $Revision: 1.1.1.1 $ $Date: 2000/02/21 21:22:36 $
*/
public class MutableStringTest extends TestCaseCollection {
  public void runTestAppendBoolean() {
    MutableString ms = new MutableString("hello, world");

    ms.append(true);
    assert(ms.equals("hello, worldtrue"), "test 1");
//
//    ms.append(false);
//    assert(ms.equals("hello, worldtruefalse"), "test 2");
  } // runTestAppendBoolean()

  public void runTestAppendChar() {
//    throw new ManualVerificationRequiredException(new ManualVerification());

    MutableString ms = new MutableString("hello, world");
    ms.append('x');
    assert(ms.equals("hello, worldx"), "test 1");
  } // runTestAppendChar()

  public void runTestAppendChars() {
    MutableString ms = new MutableString("hello, world");
    char[] characters = new char[5];
    characters[0] = 'z';
    characters[1] = 'i';
    characters[2] = 'p';
    characters[3] = 'p';
    characters[4] = 'y';

    ms.append(characters, 2, 2);
    assert(ms.equals("hello, worldpp"), "test 1");
  } // runTestAppendChars()

  public void runTestAppendString() {
    MutableString ms = new MutableString("hello, world");
    MutableString ms2 = new MutableString("hello, world");
    MutableString ms3 = new MutableString("hello, worldhello, world");

    ms.append(ms2);

    assert(ms.equals(ms3), "test 1");
  } // runTestAppendString()

  public void runTestConcatString() {
    MutableString ms = new MutableString("hello, world");
    ms.concat("hi there");
    assert(ms.equals("hello, worldhi there"), "test 1");
  } // runTestConcatString()

  public void runTestConcatMutableString() {
    MutableString ms = new MutableString("hello, world");
    ms.concat(new MutableString("hi there"));
    assert(ms.equals("hello, worldhi there"), "test 1");
  } // runTestConcatMutableString()

  public void runTestConstructThenToString() {
    MutableString ms = new MutableString("hello, world");
    String s = ms.toString();

    assert(s.equals("hello, world"), "test 1");
  } // runTestConstructThenToString()

  public void runTestEquals() {
    MutableString ms1 = new MutableString("ms");
    MutableString ms2 = new MutableString("ms");

    assert(ms1.equals(ms2), "test 1");
  } // runTestEquals()

  public void runTestEndsWithString() {
    MutableString ms = new MutableString("ms ends with");

    assert(ms.endsWith("s with"), "test 1");
  } // runTestEndsWithString()

  public void runTestEndsWithString2() {
    MutableString ms = new MutableString("ms ends with");

    assert(ms.endsWith(""), "test 1");
  } // runTestEndsWithString2()

  public void runTestStartsWithString() {
    MutableString ms = new MutableString("ms starts with");

    assert(ms.startsWith("ms star"), "test 1");
  } // runTestStartsWithString()

  public void runTestStartsWithString2() {
    MutableString ms = new MutableString("ms ends with");

    assert(ms.startsWith(""), "test 1");
  } // runTestStartsWithString2()

  public void runTestReverse() {
    MutableString ms = new MutableString("reverse me").reverse();

    assert(ms.equals("em esrever"), "test 1");
  } // runTestReverse()

  public void runTestReverse2() {
    MutableString ms = new MutableString("").reverse();

    assert(ms.equals(""), "test 1");
  } // runTestReverse2()

  public void runTestReverse3() {
    MutableString ms = new MutableString("x").reverse();

    assert(ms.equals("x"), "test 1");
  } // runTestReverse3()

  public void runTestSubstring() {
    MutableString ms = new MutableString("hello, world");

    MutableString ms2 = ms.substring(2);
    assert(ms2.equals("llo, world"), "test 1");
  } // runTestSubstring()

  public void runTestSubstring2() {
    MutableString ms = new MutableString("hello, world");

    MutableString ms2 = ms.substring(2, 7);
    assert(ms2.equals("llo, "), "test 1");
  } // runTestSubstring2()

  public void runTestLength() {
    MutableString ms = new MutableString("hello, world");

    assert(ms.length() == 12, "test 1");
  } // runTestLength()

  public void runTestReplace() {
    MutableString ms = new MutableString("hello, world");

    ms.replace('l', 'x');

    assert(ms.equals("hexxo, worxd"), "test 1");
  } // runTestReplace()

  public void runTestReplace2() {
    MutableString ms = new MutableString("hello, world");

    ms.replace(0, 2, "xyz");

    assert(ms.equals("xyllo, world"), "test 1");
  } // runTestReplace2()

  public void runTestDeleteCharAt() {
    MutableString ms = new MutableString("hello, world");

    ms.deleteCharAt(7);

    assert(ms.equals("hello, orld"), "test 1");
  } // runTestDeleteCharAt()

  public void runTestDelete() {
    MutableString ms = new MutableString("hello, world");

    ms.delete(7, 7);

    assert(ms.equals("hello, world"), "test 1");
  } // runTestDelete()

  public void runTestDelete2() {
    MutableString ms = new MutableString("hello, world");

    ms.delete(7, 9);

    assert(ms.equals("hello, rld"), "test 1");
  } // runTestDelete2()

  public void runTestTrim() {
    MutableString ms = new MutableString("    hello, world     ");

    ms.trim();

    assert(ms.equals("hello, world"), "test 1");
  } // runTestTrim()

  public void runTestToLowerCase() {
    MutableString ms = new MutableString("HeLlO, WoRlD");

    ms.toLowerCase();

    assert(ms.equals("hello, world"), "test 1");
  } // runTestToLowerCase()

  public void runTestToLowerCase2() {
    MutableString ms = new MutableString("HeLlO, WoRlD");

    ms.toLowerCase(Locale.US);

    assert(ms.equals("HeLlO, WoRlD"), "test 1");
  } // runTestToLowerCase2()

  public void runTestToUpperCase() {
    MutableString ms = new MutableString("HeLlO, WoRlD");

    ms.toUpperCase();

    assert(ms.equals("HELLO, WORLD"), "test 1");
  } // runTestToUpperCase()

  public void runTestToUpperCase2() {
    MutableString ms = new MutableString("HeLlO, WoRlD");

    ms.toUpperCase(Locale.US);

    assert(ms.equals("HeLlO, WoRlD"), "test 1");
  } // runTestToUpperCase2()

  public void runTestCompareTo() {
    MutableString ms = new MutableString("hello, world");
    MutableString ms2 = new MutableString("goodbye, world");

    assert(ms2.compareTo(ms) == 2, "test 1");
  } // runTestCompareTo()

  public void runTestCompareTo2() {
    MutableString ms = new MutableString("hello, world");
    MutableString ms2 = new MutableString("hello, world");

    assert(ms2.compareTo(ms) == 0, "test 1");
  } // runTestCompareTo2()

  public void runTestCompareTo3() {
    MutableString ms = new MutableString("hello, world");
    MutableString ms2 = new MutableString("hellx, world");

    assert(ms2.compareTo(ms) == 'x' - 'o', "test 1");
  } // runTestCompareTo3()

  public void runTestValueOf() {
    char[] characterData = new char[5];
    characterData[0] = 'h';
    characterData[1] = 'e';
    characterData[2] = 'l';
    characterData[3] = 'l';
    characterData[4] = 'o';

    MutableString ms = MutableString.valueOf(characterData);

    assert(ms.equals("hello"), "test 1");
  } // runTestValueOf()

  public void runTestIndexOf() {
    MutableString ms = new MutableString("hello, worlld");
    MutableString ms2 = new MutableString("ll");

    int position = ms.indexOf(ms2, 1);

    assert(position == 2, "test 1");
  } // runTestIndexOf()

  public void runTestIndexOf2() {
    MutableString ms = new MutableString("hello, worlld");
    MutableString ms2 = new MutableString("ll");

    int position = ms.lastIndexOf(ms2, 1);

    assert(position == 10, "test 1");
  } // runTestIndexOf2()

  public void runTestSetLength() {
    MutableString ms = new MutableString("hello, this is my string");

    ms.setLength(10);

    assert(ms.equals("hello, thi"), "test 1");
  } // runTestSetLength()

  public void runTestSetLength2() {
    MutableString ms = new MutableString("hello, string");

    ms.setLength(15);

    assert(ms.equals("hello, string" + '\u0000' + '\u0000'), "test 1");
  } // runTestSetLength2()

  public void runTestHashCode() {
    String s = "AAAbcki here is a longer string";
    MutableString ms = new MutableString(s);

    assert(s.hashCode() == ms.hashCode(), "test 1");
  } // runTestHashCode()

  public void runTestHashCode2() {
    String s = "";
    MutableString ms = new MutableString(s);

    assert(s.hashCode() == ms.hashCode(), "test 1");
  } // runTestHashCode2()

  public void runTestHashCode3() {
    String s = "this is a longer string now";
    MutableString ms = new MutableString(s);

    assert(s.hashCode() == ms.hashCode(), "test 1");
  } // runTestHashCode3()

  public void runTestGetChars() {
    MutableString ms = new MutableString("this is my string");
    char[] characters = new char[3];

    ms.getChars(2, 5, characters, 0);

    MutableString ms2 = new MutableString(characters);
    assert(ms2.equals("is "), "test 1");
  } // runTestGetChars()

  public void runTestEncoding() {
    boolean result = true;
    
    try {
      byte[] characters = new byte[3];

      characters[0] = (byte) 'h';
      characters[1] = (byte) 'i';
      characters[2] = (byte) '!';

      MutableString ms = new MutableString(characters, "UTF8");
      byte[] bytes = ms.getBytes();
    } // try
    catch (UnsupportedEncodingException e) {
      result = false;
    } // catch

    assert(result, "test 1");
  } // runTestEncoding()

  public static void main(String[] args) {
    String[] classes = {"com.simscomputing.util.MutableStringTest"};
    new com.simscomputing.testbed.DefaultTester().testAndReport(classes);
  } // main()
} // MutableStringTest
