/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */

//
package org.enhydra.tool;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.net.URLClassLoader;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.lang.reflect.Field;
import java.util.Properties;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

//
public class ToolBoxInfo {
    //
    static ResourceBundle res = ResourceBundle.getBundle("org.enhydra.tool.common.Res"); // nores
    //
    public static final String  WML_FACTORY =
        "org.enhydra.wireless.wml.WMLDomFactory"; // nores
    public static final String  CHTML_FACTORY =
        "org.enhydra.wireless.chtml.CHTMLDomFactory"; // nores
    public static final String  XHTML_FACTORY =
        "org.enhydra.xml.xhtml.XHTMLDomFactory"; // nores
    public static final String  ENHYDRA_HOME = "enhydra.home"; // nores
    public static final String  XMLC_VERSION_CLASS =
        "org.enhydra.xml.xmlc.XMLCVersion"; // nores

    //
    private static final String FILE_ENHYDRA_JAR = "enhydra.jar"; // nores
    private static final String DIR_LIB = "lib"; // nores
    private static final String VERSION = "2.0.1";  // nores
    private static final String PROPERTY_FILENAME = "toolbox.properties";  // nores
    private static final String DEFAULT_ENHYDRA_PATH =
        "/usr/local/enhydra4.0a4"; // nores
    private static final String SERVER_RUN_CLASS30 =
        "com.lutris.multiServer.MultiServer";  // nores
    private static final String SERVER_RUN_CLASS40 =
        "org.enhydra.multiServer.bootstrap.Bootstrap";  // nores

    private ToolBoxInfo() {}

    public static Properties loadProperties() throws ToolException {
        String     filename = getPropertyFilename();
        File       file = new File(filename);
        Properties properties = new Properties();

        if (file.isFile() && file.canRead()) {
            try {
                FileInputStream in = new FileInputStream(file);

                properties.load(in);
            } catch (FileNotFoundException e) {
                throw new ToolException(e,
                   ResUtil.format(res.getString("Unable_to_read"), file));
            } catch (IOException e) {
                throw new ToolException(e,
                   ResUtil.format(res.getString("Unable_to_read"), file));
            }
        }
        return properties;
    }

    /**
     * Store persisted generator options to the properties file. This is
     * public and static to allow for the creation of a valid properties
     * file without having an instance of the CodeGen object. Creating
     * an instance of CodeGen requires that a valid properties file already
     * exist.
     *
     * @param prop
     * Properties to save into the codegen.properties file.
     *
     * @return
     * A file reference to the properties file.
     *
     * @exception java.io.IOException
     * Thrown if unable to write to the codegen.properties file.
     */
    public static File storeProperties(Properties prop) throws ToolException {
        File             file = null;
        FileOutputStream out = null;

        file = new File(ToolBoxInfo.getPropertyFilename());
        if (file.getParentFile() != null) {
            file.getParentFile().mkdirs();
        }
        try {
            out = new FileOutputStream(file);
            prop.store(out, res.getString("Enhydra_ToolBox1"));
        } catch (IOException e) {
            throw new ToolException(e,
                ResUtil.format(res.getString("Unable_to_store"), file));
        }
        return file;
    }

    public static String getJavaPath() {
        final String SYS_JAVA_HOME = "java.home"; // nores
        String path = null;
        File   file = null;

        path = System.getProperty(SYS_JAVA_HOME, new String());
        file = new File(path);
        return file.getParent().replace('\\', '/');
    }

    public static String getEnhydraVersion() {
        final String THREE = (new String()) + '3';
        final String FOUR = (new String()) + '4';
        final String FILE_KERNELSM_JAR = "KernelSM.jar"; // nores
        String       version = THREE;  // nores
        StringBuffer buf = new StringBuffer();

        buf.append(ToolBoxInfo.getEnhydraPath());
        buf.append(ToolBoxInfo.DIR_LIB);
        buf.append(File.separator);
        buf.append(FILE_KERNELSM_JAR);
        File kernel = null;

        kernel = new File(buf.toString());
        if (kernel.isFile()) {
            version = FOUR;
        }
        return version;
    }

    public static boolean isEnhydra3() {
        boolean is3 = false;
        String v = ToolBoxInfo.getEnhydraVersion();
        if ((v.length() > 0) && (v.charAt(0) == '3')) {
            is3 = true;
        }
        return is3;
    }

    public static boolean isClassAvailable(String className) {
        boolean available = false;

        try {
            Class.forName(className);
            available = true;
        } catch (ClassNotFoundException e) {
            available = false;
        }
        return available;
    }

    public static void main(String args[]) {
        System.out.println(res.getString("ToolBox_Info"));
        System.out.println(new String());
        System.out.println(ResUtil.format(res.getString("ToolBox_version_0_"), ToolBoxInfo.getToolBoxVersion()));
        System.out.println(ResUtil.format(res.getString("Enhydra_home_0_"), ToolBoxInfo.getEnhydraPath()));
        System.out.println(ResUtil.format(res.getString("Enhydra_version_0_"), ToolBoxInfo.getEnhydraVersion()));
        System.out.println(ResUtil.format(res.getString("Enhydra_in_classpath"), ToolBoxInfo.isEnhydraInClassPath()));
        System.out.println(ResUtil.format(res.getString("Enhydra_main_class_0_"), ToolBoxInfo.getEnhydraMainClass()));
        System.out.println(ResUtil.format(res.getString("XMLC_version_0_"), ToolBoxInfo.getXMLCVersion()));
        System.out.println(ResUtil.format(res.getString("XMLC_in_classpath_0_"), ToolBoxInfo.isXMLCInClassPath()));
    }

    public static boolean isXMLCVersion(int v) {
      boolean is = false;
      String version = ToolBoxInfo.getXMLCVersion();
      if (version.startsWith(new String() + v)) {
        is = true;
      }
      return is;
    }

    public static String getXMLCVersion() {
        final String FIELD_VERSION = "VERSION"; // nores;
        //
        String version = res.getString("_unknown_");
        Class  verClass = null;
        Field  verField = null;

        try {
            verClass = Class.forName(ToolBoxInfo.XMLC_VERSION_CLASS);
            verField = verClass.getDeclaredField(FIELD_VERSION);
            version = verField.get(null).toString();
        } catch (Exception e) {

            // no XMLC
        }
        return version;
    }

    /**
     * Get the class to run to start MultiServer.
     */
    public static String getEnhydraMainClass() {
        String runClass = ToolBoxInfo.SERVER_RUN_CLASS30;

        try {
            Class.forName(runClass);
        } catch (ClassNotFoundException e) {
            runClass = ToolBoxInfo.SERVER_RUN_CLASS40;
        }
        return runClass;
    }

    public static boolean isXMLCInClassPath() {
        boolean inPath = true;

        try {
            Class.forName(ToolBoxInfo.XMLC_VERSION_CLASS);
        } catch (java.lang.ClassNotFoundException e) {
            inPath = false;
        }
        return inPath;
    }

    public static boolean isEnhydraInClassPath() {
        boolean inPath = true;
        String  checkClass = ToolBoxInfo.SERVER_RUN_CLASS40;

        if (ToolBoxInfo.isEnhydra3()) {
            checkClass = ToolBoxInfo.SERVER_RUN_CLASS30;
        }
        try {
            Class testClass = Class.forName(checkClass);
        } catch (java.lang.ClassNotFoundException e) {
            inPath = false;
        }
        return inPath;
    }

    public static String getToolBoxVersion() {
        return VERSION;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public static String getPropertyFilename() {
        final String SYS_USER_HOME = "user.home"; // nores
        final String DIR_ENHYDRA = ".enhydra"; // nores
        //
        StringBuffer buf = new StringBuffer();

        buf.append(System.getProperties().getProperty(SYS_USER_HOME));
        buf.append(File.separator);
        buf.append(DIR_ENHYDRA);
        buf.append(File.separator);
        buf.append(PROPERTY_FILENAME);
        return buf.toString();
    }

    public static String getEnhydraPath() {
        String path = null;

        path = getEnhydraPathByClassLoader();
        if (path == null) {

            // path = getEnhydraPathByWorkingDirectory();
        }
        if (path == null) {
            path = getEnhydraPathByPropertyFile();
        }
        if (path == null) {
            path = ToolBoxInfo.DEFAULT_ENHYDRA_PATH;
        }
        path = FileUtil.toJavaPath(path);
        return path;
    }

    public static String[] getSupportedDocTypes() {
        final String TYPE_CHTML = "chtml"; // nores
        final String TYPE_HTML = "html"; // nores
        final String TYPE_WML = "wml"; // nores
        final String TYPE_XHTML = "xhtml"; // nores
        //
        String[]  types = new String[0];
        ArrayList list = new ArrayList(Arrays.asList(types));

        list.add(TYPE_HTML);
        if (ToolBoxInfo.isClassAvailable(ToolBoxInfo.WML_FACTORY)) {
            list.add(TYPE_WML);
        }
        if (ToolBoxInfo.isClassAvailable(ToolBoxInfo.CHTML_FACTORY)) {
            list.add(TYPE_CHTML);
        }
        if (ToolBoxInfo.isClassAvailable(ToolBoxInfo.XHTML_FACTORY)) {
            list.add(TYPE_XHTML);
        }
        list.trimToSize();
        types = new String[list.size()];
        types = (String[]) list.toArray(types);
        return types;
    }

    private static String getEnhydraPathByPropertyFile() {
        String     path = null;
        Properties properties = new Properties();

        try {
            properties = ToolBoxInfo.loadProperties();
            path = properties.getProperty(ToolBoxInfo.ENHYDRA_HOME);
        } catch (ToolException e) {
            e.printStackTrace();
            path = null;
        }
        return path;
    }

    private static String getEnhydraPathByWorkingDirectory() {
        File   file = null;
        String path = null;

        file = new File((new String()) + '.');    // <enhydra home>/lib
        file = new File(file.getAbsolutePath());
        file = file.getParentFile();
        if ((file.getParentFile() != null)
                && (file.getParentFile().getParentFile() != null)) {
            path = file.getParentFile().getParent();
        }
        return path;
    }

    private static String getEnhydraPathByClassLoader() {
        final String FILE_TOOLBOX_JAR = "toolbox.jar"; // nores
        //
        boolean     found = false;
        String      path = null;
        File        file = null;
        ToolBoxInfo info = new ToolBoxInfo();
        ClassLoader loader = info.getClass().getClassLoader();

        if (loader instanceof URLClassLoader) {
            URLClassLoader urlLoader = (URLClassLoader) loader;

            for (int i = 0; i < urlLoader.getURLs().length; i++) {
                path = urlLoader.getURLs()[i].getFile().toLowerCase();
                if (path.endsWith(FILE_TOOLBOX_JAR)) {
                    if (path.charAt(2) == ':') {
                        path = path.substring(1);
                    }
                    file = new File(path);
                    if (file.getParentFile() != null) {
                        file = file.getParentFile();
                        if ((file.getParentFile() != null)
                                && (file.getParentFile().getParentFile()
                                    != null)) {
                            path = file.getParentFile().getParent();
                            file = new File(path,
                                            ToolBoxInfo.DIR_LIB + File.separator
                                            + ToolBoxInfo.FILE_ENHYDRA_JAR);
                            if (file.isFile()) {
                                found = true;
                            }
                        }
                    }
                    break;
                }
            }
            if (!found) {
                path = null;
            }
        }
        return path;
    }

    private boolean isEnhydraPathValid(String home) {
        boolean      valid = false;
        StringBuffer path = new StringBuffer();
        File         f = null;

        path.append(home);
        path.append(File.separator);
        path.append(ToolBoxInfo.DIR_LIB);
        path.append(File.separator);
        path.append(ToolBoxInfo.FILE_ENHYDRA_JAR);
        f = new File(path.toString());
        valid = f.isFile();
        f = null;
        return valid;
    }

}
