/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.archive;

// Toolbox Imports

import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.common.PathHandle;

// Standard imports

import java.io.File;
import java.io.FileFilter;
import java.util.Vector;
import java.util.ResourceBundle;

//

public class JarPlan {

    // not to be resourced

    static ResourceBundle       res =
        ResourceBundle.getBundle ("org.enhydra.tool.archive.Res");      // nores

    //

    private final String        GEN_SOURCE = "Generated Source";        // nores
    private final String        DEP_CACHE = "dependency cache";         // nores

    //

    protected Vector            fileVector = new Vector ();
    protected FileFilter        fillFilter = null;
    private String              archivePath = new String ();
    private String              classesPath = new String ();

    public String getArchivePath () {
        return archivePath;
    }

    public void setArchivePath (String in) throws ArchiveException {
        if (in == null) {
            throw new ArchiveException (res.getString ("_archive_file_is_null"));
        } else {
            PathHandle  path = PathHandle.createPathHandle (in);

            if (path.isDirectory ()) {
                String  exMess = res.getString ("_archive_file_is_a");

                exMess = ResUtil.format (exMess, path);

                throw new ArchiveException (exMess);
            } else {
                archivePath = path.getPath ();
            }
        }
    }

    public String getClasses () {
        return classesPath;
    }

    public void setClasses (String in) throws ArchiveException {
        if (in == null) {
            throw new ArchiveException (res.getString ("Classes_directory_is"));
        } else {
            PathHandle  path = PathHandle.createPathHandle (in);

            if (path.isDirectory ()) {
                classesPath = path.getPath ();
            } else {
                String  exMess = res.getString ("Classes_directory_not");

                exMess = ResUtil.format (exMess, path);

                throw new ArchiveException (exMess);
            }
        }
    }

    public File[] getFilesInClasses () {
        File[]  files = new File[0];
        File    root = new File (getClasses ());

        fileVector.removeAllElements ();

        fillFilter = new ClassesFilter ();

        fillFileVector (root);

        files = (File[]) fileVector.toArray (files);

        fileVector.removeAllElements ();

        return files;
    }

    // /

    protected void fillFileVector (File root) {
        if (root.isDirectory ()) {
            File[]      children = new File[0];

            if (fillFilter == null) {
                children = root.listFiles ();
            } else {
                children = root.listFiles (fillFilter);
            }

            for (int i = 0; i < children.length; i++) {
                if (children[i].isDirectory ()) {
                    fillFileVector (children[i]);
                } else if (children[i].isFile ()) {
                    fileVector.addElement (children[i]);
                }
            }
        }
    }

    //

    class ClassesFilter implements FileFilter {
        public boolean accept (File file) {
            boolean     include = false;

            if (file.isDirectory ()) {
                include = true;
            } else {
                String  relative = null;

                relative =
                    file.getAbsolutePath ().substring (getClasses ().length ()
                                                       + 1);

                if (relative.startsWith (GEN_SOURCE)) {

                    // exclude

                } else if (relative.startsWith (DEP_CACHE)) {

                    // exclude

                } else {
                    include = true;
                }
            }

            return include;
        }

    }

}

