/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.archive;

// ToolBox imports

import org.enhydra.tool.common.ResUtil;

// Standard imports

import java.io.File;
import java.util.ResourceBundle;

//

public class Main {

    // not to be resourced

    static ResourceBundle       res =
        ResourceBundle.getBundle ("org.enhydra.tool.archive.Res");      // nores
    private static final String PARAM_JAR_ARCHIVE = "-jarArchive";      // nores
    private static final String PARAM_WEB_ARCHIVE = "-webArchive";      // nores
    private static final String PARAM_CLASSPATH = "-classpath";         // nores

    public Main () {
        System.out.println (res.getString ("Enhydra_Archive_Tool"));
        System.out.println (new String ());
    }

    public static void main (String[] args) {
        Main    main = new Main ();

        main.init (args);
    }

    private void init (String[] args) {
        ArchiveTool     packager = null;
        File            newFile = null;
        JarPlan         plan = null;

        try {
            plan = parseArgs (args);
            packager = new ArchiveTool ();
            newFile = packager.buildArchive (plan);

            System.out.println (ResUtil.format (res.getString ("Created_file_0_"),
                                                newFile));
        } catch (ArgumentException e) {
            System.out.println (ResUtil.format (res.getString ("Error_0_"),
                                                e.getMessage ()));
            showUsage ();
        } catch (ArchiveException e) {
            System.out.println (ResUtil.format (res.getString ("Error_0_"),
                                                e.getMessage ()));
        }
    }

    private JarPlan parseArgs (String[] args) throws ArgumentException {
        JarPlan plan = null;

        if (args.length < 1) {
            throw new ArgumentException (res.getString ("Too_few_arguments"));
        } else if (args[0].equalsIgnoreCase (PARAM_JAR_ARCHIVE)) {
            plan = parseJarArgs (args);
        } else if (args[0].equalsIgnoreCase (PARAM_WEB_ARCHIVE)) {
            plan = parseWarArgs (args);
        } else {
            throw new ArgumentException (ResUtil.format (res.getString ("First_argument_must"),
                    PARAM_WEB_ARCHIVE, PARAM_JAR_ARCHIVE));
        }

        return plan;
    }

    private JarPlan parseJarArgs (String[] args) throws ArgumentException {
        JarPlan jarPlan = null;

        if (args.length < 4) {
            throw new ArgumentException (res.getString ("Too_few_arguments"));
        } else if (args.length >= 5) {
            throw new ArgumentException (ResUtil.format (res.getString ("Too_many_arguments"),
                    PARAM_JAR_ARCHIVE));
        } else if (!args[1].equals (PARAM_CLASSPATH)) {
            throw new ArgumentException (ResUtil.format (res.getString ("Second_argument_must"),
                    PARAM_CLASSPATH));
        } else {
            try {
                jarPlan = new JarPlan ();

                jarPlan.setClasses (args[2]);
                jarPlan.setArchivePath (args[3]);
            } catch (ArchiveException e) {
                throw new ArgumentException (e.getMessage ());
            }
        }

        return jarPlan;
    }

    private WarPlan parseWarArgs (String[] args) throws ArgumentException {
        WarPlan warPlan = null;

        if (args.length < 6) {
            throw new ArgumentException (res.getString ("Too_few_arguments"));
        } else if (args.length >= 7) {
            throw new ArgumentException (res.getString ("Too_many_arguments1"));
        } else if (!args[1].equals (PARAM_CLASSPATH)) {
            throw new ArgumentException (ResUtil.format (res.getString ("Second_argument_must"),
                    PARAM_CLASSPATH));
        } else {
            try {
                warPlan = new WarPlan ();

                warPlan.setClasses (args[2]);
                warPlan.setContent (args[3]);
                warPlan.setWebXML (args[4]);
                warPlan.setArchivePath (args[5]);
            } catch (ArchiveException e) {
                throw new ArgumentException (e.getMessage ());
            }
        }

        return warPlan;
    }

    private void showUsage () {
        System.out.println (new String ());
        System.out.println (res.getString ("Required_arguments_"));
        System.out.print (' ');
        System.out.print (PARAM_WEB_ARCHIVE);
        System.out.print (' ');
        System.out.print (PARAM_CLASSPATH);
        System.out.print (' ');
        System.out.print (res.getString ("_classes_root_"));
        System.out.print (' ');
        System.out.print (res.getString ("_content_root_"));
        System.out.print (' ');
        System.out.print (res.getString ("_web_xml_"));
        System.out.print (' ');
        System.out.println (res.getString ("_archive_file_"));
        System.out.print (' ');
        System.out.print (PARAM_JAR_ARCHIVE);
        System.out.print (' ');
        System.out.print (PARAM_CLASSPATH);
        System.out.print (' ');
        System.out.println (res.getString ("_archive_file_"));
    }

    class ArgumentException extends Exception {
        public ArgumentException (String msg) {
            super (msg);
        }

    }

}

