/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.archive;

// ToolBox imports

import org.enhydra.tool.common.ResUtil;

// Standard imports

import java.io.IOException;
import java.io.File;
import java.util.ResourceBundle;

//

public class WarBuilder extends JarBuilder {

    // not to be resourced

    private final String        WEB_XML = "WEB-INF/web.xml";    // nores
    private final String        WEB_INF = "WEB-INF";            // nores
    private final String        CLASSES = "classes";            // nores

    //

    private WarPlan             warPlan = null;

    //

    public WarBuilder (){}

    public WarPlan getWarPlan () {
        return warPlan;
    }

    public void setWarPlan (WarPlan p) {
        warPlan = p;
    }

    public File buildArchive () throws ArchiveException {
        File    war = null;

        war = openJarStream (getWarPlan ().getArchivePath ());

        addWarClasses ();
        addContent ();

        if (isWebXMLinContent ()) {
            closeJarStream ();
        } else if (getWarPlan ().getWebXML ().length () > 0) {
            addWebXML ();
            closeJarStream ();
        } else {
            closeJarStream ();

            throw new ArchiveException (ResUtil.format (res.getString ("Archive_does_not"),
                                                        WEB_XML));
        }

        return war;
    }

    private boolean isWebXMLinContent () {
        boolean         in = false;
        StringBuffer    jarPath = new StringBuffer ();
        String          contentRoot = null;
        String          relative = null;
        File[]          files = getWarPlan ().getFilesInContent ();

        for (int i = 0; i < files.length; i++) {
            contentRoot = getWarPlan ().getContent ();
            relative =
                files[i].getAbsolutePath ().substring (contentRoot.length ()
                                                       + 1);
            relative = relative.replace ('\\', '/');

            if (relative.equals (WEB_XML)) {
                in = true;

                break;
            }
        }

        return in;
    }

    private void addContent () throws ArchiveException {
        File[]  files = getWarPlan ().getFilesInContent ();

        for (int i = 0; i < files.length; i++) {
            addContentFile (files[i]);
        }
    }

    private void addContentFile (File source) throws ArchiveException {
        StringBuffer    entryPath = new StringBuffer ();
        String          contentRoot = null;
        String          relative = null;

        contentRoot = getWarPlan ().getContent ();
        relative = source.getAbsolutePath ().substring (contentRoot.length ()
                + 1);
        relative = relative.replace ('\\', '/');

        entryPath.append (relative);

        try {
            createEntry (source, entryPath.toString ());
        } catch (IOException e) {
            throw new ArchiveException (e,
                                        ResUtil.format (res.getString ("WarBuilder_Unable_to"),
                                                        source));
        }
    }

    private void addWarClasses () throws ArchiveException {
        File[]  files = getWarPlan ().getFilesInClasses ();

        for (int i = 0; i < files.length; i++) {
            addWarClassesFile (files[i]);
        }
    }

    private void addWarClassesFile (File source) throws ArchiveException {
        StringBuffer    entryPath = new StringBuffer ();
        String          classRoot = null;
        String          relative = null;

        //

        classRoot = getWarPlan ().getClasses ();
        relative = source.getAbsolutePath ().substring (classRoot.length ()
                + 1);
        relative = relative.replace ('\\', '/');

        //

        entryPath.append (WEB_INF);
        entryPath.append ('/');
        entryPath.append (CLASSES);
        entryPath.append ('/');
        entryPath.append (relative);

        //

        try {
            createEntry (source, entryPath.toString ());
        } catch (IOException e) {
            throw new ArchiveException (e,
                                        ResUtil.format (res.getString ("WarBuilder_Unable_to1"),
                                                        source));
        }
    }

    private void addWebXML () throws ArchiveException {
        StringBuffer    entryPath = new StringBuffer ();
        String          classRoot = null;
        String          relative = null;
        File            source = null;

        source = new File (getWarPlan ().getWebXML ());

        entryPath.append (WEB_XML);

        try {
            createEntry (source, entryPath.toString ());
        } catch (IOException e) {
            throw new ArchiveException (e,
                                        ResUtil.format (res.getString ("WarBuilder_Unable_to2"),
                                                        getWarPlan ().getWebXML ()));
        }
    }

}

