/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;

// ToolBox imports

import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.codegen.wizard.AppOptionPanel1;
import org.enhydra.tool.codegen.wizard.AppOptionPanel2;
import org.enhydra.tool.codegen.wizard.AppOptionPanel3;
import org.enhydra.tool.codegen.wizard.CodeGenPanel;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.ReplacementSet;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.common.TemplateFile;
import org.enhydra.tool.common.TemplateTool;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.event.TemplateEvent;
import org.enhydra.tool.common.event.TemplateListener;
import org.enhydra.tool.configure.ConfigTool;

// Standard imports

import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

//

abstract public class AppGenerator extends TemplateDrivenGenerator
    implements Constants {

    //

    private CodeGenPanel[]      panels = null;
    private TemplateFilter      templateFilter = null;
    private TemplateFilter      copyFilter = null;
    private String              client = null;
    private String              javaPath = ToolBoxInfo.getJavaPath ();
    private String[]            copyright = new String[0];
    private String[]            kelpSteps = getDefaultKelpSteps ();
    private String[]            shellSteps = getDefaultShellSteps ();
    private String[][]          kelpMap = null;
    private boolean             make = true;
    private boolean             script = true;

    abstract public String[] getExcludeEnds ();

    //

    public AppGenerator () {
        super ();

        setReplacementSet (new AppReplacementSet ());
        setOptionSet (new AppOptionSet ());
    }

    public TemplateFile[] preprocessTemplates (TemplateFile[] temps) {
        TemplateFile    cursor = null;
        String          path = new String ();
        String          newPath = new String ();
        StringBuffer    clientExt = new StringBuffer ();
        ArrayList       list = null;
        GeneratorOption option = null;
        boolean         e4 = false;

        try {
            option = getOptionSet ().lookup (AppOptionSet.E4);
            e4 = option.isValue ();
        } catch (GeneratorException e) {
            e.printStackTrace ();

            e4 = false;
        }

        clientExt.append ('.');
        clientExt.append (getClient ().toLowerCase ());
        clientExt.append (CLIENT_SUFFIX);

        list = new ArrayList (Arrays.asList (temps));

        for (int i = 0; i < list.size (); i++) {
            cursor = (TemplateFile) list.get (i);
            path = cursor.getOutput ().getAbsolutePath ();
            path = path.toLowerCase ();

            if (path.endsWith (clientExt.toString ())) {
                newPath = cursor.getOutput ().getAbsolutePath ();
                newPath =
                    newPath.substring (0,
                                       newPath.length ()
                                       - clientExt.toString ().length ());

                cursor.setOutput (new File (newPath));
                list.set (i, cursor);
            } else if (path.endsWith (CLIENT_SUFFIX)) {
                list.remove (i);
                list.trimToSize ();

                i--;
            } else if (!e4) {
                if (path.endsWith (FILE_BOOTSTRAP4_CONF)
                        || path.endsWith (FILE_JAVA_POLICY)
                        || path.endsWith (FILE_START4_IN)) {
                    list.remove (i);
                    list.trimToSize ();

                    i--;
                }
            }
        }

        list.trimToSize ();

        temps = new TemplateFile[list.size ()];
        temps = (TemplateFile[]) list.toArray (temps);

        list.clear ();

        return temps;
    }

    public File[] preprocessCopyList (File[] files) {
        File            cursor = null;
        String          path = new String ();
        ArrayList       list = null;
        GeneratorOption option = null;
        boolean         e4 = false;

        try {
            option = getOptionSet ().lookup (AppOptionSet.E4);
            e4 = option.isValue ();
        } catch (GeneratorException e) {
            e.printStackTrace ();

            e4 = false;
        }

        list = new ArrayList (Arrays.asList (files));

        for (int i = 0; i < list.size (); i++) {
            cursor = (File) list.get (i);
            path = cursor.getAbsolutePath ();
            path = path.toLowerCase ();

            if (!e4) {
                if (path.endsWith (FILE_BOOTSTRAP4_CONF)
                        || path.endsWith (FILE_JAVA_POLICY)
                        || path.endsWith (FILE_START4_IN)) {
                    list.remove (i);
                    list.trimToSize ();

                    i--;
                }
            }
        }

        list.trimToSize ();

        files = new File[list.size ()];
        files = (File[]) list.toArray (files);

        list.clear ();

        return files;
    }

    public File[] generate () throws GeneratorException {
        File[]          files = new File[0];
        File            readme = null;
        ArrayList       list = null;

        setOverwrite (getOptionSet ().lookup (AppOptionSet.OVERWRITE).isValue ());

        make = (!getOptionSet ().lookup (AppOptionSet.NOMAKE).isValue ());
        script = (!getOptionSet ().lookup (AppOptionSet.NOSCRIPT).isValue ());
        files = super.generate ();
        list = new ArrayList (Arrays.asList (files));
        readme = generateReadme (getDestinationPath ());

        list.add (readme);

        files = new File[list.size ()];
        files = (File[]) list.toArray (files);

        list.clear ();

        return files;
    }

    /**
     * Get the default option panels and preset the option set
     * for each panel.
     *
     * @return
     * An array containing the three default option panels.
     */
    public CodeGenPanel[] getWizardPanels () throws GeneratorException {

        // don't initialize until called.

        if (panels == null && isSwing ()) {
            File        eHome = null;

            eHome = new File (ToolBoxInfo.getEnhydraPath ());
            panels = new CodeGenPanel[3];
            panels[0] = new AppOptionPanel1 ();
            panels[1] = new AppOptionPanel2 ();
            panels[2] = new AppOptionPanel3 ();

            for (int i = 0; i < panels.length; i++) {
                panels[i].setOptionSet (getOptionSet ());
                panels[i].readOptionSet ();
                panels[i].setEnyhdraHome (eHome);
            }
        }

        return panels;
    }

    public void storeProperties () {
        try {
            getOptionSet ().lookup (AppOptionSet.OVERWRITE).setValue (isOverwrite ());
        } catch (GeneratorException e) {
            e.printStackTrace ();
        }

        super.storeProperties ();
    }

    public void setProperties (Properties p) throws GeneratorException {
        super.setProperties (p);

        // Ignore persisted make and script flags

        getOptionSet ().lookup (AppOptionSet.NOMAKE).setValue (false);
        getOptionSet ().lookup (AppOptionSet.NOSCRIPT).setValue (false);

        // ensure unique projectName

        PathHandle      handle = null;
        File            destDir = null;
        String          destPath = new String ();
        String          projectName = new String ();
        String[]        fileList = new String[0];
        boolean         newName = false;

        handle =
            PathHandle.createPathHandle (getOptionSet ().lookup (AppOptionSet.ROOT).getValue ());
        destPath = handle.getPath ();
        projectName =
            getOptionSet ().lookup (AppOptionSet.PROJECT).getValue ();
        destDir = handle.getFile ();

        if (!destDir.exists ()) {
            destDir.mkdirs ();
        }

        fileList = destDir.list ();

        for (int i = 0; i < fileList.length; i++) {
            if (fileList[i].equalsIgnoreCase (projectName)) {
                newName = true;

                char    lastChar = projectName.charAt (projectName.length ()
                                                       - 1);
                String  digits = new String ();

                while (Character.isDigit (lastChar)) {
                    projectName = projectName.substring (0,
                                                         projectName.length ()
                                                         - 1);
                    digits = lastChar + digits;
                    lastChar = projectName.charAt (projectName.length () - 1);
                }

                int     next = 1;

                try {
                    next = Integer.parseInt (digits);
                } catch (NumberFormatException e) {
                    next = 1;
                }

                next++;
                projectName = projectName + next;
                i = 0;          // start the search over again
            }
        }

        if (newName) {
            getOptionSet ().lookup (AppOptionSet.PROJECT).setValue (projectName);
        }
    }

    public FileFilter getCopyFilter () {
        if (copyFilter == null) {
            copyFilter = new TemplateFilter (false);
        }

        return copyFilter;
    }

    public FileFilter getTemplateFilter () {
        if (templateFilter == null) {
            templateFilter = new TemplateFilter (true);
        }

        return templateFilter;
    }

    public String getDestinationPath () throws GeneratorException {
        StringBuffer    buf = new StringBuffer ();
        PathHandle      path = null;

        buf.append (getOptionSet ().lookup (AppOptionSet.ROOT).getValue ());
        buf.append (File.separator);
        buf.append (getOptionSet ().lookup (AppOptionSet.PROJECT).getValue ());

        path = PathHandle.createPathHandle (buf.toString ());

        return path.getPath ();
    }

    public void setJavaPath (String p) {
        javaPath = PathHandle.createPathString (p);
    }

    public String getJavaPath () {
        return javaPath;
    }

    /**
     * Get steps required to build the project from
     * a unix shell.
     *
     * @return
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project using make files.
     */
    public String[] getShellSteps () {
        return shellSteps;
    }

    public String getClient () {
        if (client == null) {
            try {
                client =
                    getOptionSet ().lookup (AppOptionSet.CLIENT).getValue ().toUpperCase ();
            } catch (GeneratorException e) {
                e.printStackTrace ();

                client = null;

                return TYPE_HTML;
            }
        }

        return client;
    }

    /**
     * Set the steps required to build the project from
     * a unix shell.
     *
     * @param steps
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project from a unix shell.
     */
    public void setShellSteps (String[] steps) {
        shellSteps = steps;
    }

    /**
     * Get the default table for mapping directories to
     * packages that maps resources directory to the
     * presentaion package and the resource media directory
     * to the a media package.
     *
     * @param appPackage
     * This is the application package to prefix to each mapped
     * package name.
     *
     * @return
     * The default package map for template driven generators.
     */
    public String[][] expandKelpPackageMap () throws GeneratorException {
        String[][]      exMap = new String[0][0];
        TemplateTool    tool = null;

        tool = new TemplateTool ();

        tool.setReplacements (getReplacementSet ().toArray ());

        if (kelpMap == null) {
            initKelpPackageMap ();
        }

        exMap = new String[kelpMap.length][kelpMap[0].length];

        for (int i = 0; i < exMap.length; i++) {
            for (int j = 0; j < exMap[i].length; j++) {
                exMap[i][j] = kelpMap[i][j];
            }
        }

        for (int i = 0; i < exMap.length; i++) {
            for (int j = 0; j < exMap[i].length; j++) {
                exMap[i][j] = tool.lineSearchAndReplace (exMap[i][j]);
            }
        }

        return exMap;
    }

    public String[][] getKelpPackageMap () throws GeneratorException {
        if (kelpMap == null) {
            initKelpPackageMap ();
        }

        return kelpMap;
    }

    public void setKelpPackageMap (String[][] m) {
        kelpMap = m;
    }

    /**
     * Get steps required to build the project with Kelp.
     *
     * @return
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project within an IDE.
     */
    public String[] getKelpSteps () {
        return kelpSteps;
    }

    /**
     * Set the steps required to build the project from
     * within an IDE.
     *
     * @param steps
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project within an IDE.
     */
    public void setKelpSteps (String[] steps) {
        kelpSteps = steps;
    }

    /**
     * Get the path where Java source files were generated.
     *
     * @return
     * The java source directory.
     */
    public String getProjectSourcePath () throws GeneratorException {
        String  path = new String ();
        String  lookFor = AppReplacementSet.at_PACKAGE_DIR_at.toLowerCase ();
        File    rootDir = new File (getDestinationPath ());

        path = findProjectSourcePath (lookFor, getTemplateDirectory (),
                                      rootDir);

        if (path == null) {
            throw new GeneratorException (res.getString ("Project_source_not"));
        }

        return path;
    }

    /**
     * Method declaration
     *
     *
     * @exception GeneratorException
     */
    public void initReplacementSet () throws GeneratorException {
        try {
            String      packageName = null;
            String      shellPackageDir = null;
            String      systemPackageDir = null;
            String      content = null;
            int         index = 0;

            //

            packageName =
                getOptionSet ().lookup (AppOptionSet.PACKAGE).getValue ();
            shellPackageDir = packageName.replace ('.', '/');
            systemPackageDir = packageName.replace ('.', File.separatorChar);

            //

            readCopyright ();
            getReplacementSet ().lookup (AppReplacementSet.COPYRIGHT).setReplaceWith (copyright);
            getReplacementSet ().lookup (AppReplacementSet.at_PROJECT_at).setReplaceWith (getOptionSet ().lookup (AppOptionSet.PROJECT).getValue ());
            getReplacementSet ().lookup (AppReplacementSet.at_PACKAGE_DIR_at).setReplaceWith (systemPackageDir);
            getReplacementSet ().lookup (AppReplacementSet.SHELL_PACKAGE_DIR).setReplaceWith (shellPackageDir);
            getReplacementSet ().lookup (AppReplacementSet.JAVA_PACKAGE).setReplaceWith (packageName);

            // client

            if (getClient ().equalsIgnoreCase (TYPE_WML)) {
                content = CONTENT_WML;
            } else {

                // chtml, html, xhtml, etc.

                content = CONTENT_HTML;
            }

            getReplacementSet ().lookup (AppReplacementSet.CLIENT).setReplaceWith (getClient ().toUpperCase ());
            getReplacementSet ().lookup (AppReplacementSet.CONTENT_TYPE).setReplaceWith (content);
        } catch (ToolException e) {
            e.printStackTrace ();

            throw new GeneratorException (e,
                                          res.getString ("Unable_to_init_rep"));
        }
    }

    public void onTemplate (TemplateEvent event) {
        if (copyright.length >= 1) {
            int                 last = 0;
            Replacement[]       reps = event.getTool ().getReplacements ();
            String[]            newCR = new String[0];
            PathHandle          pathOut = event.getPathOut ();

            newCR = new String[copyright.length + 2];
            newCR[0] = new String ();

            if (newCR.length >= 1) {
                last = (newCR.length - 1);
            }

            if (pathOut.endsWith ('/' + FILE_MAKEFILE)
                    || pathOut.hasExtension (TYPE_MK)) {
                for (int i = 1; i < (newCR.length - 1); i++) {
                    newCR[i] = "#  " + copyright[i - 1];        // nores
                }

                newCR[last] = "#";                              // nores
            } else if (pathOut.hasExtension (TYPE_JAVA)) {
                for (int i = 1; i < (newCR.length - 1); i++) {
                    newCR[i] = " * " + copyright[i - 1];        // nores
                }

                newCR[last] = " *";                             // nores
            } else {
                newCR = new String[1];
                newCR[0] = new String ();
            }

            try {
                getReplacementSet ().lookup (AppReplacementSet.COPYRIGHT).setReplaceWith (newCR);
                event.getTool ().setReplacements (reps);
            } catch (ToolException e) {
                e.printStackTrace ();
            }
        }
    }

    private void readCopyright () {
        String          copyText = new String ();
        String          copyPath = new String ();
        StringTokenizer tokenizer = null;
        File            copyFile = null;
        BufferedReader  reader = null;
        Vector          readVector = new Vector ();

        try {
            copyText =
                getOptionSet ().lookup (AppOptionSet.COPYRIGHT).getValue ();

            if (copyText.trim ().length () > 0) {
                final String    NEWLINE = "\n";         // nores

                tokenizer = new StringTokenizer (copyText, NEWLINE);
                copyright = new String[tokenizer.countTokens ()];

                for (int i = 0; i < copyright.length; i++) {
                    copyright[i] = tokenizer.nextToken ();
                }
            }
        } catch (GeneratorException e) {
            e.printStackTrace ();
        }

        if (copyright.length == 0) {
            try {
                copyPath =
                    getOptionSet ().lookup (AppOptionSet.COPYRIGHTFILE).getValue ();

                if (FileUtil.isFile (copyPath)) {
                    copyFile = new File (copyPath);
                    reader = new BufferedReader (new FileReader (copyFile));

                    String      line = reader.readLine ();

                    while (line != null) {
                        readVector.addElement (new String (line));

                        line = reader.readLine ();
                    }

                    reader.close ();

                    copyright = new String[readVector.size ()];
                    copyright = (String[]) readVector.toArray (copyright);

                    readVector.clear ();
                }
            } catch (GeneratorException e) {
                e.printStackTrace ();

                copyright = new String[0];
            } catch (FileNotFoundException e) {
                e.printStackTrace ();

                copyright = new String[0];
            } catch (IOException e) {
                e.printStackTrace ();

                copyright = new String[0];
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param target
     *
     * @exception GeneratorException
     */
    public Replacement[] createReplacementsForDirectory (PathHandle path)
            throws GeneratorException {
        Replacement[]   pathReps = new Replacement[0];
        StringBuffer    rootPath = new StringBuffer ();
        PathHandle      destHandle = null;
        PathHandle      target = null;
        PathHandle      cursor = null;

        destHandle = PathHandle.createPathHandle (getDestinationPath ());
        target = PathHandle.createPathHandle (path.getPath ());
        cursor = PathHandle.createPathHandle (path.getPath ());

        if (destHandle.parentOf (cursor)) {
            rootPath.append ('.');
            rootPath.append ('.');

            cursor =
                PathHandle.createPathHandle (cursor.getFile ().getParent ());

            while (destHandle.parentOf (cursor)) {
                rootPath.append ('/');
                rootPath.append ('.');
                rootPath.append ('.');

                cursor =
                    PathHandle.createPathHandle (cursor.getFile ().getParent ());
            }
        } else {
            rootPath.append ('.');
        }

        try {
            getReplacementSet ().lookup (AppReplacementSet.SHELL_ROOT_PATH).setReplaceWith (rootPath.toString ());
        } catch (ToolException e) {
            throw new GeneratorException (e,
                                          res.getString ("Unable_to_init_root"));
        }

        pathReps = getReplacementSet ().toArray ();

        StringBuffer    inputBuf = new StringBuffer ();
        PathHandle      inputPath = null;

        inputBuf.append (getDestinationPath ().toLowerCase ());
        inputBuf.append (File.separator);
        inputBuf.append (DIR_INPUT);
        inputBuf.append (File.separator);

        inputPath = PathHandle.createPathHandle (inputBuf.toString ());

        if (!inputPath.parentOf (target)) {
            try {
                Replacement[]   confReps = new Replacement[0];
                ArrayList       newList = null;
                ReplacementSet  pathSet = null;

                confReps =
                    ConfigTool.createReplacements (getDestinationPath ()
                                                   + File.separator
                                                   + DIR_OUTPUT);
                newList = new ArrayList ();

                newList.addAll (Arrays.asList (pathReps));
                newList.addAll (Arrays.asList (confReps));

                pathReps = (Replacement[]) newList.toArray (pathReps);

                newList.clear ();

                pathSet = new ReplacementSet (pathReps);

                pathSet.lookup (ConfigTool.JAVA_PATH).setReplaceWith (getJavaPath ());

                pathReps = pathSet.toArray ();
            } catch (ToolException e) {
                throw new GeneratorException (e,
                                              res.getString ("Unable_to_create_path"));
            }
        }

        return pathReps;
    }

    // /
    // /

    private String findProjectSourcePath (String lookFor, File source,
                                          File dest) {
        String          found = null;
        String          destPath = null;
        File[]          list = new File[0];
        TemplateTool    tool = null;

        list = source.listFiles ();
        tool = new TemplateTool ();

        tool.setReplacements (getReplacementSet ().toArray ());

        destPath = dest.getAbsolutePath ();
        destPath = tool.lineSearchAndReplace (destPath);
        dest = new File (destPath);

        for (int i = 0; i < list.length; i++) {
            if (list[i].isDirectory ()) {
                String  lookIn = new String ();

                lookIn = list[i].getAbsolutePath ();
                lookIn = lookIn.replace ('\\', '/');
                lookIn = lookIn.toLowerCase ();

                if (lookIn.endsWith (lookFor.toLowerCase ())) {
                    found = dest.getAbsolutePath ().replace ('\\', '/');
                } else {
                    File        nextDest = null;

                    nextDest = new File (dest, list[i].getName ());
                    found = findProjectSourcePath (lookFor, list[i],
                                                   nextDest);
                }
            }

            if (found != null) {
                break;
            }
        }

        return found;
    }

    private void initKelpPackageMap () throws GeneratorException {
        kelpMap = new String[2][2];

        StringBuffer    buf = new StringBuffer ();

        buf.append (getDestinationPath ());
        buf.append (File.separator);
        buf.append (DIR_SRC);
        buf.append (File.separator);
        buf.append (DIR_AT_PACKAGE_DIR_AT);
        buf.append (File.separator);
        buf.append (DIR_RESOURCES);

        kelpMap[0][0] = buf.toString ();

        buf.setLength (0);
        buf.append ("@JAVA_PACKAGE@");          // nores
        buf.append ('.');
        buf.append (DIR_PRESENTATION);

        kelpMap[0][1] = buf.toString ();

        buf.setLength (0);
        buf.append (getDestinationPath ());
        buf.append (File.separator);
        buf.append (DIR_SRC);
        buf.append (File.separator);
        buf.append (DIR_AT_PACKAGE_DIR_AT);
        buf.append (File.separator);
        buf.append (DIR_RESOURCES);
        buf.append (File.separator);
        buf.append (DIR_MEDIA);

        kelpMap[1][0] = buf.toString ();
        kelpMap[1][1] = DIR_MEDIA;
    }

    /**
     * Method declaration
     *
     *
     * @param destDir
     *
     * @exception java.io.IOException
     */
    private File generateReadme (String destPath) throws GeneratorException {
        File            readme = null;
        PrintWriter     out = null;
        Replacement     rep = null;
        String          projName = new String ();

        readme = new File (destPath + File.separator + FILE_README_HTML);

        try {
            out = new PrintWriter (new FileWriter (readme));
        } catch (IOException e) {
            throw new GeneratorException (e,
                                          res.getString ("Unable_to_create_readme"));
        }

        try {
            rep =
                getReplacementSet ().lookup (AppReplacementSet.at_PROJECT_at);
            projName = rep.getReplaceWith ()[0];
        } catch (ToolException e) {
            projName = res.getString ("The_project");
        }

        out.println ("<html>");         // nores
        out.println ("<head>");         // nores
        out.println ("  "
                     + ResUtil.format (res.getString ("Title_readme"),
                                       projName));
        out.println ("</head>");        // nores
        out.println ("<body>");         // nores
        out.println ("<br>");           // nors

        if (getKelpSteps () != null && getKelpSteps ().length >= 1) {
            out.println ("<h2>"
                         + ResUtil.format (res.getString ("To_build_with_kelp"), projName)
                         + "</h2>");
            out.println ("<ol>");                               // nores

            for (int i = 0; i < getKelpSteps ().length; i++) {
                out.println ("  <li>");                         // nores
                out.println ("    " + getKelpSteps ()[i]);      // nores
                out.println ("  </li>");                        // nores
            }

            out.println ("</ol>");                              // nores
        }

        if ((!script) || (!make)) {

            // don't show shell build steps if not all required
            // files are present.

        } else if (getShellSteps () != null && getShellSteps ().length >= 1) {
            out.println ("<h2>"
                         + ResUtil.format ("To_build_with_shell", projName)
                         + "</h2>");
            out.println ("<ol>");                               // nores

            for (int i = 0; i < getShellSteps ().length; i++) {
                out.println ("  <li>");                         // nores
                out.println ("    " + getShellSteps ()[i]);     // nores
                out.println ("  </li>");                        // nores
            }

            out.println ("</ol>");                              // nores
        }

        out.println ("</body>");        // nores
        out.println ("</html>");        // nores
        out.flush ();
        out.close ();

        return readme;
    }

    private String[] getDefaultKelpSteps () {
        String[]        steps = new String[4];

        steps[0] = res.getString ("StepsKelp0");
        steps[1] = res.getString ("StepsKelp1");
        steps[2] = res.getString ("StepsKelp2");
        steps[3] = res.getString ("StepsKelp3");

        return steps;
    }

    private String[] getDefaultShellSteps () {
        String[]        steps = new String[3];
        StringBuffer    buf = new StringBuffer ();

        buf.append (res.getString ("StepsShell0a"));
        buf.append (res.getString ("StepsShell0b"));
        buf.append (res.getString ("StepsShell0c"));
        buf.append (res.getString ("StepsShell0d"));
        buf.append (res.getString ("StepsShell0e"));
        buf.append (res.getString ("StepsShell0f"));
        buf.append (res.getString ("StepsShell0g"));
        buf.append (res.getString ("StepsShell0h"));

        steps[0] = buf.toString ();
        steps[1] = res.getString ("StepsShell1");

        buf.setLength (0);
        buf.append (res.getString ("StepsShell2a"));
        buf.append (res.getString ("StepsShell2b"));

        steps[2] = buf.toString ();

        return steps;
    }

    class TemplateFilter implements FileFilter {
        private boolean         tempIn = true;
        private String[]        exclude = getExcludeEnds ();

        public TemplateFilter (boolean b) {
            tempIn = b;
        }

        public boolean accept (File file) {
            boolean     include = false;

            if (file.isDirectory ()) {
                include = tempIn;
            } else {
                PathHandle      path = null;

                path = PathHandle.createPathHandle (file);

                if (tempIn) {
                    include = path.hasExtension (TYPE_TEMPLATE);
                } else {
                    include = (!path.hasExtension (TYPE_TEMPLATE));
                }

                if (include) {
                    if (path.hasExtension (TYPE_MK)
                            || path.endsWith ('.' + TYPE_MK + '.' + TYPE_TEMPLATE)
                            || path.endsWith (FILE_MAKEFILE)
                            || path.endsWith (FILE_MAKEFILE + '.'
                                              + TYPE_TEMPLATE)) {
                        include = make;
                    } else if (path.hasExtension (TYPE_IN)
                               || path.endsWith ('.' + TYPE_IN + '.'
                               + TYPE_TEMPLATE)) {
                        if ((!path.endsWith ('.' + TYPE_CONF + '.' + TYPE_IN))
                                && (!path.endsWith ('.' + TYPE_CONF + '.'
                                                    + TYPE_IN + '.'
                                                    + TYPE_TEMPLATE))) {
                            include = script;
                        }
                    }
                }

                if (include) {
                    for (int i = 0; i < exclude.length; i++) {
                        if (path.endsWith (exclude[i])) {
                            include = false;

                            break;
                        }
                    }
                }
            }

            return include;
        }

    }

}

