/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.codegen.CodeGen;

// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.File;
import java.util.ResourceBundle;

/**
 * Swing layout panel for configuration dialog.
 *
 * @author Paul Mahar
 */
public class ConfigPanel extends JPanel {
    //
    static ResourceBundle res = ResourceBundle.getBundle("org.enhydra.tool.codegen.Res"); // nores
    //
    transient protected JButton           buttonConfig;
    transient protected JButton           buttonCancel;
    //
    transient private JPanel              panelDir;
    transient private JPanel              panelButton;
    transient private GridBagLayout       layoutMain;
    transient private JButton             buttonHome;
    transient private JTextField          textHome;
    transient private GridBagLayout       buttonLayout;
    transient private GridBagLayout       layoutDir;
    transient private JPanel              panelFiller;
    transient private TitledBorder        border;
    transient private BorderLayout        layoutFiller;
    transient private LocalButtonListener buttonListener;

    /**
     * Create a panel containing controls required
     * to configure CodeGen.
     */
    public ConfigPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    ///
    /// PRIVATE METHODS
    ///

    /**
     * Method declaration
     *
     *
     */
    private void pmInit() {
        border = BorderFactory.createTitledBorder(res.getString("Enhydra_home"));
        panelDir.setBorder(border);
        buttonListener = new LocalButtonListener();
        buttonHome.addActionListener(buttonListener);
        textHome.setText(PathHandle.createPathString(ToolBoxInfo.getEnhydraPath()));
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit() throws Exception {
        panelDir = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        panelButton = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonHome = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        textHome = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonConfig =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonCancel =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonLayout =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutDir =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        layoutFiller =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        buttonHome.setText(res.getString("Set_"));
        buttonConfig.setText(res.getString("Configure"));
        buttonCancel.setText(res.getString("Cancel"));
        panelButton.setLayout(buttonLayout);
        panelDir.setLayout(layoutDir);
        panelFiller.setLayout(layoutFiller);
        panelDir.add(textHome,
                     new GridBagConstraints(0, 0, 1, 1, 0.8, 0.8,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(10, 10, 10, 10), 200,
                                            0));
        panelDir.add(buttonHome,
                     new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.NONE,
                                            new Insets(10, 10, 10, 10), 0,
                                            0));
        panelButton.add(buttonConfig,
                        new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(buttonCancel,
                        new GridBagConstraints(2, 0, 1, 1, 0.1, 0.1,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(panelFiller,
                        new GridBagConstraints(0, 0, 1, 1, 0.4, 0.4,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.BOTH,
                                               new Insets(0, 0, 0, 0), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelDir,
                 new GridBagConstraints(0, 0, 1, 1, 0.6, 0.6,
                                        GridBagConstraints.NORTH,
                                        GridBagConstraints.BOTH,
                                        new Insets(20, 20, 10, 20), 0, 0));
        this.add(panelButton,
                 new GridBagConstraints(0, 2, 1, 1, 0.4, 0.4,
                                        GridBagConstraints.SOUTH,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected File getEnhydraHome() {
        File file = new File(textHome.getText());

        return file;
    }

    protected void setEnhydraHome(File f) {
      if (f != null && f.isDirectory()) {
        textHome.setText(PathHandle.createPathString(f));
      } else {
        textHome.setText(new String());
      }
    }

    /**
     * Method declaration
     *
     */
    private void browseForHome() {
        File file = null;

        file =
            SwingUtil.getDirectoryChoice(this, textHome.getText(),
                                         res.getString("Select_the_Enhydra"));
        if (file != null && file.exists()) {
            textHome.setText(file.getAbsolutePath());
        }
    }

    /**
     */
    private class LocalButtonListener implements ActionListener {

        /**
         * ActionListener implementation.
         *
         * @param event
         * Event that triggered this listener.
         *
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonHome) {
                browseForHome();
            }
        }

    }
}
