/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

// Standard imports

import java.io.File;
import java.util.Vector;
import javax.swing.filechooser.FileFilter;

/**
 * An implementation of FileFilter that filters out all files
 * except for those matching a set of given extensions.
 */
public class ExtensionFilter extends FileFilter
    implements java.io.FileFilter {
    private Vector      include = new Vector ();
    private Vector      exclude = new Vector ();
    private String      descTitle = new String ();
    private boolean     directoryValid = true;
    private boolean     excludeOnly = false;

    /**
     * Creates an extension file filter without an extensions set.
     */
    public ExtensionFilter (){}

    /**
     * Return true if this file should be shown in a file chooser.
     */
    public boolean accept (File f) {
        boolean         acceptFile = false;
        int             count = include.size ();
        PathHandle      handle = null;

        if (f == null) {
            acceptFile = true;
        } else if (f.isDirectory ()) {
            acceptFile = isDirectoryValid ();
        } else {
            handle = PathHandle.createPathHandle (f);

            if (isExcludeOnly ()) {
                acceptFile = (!exclude (handle));
            } else {
                for (int i = 0; i < count; i++) {
                    String      current = include.elementAt (i).toString ();

                    if (handle.hasExtension (current)) {
                        acceptFile = (!exclude (handle));
                    }
                }
            }
        }

        return acceptFile;
    }

    private boolean exclude (PathHandle handle) {
        boolean ex = false;
        String  cursor = new String ();

        for (int i = 0; i < exclude.size (); i++) {
            cursor = exclude.elementAt (i).toString ();

            if (handle.endsWith (cursor)) {
                ex = true;

                break;
            }
        }

        return ex;
    }

    /**
     * Adds an an extension to the list of acceptable file
     * types.
     */
    public void addExtension (String in) {
        if (in != null) {
            in = in.toLowerCase ().trim ();
        }

        if (!include.contains (in)) {
            include.addElement (in);
        }
    }

    public void addExclusion (String ex) {
        if (ex != null) {
            ex = ex.toLowerCase ().trim ();
        }

        if (!exclude.contains (ex)) {
            exclude.addElement (ex);
        }
    }

    /**
     * Returns the human readable description of this filter.
     */
    public String getDescription () {
        StringBuffer    desc = new StringBuffer ();

        desc.append (getDescriptionTitle ());
        desc.append (' ');
        desc.append ('(');

        if (include.size () == 0) {
            desc.append ('*');
            desc.append ('.');
            desc.append ('*');
        } else {
            desc.append ('*');
            desc.append ('.');

            int count = include.size ();

            for (int i = 0; i < count; i++) {
                desc.append ((String) include.elementAt (i));

                if (i < (count - 1)) {
                    desc.append (',');
                    desc.append (' ');
                }
            }
        }

        desc.append (')');

        return desc.toString ();
    }

    /**
     * Sets the portion of the description that does not
     * include extensions.
     */
    public void setDescriptionTitle (String d) {
        this.descTitle = d;
    }

    /**
     * Gets the portion of the description that does not
     * include extensions.
     */
    public String getDescriptionTitle () {
        return descTitle;
    }

    public void setDirectoryValid (boolean valid) {
        directoryValid = valid;
    }

    public boolean isDirectoryValid () {
        return directoryValid;
    }

    public boolean isExcludeOnly () {
        return excludeOnly;
    }

    public void setExcludeOnly (boolean b) {
        excludeOnly = b;
    }

}

