/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

// Standard imports

import java.io.File;

//

public class PathHandle {
    private String      path = null;

    public static PathHandle createPathHandle (File inFile) {
        PathHandle      handle = null;

        if (inFile == null) {
            handle = new PathHandle ();
        } else {
            handle = PathHandle.createPathHandle (inFile.getAbsolutePath ());
        }

        return handle;
    }

    public static PathHandle createPathHandle (String inPath) {
        PathHandle      handle = null;

        handle = new PathHandle ();

        handle.setPath (inPath);

        return handle;
    }

    public static String createPathString (String inPath) {
        PathHandle      handle = null;

        handle = PathHandle.createPathHandle (inPath);

        return handle.getPath ();
    }

    public static String createPathString (File inFile) {
        PathHandle      handle = null;

        handle = PathHandle.createPathHandle (inFile);

        return handle.getPath ();
    }

    public PathHandle (){}

    public boolean isFile () {
        return FileUtil.isFile (getPath ());
    }

    public boolean isDirectory () {
        return FileUtil.isDirectory (getPath ());
    }

    public PathHandle getParent () {
        PathHandle      parent = null;
        File            parentFile = null;

        if (getFile () == null) {
            parent = null;
        } else {
            parentFile = getFile ().getParentFile ();

            if (parentFile == null) {
                parent = null;
            } else {
                parent = PathHandle.createPathHandle (parentFile);
            }
        }

        return parent;
    }

    public File getFile () {
        File    file = null;

        if (getPath () == null) {

            // done

        } else {
            file = new File (getPath ());
        }

        return file;
    }

    public void setPath (final String inPath) {
        if (inPath == null) {

            // done

        } else if (inPath.trim ().length () == 0) {
            path = new String ();
        } else {
            path = new String (inPath);
            path = FileUtil.toCanonicalPath (path);
            path = path.replace ('\\', '/').trim ();
        }
    }

    public String getPath () {
        return path;
    }

    public boolean parentOf (File childFile) {
        PathHandle      childHandle = null;

        childHandle = PathHandle.createPathHandle (childFile);

        return parentOf (childHandle);
    }

    public boolean parentOf (String childPath) {
        PathHandle      childHandle = null;

        childHandle = PathHandle.createPathHandle (childPath);

        return parentOf (childHandle);
    }

    public boolean parentOf (PathHandle childHandle) {
        boolean isParent = false;

        if (getPath () == null || childHandle.getPath () == null) {

            // Done

        } else {
            File        parentFile = null;

            parentFile = new File (getPath ());

            if (parentFile.isDirectory ()) {
                isParent =
                    childHandle.getComparePath ().startsWith (getComparePath ()
                        + '/');
            }
        }

        return isParent;
    }

    public boolean equals (String comparePath) {
        PathHandle      compareHandle = null;

        compareHandle = PathHandle.createPathHandle (comparePath);

        return equals (compareHandle);
    }

    public boolean equals (PathHandle compare) {
        boolean equal = false;

        if (getPath () == null || compare == null
                || compare.getPath () == null) {

            // done.

        } else {
            equal = getComparePath ().equals (compare.getComparePath ());
        }

        return equal;
    }

    public boolean hasExtension (String[] exts) {
        boolean has = false;

        for (int i = 0; i < exts.length; i++) {
            if (hasExtension (exts[i])) {
                has = true;

                break;
            }
        }

        return has;
    }

    public boolean hasExtension (String ext) {
        boolean has = false;

        if (getPath () == null || ext == null) {

            // done

        } else {
            if (isWindows ()) {
                has = ext.equalsIgnoreCase (getExtension ());
            } else {
                has = ext.equals (getExtension ());
            }
        }

        return has;
    }

    public boolean endsWith (String inPath) {
        boolean endsWith = false;

        if (inPath == null || getPath () == null) {

            // done

        } else {
            inPath = inPath.replace ('\\', '/');

            if (isWindows ()) {
                endsWith = getComparePath ().endsWith (inPath.toLowerCase ());
            } else {
                endsWith = getComparePath ().endsWith (inPath);
            }
        }

        return endsWith;
    }

    public String getExtension () {
        String  ext = new String ();
        int     index = -1;

        if (getPath () == null) {

            // done

        } else {
            index = getPath ().lastIndexOf ('.');

            if ((index > -1) && (getPath ().length () > index)) {
                ext = getPath ().substring (index + 1);
            } else {
                ext = new String ();
            }
        }

        return ext;
    }

    private String getComparePath () {
        String  thisPath = getPath ();

        if (getPath () == null) {

            // done

        } else if (isWindows ()) {
            thisPath = getPath ().toLowerCase ();
        } else {
            thisPath = new String (getPath ());
        }

        return thisPath;
    }

    private boolean isWindows () {
        return File.separatorChar == '\\';
    }

}

