/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

// Standard imports

import java.awt.*;
import java.beans.*;
import javax.swing.*;

/**
 * The CodeGenPage is a panel that you can add to a WizardDialog. It displays
 * a page title, instructions and a CodeGenPanel.
 * <P><I>
 * This class is used when creating a standalone wizard. Use the
 * OpenTools BasicWizardPage when developing JBuilder add-ins.
 * </I></P>
 */
public class TBWizardPage extends JPanel {
    transient private BorderLayout      layoutCustom;
    transient private BorderLayout      layoutMain;
    transient private GridBagLayout     layoutWizard;
    transient private JLabel            image;
    transient private JLabel            pageTitle;
    transient private JPanel            panelCustom;
    transient private JPanel            panelWizard;
    transient private JTextArea         instructions;
    transient private TBWizardPanel     wizardPanel = null;

    /**
     * Create a ToolBoxWizardPage.
     */
    public TBWizardPage () {
        try {
            jbInit ();
            pmInit ();
        } catch (Exception e) {
            e.printStackTrace ();
        }
    }

    /**
     * Get the instructions that display in a normal font under the page title.
     */
    public String getInstructions () {
        return instructions.getText ();
    }

    /**
     * Set the instructions that display in a normal font under the page title.
     * The instructions will word wrap.
     *
     * @param s
     * A paragraph to display in a read only word wrapped text area
     */
    public void setInstructions (String s) {
        instructions.setText (s);
    }

    /**
     * Get the text that appears in bold at the top of the page.
     */
    public String getPageTitle () {
        return pageTitle.getText ();
    }

    /**
     * Set the text that appears in bold at the top of the page.
     */
    public void setPageTitle (String s) {
        pageTitle.setText (s);
    }

    /**
     * Add a CodeGenPanel to a panel below the instruction text.
     *
     * @param panel
     * The CodeGenPanel containing swing controls for gathering
     * option values.
     */
    public void add (TBWizardPanel panel) {
        wizardPanel = panel;

        panelCustom.add (wizardPanel, BorderLayout.CENTER);
    }

    public TBWizardPanel getWizardPanel () {
        return wizardPanel;
    }

    public boolean checkPage () throws ToolException {
        return true;
    }

    // /
    // / PRIVATE METHODS
    // /

    /**
     * Do construtor operations that are not generated by
     * the JBuilder designer.
     */
    private void pmInit () {
        image.setText (new String ());
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit () throws Exception {
        panelWizard =
            (JPanel) Beans.instantiate (getClass ().getClassLoader (),
                                        JPanel.class.getName ());
        layoutWizard =
            (GridBagLayout) Beans.instantiate (getClass ().getClassLoader (),
                                               GridBagLayout.class.getName ());
        pageTitle = (JLabel) Beans.instantiate (getClass ().getClassLoader (),
                                                JLabel.class.getName ());
        instructions =
            (JTextArea) Beans.instantiate (getClass ().getClassLoader (),
                                           JTextArea.class.getName ());
        panelCustom =
            (JPanel) Beans.instantiate (getClass ().getClassLoader (),
                                        JPanel.class.getName ());
        layoutCustom =
            (BorderLayout) Beans.instantiate (getClass ().getClassLoader (),
                                              BorderLayout.class.getName ());
        layoutMain =
            (BorderLayout) Beans.instantiate (getClass ().getClassLoader (),
                                              BorderLayout.class.getName ());
        image = (JLabel) Beans.instantiate (getClass ().getClassLoader (),
                                            JLabel.class.getName ());

        this.setLayout (layoutMain);
        panelWizard.setLayout (layoutWizard);
        pageTitle.setFont (new Font (pageTitle.getFont ().getName (), 1, 12));
        pageTitle.setText ("PAGE TITLE");               // nores
        instructions.setLineWrap (true);
        instructions.setWrapStyleWord (true);
        instructions.setDisabledTextColor (SystemColor.windowText);
        instructions.setBackground (SystemColor.inactiveCaptionBorder);
        instructions.setEnabled (false);
        instructions.setText ("INSTRUCTIONS");          // nores
        instructions.setEditable (false);
        instructions.setFont (new Font (pageTitle.getFont ().getName (), 0,
                                        12));
        panelCustom.setLayout (layoutCustom);
        this.add (image, BorderLayout.WEST);
        this.add (panelWizard, BorderLayout.CENTER);
        panelWizard.add (pageTitle,
                         new GridBagConstraints (0, 0, 1, 1, 0.1, 0.1,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.BOTH,
                                                 new Insets (2, 5, 2, 5), 5,
                                                 5));
        panelWizard.add (instructions,
                         new GridBagConstraints (0, 1, 1, 1, 0.2, 0.2,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.BOTH,
                                                 new Insets (2, 5, 2, 5), 5,
                                                 5));
        panelWizard.add (panelCustom,
                         new GridBagConstraints (0, 2, 1, 2, 0.7, 0.7,
                                                 GridBagConstraints.CENTER,
                                                 GridBagConstraints.BOTH,
                                                 new Insets (2, 5, 2, 5), 5,
                                                 5));
    }

}

