/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

import java.util.ResourceBundle;

public class ToolException extends Exception {
    static ResourceBundle       res =
        ResourceBundle.getBundle ("org.enhydra.tool.common.Res");       // nores

    //

    transient private Throwable parent;

    /**
     * Create a ToolException with a specified error message.
     *
     * @param message
     * Message providing information on the nature of the exception.
     */
    public ToolException (String message) {
        super (message);
    }

    /**
     * Create a ToolException linked to a parent exception.
     *
     * @param chain
     * Parent exception to include in stack trace.
     * @param message
     * Message providing information on the nature of the exception.
     */
    public ToolException (Throwable chain, String message) {
        super (message);

        parent = chain;
    }

    /**
     * Get a string containing messages from all members of
     * exception chain.
     *
     * @return
     * All messages from exceptions in the chain with line feed delimiters.
     */
    public String getMessage () {
        StringBuffer    buf = new StringBuffer ();

        buf.append (ResUtil.format (res.getString ("Tool_Exception_n_0_"),
                                    super.getMessage ()));

        if (parent != null) {
            buf.append ('\n');
            buf.append ("    " + parent.getMessage ());         // nores
        }

        return buf.toString ();
    }

    /**
     * Print a recursive stack trace including all members of
     * exception chain.
     */
    public void printStackTrace () {
        super.printStackTrace ();

        if (parent != null) {
            parent.printStackTrace ();
        }
    }

}

