/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.xml.xmlc.dom.lazydom;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.codegen.*;
import org.enhydra.xml.xmlc.dom.*;
import org.enhydra.xml.xmlc.dom.generic.GenericDocCreateGenerator;
import org.enhydra.xml.xmlc.compiler.ClassGenerator;
import org.enhydra.xml.xmlc.compiler.ElementInfo;
import org.enhydra.xml.xmlc.compiler.ElementTable;
import org.w3c.dom.*;
import org.w3c.dom.html.HTMLDocument;

/**
 * Class to create code that creates documents using XMLCDomFactory objects.
 */
public class LazyDOMDocCreateGenerator extends GenericDocCreateGenerator {
    /**
     * Constructor.
     */
    public LazyDOMDocCreateGenerator(LazyDomFactory domFactory) {
        super(domFactory);
    }

    /**
     * Generate code to create a document from a TemplateDOM.
     * @param document Document that will be recreated.
     * @param templateFieldName Name of template field.
     * @param docFieldName Name of field for document object.
     */
    public void generateDocCreateFromTemplate(Document document,
                                              String templateFieldName,
                                              String docFieldName,
                                              JavaCode body) {
        String docClass = document.getClass().getName();

        body.addln(docFieldName + " = (" + docClass + ")(("
                   + LazyDomFactory.class.getName()
                   + ")" + ClassGenerator.DOM_FACTORY_FIELD_NAME
                   + ").createDocument(" + templateFieldName + ");");
    }
}

