/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ClassPathBuilder.java,v 1.1.2.1 2000/11/16 06:36:31 markd Exp $
 */

package org.enhydra.xml.xmlc.reloading;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;

/**
 * Class used to assemble a class path from a ClassLoader delegation chain.
 */
class ClassPathBuilder {
    /** Class path being built */
    private ArrayList fClassPath = new ArrayList();

    /**
     * Constructor.
     */
    public ClassPathBuilder(boolean initWithSystemClassPath) {
        if (initWithSystemClassPath) {
            addSystemClassPath();
        }
    }

    /**
     * Constructor, initializing with system class path.
     */
    public ClassPathBuilder() {
        this(true);
    }

    /**
     * Add a URL to the class path.
     */
    public void addURL(URL url) {
        fClassPath.add(url);
    }

    /**
     * Add URLs to the end of the class path.
     */
    public void addURLs(URL[] urls) {
        if (urls != null) {
            for (int idx = 0; idx < urls.length; idx++) {
                fClassPath.add(urls[idx]);
            }
        }
    }

    /**
     * Add the system CLASSPATH to the classpath being assembled.
     */
    public void addSystemClassPath() {
        try {
            String path = System.getProperty("java.class.path");
            if (path != null) {
                StringTokenizer tokenizer = new StringTokenizer(path, File.pathSeparator);
                while (tokenizer.hasMoreTokens()) {
                    addURL(new URL("file", "", tokenizer.nextToken()));
                }
            }
        } catch (MalformedURLException except) {
            throw new XMLCReloadingException("Error parsing system CLASSPATH",
                                             except);
        }
    }

    /**
     * Get the assembled CLASSPATH as an array of URLs.
     */
    public URL[] getClassPath() {
        return (URL[])fClassPath.toArray(new URL[fClassPath.size()]);
    }
}
