/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package tests.org.enhydra.xml.lazydom;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.FileWriter;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.enhydra.xml.lazydom.*;
import org.enhydra.xml.io.DOMParser;
import org.enhydra.xml.dom.DOMInfo;

/**
 * Basic tests of Lazy DOM.
 *    BasicTest input outputPrefix
 */
public class BasicTests extends TestSupport {
    /** Parameters */
    private String fInput;
    private String fOutputPrefix;

    /**
     * Constructor.  Parse arguments.
     */
    private BasicTests(String[] args) {
        if (args.length != 2) {
            System.err.println("Wrong # args: BasicTest input outputPrefix");
        }
        fInput = args[0];
        fOutputPrefix = args[1];
    }

    /**
     * Simple test on insulin3.bioml
     */
    private void test1_1(LazyDocument testDoc) throws SAXException, IOException {
        LazyDocument lazyDoc = LazyDOMParser.build(LazyDocument.class,
                                                   testDoc);
        printNode("Unexpanded Lazy DOM", lazyDoc,
                  fOutputPrefix + ".simple.unexpanded-lazy.info");
        traverseTree(lazyDoc);
        printNode("Expanded Lazy DOM", lazyDoc,
                  fOutputPrefix + ".simple.expanded-lazy.info");
    }

    /**
     * Basic modifications on insulin3.bioml, 
     */
    private void test1_2(LazyDocument testDoc) throws SAXException, IOException {
        LazyDocument lazyDoc = LazyDOMParser.build(LazyDocument.class,
                                                    testDoc);
        DocInfo docInfo = new DocInfo(lazyDoc);
        if (false) {//FIXME        
        // Add a text node to an element.  
        Element lazyElem = docInfo.getByAttrValue(lazyDoc, "label", "HUMINS locus");
        lazyElem.appendChild(lazyDoc.createTextNode("@ new node #1 @"));

        // Add a attribute
        lazyElem = docInfo.getByAttrValue(lazyDoc, "end", "4992");
        lazyElem.setAttribute("newAttr", "new value");
        }
        // Expand the parent
        Element lazyElem = docInfo.getByAttrValue(lazyDoc, "format", "PIR");

        Node parent = (Element)lazyElem.getParentNode();
        String name = parent.getNodeName();  // Make sure we got something
        
        printNode("Modified, unexpanded Lazy DOM", lazyDoc,
                  fOutputPrefix + ".mod.unexpanded-lazy.info");
        traverseTree(lazyDoc);
        printNode("Modified and expanded Lazy DOM", lazyDoc,
                  fOutputPrefix + ".mod.expanded-lazy.info");
    }

    /**
     * Test cloning of attribute on shallow clone.  This failed at one
     * time.
     */
    private void test1_3(LazyDocument testDoc) throws SAXException, IOException {
        String tagName = "organism";
        LazyDocument lazyDoc = LazyDOMParser.build(LazyDocument.class,
                                                   testDoc);
        // Find an Element, without attributes expanded
        LazyElement elem = (LazyElement)getElementByTagName(lazyDoc, lazyDoc, tagName);
        if (elem == null) {
            fail("test1_3: element null for tag: " + tagName);
        }
        if (elem.areAttributesExpanded()) {
            fail("test1_3: attributes expanded");
        }
        LazyElement cloneElem = (LazyElement)elem.cloneNode(false);
        if (!elem.areAttributesExpanded()) {
            //FIXME:            fail("test1_3: attributes not expanded");
        }
        String labelValue = cloneElem.getAttribute("label");
        if ((labelValue == null) || !labelValue.equals("Homo sapiens (human)")) {
            fail("test1_3: attribute \"label\" has wrong value: \"" + labelValue + "\"");
        }
    }

    /**
     * Tests on insulin3.bioml.
     */
    private void test1() throws SAXException, IOException {
        LazyDocument testDoc = LazyDOMParser.parse(LazyDocument.class,
                                                   fInput);
        printNode("Parsed DOM", testDoc,
                  fOutputPrefix + ".orig-dom.info");
        test1_1(testDoc);
        test1_2(testDoc);
        test1_3(testDoc);
    }

    /**
     * Entry.
     */
    public static void main(String args[]) {
        try {
            BasicTests tests = new BasicTests(args);
            tests.test1();
        } catch(Exception except) {
            except.printStackTrace();
            System.exit(1);
        }
    }
}
