#
# Enhydra Java Application Server Project
# 
# The contents of this file are subject to the Enhydra Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License on
# the Enhydra web site ( http://www.enhydra.org/ ).
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
# the License for the specific terms governing rights and limitations
# under the License.
# 
# The Initial Developer of the Enhydra Application Server is Lutris
# Technologies, Inc. The Enhydra Application Server and portions created
# by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
# All Rights Reserved.
# 
# Contributor(s):
# 
# $Id$
#

#
# Tests are executed and compared to a set of know results.  The main set of
# tests are executed several times with different sets of parameters.  Each
# execution is refered to as a `test group'.  The resulting output can be
# different for each parser.  This is due to warnings and error messages
# generated, as well as different ways HTML is interpreted.  so expected HTML
# results are kept for each parser.  Different DOMs produce different DOM
# dumps. A subdirectory of expected DOM dumps is kept for each DOM type.
#
# The structure of the expected results is:
#   - expected/
#     - $PARSER/
#       - $DOM/
#
# Different test group are executed by recursively calling this makefile.  The
# make parameters that define a test group are:
# 
#  PARSER - The name of the parser.
#  DOM - The name of the DOM.
#  RECOMP - `recomp' or `comp'. Compile with or without the -for-recomp
#            option.  As many tests as possible are run with the -for-recomp
#            option, since it has to save and process options.
#  STANDALONE - `sa' or `eh' - Run standalone jar or with enhydra jar.  This
#             file only runs the standalong tests if jar has been built.
#
# This make file recursively calls itself twice with the PARSER
# variable set to the different parsers.
#
# Make is recursively called to combine tests with different parameters
# The following is the test hierarchy.  They lower down on the test
# tree, the quicker the test will run.
#
# The mode that is run is controlled by parameters passed on the command line:
#
#   1) STANDALONE=eh|sa
#   2) RECOMP=comp|recomp (recomp not done on standalone)
#   3) DOM=lazydom|xerces
#   4) PARSER=tidy|swing for HTML, PARSER=xerces for XML
#
#  FIXME: this should be a passed parameter, not an env var..
#  Setting the environment variable
#     XMLC_TEST_NONSTOP=YES
#  will keep tests from exiting non-zero on a failure.

CLASS_OUTPUT = ${XMLC_TEST_CLASSES}

include $(MODROOT)/config.mk

ERROR_EXIT=exit 1
ifeq (${XMLC_TEST_NONSTOP},YES)
    # Can grep for exit=1
    ERROR_EXIT=echo would exit=$$?
endif

# Disable messages from TYA JIT on *BSD/Linux
TYA_LOGFILE=/dev/null
export TYA_LOGFILE

# There are some problems with certain tests run using the Linux JDK on Linux
# or FreeBSD.  Set a variable that is used to disable these tests.
# FIXME: Determine if there is a workaround.
SYSTEM=$(shell uname -s)
ifeq ($(SYSTEM), Linux)
   BSD_OR_LINUX=YES
else
ifeq ($(SYSTEM), FreeBSD)
   BSD_OR_LINUX=YES
else
   BSD_OR_LINUX=NO
endif
endif

#
# Default parameters if not set.
#
ifeq (${RECOMP},)
    RECOMP=comp
endif
ifeq (${XMLC_STANDALONE},YES)
    STANDALONE=sa
endif
ifeq (${STANDALONE},)
    STANDALONE=eh
endif

GROUP_PKG=${PARSER}.${DOM}.${RECOMP}.${STANDALONE}
GROUP_DIR=${PARSER}/${DOM}/${RECOMP}/${STANDALONE}
TEST_PACKAGE=${PACKAGE}.${GROUP_PKG}

TEST_PACKAGE_DIR=${XMLC_TEST_CLASSES}/$(subst .,/,${TEST_PACKAGE})

OUTDIR=output/${GROUP_DIR}
EXPECTDIR=expected/${PARSER}
DOMEXPECTDIR=expected/${PARSER}/${DOM}
TMP_DIR = tmp

# XMLC common options.
XMLC_FLAGS = -sourceout ${TMP_DIR} -parser ${PARSER}

# Use compiler to dump
DUMPDOM = ${XMLC_CMD} -parser ${PARSER} -dump -methods -docout docout.tmp
RUNOBJECT = ../run-object

# Explicty set the class path before classing executing a command
# Turn on -x, so command can start with @ to avoid echo of long path.
SET_CLASSPATH = CLASSPATH="$(TEST_CLASSPATH)"; export CLASSPATH ; set -x
