/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package tests.org.enhydra.xml.xmlc.perf;

import org.enhydra.xml.xmlc.*;
import org.enhydra.xml.xmlc.html.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import java.util.Date;

/**
 * Run XMLC generated objects for performance testing.
 *   PerfRunner class
 */
public class PerfRunner {
    /**
     * Constants controling number of tests.
     */
    private static final int TEST_COUNT = 10; //4;
    private static final int REPEAT_COUNT = 10; //20;


    /**
     * Create a XMLC document object and convert it to a string a specified
     * number of times.
     */
    private void testInstantiation(Class docClass,
                                   int repeatCount) {
        XMLCFactory factory = new XMLCStdFactory(null, null);
        
        for (int cnt = 0; cnt < repeatCount; cnt++) {
            XMLObject xmlcObj = factory.create(docClass);
            String docStr = xmlcObj.toDocument();
        }
    }

    /**
     *  Run testInstantiation a number of times.
     */
    private void runTestInstantiation(String docClassName,
                                      int testCount,
                                      int repeatCount) 
        throws ClassNotFoundException {
        Class docClass = Class.forName(docClassName);
        double timeSec[] = new double[testCount];
        
        for (int cnt = 0; cnt < testCount; cnt++) {
            long start = System.currentTimeMillis();

            testInstantiation(docClass, repeatCount);
            
            timeSec[cnt] = (System.currentTimeMillis() - start)/1000.0;
        }

        System.out.println("Test class = " + docClass.getName());
        System.out.println("Test = " + TEST_COUNT
                           + ", Repeat = " + REPEAT_COUNT);
        double totalSec = 0;
        for (int cnt = 0; cnt < testCount; cnt++) {
            System.out.println("Pass " + cnt + ": "
                               + timeSec[cnt]/repeatCount
                               + "\tsec/iteration");
            totalSec += timeSec[cnt];
        }
        double totalAveTimeSec = totalSec/(repeatCount*testCount);
        System.out.println("Ave:  " + totalAveTimeSec + " sec");
        System.out.println("Rate: " + 1.0/totalAveTimeSec + "/sec");
    }


    public static void main(String[] args) throws ClassNotFoundException {
        if (args.length != 1) {
            System.err.println("wrong # args: PerfRunner class");
            System.exit(1);
        }
        PerfRunner tests = new PerfRunner();
        tests.runTestInstantiation(args[0], TEST_COUNT, REPEAT_COUNT);
    }
}
